<?php defined('SYSPATH') or die('No direct script access.');

// -- Environment setup --------------------------------------------------------

// Load the core Kohana class
require SYSPATH.'classes/kohana/core'.EXT;

if (is_file(APPPATH.'classes/kohana'.EXT))
{
	// Application extends the core
	require APPPATH.'classes/kohana'.EXT;
}
else
{
	// Load empty core extension
	require SYSPATH.'classes/kohana'.EXT;
}

/**
 * Set the default time zone.
 *
 * @see  http://kohanaframework.org/guide/using.configuration
 * @see  http://php.net/timezones
 */
date_default_timezone_set('America/Toronto');

/**
 * Set the default locale.
 *
 * @see  http://kohanaframework.org/guide/using.configuration
 * @see  http://php.net/setlocale
 */
setlocale(LC_ALL, 'en_US.utf-8');

/**
 * Enable the Kohana auto-loader.
 *
 * @see  http://kohanaframework.org/guide/using.autoloading
 * @see  http://php.net/spl_autoload_register
 */
// custom autoloader to deal with namespace '\' character
function custom_auto_load($class)
{
	try
	{
		// Transform the class name into a path
//			$file = str_replace('_', '/', strtolower($class));
		$file = str_replace(array('_','\\'), '/', strtolower($class));

		if ($path = Kohana::find_file('classes', $file))
		{
			// Load the class file
			require $path;

			// Class has been found
			return TRUE;
		}

		// Class is not in the filesystem
		return FALSE;
	}
	catch (Exception $e)
	{
		Kohana_Exception::handler($e);
		die;
	}
}

//spl_autoload_register(array('Kohana', 'auto_load'));
spl_autoload_register('custom_auto_load');

/**
 * Enable the Kohana auto-loader for unserialization.
 *
 * @see  http://php.net/spl_autoload_call
 * @see  http://php.net/manual/var.configuration.php#unserialize-callback-func
 */
ini_set('unserialize_callback_func', 'spl_autoload_call');

// -- Configuration and initialization -----------------------------------------

/**
 * Set the default language
 */
I18n::lang('en-us');

/**
 * Set Kohana::$environment if a 'KOHANA_ENV' environment variable has been supplied.
 *
 * Note: If you supply an invalid environment name, a PHP warning will be thrown
 * saying "Couldn't find constant Kohana::<INVALID_ENV_NAME>"
 */
if (!empty($_SERVER['KOHANA_ENV'])) {
	Kohana::$environment = constant('Kohana::'.strtoupper($_SERVER['KOHANA_ENV']));
} else {
//	Kohana::$environment = Kohana::DEVELOPMENT;
	Kohana::$environment = Kohana::PRODUCTION;
}
/**
 * Initialize Kohana, setting the default options.
 *
 * The following options are available:
 *
 * - string   base_url    path, and optionally domain, of your application   NULL
 * - string   index_file  name of your index file, usually "index.php"       index.php
 * - string   charset     internal character set used for input and output   utf-8
 * - string   cache_dir   set the internal cache directory                   APPPATH/cache
 * - boolean  errors      enable or disable error handling                   TRUE
 * - boolean  profile     enable or disable internal profiling               TRUE
 * - boolean  caching     enable or disable internal caching                 FALSE
 */
Kohana::init(array(
	'base_url'   => '/',
	'profile' => (Kohana::$environment == Kohana::DEVELOPMENT),
	'index_file' => FALSE,
	'caching' => (Kohana::$environment == Kohana::PRODUCTION),
));

/**
 * Attach the file write to logging. Multiple writers are supported.
 */
Kohana::$log->attach(new Log_File(APPPATH.'logs'));

/**
 * Attach a file reader to config. Multiple readers are supported.
 */
Kohana::$config->attach(new Config_File);

/**
 * Enable modules. Modules are referenced by a relative or absolute path.
 */
Kohana::modules(array(
	// 'auth'       => MODPATH.'auth',       // Basic authentication
	// 'cache'      => MODPATH.'cache',      // Caching with multiple backends
	// 'codebench'  => MODPATH.'codebench',  // Benchmarking tool
	 'musterbase'      => MODPATH.'musterbase',      // Musterdb application module
	 'muster'      => MODPATH.'muster',      // Musterdb code module
	 'database'   => MODPATH.'database',   // Database access
	 'image'      => MODPATH.'image',      // Image manipulation
	 'purifier'      => MODPATH.'purifier',      // Musterdb application module
	// 'orm'        => MODPATH.'orm',        // Object Relationship Mapping
	// 'unittest'   => MODPATH.'unittest',   // Unit testing
	// 'userguide'  => MODPATH.'userguide',  // User guide and API documentation
	));

/**
 * Set the routes. Each route must have a minimum of a name, a URI and a set of
 * defaults for the URI.
 */
Route::set('nodes','<name>/<id>',
	array('name' => 'node'))
	->defaults(array(
		'controller' => 'documents' // redirects to simplest controller for further redirect to proper controller
	));
Route::set('blogs','<controller>(/<code>(/<selector>(/<id>)))',
	array('controller' => 'blogs'));
Route::set('pages','<controller>(/<selector>(/<id>))',
	array('controller' => 'pages'));
Route::set('folders','<controller>(/<selector>(/<id>))',
	array('controller' => '(folders|topics)'));
Route::set('library','<controller>(/<selector>(/<id>(/<code>)))',
	array('controller' => 'library'));
Route::set('error','<controller>(/<id>)',
	array('controller' => 'error'));
Route::set('lists','<controller>/<type>(/<selector>(/<code>(/<id>)))',
	array('controller' => '(sublists|subitems)'));
Route::set('documents', '<controller>(/<id>)',
	array('controller' => 'documents',
			'id' => '([0-9]++)'));
Route::set('profiles','<controller>(/<id>)',
	array('controller' => 'profiles'));
Route::set('default', '(<controller>(/<action>(/<id>)))')
	->defaults(array(
		'controller' => 'home',
		'action'     => 'index',
	));
Cookie::$salt = "enigma";
