
/* dlml dynamic menu manager version May 2009
copyright (c) 2007 - 2009 Henrik Bechmann, Bechmann Software Services, Toronto Canada, all rights reserved.
www.bechmann.ca. General Public Licence.
*/
/*
- register with DLManager "select" to detect when to close the menu for modes other than flyover
- menu should first appear with slow expand with ready event.
- add MenuPanel, MenuPanelItem, MenuTab, MenuTabStrip, MenuTabColumn
- add: popupmenu, dlmodes, menu responses and response registration
*/
//======================================================================================
// ---------[ Menu Manager ]-----------
//======================================================================================

MenuManager = new (DLPrototype({
$extends:DLModule,
$prototype:{
 name:"MenuManager",
 menuactions:{},
 // library of event  functions (closures) for re-use
 activateevent:DLEventTerminal("activate"), // cursor inside element (bar or column)
 deactivateevent:DLEventTerminal("deactivate"), // cursor leaves element (bar or column)
 selectevent:DLEventTerminal("select"), // menu item active
 deselectevent:DLEventTerminal("deselect") // menuitem deactive
}
}))();

//==========================
//------[ Prototypes ]------
//==========================

//--------------[Menu]----------------

Menu = DLPrototype({
$extends:DLPlatform,
$constructor:function () {
	// for control of appearance/disappearance of menu tree
	this.timeoutvalue=500;
	this.istimeout=false;
	this.openmenubars=[];
	var menu=this; // for the following closure
	this.closemenu=function () {
		var length = menu.openmenubars.length;
		for (var i=0;i<length;i++) {
			var menubar = menu.openmenubars[i];
			//for popup close all including top, otherwise not including top
			((!(menubar.container instanceof Menu)) && (!(menubar.container instanceof PopupMenu))) 
				&& menubar.act("hide");
		}
		menu.openmenubars=[];
	}
},
$prototype:{
 //data
	type:'Menu',
	dlmode:"flyover",// or 'select-and-flyover' or 'select'
	html:'<div class="menublock-platform"></div>',
	metadata:{
		ATTRIBUTES:['dlstyle','dlclass','dlmode'],
		PARSERS:{dlstyle:DLParse.dlstyle,dlclass:DLParse.dlclass},
		SETTINGS:{
			defaults:{
				COLLECTIONS:{
					defaults:DLData.defaultstyleattributetree
				}
			}
		},
		COLLECTIONS:{
			defaults:DLData.defaultstyleattributetree
		}
	},
	//functions
	setMenuBranchTimeout: function () {
		if (!this.istimeout) {
			this.timeoutindex=setTimeout(this.closemenu,this.timeoutvalue);
			this.istimeout=true;
		}
	},
	clearMenuBranchTimeout: function () {
		clearTimeout(this.timeoutindex);
		this.istimeout=false;
	}
}
});
// -------------[PopupMenu]--------------
PopupMenu = DLPrototype(
{$extends:Menu
});

//--------------[MenuBar]----------------

//abstraction for menustrip and menucolumn, ie vertical or horizontal menu bar
MenuBar = DLPrototype({ // abstract
$extends:DLContainer,
$prototype:{
	//data
	type:'MenuBar',
	index:{frame:{styles:{display:"none"}}},
	states:{
		"ready":{index:{
			frame:{effects:{"show":"fast"}},
			box:{styles:{"border-color":"#eee gray gray #eee"}}}
		},
		"hover":{index:{
			box:{styles:{"border-color":"aqua blue blue aqua"}}}
		},
		"invisible":{index:{
			box:{styles:{"border-color":"#eee gray gray #eee"}},
			frame:{effects:{"hide":"fast"}}}
		}
	},
	transitions: {
		"activate": { // element has focus
			"ready":"hover"
		},
		"deactivate": { // element loses focus
			"hover":"ready"
		},
		"show": { // element is shown
			"invisible":"ready"
		},
		"hide": { // element is hidden
			"ready":"invisible",
			"hover":"invisible"
		}
	},
	metadata:DLServices.extend(
		DLServices.clone(DLData.stateattributetree),
		DLData.defaultstateattributetree),
	//functions
	initialize:function() {
		var menuitem;
		// cross-link related menuitem and menubar
		if ((this.parent instanceof MenuNode) &&
			((menuitem = this.parent.children[0]) instanceof MenuItem)) {
			this.menuitem=menuitem;
			menuitem.menubar=this; 
			menuitem.setstate('closed').renderstate(); // follows init of menuitem by structure
		}
		// initialize state
		((this.container instanceof Menu) && (!(this.container instanceof PopupMenu))) 
			? this.setstate("ready")
			: this.setstate("invisible");
		this.renderstate();
		// set events
		$(this.elements.frame)
			.bind("mouseover",MenuManager.activateevent)
			.bind("mouseout",MenuManager.deactivateevent);
	},
	act: function(action,data) {
		this.changestate(action).renderstate();
		switch (action) {
			case "activate": this.activate(data); break;
			case "deactivate": this.deactivate(data); break;
			case "show": this.show(data); break;
			case "hide": this.hide(data); break;
		}
		return this;
	},
	activate:function(data) {
		//activate in MenuBar implies deactivate of parent MenuBar
		(this.container instanceof MenuBar) && this.container.act("deactivate");
		(this.platform.dlmode=="flyover") && this.platform.clearMenuBranchTimeout();
	},
	deactivate:function(data) {
		(this.platform.dlmode=="flyover") && this.platform.setMenuBranchTimeout(); 
	},
	show:function(data) { 
		this.menuitem && this.menuitem.act("open"); // synchronize menuitem display
	},
	hide:function(data) { 
		this.menuitem && this.menuitem.act("close"); // synchronize menuitem display
	},
	setbranchdisplay:function() {
		var menubar=this, menu=this.platform,
		openmenubars=menu.openmenubars,closemenubars,
		len,sharedoffset;
		//save current branch list
		closemenubars=openmenubars.shallowCopy();
		//collect list of branches to root (Menu) from reference branch
		openmenubars=[menubar];
		while (menubar.container && (menubar.container instanceof MenuBar)) {
			menubar=menubar.container;
			openmenubars.push(menubar);
		}
		openmenubars=openmenubars.reverse();//top down, root is first
		//leave aside shared part of the new and old branches
		sharedoffset=-1;
		len=Math.min(openmenubars.length,closemenubars.length);
		for (var index=0;index<len;index++) {
			if (openmenubars[index]===closemenubars[index])
				sharedoffset=index;
			else break;
		}
		//hide branch nodes of previous branch beyond shared, starting with leaf;
		for (var index=closemenubars.length-1;index>sharedoffset;index--) {
			menubar=closemenubars[index];
			menubar.act("hide");
		}
		//coerce display of current branch nodes beyond shared
		len=openmenubars.length;
		for (var index=sharedoffset+1;index<len;index++) {
			menubar=openmenubars[index];
			menubar.act("show");
		}
		menu.openmenubars=openmenubars;
	}
}}
);

//--------------[MenuStrip]----------------

MenuStrip = DLPrototype({
$extends:MenuBar,
$prototype:{
	//data
	type:'MenuStrip',
	html:'<div class="menustrip-frame"><div class="menustrip-box" dlindex="box"><div dlindex="liner" class="menustrip-liner"><div style="clear:both;"></div></div></div></div>',
	//functions
	initialize:function() {
		(this.container && (!(this.container instanceof Menu))) && $(this.elements.frame).addClass("menustripdropdown");
		this.base.initialize && this.base.initialize.call(this);
	}
}
});

//--------------[MenuColumn]----------------

MenuColumn = DLPrototype({
$extends:MenuBar,
$prototype:{
	//data
	type:'MenuColumn',
	html:'<div class="menucolumn-frame"><div class="menucolumn-box" dlindex="box"><div class="menucolumn-liner"></div></div></div>',
	//functions
	initialize:function() {
		(this.container instanceof MenuStrip)
			? $(this.elements.frame).addClass("menudropdown")
			: $(this.elements.frame).addClass("menuflyout");
		this.base.initialize && this.base.initialize.call(this);
	}
}
});

//--------------[MenuItem]----------------

MenuItem = DLPrototype({
$extends:DLObject,
$prototype:{
	//data
	type:'MenuItem',
	html:'<div class="menuitem-frame"><div dlindex="box"><div class="menuitem-liner"></div></div></div>',
	// orthogonal state sets: ready, open, closed; hover, active
	states: {
		"ready":{index:{box:{classes:{addClass:"menuitem-ready",removeClass:null}}}},
		"ready-hover":{index:{box:{classes:{addClass:"menuitem-ready-hover",removeClass:null}}}},
		"ready-active":{index:{box:{classes:{addClass:"menuitem-ready-active",removeClass:null}}}},
		"ready-hover-active":{index:{box:{classes:{addClass:"menuitem-ready-hover-active",removeClass:null}}}},

		"open":{index:{box:{classes:{addClass:"menuitem-open",removeClass:null}}}},
		"open-hover":{index:{box:{classes:{addClass:"menuitem-open-hover",removeClass:null}}}},
		"open-active":{index:{box:{classes:{addClass:"menuitem-open-active",removeClass:null}}}},
		"open-hover-active":{index:{box:{classes:{addClass:"menuitem-open-hover-active",removeClass:null}}}},

		"closed":{index:{box:{classes:{addClass:"menuitem-closed",removeClass:null}}}},
		"closed-hover":{index:{box:{classes:{addClass:"menuitem-closed-hover",removeClass:null}}}},
		"closed-active":{index:{box:{classes:{addClass:"menuitem-closed-active",removeClass:null}}}},
		"closed-hover-active":{index:{box:{classes:{addClass:"menuitem-closed-hover-active",removeClass:null}}}}
	},
	transitions: {
		"activate": {
			"ready": "ready-hover",
			"ready-active": "ready-hover-active",
			"open": "open-hover",
			"open-active": "open-hover-active",
			"closed": "closed-hover",
			"closed-active": "closed-hover-active"
		},
		"deactivate": {
			"ready-hover": "ready",
			"ready-hover-active": "ready-active",
			"open-hover": "open",
			"open-hover-active": "open-active",
			"closed-hover": "closed",
			"closed-hover-active": "closed-active"
		},
		"select": {
			"ready": "ready-active",
			"ready-hover": "ready-hover-active",
			"open": "open-active",
			"open-hover": "open-hover-active",
			"closed": "closed-active",
			"closed-hover": "closed-hover-active"
		},
		"deselect": {
			"ready-active": "ready",
			"ready-hover-active": "ready-hover",
			"open-active": "open",
			"open-hover-active": "open-hover",
			"closed-active": "closed",
			"closed-hover-active": "closed-hover"
		},
		"open": {
			"ready": "open",
			"ready-hover": "open-hover",
			"ready-active": "open-active",
			"ready-hover-active": "open-hover-active",
			"closed": "open",
			"closed-hover": "open-hover",
			"closed-active": "open-active",
			"closed-hover-active": "open-hover-active"
		},
		"close": {
			"ready": "closed",
			"ready-hover": "closed-hover",
			"ready-active": "closed-active",
			"ready-hover-active": "closed-hover-active",
			"open": "closed",
			"open-hover": "closed-hover",
			"open-active": "closed-active",
			"open-hover-active": "closed-hover-active"
		}
	},
	metadata:DLData.commonattributetree,
	//functions
	initialize: function () {
		// hook events
		$(this.elements.frame)
			.toggle(MenuManager.selectevent,MenuManager.deselectevent)
			.bind("mouseover",MenuManager.activateevent)
			.bind("mouseout",MenuManager.deactivateevent);
		// set float by orientation
		(this.parent instanceof MenuStrip) && $(this.elements.frame).css("float","left");
		this.setstate('ready').renderstate();
	},
	act: function(action,data) {
		this.changestate(action).renderstate();
		switch (action) {
			case "activate": this.activate(data); break;
			case "deactivate": this.deactivate(data); break;
			case "select": this.select(data); break;
			case "deselect": this.deselect(data); break;
			case "open": break;
			case "close": break;
		}
		return this;
	},
	activate:function(data) {
		var menubar = this.container;
		(menubar && (menubar instanceof MenuBar)) && menubar.act("activate");
		menubar=(this.menubar || menubar); // to open...
		(menubar && (menubar instanceof MenuBar)) && menubar.setbranchdisplay();
	},
	deactivate:function(data) {
		var menubar = this.container;
		(menubar.platform.dlmode=="flyover") && menubar.platform.setMenuBranchTimeout(); 
	},
	select:function(data) {
		this.active=true;
		var target = data.event.target;
		if (target.tagName != 'A')
			target = $(target,this.elements.frame).parents('a').get(0);
		if (target) {
			$(this.elements.frame).click(); // revert state by forcing toggle
			this.platform.closemenu();
			window.location = target.href; // click bubbling disabled
		}
		else if (this.menuaction) { // check for menuaction *** NOT YET IMPLEMENTED ***
			// create menuaction object, run MenuManager.menuactions[this.menuaction](menuaction)
		}
	},
	deselect:function(data) {
		this.active=false;
	}
} // $prototype
}); // MenuItem = DLProtypye

//--------------[MenuScaffold]----------------
// to properly set submenus when entering non-menuitem areas
MenuScaffold = DLPrototype({
$extends:DLObject,
$prototype:{
	//data
	type:'MenuScaffold',
	initialize: function () {
		$(this.elements.frame)
			.bind("mouseover",MenuManager.activateevent);
	},
	act: function(action,data) {
		switch (action) {
			case "activate": this.activate(data); break;
		}
	},
	activate: function(data) {
		var menubar=this.container; // to set - close any open menubars
		if (menubar && (menubar instanceof MenuBar)) {
			menubar.act('activate');
			menubar.setbranchdisplay();
		}; 
	}
}
});

//--------------[MenuNode]----------------

MenuNode = DLPrototype({
$extends:MenuScaffold,
$prototype:{
	//data
	type:'MenuNode',
	html:'<div class="menunode-frame"></div> ',
	// metadata:{},
	//functions
	initialize:function(){
		this.base.initialize && this.base.initialize.call(this);
		(this.parent instanceof MenuStrip) && $(this.elements.frame).css("float","left");
	}
}
});

//--------------[MenuDivider]----------------

MenuDivider = DLPrototype({
$extends:MenuScaffold,
$prototype:{
	//data
	type:'MenuDivider',
	html:'<div class="menudivider"></div>',
	metadata:{
		ATTRIBUTES:['dlstyle','dlclass'],
		PARSERS:{dlstyle:DLParse.dlstyle,dlclass:DLParse.dlclass}
	}
}
});

//--------------[MenuContent]----------------

MenuContent = DLPrototype({
$extends:MenuScaffold,
$prototype:{
	//data
	type:'MenuContent',
	html:'<div></div>',
	metadata:{
		ATTRIBUTES:['dlstyle','dlclass'],
		PARSERS:{dlstyle:DLParse.dlstyle,dlclass:DLParse.dlclass}
	}
}
});

//=================================================================
//-------------[ REGISTER MENU MANAGER PROTOTYPES ]----------------
//=================================================================

MenuManager.register(
[
Menu,
PopupMenu,
// MenuBar, // abstract
MenuStrip,
MenuColumn,
MenuItem,
// MenuScaffold // abstract
MenuNode,
MenuDivider,
MenuContent
]
);
