/*
Gargoyle components (dloutlines.js) copyright (c) 2007 - 2011 Henrik Bechmann, bechmann.ca, 
	Bechmann Software Services, Toronto Canada, all rights reserved.
website: www.dlml.org/gargoyle
licence: BSD
dl = "dynamic layout"
*/
/*
add drag & drop internal, and drag & drop with trays
selectable
*/
//================[ OUTLINES ]=====================//

OutlineManager = new (DLPrototype({
$extends:DLModule,
$prototype:{
 name:"OutlineManager"
}
}))();

Outlines = DLPrototype({
$extends:DLPlatform,
$contructor: function() {
	this.selected = null;
	this.dllocked = false;
},
$prototype:{
	type:'Outlines',
	html:'<div class="outlines"><div class="outlines-box"><div class="outlines-liner"></div></div></div>',
	metadata:{ATTRIBUTES:['dldocked'],DLPARSERS:DLParse['boolean']},
	initialize: function () {
		if (this.dldocked) $(this.elements.frame).addClass('docked');
		// look for window;
		var container = this.container;
		while (container) {
			if (container instanceof Window) {
				this.registercomponent('window',container);
				break;
			}
			container = container.container;
		}
	},
	setselected:function(dlobject) {
		var subtitle;
		if (this.selected === dlobject) return;
		if (this.selected) $(this.selected.elements.frame).removeClass('selected');
		this.selected = dlobject;
		this.notify('selected',{dlobject:dlobject});
		this.components.window && (subtitle = this.components.window.components.subtitle) &&
			$(subtitle.elements.liner).text(" - " + $(dlobject.elements.liner).text());
	}
}
});

Outline = DLPrototype({
$constructor:function () {
	this.dlmode = 'toggle-each'; // or "toggle"
	this.dltheme = 'winter';
},
$extends:DLContainer,
$prototype:{
	type:'Outline',
	html:'<div class="outline"><div class="outline-box" dlindex="box"><div class="outline-liner"></div></div></div>',
	metadata:
		DLServices.extend(
			DLServices.clone(DLData.styleattributetree),
			{
				ATTRIBUTES:['dlmode','dltheme','dlconfig'],
				PARSERS:{
					dlconfig:DLParse.jsonproperty('config')
				}
			}
		),
	initialize:function () {
		var effect = "slide";
		var mode = this.dlmode;
		(mode == "default") || (mode == "toggle") || (mode == "toggle-each") || (mode = "toggle-each");
		var config = 
		{
			tabs:'>.outline-box>.outline-liner>.outlinetab',
			effect:effect,
			initialIndex:null
		};
		this.config && $.extend(config,this.config);
		var children = this.children;
		var count = children.length;
		var length = children.length;
		for (var i=0;i<length;i++) {
			var child = children[i];
			var index = i;
			if (child instanceof OutlineTab) {
				var nextindex = index + 1;
				if (nextindex < count) {
					var nextchild = children[nextindex];
					if  (nextchild instanceof OutlinePane) {
						child.pane = nextchild;
						nextchild.tab = child; // cross-link
					}
				}
				child.pane || $(child.elements.frame).addClass("empty");
			}
		};
	}
}
});

OutlineTab = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'OutlineTab',
	metadata:DLServices.extend(DLServices.clone(DLData.styleattributetree),
		{ATTRIBUTES:['dlreferenceid','dlicon']}),
	html:
	'<div class="outlinetab">' + 
	'<div dlindex="status" class="outlinetab-status"></div>' + 
	'<div dlindex="content" class="outlinetab-content">' + 
		'<div dlindex="icon" class="outlinetab-icon"></div>' + 
		'<div dlindex="marker" class="outlinetab-marker"></div>' + 
		'<div dlindex="liner" class="outlinetab-caption"></div>' + 
	'</div>' + 
	'<div style="clear:both"></div>' + 
	'</div>',
	initialize:function() {
		var icon = (this.dlicon || 'folder');
		$(this.elements.icon).addClass(icon);
		$(this.elements.content).bind("dblclick",this.actiontrigger("drill"));
		$(this.elements.content).bind("click",this.actiontrigger("select"));
		$(this.elements.status).bind("click",this.actiontrigger("toggle"));
	},
	actiontrigger: function(action) {
		var self = this;
		return function (event) {
				DLManager.objects[self.dlobjectid]
					.act(action, new DLAction(action,{event:event}));
			}
	},
	act: function(action,data) {
		switch (action) {
			case "drill": this.drill(data); 
				break;
			case "select": this.select(data);
				break;
			case "toggle": this.toggle(data);
				break;
		}
	},
	drill: function (data) {
		// remove unwanted selection from doubleclick;
		var selection ;
		if (document.selection && document.selection.empty) {
			document.selection.empty() ;
		} else {
			if (window.getSelection) {
				selection=window.getSelection();
				if (selection && selection.removeAllRanges)
					selection.removeAllRanges();
			}
		}
		this.toggle(data);
		this.select(data);
	},
	toggle: function (data) {
		if (!this.pane) return;
		var $frame = $(this.elements.frame);
		var $pane = $(this.pane.elements.frame);
		if ($frame.hasClass('open')) {
			$frame.removeClass('open');
			$pane.slideUp('fast');
		} else {
			$frame.addClass('open');
			$pane.slideDown('fast');
		}
	},
	select: function (data) {
		this.platform.setselected(this);
		$(this.elements.frame).addClass("selected");
	}
}
});

OutlinePane = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'OutlinePane',
	html:'<div class="outlinepane"><div class="outlinepane-box" dlindex="box"><div class="outlinepane-liner"></div></div></div>'
}
});

OutlineManager.register(
[
Outlines,
Outline,
OutlineTab,
OutlinePane //,
//OutlineControls, // expand/collapse
//OutlineControl
]
);
