/*
Gargoyle components (dltabs.js) copyright (c) 2007 - 2011 Henrik Bechmann, bechmann.ca, 
	Bechmann Software Services, Toronto Canada, all rights reserved.
website: www.dlml.org/gargoyle
licence: BSD
dl = "dynamic layout"
*/
/*
Tabs

Future: 
- horizontal accordion, 
- tabbarpane - interspersed notes between tabs
- toggleplay

Tasks:
- tabicons (multiple tab sets?), tabsiconbar
- add attribute trees for defaults for DLPlatform and DLContainers
- history
- over-ride tab color scheme, inherited by associated pane (acts as divider)

Bugs: 
- JQuery tools has a bug whereby next sometimes rotates without rotate set to true

*/
// set up tabs model
TabsManager = new (DLPrototype({
$extends:DLModule,
$prototype:{
 name:"TabsManager"
}
}))();

//===============[ ABTRACT PROTOTYPES ]==================//

AbstractTabs = DLPrototype({
$extends:DLPlatform,
$prototype:{
	type:'AbstractTabs',
	metadata:
		DLServices.extend(
			DLServices.clone(DLData.styleattributetree),
			{
				ATTRIBUTES:['dlorientation','dlmode','dltheme','dlconfig','dlslideshow'],
				PARSERS:{
					dlconfig:DLParse.jsonproperty('config'),
					dlslideshow:DLParse.jsonproperty('slideshow')
				}
			}
		),
	initialize:function () {
		var slideshow;
		if (slideshow = this.slideshow) {
			var slideshowconfig = (typeof slideshow === 'object')? slideshow:{};
			this.tabs.slideshow(slideshowconfig);
		}
	}
}
});

AbstractTab = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'AbstractTab',
	metadata:DLData.styleattributetree
}
});

AbstractTabPane = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'AbstractTabPane',
	metadata:DLData.styleattributetree
}
});

//====================[ TABS ]======================//

Tabs = DLPrototype({
$constructor:function () { // defaults
	this.dlorientation = 'top';
	this.dltheme = 'winter';
	this.dldocked = false;
},
$extends:AbstractTabs,
$prototype:{
	type:'Tabs',
	html:'<div dlindex="liner" class="tabs"><div style="clear:both"></div>',
	metadata:{ATTRIBUTES:['dldocked'],DLPARSERS:DLParse['boolean']},
	initialize:function () { // call JQuery Tools setup
		var orientation = this.dlorientation,config;
		if (this.dldocked) $(this.elements.frame).addClass('docked');
		if (orientation == "top" || orientation == "bottom") orientation += " horizontal";
			else orientation += " vertical";
		this.render({frame:{classes:{addClass:orientation + ' ' + this.dltheme}}});
		this.render();
		var $tabbar = $(">.tabbar",this.elements.frame);
		if (!$tabbar.length) {
			$tabbar = $(">.tabiconbar",this.elements.frame);
			if (!$tabbar.length) $tabbar = $(">.tabcontrols .tabiconbar",this.elements.frame);
			if (!$tabbar.length) return;
			config = {tabs:'.tabicon',rotate:true};
		} else {
			config = {tabs:'.tab',rotate:true};
		}
		this.tabs = $tabbar;
		this.config && $.extend(config,this.config);
		var $panes = $(">.tabpanes>.tabpanes-box>.tabpanes-liner>.tabpane",this.elements.frame);
		if (!($tabbar.length && $panes.length)) return;
		DLServices.initializechildren(this);
		$tabbar.tabs($panes,config);
		this.base.initialize.call(this);
	}
}
});

TabBar = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TabBar',
	html:'<div class="tabbar"><div class="tabbar-box" dlindex="box"><div class="tabbar-liner"></div></div></div>',
	metadata:DLData.styleattributetree,
	initialize: function () {
		DLServices.initializechildren(this);
	}
}
});

TabIconBar = DLPrototype({ // for autoplay
$extends:TabBar,
$prototype:{
	type:'TabIconBar',
	html:'<div class="tabiconbar"></div>',
	metadata:DLData.styleattributetree,
	initialize: function () {
		DLServices.initializechildren(this);
	}
}
});

TabPanes = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TabPanes',
	html:'<div class="tabpanes"><div class="tabpanes-box" dlindex="box"><div dlindex="liner" class="tabpanes-liner"></div><div class="paneblock-divider" style="clear:both"></div></div></div>',
	metadata:DLData.styleattributetree
}
});

TabPane = DLPrototype({
$extends:AbstractTabPane,
$constructor: function () {
	this.dldock = false;
},
$prototype:{
	type:'TabPane',
	html:'<div class="tabpane"><div class="tabpane-box" dlindex="box"><div class="tabpane-liner"></div></div></div>',
	metadata:{ATTRIBUTES:['dldock'],PARSERS:{dldock:DLParse['boolean']}},
	initialize: function () {
		if (this.dldock) $(this.elements.liner).addClass('dock');
	}
}
});

Tab = DLPrototype({
$extends:AbstractTab,
$prototype:{
	type:'Tab',
	html:'<div class="tab"><div dlindex="left" class="tableft"></div><a dlindex="liner" href="javascript:void(0);"></a><div dlindex="right" class="tabright"></div></div>'
}
});

TabIcon = DLPrototype({
$extends:AbstractTab,
$prototype:{
	type:'TabIcon',
	html:'<div class="tabicon"></div>'
}
});
TabControls = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TabControls',
	html:'<div class="tabcontrols"><div class="tabcontrols-box" dlindex="box"><div dlindex="liner" class="tabcontrols-liner"><div class="tabcontrols-divider"></div></div></div></div>',
	metadata:DLServices.extend(DLServices.clone(DLData.styleattributetree),
		{
			SETTINGS:{
				defaults:{
					COLLECTIONS:{
						defaults:DLData.styleattributetree
					}
				}
			},
			COLLECTIONS:{
				defaults:DLData.styleattributetree
			}
		})
}
});
//first, previous, next, last, stop, play, expand, collapse,
TabControl = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TabControl',
	html:'<div class="tabcontrol"><div class="tabcontrol-box" dlindex="box"><div class="tabcontrol-liner"></div></div></div>',
	metadata:DLServices.extend(DLServices.clone(DLData.styleattributetree),
		{ATTRIBUTES:['dlaction']}),
	dlactions:['first', 'previous', 'next', 'last', 'stop', 'play', 'expand', 'collapse'],
	initialize: function() {
		var action = this.dlaction;
		var frame = this.elements.frame;
		if (action && this.dlactions.inArray(action)) {
			$(frame).bind('click',DLEventTerminal(action));
			(action == 'previous') && ($(frame).addClass('backward'));
			(action == 'next') && ($(frame).addClass('forward'));
		}
		this.api = this.platform.tabs.data('tabs');
	},
	act: function (action,data)	{
		this[action] && this[action](data);
	},
	first: function (data) {
		this.api.click(0);
	},
	previous: function (data) {
		this.api.prev();
	},
	next: function (data) {
		this.api.next();
	},
	last: function (data) {
		this.api.click(this.api.getTabs().length - 1);
	},
	stop: function (data) {
		this.platform.tabs.data('slideshow').stop();
	},
	play: function (data) {
		this.api.next();
		this.platform.tabs.data('slideshow').play()
	},
	expand: function (data) {
		this.api.getPanes().show("fast");
	},
	collapse: function (data) {
		this.api.getPanes().hide("fast").eq(this.api.getIndex()).show("fast");
	}
}
});
// ===================[ ACCORDIONS ]========================//
Accordion = DLPrototype({
$constructor:function () {
	this.dlorientation = 'vertical';
	this.dlmode = 'default'; // or "toggle" or "toggle-each"
	this.dltheme = 'winter';
},
$extends:AbstractTabs,
$prototype:{
	type:'Accordion',
	html:'<div class="accordion"><div class="accordion-box" dlindex="box"><div class="accordion-liner"></div></div></div>',
	initialize:function () {
		var orientation = this.dlorientation;
		(orientation == "vertical" || orientation == "horizontal") || (orientation = "vertical");
		var effect = (orientation == "vertical")? "slide":"horizontal";
		var mode = this.dlmode;
		var modeclass;
		if (!(mode == 'default')) {
			effect = mode; // *** temporary hack for toggle modes
			modeclass = 'toggle'; // for icon
		}
		var $accordion = $(this.elements.frame);
		this.tabs = $accordion;
		var config = {tabs:'>.accordion-box>.accordion-liner>.accordiontab',effect:effect,rotate:true}; // immediate descendents only
		this.config && $.extend(config,this.config);
		var $panes = $(">.accordion-box>.accordion-liner>.accordionpane",this.elements.frame);
		if (!($accordion.length && $panes.length)) return;
		$accordion.tabs($panes,config);
		var class_string = orientation + ' ' + this.dltheme;
		if (modeclass) class_string+= ' ' + modeclass;
		this.render({frame:{classes:{addClass:class_string}}});
		this.base.initialize.call(this);
	}
}
});

AccordionTab = DLPrototype({
$extends:AbstractTab,
$prototype:{
	type:'AccordionTab',
	html:'<div class="accordiontab"><div class="accordiontab-box"><div class="accordiontab-liner"></div></div></div>'
}
});

AccordionPane = DLPrototype({
$extends:AbstractTabPane,
$prototype:{
	type:'AccordionPane',
	html:'<div class="accordionpane"><div class="accordionpane-box" dlindex="box"><div class="accordionpane-liner"></div></div></div>'
}
});

TabsManager.register(
[
// AbstractTabs 
// AbstractTab
// AbstractTabPane
Tabs,
TabBar,
TabIconBar, // for tab icons
TabPanes,
TabPane,
Tab,
TabIcon, // for controls
TabControls, // toolbar
TabControl,

Accordion,
AccordionTab,
AccordionPane
]
);
