/*
Gargoyle components (dltrays.js) copyright (c) 2007 - 2011 Henrik Bechmann, bechmann.ca, 
	Bechmann Software Services, Toronto Canada, all rights reserved.
website: www.dlml.org/gargoyle
licence: BSD
dl = "dynamic layout"
*/
/*
- drag & drop with outlines
- allow dblclick on namepacks; remove text selection as in outline

- deferred:
	selectable
	lazy loading
	zebrastripes
	sort
	
*/
//================[ TRAYS ]=====================//
TrayManager = new (DLPrototype({
$extends:DLModule,
$prototype:{
 name:"TrayManager"
}}))();

Tray = DLPrototype({
$extends:DLPlatform,
$constructor: function () {
	this.dldocked = false;
},
$prototype:{
	type:'Tray',
	html:
	'<div class="tray">' + 
	'<div class="tray-box" dlindex="box">' + 
	'<div class="tray-liner">\n</div>' + 
	'</div></div>\n',
	metadata:{ATTRIBUTES:['dltray','dldocked'],PARSERS:{dldocked:DLParse['boolean']}},// for absence of controls
	initialize: function () {
		var defaulttrayname, defaulttray, controls;
		if (this.dldocked) $(this.elements.frame).addClass('docked');
		DLServices.initializechildren(this);
		if (controls = this.components.controls) defaulttrayname = controls.dldefault;
		else (this.dltray && (defaulttrayname = this.dltray)) || (defaulttrayname = 'list');
		(this.components[defaulttrayname]) && (defaulttray = this.components[defaulttrayname])
		defaulttray && $(defaulttray.elements.frame).css('display','block');
		this.selectedtray = defaulttray;
	},
	selecttray: function (data) {
		var trayname = data.trayname, tray;
		(this.components[trayname]) && (tray = this.components[trayname])
		if (tray) {
			this.selectedtray && $(this.selectedtray.elements.frame).css('display','none');
			$(tray.elements.frame).css('display','block');
			this.selectedtray = tray;
		}
	},
	destroy: function () {
		delete this.selectedtray;
	}
}
});

TrayControls = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TrayControls',
	html:
	'<div class="traycontrols">' + 
	'<div class="traycontrols-box" dlindex="box">' + 
	'<div dlindex="liner" class="traycontrols-liner">' + 
	'<div style="clear:both">' + 
	'</div></div></div></div>\n',
	metadata:{ATTRIBUTES:['dldefault']},
	initialize: function () {
		this.dldefault || (this.dldefault = 'list');
		DLServices.initializechildren(this);
		if (this.components[this.dldefault]) {
			this.selectedtray = this.components[this.dldefault];
			$(this.selectedtray.elements.icon).addClass('selectedtray');
		}
		this.platform.registercomponent('controls',this);
	},
	selecttray: function (data) {
		var selectedtray = data.control;
		(this.selectedtray) && $(this.selectedtray.elements.icon).removeClass('selectedtray');
		this.selectedtray = selectedtray;
		$(selectedtray.elements.icon).addClass('selectedtray');
		this.platform.selecttray({trayname:selectedtray.dlaction});
	},
	destroy: function () {
		delete this.selectedtray;
	}
}
});

TrayControl = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayControl',
	html:
	'<div class="traycontrol">' + 
	'<div class="traycontrol-box" dlindex="box">' + 
	'<div class="traycontrol-icon" dlindex="icon"></div>' + 
	'<div dlindex="liner" class="traycontrol-liner">' + 
	'</div></div></div>\n',
	metadata: {ATTRIBUTES:['dlaction','dlattribute'],PARSERS:{dlattribute:DLParse.dlattribute}},
	initialize: function () {
		var dlaction = this.dlaction;
		dlaction || (this.dlaction = 'list');
		['list','details','tiles','matrix_small','matrix_medium','matrix_large'].inArray(dlaction) 
			|| (this.dlaction = dlaction = 'list');
		$(this.elements.icon).addClass(dlaction);
		if (!this.dlindex) {
			this.dlindex = dlaction;
			this.container.registercomponent(this.dlindex,this);
		}
		$(this.elements.frame).bind("click",DLEventTerminal('selecttray'));
	},
	act: function (action,data) {
		this[action](data);
	},
	selecttray: function (data) {
		data.control = this;
		this.container.selecttray(data);
	}
}
});

TrayList = DLPrototype({
$extends:DLContainer,
$constructor: function () {
	this.selected = null;
	this.dldock = false;
},
$prototype:{
	type:'TrayList',
	html:
	'<div class="traylist">\n' + 
		'<div class="traylist-box" dlindex="box">\n' + 
		'<div class="traylist-liner">\n' + 
	'</div></div></div>\n',
	metadata:{ATTRIBUTES:['dldock'],PARSERS:{dldock:DLParse['boolean']}},
	initialize: function () {
		this.platform.registercomponent("list",this);
		if (this.dldock) $(this.elements.liner).addClass('dock');
	},
	select:function(data) {
		var dlobject = data.item;
		if (this.selected === dlobject) return;
		if (this.selected) $(this.selected.elements.frame).removeClass('selected');
		this.selected = dlobject;
		$(dlobject.elements.frame).addClass('selected');
	},
	destroy: function () {
		delete this.selected;
	}
}
});

TrayListItem = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayListItem',
	html:
	'<div class="traylistitem">\n' + 
		'<div class="traylistitem-box" dlindex="box">\n' + 
			'<div dlindex="namepack" class="traylistitem-namepack">\n' + 
				'<div dlindex="icon" class="traylistitem-icon"></div>\n' + 
				'<div dlindex="liner" class="traylistitem-name"></div>\n' + 
				'<div style="clear:both"></div>' +
			'</div>\n' + 
		'</div>\n' + 
	'</div>\n',
	metadata:{ATTRIBUTES:['dlicon','dlreferenceid']},
	initialize: function () {
		var icon = this.dlicon;
		icon || (icon = this.dlicon = 'document');
		$(this.elements.icon).addClass(icon);
		$(this.elements.namepack).bind("click",DLCallTrigger.call(this.container,"select",{item:this}));
	}
}
});

TrayTiles = DLPrototype({
$extends:DLContainer,
$constructor: function () {
	this.selected = null;
	this.dldock=false;
},
$prototype:{
	type:'TrayTiles',
	html:
	'<div class="traytiles">' + 
	'<div class="traytiles-box" dlindex="box">' + 
	'<div class="traytiles-liner">' + 
	'</div></div></div>\n',
	metadata:{ATTRIBUTES:['dldock'],PARSERS:{dldock:DLParse['boolean']}},
	initialize: function () {
		this.platform.registercomponent("tiles",this);
		if (this.dldock) $(this.elements.liner).addClass('dock');
	},
	select:function(data) {
		var dlobject = data.item;
		if (this.selected === dlobject) return;
		if (this.selected) $(this.selected.elements.frame).removeClass('selected');
		this.selected = dlobject;
		$(dlobject.elements.frame).addClass('selected');
	},
	destroy: function () {
		delete this.selected;
	}
}
});

TrayTile = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TrayTile',
	html:
	'<div class="traytile">' + 
	'<div class="traytile-box" dlindex="box">' + 
	'<div class="traytile-liner" dlindex="liner">' + 
	'</div>\n' + 
	'</div></div>\n',
	metadata:{ATTRIBUTES:['dlicon','dlreferenceid']},
	initialize: function () {
		DLServices.initializechildren(this);
		var icon = (this.dlicon || 'document');
		(icon == this.dlicon) || (this.dlicon = icon);
		if (this.components.namepack)
			$(this.components.namepack.elements.icon).addClass(icon);
	}
}
});

TrayTileName = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayTileName',
	html:
	'<div dlindex="namepack" class="traytile-namepack">\n' + 
		'<div dlindex="icon" class="traytile-icon"></div>\n' + 
		'<div dlindex="liner" class="traytile-name"></div>\n' + 
//		'<div style="clear:both"></div>' +
	'</div>\n',
	initialize: function () {
		this.container.registercomponent("namepack",this);
		$(this.elements.namepack).bind("click",DLCallTrigger.call(this.container.container,"select",{item:this.container}));
	}
}
});

TrayDetails = DLPrototype({
$extends:DLContainer,
$constructor: function () {
	this.headerfieldsadjustment = 1;
	this.selected = null;
},
$prototype:{
	type:'TrayDetails',
	html:
		'<div class="traydetails">' + 
			'<div dlindex="wrapper" class="traydetails-wrapper">' + 
				'<div dlindex="box" class="traydetails-box">' + 
					'<div dlindex="liner" class="traydetails-liner"></div>' + 
				'</div>' + 
			'</div>' +
		'</div>',
	initialize: function () {
		this.platform.registercomponent("details",this);
		DLServices.initializechildren(this);
		var $wrapper = $(this.elements.wrapper)
		var width = this.getcolumnswidth();
		$wrapper.width(width);
	},
	select:function(data) {
		var dlobject = data.item;
		if (this.selected === dlobject) return;
		if (this.selected) $(this.selected.elements.frame).removeClass('selected');
		this.selected = dlobject;
		$(dlobject.elements.frame).addClass('selected');
	},
	act: function act(action, data) {
		this[action](data);
	},
	actiontrigger:DLActionTrigger,
	startresizecol: function (data) {
		var event = data.event,columnswidth=0,width,length,i,wrapperwidth;
		wrapperwidth=$(this.elements.wrapper).width();
		var headerfields = this.components.header.children;
		this.resizecoldata = {
			startx:event.pageX,
			headercolumn:data.column,
			startwidth:$(data.column.elements.frame).width(),
			columnoffset:headerfields.indexOf(data.column),
			columnswidth:columnswidth,
			wrapperwidth:wrapperwidth
		};
		$(document).bind('mousemove.resizecol',this.actiontrigger('resizecol'));
		$(document).bind('mouseup.resizecol',this.actiontrigger('stopresizecol'));
	},
	getcolumnswidth: function () {
		var columnswidth = 0;
		var headerfields = this.components.header.children;
		var length = headerfields.length;
		for (var i=0;i<length;i++) {
			var width = $(headerfields[i].elements.frame).width();
			columnswidth+= width;
		}
		columnswidth += this.headerfieldsadjustment;
		return columnswidth;
	},
	resizecol: function (data) {
		var event = data.event,resizecoldata = this.resizecoldata,
			diffx,newwidth,newcolumnswidth,
			row,row,length,i,columns,column,footercol;
		diffx = event.pageX - resizecoldata.startx;
		newwidth = resizecoldata.startwidth + diffx;
		if (newwidth < 10) {
			newwidth = 10;
			diffx = newwidth - resizecoldata.startwidth;
		}
		newcolumnswidth = resizecoldata.wrapperwidth + diffx;
		if (newcolumnswidth > resizecoldata.wrapperwidth) {
			$(this.elements.wrapper).width(newcolumnswidth);
		}
		$(resizecoldata.headercolumn.elements.frame).width(newwidth);
		rows = this.components.body.children;
		length = rows.length;
		for (i=0;i<length;i++) {
			row = rows[i];
			columns = row.children;
			column = columns[resizecoldata.columnoffset];
			$(column.elements.frame).width(newwidth);
		}
		this.components.footer && 
			this.components.footer.children && 
			(footercol = this.components.footer.children[resizecoldata.columnoffset]);
		footercol && $(footercol.elements.frame).width(newwidth);
		if (newcolumnswidth < resizecoldata.wrapperwidth)
			$(this.elements.wrapper).width(newcolumnswidth);
	},
	stopresizecol: function (data) {
		$(document).unbind('mousemove.resizecol');
		$(document).unbind('mouseup.resizecol');
		delete this.resizecoldata;
	}
}
});

TrayDetailsHeader = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayDetailsHeader',
	html:
	'<div class="traydetails-header">' + 
	'<div class="traydetails-header-box">' + 
		'<div dlindex="liner" class="traydetails-header-liner"></div>' + 
		'<div style="clear:both"></div>' +
	'</div></div>\n',
	initialize:function () {
		this.container.registercomponent('header',this);
	}
}
});

// container splitter
TrayDetailsHeaderItem = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayDetailsHeaderItem',
	html:
	'<div class="traydetails-headeritem">' + 
	'<div class="traydetails-headeritem-box">' + 
		'<div dlindex="liner" class="traydetails-headeritem-content"></div>' + 
		'<div dlindex="splitter" class="traydetails-headeritem-splitter"></div>' + 
	'</div>\n' + 
	'</div>\n',
	initialize: function () {
		$(this.elements.splitter).bind('mousedown',this.container.actiontrigger('startresizecol',{column:this}));
	}
}
});

TrayDetailsBody = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayDetailsBody',
	html:
	'<div class="traydetails-body">' + 
	'<div class="traydetails-body-box">' + 
		'<div class="traydetails-body-liner"></div>' + 
	'</div></div>\n',
	initialize:function () {
		this.container.registercomponent('body',this);
	}
}
});

TrayDetailsRow = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TrayDetailsRow',
	html:
	'<div class="traydetails-row">' + 
	'<div class="traydetails-row-box">' + 
		'<div dlindex="liner" class="traydetails-row-liner"></div>' + 
	'</div><div style="clear:both"></div></div>\n',
	metadata:{ATTRIBUTES:['dlicon','dlreferenceid']},
	initialize: function () {
		DLServices.initializechildren(this);
		var icon = (this.dlicon || 'document');
		(icon == this.dlicon) || (this.dlicon = icon);
		if (this.components.namepack) {
			var $icon = $(this.components.namepack.elements.icon);
			$icon.addClass(icon);
		}
	}
}
});


TrayDetailsItem = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayDetailsItem',
	html:
	'<div class="traydetails-rowitem">\n' +
		'<div class="traydetails-rowitem-box">\n' +
			'<div dlindex="liner" class="traydetails-rowitem-liner"></div>\n' + 
		'</div>' +
	'</div>'
}
});

TrayDetailsName = DLPrototype({
$extends:TrayDetailsItem,
$prototype:{
	type:'TrayDetailsName',
	html:
	'<div dlindex="namepack" class="traydetails-rowitem traydetails-rowitem-namepack">\n' + 
		'<div class="traydetails-rowitem-box">\n' +
			'<div dlindex="icon" class="traydetails-rowitem-icon"></div>\n' + 
			'<div dlindex="liner" class="traydetails-rowitem-caption"></div>\n' + 
			'<div style="clear:both"></div>' +
		'</div>\n' +
	'</div>\n',
	initialize: function () {
		this.container.registercomponent("namepack",this);
		$(this.elements.namepack).bind("click",DLCallTrigger.call(this.container.container,"select",{item:this.container}));
	}
}
});

TrayDetailsFooter = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayDetailsFooter',
	html:
	'<div class="traydetails-footer">' + 
		'<div class="traydetails-footer-box">' + 
			'<div dlindex="liner" class="traydetails-footer-liner">' + 
				'<div style="clear:both"></div>' + 
			'</div>' +
		'</div>\n' +
	'</div>',
	initialize:function () {
		this.container.registercomponent('footer',this);
	}
}
});

TrayDetailsFooterItem = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayDetailsFooterItem',
	html:
	'<div class="traydetails-footeritem">' + 
		'<div class="traydetails-footeritem-box">' + 
			'<div class="traydetails-footeritem-content">' + 
		'</div>' + 
	'</div>' + 
	'</div>\n'
}
});

TrayMatrix = DLPrototype({
$extends:DLContainer,
$constructor: function () {
	this.selected = null;
	this.dldock=false;
},
$prototype:{
	type:'TrayMatrix',
	html:
	'<div class="traymatrix">' + 
		'<div class="traymatrix-box" dlindex="box">' + 
			'<div class="traymatrix-liner"></div>' + 
		'</div>' +
	'</div>\n',
	metadata:{ATTRIBUTES:['dlsize','dldock'],PARSERS:{dldock:DLParse['boolean']}},
	initialize: function () {
		var matrixtype = this.dlsize;
		if (this.dldock) $(this.elements.liner).addClass('dock');
		DLServices.initializechildren(this);
		matrixtype || (this.dlsize = matrixtype = 'small');
		['small','medium','large'].inArray(matrixtype) || (this.dlsize = matrixtype = 'small');
		this.platform.registercomponent('matrix_' + matrixtype,this);
		$(this.elements.frame).addClass(matrixtype);
	},
	select:function(data) {
		var dlobject = data.item;
		if (this.selected === dlobject) return;
		if (this.selected) $(this.selected.elements.frame).removeClass('selected');
		this.selected = dlobject;
		$(dlobject.elements.frame).addClass('selected');
	}
}
});
TrayMatrixItem = DLPrototype({
$extends:DLContainer,
$prototype:{
	type:'TrayMatrixItem',
	html:'<div class="traymatrix-item"><div class="traymatrix-item-box"><div class="traymatrix-item-liner"></div></div></div>\n',
	metadata:{ATTRIBUTES:['dlicon','dlreferenceid']},
	initialize: function () {
		var icon = this.dlicon;
		DLServices.initializechildren(this);
		icon || (icon = this.dlicon = 'document');
		this.components.namepack && 
			$(this.components.namepack.elements.icon).addClass(icon);
	}
}
});

// small image about 48 x 48; large image about 96 x 96.
TrayMatrixImage = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayMatrixImage',
	html:
	'<div dlindex="liner" class="traymatrix-item-image">' + 
		'<div dlindex="liner" class="traymatrix-item-image-outer">' + 
			'<div dlindex="liner" class="traymatrix-item-image-box">' + 
				'<div dlindex="liner" class="traymatrix-item-image-liner">' + 
				'</div>\n' +
			'</div>\n' +
		'</div>\n' +
	'</div>\n'
}
});

TrayMatrixItemName = DLPrototype({
$extends:DLObject,
$prototype:{
	type:'TrayMatrixItemName',
	html:
	'<div dlindex="namepack" class="traymatrix-item-namepack">\n' + 
		'<div dlindex="icon" class="traymatrix-item-icon"></div>\n' + 
		'<div dlindex="liner" class="traymatrix-item-caption"></div>\n' + 
		'<div style="clear:both"></div>' +
	'</div><br />\n',
	initialize: function () {
		this.container.registercomponent("namepack",this);
		$(this.elements.namepack).bind("click",DLCallTrigger.call(this.container.container,"select",{item:this.container}));
	}
}
});

TrayManager.register(
[
Tray,
// controls
TrayControls,
TrayControl,
// list
TrayList,
TrayListItem,
// tiles
TrayTiles,
TrayTile,
TrayTileName,
// details
TrayDetails,
TrayDetailsHeader,
TrayDetailsHeaderItem,
TrayDetailsBody,
TrayDetailsRow,
TrayDetailsItem,
TrayDetailsName,
TrayDetailsFooter,
TrayDetailsFooterItem,
// matrix
TrayMatrix,
TrayMatrixItem,
TrayMatrixImage,
TrayMatrixItemName
]
);
