<?php 
namespace Muster;

use Kohana, DB, Database;
use User, System, Services;
use Muster\Access\Token as Token;

defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
class Access {

	protected static $_instance;
	
	protected $_user;
	protected $_db;
	protected $_userid;
	
	protected $_groups;
	protected $_user_roles;
	protected $_owner_roles;
	protected $_collaborator_roles;
	
	protected $_token_cache;
	protected $_owner_token_cache;
	protected $_article_collaborator_token_cache;
	
	protected $_has_permission_table = FALSE;
	
	public static function instance()
	{
		if (!self::$_instance) 
			return new Access();
		return self::$_instance;
	}
	public static function combine_tokens() // any number of tokens
	{
		$tokenlist = func_get_args();
		$return_token = new Token();
		foreach ($tokenlist as $token) {
			$return_token->Create += $token->Create;
			$return_token->Read += $token->Read;
			$return_token->Edit += $token->Edit;
			$return_token->Delete += $token->Delete;
			$return_token->Deny += $token->Deny;
		}
		return $return_token;
	}
	public function __construct()
	{
		self::$_instance = $this;
		
		$user = $this->_user = User::instance();
		$system = System::instance();
		$this->_db = $user->db();
		if ($user->authenticated())	{
			$this->_userid = $user->userid();
			$user_data = $user->user_data();
			if (!$system->MultipleGroups) {
				$groupid = $user_data->GroupID;
				if (is_null($groupid)) {
					$groupid = $system->DefaultGroupID;
				}
				$roleid = $user_data->RoleID;
				if (is_null($roleid)) {
					$roleid = $system->DefaultRoleID;
				}
				$this->_groups = array($groupid);
				$this->_user_roles = array($roleid);
				$ownerrole = $system->RecordOwnerRole;
				$result = DB::select('ID')
					->from('Access_Group_Roles')
					->where('Code','=',$ownerrole)
					->and_where('GroupID','IN',$this->_groups)
					->as_object()
					->execute($this->_db);
				if ($result)
					$record = $result->current();
				if ($record) {
					$ownerroleid = $record->ID;
					$this->_owner_roles = array($ownerroleid);
				}
				$collaboratorrole = $system->RecordCollaboratorRole;
				$result = DB::select('ID')
					->from('Access_Group_Roles')
					->where('Code','=',$collaboratorrole)
					->and_where('GroupID','IN',$this->_groups)
					->as_object()
					->execute($this->_db);
				if ($result)
					$record = $result->current();
				if ($record) {
					$collaboratorroleid = $record->ID;
					$this->_collaborator_roles = array($collaboratorroleid);
				}
			}
		} else { // not authenticated
			$this->_user_roles = array($system->PublicRoleID);
		}
	}
	// returns boolean
	public function permission($token_name,$requires,$ownerid = NULL,$collaborationarticleid = NULL)
	{
		$token = $this->get_token($token_name,$ownerid,$collaborationarticleid);
		if ($token->deny) return FALSE;
		$stringtoken = $token->get_string();
		if (empty($stringtoken)) return FALSE;
		$unfulfilled = preg_replace('/[' . $stringtoken. ']/','',$requires);
		return empty($unfulfilled); // all requirements met
	}
	public function get_token($token_name,$ownerid = NULL,$collaborationarticleid = NULL)
	{
		$userid = $this->_userid;
		$merge_owner_roles = FALSE;
		$merge_collaborator_roles = FALSE;
		if (isset($this->_token_cache[$token_name]) and (is_null($ownerid))) {
			$token = $this->_token_cache[$token_name];
		} elseif ((!is_null($ownerid)) and ($ownerid == $userid) and isset($this->_owner_token_cache[$ownerid][$token_name])){
			$token = $this->_owner_token_cache[$ownerid][$token_name];
		} elseif ((!is_null($collaborationarticleid)) and isset($this->_article_collaborator_token_cache[$collaborationarticleid][$token_name])){
			$token = $this->_article_collaborator_token_cache[$collaborationarticleid][$token_name];
		} else {
			$rolelist = $this->_user_roles;
			if (!is_null($ownerid)) {
				if ($ownerid == $userid) {
					$merge_owner_roles = TRUE;
				} elseif (isset($collaborationarticleid) and 
					(is_numeric($collaborationarticleid))) {
					$operation = DB::select('ID')
						->from('Articles_Authors')
						->where('ArticleID','=',$collaborationarticleid)
						->where('UserID','=',$userid)
						->as_object()
						->prepare()
						->execute();
					if (!$operation->success())
						die(var_dump($operation->error_info()));
					$record = $operation->fetch();
					$merge_collaborator_roles = (bool) $record;
				}
				if ($merge_owner_roles) {
					$rolelist = array_merge($rolelist,$this->_owner_roles);
				} elseif ($merge_collaborator_roles) {
					$rolelist = array_merge($rolelist,$this->_collaborator_roles);
				}
			}
			$tokenid = Services::dblookup('Access_Tokens','ID','Code',$token_name);
			if ($tokenid == NULL) {
				$token = new Token();
				if ($merge_owner_roles)
					$this->_owner_token_cache[$ownerid][$token_name] = $token;
				elseif ($merge_collaborator_roles)
					$this->_article_collaborator_token_cache[$collaborationarticleid][$token_name] =  $token;
				else
					$this->_token_cache[$token_name] =  $token;
				$token->name = $token_name;
				return $token; // everything FALSE;
			}
			$operation = DB::select_array(
				array(
					DB::expr('sum(`Create`) as `Create`'),
					DB::expr('sum(`Read`) as `Read`'),
					DB::expr('sum(`Edit`) as `Edit`'),
					DB::expr('sum(`Delete`) as `Delete`'),
					DB::expr('sum(`Deny`) as `Deny`')
				)
			)->from('Access_Role_Tokens')
			->where('RoleID','IN',$rolelist)
			->and_where('TokenID','=',$tokenid)
			->as_object()
			->prepare($this->_db)
			->execute();
			if (!$operation->success()) {
				var_dump($operation->sql());
				var_dump($operation->error_info());
			}
			$tokenrecord = $operation->current(); 
			$token = new Token($tokenrecord);
			if ($merge_owner_roles) {
				$this->_owner_token_cache[$ownerid][$token_name] =  $token;
			} elseif ($merge_collaborator_roles) {
				$this->_article_collaborator_token_cache[$collaborationarticleid][$token_name] =  $token;
			} else {
				$this->_token_cache[$token_name] =  $token;
			}
			$token->name = $token_name;
		}
		return $token;
	}
	// content, hidden, trash, unpublished read permission records for each record type
	public function create_readpermission_tables($db = NULL)
	{
		if ($this->_has_permission_table) return;
		
		$sql = // temp tables can only be referenced once per sql statement, so 4 are required
"
CREATE TEMPORARY TABLE IF NOT EXISTS temp_permissions_content (
  `RecordType` varchar(30) NOT NULL,
  `TokenCode` varchar(30) NOT NULL,
  `Read` tinyint(1) NOT NULL DEFAULT '0',
  PRIMARY KEY (`RecordType`)
);
CREATE TEMPORARY TABLE IF NOT EXISTS temp_permissions_hidden (
  `RecordType` varchar(30) NOT NULL,
  `TokenCode` varchar(30) NOT NULL,
  `Read` tinyint(1) NOT NULL DEFAULT '0',
  PRIMARY KEY (`RecordType`)
);
CREATE TEMPORARY TABLE IF NOT EXISTS temp_permissions_trash (
  `RecordType` varchar(30) NOT NULL,
  `TokenCode` varchar(30) NOT NULL,
  `Read` tinyint(1) NOT NULL DEFAULT '0',
  PRIMARY KEY (`RecordType`)
);
CREATE TEMPORARY TABLE IF NOT EXISTS temp_permissions_unpublished (
  `RecordType` varchar(30) NOT NULL,
  `TokenCode` varchar(30) NOT NULL,
  `Read` tinyint(1) NOT NULL DEFAULT '0',
  PRIMARY KEY (`RecordType`)
);
";
		$db = Database::instance($db);
		$result = $db->connection()->exec($sql);
		if ($result === FALSE) {
			var_dump($db->error_info());
			exit;
		}
		$roleid = $this->_user->user_data()->RoleID;
		
		// assemble data for insert
		$selection = DB::select('TokenCode','Read')->from('Access_Role_Tokens_View')->where('RoleID','=',$roleid)
			->prepare()->execute();
		$tokenvalues = $selection->as_array('TokenCode','Read');
		$tokensets = Kohana::$config->load('recordtypetokensets')->getArrayCopy();
		$tokentypelist = array('content','hidden','trash','unpublished');
		$parmarray = array(':recordtype'=>NULL,':tokencode'=>NULL,':read'=>NULL);
		// prepare insert statements
		$contentoperation = DB::insert('temp_permissions_content',array('RecordType','TokenCode','Read'))
			->values(array(':recordtype',':tokencode',':read'))
			->prepare($db);
		$hiddenoperation = DB::insert('temp_permissions_hidden',array('RecordType','TokenCode','Read'))
			->values(array(':recordtype',':tokencode',':read'))
			->prepare($db);
		$trashoperation = DB::insert('temp_permissions_trash',array('RecordType','TokenCode','Read'))
			->values(array(':recordtype',':tokencode',':read'))
			->prepare($db);
		$unpublishedoperation = DB::insert('temp_permissions_unpublished',array('RecordType','TokenCode','Read'))
			->values(array(':recordtype',':tokencode',':read'))
			->prepare($db);
		foreach($tokensets as $recordtype => $tokentypes) {
			foreach($tokentypelist as $tokentype) {
				if (!empty($tokentypes[$tokentype])) {
					$tokenname = $tokentypes[$tokentype];
					if (!empty($tokenvalues[$tokenname])) {
						$readvalue = $tokenvalues[$tokenname];
						$parmarray[':recordtype'] = $recordtype;
						$parmarray[':tokencode'] = $tokenname;
						$parmarray[':read'] = $readvalue;
						switch ($tokentype) {
							case 'content': 
								$contentoperation->execute($parmarray);
								break;
							case 'hidden': 
								$hiddenoperation->execute($parmarray);
								break;
							case 'trash': 
								$trashoperation->execute($parmarray);
								break;
							case 'unpublished': 
								$unpublishedoperation->execute($parmarray);
								break;
						}
					}
				}
			}
		}
		$this->_has_permission_table = TRUE;
//		$this->dump_permission_tables();
	}
	public function dump_permission_tables() // for debug
	{
		$selection = DB::select('*')
			->from('temp_permissions_content')
			->as_object()
			->prepare()
			->execute();
		var_dump($selection->as_array());
		$selection = DB::select('*')
			->from('temp_permissions_hidden')
			->as_object()
			->prepare()
			->execute();
		var_dump($selection->as_array());
		$selection = DB::select('*')
			->from('temp_permissions_trash')
			->as_object()
			->prepare()
			->execute();
		var_dump($selection->as_array());
		$selection = DB::select('*')
			->from('temp_permissions_unpublished')
			->as_object()
			->prepare()
			->execute();
		var_dump($selection->as_array());
	}
}
