<?php defined('SYSPATH') or die('No direct script access.');
// replacement for  Kohana_Database_PDO, version 3.0.7
/**
 * PDO database connection.
 *
 * original attributions of Kohana_Database_PDO, version 3.0.7:
 * @package    Kohana/Database
 * @category   Drivers
 * @author     Kohana Team
 * @copyright  (c) 2008-2009 Kohana Team
 * @license    http://kohanaphp.com/license
 
 * This file very heavily Modified by Henrik Bechmann, Toronto, Canada, July/August 2010; 
	- replaces Kohana_Database_PDO version 3.0.7
	- supports prepare() (value parameters and bound parameters)
	- surfaces some features (notably error_info() and attribute assignment)
 * changes copyright (c) Henrik Bechmann, Toronto, Canada, 2010. bechmann.ca
 * New BSD licence
 */
class Muster_Database_PDO extends Database {

	const QUERY = 1;
	const PREPARE = 2;
	protected $_calltype; // QUERY or PREPARE;
	protected $_identifier = ''; // default
	# new properties
	protected $_config; // array
	protected $_result_config; // array
	protected $_operation_config; // array
	protected $_last_statement; // (object) from query or prepare
	protected $_statement_error; // last statement error;
	
	// modified
	public function connect()
	{
		if ($this->_connection)
			return;
		// Extract the connection parameters, adding required variables
		extract($this->_config['connection'] + array(
			'dsn'        => '',
			'username'   => NULL,
			'password'   => NULL,
			'attributes' => array(),
		));

		// Clear the connection parameters for security
		unset($this->_config['connection']);
		// new array items
		$this->_config = $this->_config + array(
			'result' => array( // result statement defaults
				'attributes' => array(),
			),
			'operation' => array( // operation statement defaults
				'attributes' => array(),
			),
		);
		$result_config = $this->_config['result'];
		unset($this->_config['result']);
		$result_config['db'] = $this;
		$result_config['cursorscroll'] = 
			(boolean) (isset($result_config['attributes'][PDO::ATTR_CURSOR]) and 
				($result_config['attributes'][PDO::ATTR_CURSOR] === PDO::CURSOR_SCROLL));
		$this->_result_config = $result_config;
		$this->_operation_config = $this->_config['operation'];
		unset($this->_config['operation']);
		$this->_operation_config['db'] = $this;

		// Create a new PDO connection
		$this->_connection = new PDO($dsn, $username, $password, $attributes);

		if (isset($this->_config['identifier']))
		{
			// Allow the identifier to be overloaded per-connection
			$this->_identifier = (string) $this->_config['identifier'];
		}

		if ( ! empty($this->_config['charset']))
		{
			// Set the character set
			$this->set_charset($this->_config['charset']);
		}
	}
	// modified
	public function disconnect()
	{
		// Destroy the PDO object
		$this->_last_statement = NULL;
		$this->_connection = NULL;

		return TRUE;
	}
	// unmodified
	public function set_charset($charset)
	{
		// Make sure the database is connected
		$this->_connection or $this->connect();

		// Execute a raw SET NAMES query
		$this->_connection->exec('SET NAMES '.$this->quote($charset));
	}
	// heavily modified
	public function query($type, $sql, $as_object = FALSE, array $config = NULL)
	{
		$this->_calltype = self::QUERY;
		return $this->process_sql($type, $sql, $as_object, $config);
	}
	// new: identical to query() except no execute
	public function prepare($type, $sql, $as_object, $bindings = NULL, $binding_datatypes = NULL, $config = NULL)
	{
		$this->_calltype = self::PREPARE;
		return $this->process_sql($type, $sql, $as_object, $config, $bindings, $binding_datatypes);
	}
	// new, but based on previous
	protected function process_sql($type, $sql, $as_object, $config = NULL, $bindings = NULL, $binding_datatypes = NULL)
	{
		$this->_statement_error = NULL;
		// Make sure the database is connected
		$this->_connection or $this->connect();
		if ( ! empty($this->_config['profiling']))
		{
			// Benchmark this query for the current instance
			$benchmark = Profiler::start("Database ({$this->_instance})", $sql);
		}
		
		if ($type === Database::SELECT)
			$statement_config = $this->_result_config;
		else
			$statement_config = $this->_operation_config;
		if (isset($config))
			$statement_config = array_merge($statement_config,$config);
		try
		{
			$statement = $this->prepare_statement($sql, $statement_config);
			if ($statement)
			{
//				echo 'SUCCEEDED PREPARE STATEMENT ';
				if ($this->_calltype === self::QUERY) 
				{
					$result = $statement->execute();
					if (!$result)
					{
						$this->_statement_error = $statement->errorInfo();
						$this->_last_statement = NULL;
						$statement = $result;
					}
				}
				else
				{
					if (!empty($bindings))
					{
						$binding_datatypes or ($binding_datatypes = array());
						foreach ($bindings as $key => &$value)
						{
							if (!isset($binding_datatypes[$key]))
								$statement->bindParam($key,&$value);
							else
								$statement->bindParam($key,&$value,$binding_datatypes[$key]);
						}
					}
				}
			} else { 
//				echo 'FAILED PREPARE STATEMENT ';
			}
		}
		catch (Exception $e)
		{
//			echo 'EXCEPTION PREPARE STATEMENT ';
			if (isset($benchmark))
			{
				// This benchmark is worthless
				Profiler::delete($benchmark);
			}
			$dberror = (isset($statement) and (!empty($statement)))? $statement->errorCode():0;
			// Rethrow the exception
			throw new Database_Exception($dberror,':error',
				array(':error' => $e->getMessage(),	$e->getCode()));
		}

		if (isset($benchmark))
		{
			Profiler::stop($benchmark);
		}
		if (!$statement) return $statement;
		return $this->create_return_object($type, $sql, $as_object, $statement_config);
	}
	// new
	protected function prepare_statement($sql, $config)
	{
		$this->_last_statement = NULL;
		if (isset($config['attributes']))
			$attributes = $config['attributes'];
		if (empty($attributes)) $attributes = array();
		return $this->_last_statement = $this->_connection->prepare($sql, $attributes);
	}
	// new
	protected function create_return_object($type, $sql, $as_object, $config)
	{
		$statement = $this->_last_statement;
		// Set the last query
		$this->last_query = $sql;

		if ($type === Database::SELECT)
		{
			if ($as_object === FALSE)
			{
				$statement->setFetchMode(PDO::FETCH_ASSOC);
			}
			elseif (is_string($as_object))
			{
				$statement->setFetchMode(PDO::FETCH_CLASS, $as_object);
			}
			else
			{
				$statement->setFetchMode(PDO::FETCH_CLASS, 'stdClass');
			}
			$resultclass = 'Database_Result_' . $config['type'];
			return new $resultclass($statement, $sql, $as_object, $config);
		}
		elseif ($this->_calltype === self::PREPARE)
		{
			$operationclass = 'Database_Operation_' . $config['type'];
			return new $operationclass($statement, $sql, $config);
		}
		elseif ($type === Database::INSERT)
		{
			// Return a list of insert id and rows created
			return array( // Kohana legacy
				$this->insert_id(),
				$this->row_count(),
			);
		}
		else // update or delete
		{
			// Return the number of rows affected
			return $this->row_count(); // Kohana legacy
		}
	}
	// isolated to allow subclass over-ride
	public function insert_id($name = NULL)
	{
		return $this->_connection->lastInsertId($name);
	}
	public function row_count()
	{
		$statement = $this->_last_statement;
		if ($statement) return $statement->rowCount();
		else return NULL;
	}
	public function error_code()
	{
		return $this->_connection->errorCode(); // SQLSTATE
	}
	public function error_info()
	{
		return $this->_connection->errorInfo(); // array(0=>SQLSTATE,1=>driver_error_code,2=>errormessage)
	}
	public function statement_error($error = NULL)
	{
		if (!is_null($error))
			$this->_statement_error = $error;
		return $this->_statement_error;
	}
	public function begin_transaction()
	{
		return $this->_connection->beginTransaction();
	}
	public function commit()
	{
		return $this->_connection->commit();
	}
	public function rollback()
	{
		return $this->_connection->rollBack();
	}
	// quote filters out parameter processing
	public function quote($value)
	{
		if (($this->_calltype === self::PREPARE) and (is_string($value)) and (!empty($value)))
		{
			if (($value == '?') or ($value[0] == ':')) return $value; // parameter placeholder, do not modify
		}
		return parent::quote($value); // escape
	}
	// unmodified
	public function escape($value)
	{
		// Make sure the database is connected
		$this->_connection or $this->connect();

		return $this->_connection->quote($value);
	}
	// placeholders, unmodified
	public function list_tables($like = NULL)
	{
		throw new Kohana_Exception('Database method :method is not supported by :class',
			array(':method' => __FUNCTION__, ':class' => __CLASS__));
	}

	public function list_columns($table, $like = NULL, $add_prefix = TRUE) // add_prefix is new Ko 3.1.2
	{
		throw new Kohana_Exception('Database method :method is not supported by :class',
			array(':method' => __FUNCTION__, ':class' => __CLASS__));
	}
	
	// modified
	public function exec($sql)
	{
		$this->_calltype = self::QUERY;
		return $this->_connection->exec($sql);
	}
	# property accessors
	/* connection: 
	   use for 

		->getAttribute(...
		->setAttribute(...
		
	   do not use for
		->lastInsertId - returned with query or prepare; available through this API
		->prepare - wrapped in this->prepare
	    ->exec(... - available here
		->query - wrapped in this->query
		->quote - handled with query builder; available from database object
		->errorCode(... handled by pdo driver
		->errorInfo(... handled by pdo driver
	    ->beginTransaction(...
		->commit(...
		->rollback(...
		
	*/
	public function connection()
	{
		return $this->_connection;
	}
	public function last_statement() // returns PDOStatement object
	{
		return $this->_last_statement;
	}
	public function config()
	{
		return $this->_config;
	}
	public function result_config()
	{
		return $this->_result_config;
	}
	public function operation_config()
	{
		return $this->_operation_config;
	}
	public function is_query()
	{
		$this->_calltype = self::QUERY;
	}
	public function is_prepare()
	{
		$this->_calltype = self::PREPARE;
	}
	public function call_type()
	{
		return $this->_calltype;
	}
	// new abstract functions in Kohana_Database, march 2011
	// $mode is ignored here because isolation level in pdo
	// has to be set before begin transaction level.
	public function begin($mode = NULL)
	{
		$this->begin_transaction();
	}
} // End Database_PDO
