<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
class Muster_Database_Query extends Kohana_Database_Query 
{	
	protected $_db; 
	protected $_config;
	protected $_sql;
	// for execute of prepared statement	
	/**
	 * Execute the current query on the given database.
	 *
	 * @param   mixed  Database instance or name of instance
	 * @return  object   Database_Result for SELECT queries
	 * @return  mixed    the insert id for INSERT queries
	 * @return  integer  number of affected rows for all other queries
	 */
	public function sql()
	{
		return $this->_sql;
	}
	public function statement_config($config)
	{
		$this->_config = $config;
		return $this;
	}
	public function execute($db = 'default',$as_object = NULL, $config = NULL) 
	{
		if ( ! is_object($db))
		{
			// Get the database instance
			$db = Database::instance($db);
		}
		if ($as_object === NULL)
		{
			 $as_object = $this->_as_object;
		}

		if ($config === NULL)
		{
			$config = $this->_config;
		}
		$db->is_query(); // notify to use quote
		// Compile the SQL query
		$sql = $this->_sql = $this->compile($db);
		
		if ( ! empty($this->_lifetime) AND $this->_type === Database::SELECT)
		{
			// Set the cache key based on the database instance name and SQL
			$cache_key = 'Database::query("'.$db.'", "'.$sql.'")';

			if ($result = Kohana::cache($cache_key, NULL, $this->_lifetime))
			{
				// Return a cached result
				return new Database_Result_Cached($result, $sql, $this->_as_object);
			}
		}

		// Execute the query
		$result = $db->query($this->_type, $sql, $as_object,$config); // adds config parameter
		if (isset($cache_key))
		{
			// Cache the result array
			Kohana::cache($cache_key, $result->as_array(), $this->_lifetime);
		}

		return $result;
	}
	# new function
	public function prepare($db = 'default',$bindings = NULL,$binding_datatypes = NULL)	
	{		
		$config = $this->_config;
		if ( ! is_object($db))		
		{			
			// Get the database instance			
			$db = Database::instance($db);		
		}		
		$this->_db = $db;
		$db->is_prepare(); // notify not to use quote for compile
		// Compile the SQL query
		$sql = $this->_sql = $this->compile($db);
		// Execute the query
		$result = $db->prepare($this->_type, $sql, $this->_as_object, $bindings, $binding_datatypes, $config);
		
		return $result;	
	}
	public function clear_select()
	{
		$this->_select = array();
		return $this;
	}
}