<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
// extension of Kohana_Database_Result
// statement wrapper
/*
	- provides two way cursor if supported, cursor emulation if so configured
	- provides execute($parameters) for select prepared statements
	- surfaces informational functions
*/
class Muster_Database_Result_PDO extends Database_Result 
{
	protected $_sql;
	protected $_statement;
	protected $_cursor_scroll; // Boolean
	protected $_emulate_cursor; // Boolean
	protected $_fetch_style; // pdo constant
//	protected $_parameters; // save for rewind
	protected $_db;
	// currency
	protected $_total_rows  = 0;
	protected $_current_row = 0;
//	protected $_internal_row = -1;
	protected $_BOF = TRUE;
	protected $_row_data; // current row data
	protected $_success = TRUE;
	
	// based on the Kohana original
	public function __construct($statement, $sql, $as_object, $config)
	{
		$config = (array) $config;
		$config = $config + array(
			'cursorscroll' => FALSE,
			'emulatecursor' => FALSE,
		);
		$this->_cursor_scroll = $config['cursorscroll'];
		$this->_emulate_cursor = $config['emulatecursor'];
		// Store the statement locally
		$this->_statement = $this->_result = $statement; // _result = Kohana legacy

		// Store the SQL locally
		$this->_sql = $this->_query = $sql; // _query = Kohana legacy
		$this->_db = $config['db'];

		if (is_object($as_object))
		{
			// Get the object class name
			$as_object = get_class($as_object);
		}

		// Results as objects or associative arrays
		$this->_as_object = $as_object;
		
		$this->_total_rows = $statement->rowCount();
		
		$this->_fetch_style = ($this->_as_object === FALSE)? PDO::FETCH_ASSOC : PDO::FETCH_CLASS;
		
	}
	public function __destruct()
	{
		$this->_statement = NULL;
		$this->_db = NULL;
		$this->_row_data = NULL;
	}

	// based on the Kohana original
	// $value is column name for single value
	public function as_array($key = NULL, $value = NULL)
	{
		$rows = $this->_statement->fetchAll($this->_fetch_style);
		$results = array();

		if ($key === NULL AND $value === NULL) // returns indexed array of rows
		{
			// Indexed rows
			$results = $rows;

		}
		elseif ($key === NULL) // $value = column name used for value of indexed array
		// returns indexed array of single values
		{
			// Indexed columns

			if ($this->_as_object)
			{
				foreach ($rows as $row)
				{
					$results[] = $row->$value;
				}
			}
			else
			{
				foreach ($rows as $row)
				{
					$results[] = $row[$value];
				}
			}
		}
		elseif ($value === NULL) // $key = column name used for associative index
		// returns dictionary of rows
		{
			// Associative rows

			if ($this->_as_object)
			{
				foreach ($rows as $row)
				{
					$results[$row->$key] = $row;
				}
			}
			else
			{
				foreach ($rows as $row)
				{
					$results[$row[$key]] = $row;
				}
			}
		}
		else // $key used as column name for associative key, $value used for record value
		// returns dictionary of single values
		{
			// Associative columns

			if ($this->_as_object)
			{
				foreach ($rows as $row)
				{
					$results[$row->$key] = $row->$value;
				}
			}
			else
			{
				foreach ($rows as $row)
				{
					$results[$row[$key]] = $row[$value];
				}
			}
		}
		// this is different from the original
		if ($this->_cursor_scroll or $this->_emulate_cursor)
			$this->rewind();

		return $results;
	}
	public function seek($offset)
	{
		if (!$this->offsetExists($offset))
			return FALSE;
		if ($this->_cursor_scroll)
		{
			if ($this->_BOF) $this_BOF = FALSE;
			$this->_current_row = $offet; // $this->_internal_row = $offset;
			return TRUE;
		}
		elseif ($offset < $this->_current_row)
		{
			if ($this->_emulate_cursor)
			{
				$this->rewind();
			}
			else
				throw new Kohana_Exception('Database statement is forward moving only, cannot seek backwards');
		}
		// increment next to offset
		while ($this->_current_row < $offset)
			$this->next();
		return TRUE;
	}
	public function current()
	{
		if ($this->_cursor_scroll)
		{
			return $this->_statement->fetch($this->_fetch_style,PDO::FETCH_ORI_ABS,$this->_current_row);
		}
		else
		{
			if ($this->_BOF) // get first
				$this->next();
			return $this->_row_data;
		}
	}
	public function next()
	{
		if ($this->_BOF)
			$this->_BOF = FALSE;
		else
		{
			$this->_current_row++;
		}
		if (!$this->_cursor_scroll)
		{
			if ($this->_current_row < $this->_total_rows)
			{
				$this->_row_data = $this->_statement->fetch($this->_fetch_style,PDO::FETCH_ORI_NEXT);
			}
			else
			{
				$this->_row_data = NULL;
			}
		}
		return $this;
	}
	public function prev()
	{
		if ($this->_BOF) return $this;
		if ($this->_cursor_scroll)
		{
			if ($this->_current_row == 0)
				$this->_BOF = TRUE;
			elseif ($this->_current_row > 0)
				$this->_current_row--;
		}
		elseif ($this->_emulate_cursor)
		{
			if ($this->_current_row == 0)
				$this->_BOF = TRUE;
			else
				$this->seek($this->_current_row - 1);
		}
		else
		{
			throw new Kohana_Exception('Database statement is forward moving only, cannot go to previous');
		}
		return $this;
	}
	// fetch next
	public function fetch()
	{
		return $this->next()->current();
	}
	public function rewind()
	{
		if ($this->_BOF) return $this;
		$this->_BOF = TRUE;
		$this->_current_row = 0;
		if ($this->_emulate_cursor)
		{
			$this->requery();
		}
		else
			throw new Kohana_Exception('Database statement is forward moving only, cannot rewind');
		return $this;
	}
	// new...
	public function requery()
	{
		$this->_statement->closeCursor();
		$this->execute(); // $this->_parameters); // PDO statement uses previous parameters
		return $this;
	}
	public function execute($parameters = NULL) // for prepare
	{
		$this->_total_rows = 0;
		try
		{
			$statement = $this->_statement;
//			$this->_parameters = $parameters; // save for requery - nor required, PDO statement saves parameters
			if (empty($parameters))
			{
				$retval = $statement->execute();
			}
			else
			{
				$retval = $statement->execute($parameters);
			}
			$this->_db->statement_error($statement->errorInfo());
			$this->_success = (bool) $retval; 
		}
		catch (Exception $e)
		{
			$this->_success = FALSE;
//			var_dump('***FALSE***');
			$this->_db->statement_error($statement->errorInfo());
			// Rethrow the exception
			$dberror = (isset($statement) and (!empty($statement)))? $statement->errorCode():0;
			throw new Database_Exception($dberror,':error (:code)',
				array(':error' => $e->getMessage(),	':code'=>$e->getCode()));
		}
		$this->_BOF = TRUE;
		$this->_current_row = 0;
		$this->_total_rows = $statement->rowCount();
		return $this;
	}
	public function success()
	{
		return $this->_success;
	}
	public function error_code()
	{
		return $this->_statement->errorCode(); // SQLSTATE
	}
	public function error_info()
	{
		return $this->_statement->errorInfo(); // array(0=>SQLSTATE,1=>drivererrorcode,2=>errormessage)
	}
	public function statement()
	{
		return $this->_statement;
	}
	public function sql()
	{
		return $this->_statement->queryString;
	}
	public function valid()
	{
		return (boolean) (($this->_current_row >= 0) and ($this->_current_row < $this->_total_rows));
	}
	public function row_count()
	{
		return $this->_total_rows;
	}
	public function current_row()
	{
		return $this->_current_row;
	}
	public function db()
	{
		return $this->_db;
	}
}
