<?php 
namespace Muster;

use StdClass;
use Point;
use 
	Muster\Point\Control, 
	Muster\Point\Parameter;

defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
/*
	todo:
	- error handling
	- create separate template dictionary for containers (navigation)

	notes:
	- is handed frames for data, controls, and containers
	- calls input gateway to obtain data for these
	- makes decisions based on control points regarding data transformations and storage based on controls
	- calls output gateway to create dictionary
*/
class DataModel {
	// the gateway used to manage data
	public $gateway;
	// the data set
	public $dataframe;
	// the control set
	public $controlframe;
	// the navigation set
	public $containerframe;
	// the data output for template
	public $dictionary = array();
	// the navigation tree for template
	public $container_dictionary;
	// responses to control requests
	public $responses = array();
	// the calling controller
	public $controller;
	// data sources
	public $datapool;
	// data cache
	public $cache = array();
	// ownerid for token assessment
	public $ownerid;
	// articleid for token assessment
	public $articleid;
	// record caching control
	protected $_recordcaching = FALSE;
	// datapool variables error message index
	protected $_errordictionaryindex;
	protected $_errormessageindex;
	public $errorstate = 0;
	
	public static function renumber_listitems($listitems,$prefix,$errorindex='errorindex',$referenceindex='reference') {
		$count = -1;
		foreach ($listitems as $listitem) {
			$count++;
			$listitem->index = $count;
			$reference = $listitem->reference = $prefix . $count;
			if ($listitem->pointlist[$errorindex]->value)
				$listitem->pointlist[$errorindex]->value = $count + 1;
			$listitem->pointlist[$referenceindex]->value = $reference;
		}
	}
	
	public static function set_listitem_errorstate($listitem,
		$recordstatusindex='recordstatus',$errorindex='errorindex')
	{
		$listitem->errorstate = Point::ERRORSTATE_ERROR;
		$listitem->recordstatus = 'error';
		$listitem->pointlist['recordstatus']->value = 'error';
		$listitem->pointlist['errorindex']->value = $listitem->index + 1;
	}
	public function __construct($controller)
	{
		$this->controller = $controller;
		$this->gateway = new Gateway($this);
		$datapool = $this->datapool = new StdClass;
		// the set of constraints limiting processing of metadata
		$datapool->constraints = array('mode' => 'browse');
		$datapool->arrays = array('get' => $_GET,'post' => $_POST, 'files' => $_FILES);
		$datapool->variables = array();
		$datapool->records = array();
		$datapool->recordsets = array();
		$datapool->functions = array();
		$datapool->objects = array();
		$datapool->activerecords = array();
		$datapool->tables = array();
		$this->cache = clone $datapool;
	}
	/* if $constraint is an array, passed to ->constraint(...
	if $constraint is an object, then looks for $constraint->constraints and processes multiple constraints
	the object $constraint can also have the property ->condition being 'AND' or 'OR' (defaults to 'AND')
	*/
	public function constraints($constraint)
	{
		if (is_array($constraint)) {
			$constraint = (object) $constraint;
			return $this->constraint($constraint->name,$constraint->value);
		} elseif (is_object($constraint)) {
			$constraints = $constraint->constraints;
			if (property_exists($constraint,'condition'))
				$condition = $constraint->condition;
			else 
				$condition = 'AND';
			if (!(($condition == 'AND') or ($condition == 'OR'))) $condition = 'AND';
			$retval = NULL;
			foreach ($constraints as $constraint) {
				if (is_object($constraint)) {
					$result = $this->constraints($constraint);
 					if (is_null($retval)) $retval = $result;
					else $retval = ($result or $retval);
				} else {
					$constraint = (object) $constraint;
					$result = $this->constraint($constraint->name,$constraint->value);
					if (is_null($retval))
						$retval = $result;
					else {
						if ($condition == 'AND')
							$retval = ($retval and $result);
						else
							$retval = ($retval or $result);
					}
				}
			}
			return (bool) $retval;
		} else {
			return FALSE;
		}
	}
	// if the index does not exist, FALSE
	// if the index exists, and no comparison value is provided, TRUE (exists)
	// otherwise the constraint value is compared with $compare, and must be equivalent to be TRUE
	public function constraint($index, $compare = NULL)
	{
		$constraints = $this->datapool->constraints;
		if (!isset($constraints[$index])) return FALSE;
		if (is_null($compare)) return TRUE;
		return $constraints[$index] === $compare;
	}
	public function recordcaching($recordcaching = NULL)
	{
		if (is_bool($recordcaching)) {
			$this->_recordcaching = $recordcaching;
			return $this;
		}
		return $this->_recordcaching;
	}
	public function dataframe($frame = NULL)
	{
		if (!is_null($frame)) {
			if (is_string($frame))
				$frame = Point::get_frame($frame);
			$this->dataframe = (object) $frame;
			return $this;
		} else {
			return $this->_dataframe;
		}
	}
	public function errordictionaryindex($index = NULL)
	{
		if (is_null($index)) {
			return $this->_errordictionaryindex;
		} else {
			$this->_errordictionaryindex = $index;
			return $this;
		}
	}
	public function errormessageindex($file = NULL)
	{
		if (is_null($file)) {
			return $this->_errormessageindex;
		} else {
			$this->_errormessageindex = $file;
			return $this;
		}
	}
	public function controlframe($frame = NULL)
	{
		if (!is_null($frame)) {
			if (is_string($frame))
				$frame = Point::get_frame($frame);
			$this->controlframe = (object) $frame;
			return $this;
		} else {
			return $this->_controlframe;
		}
	}
	public function containerframe($frame = NULL)
	{
		if (!is_null($frame)) {
			if (is_string($frame))
				$frame = Point::get_frame($frame);
			$this->containerframe = (object) $frame;
			return $this;
		} else {
			return $this->_containerframe;
		}
	}
	public function responses($responses = NULL)
	{
		if (!is_null($responses)) {
			foreach ($responses as $index => $response) {
				$this->responses[$index] = $response;
			}
			return $this;
		} else {
			return $this->responses;
		}
	}
	public function command()
	{
		// call input gateway for data
		if ($this->controlframe) {
			$this->gateway->process_input()->with($this->controlframe)->execute();
			$command = $this->get_command();
			return $command;
		}
	}
	public function get_command()
	{
		if ($this->controlframe) {
			$controls = $this->controlframe->pointlist;
			if ($controls) {
				foreach ($controls as $index => $control) {
					if (($control instanceof Control) and ($value = $control->value)) {
						return $value;
					}
				}
			}
		}
	}
	public function get_command_control()
	{
		if ($this->controlframe) {
			$controls = $this->controlframe->pointlist;
			if ($controls) {
				foreach ($controls as $index => $control) {
					if (($control instanceof Control) and ($value = $control->value)) {
						return $control;
					}
				}
			}
		}
	}
	public function get_parameters()
	{
		if ($this->controlframe) {
			$controls = $this->controlframe->pointlist;
			$parameters = array();
			if ($controls) {
				foreach ($controls as $index => $item) {
					if ($item instanceof Parameter) {
						$parameters[$index] = $item->value;
					}
				}
			}
			return $parameters;
		}
		return NULL;
	}
	// dispatcher
	public function respond()
	{
		// process controls
		$command = $this->command();
		$responses = $this->responses;
		if (isset($responses[$command])) {
			return call_user_func($responses[$command],$this);
		} else {
			return NULL;
		}
	}
	public function retrieve()
	{
		$gateway = $this->gateway;
		if ($this->dataframe)
			$gateway->process_retrieve()->with($this->dataframe)->execute();
		return $this;
	}
	public function output()
	{
		$gateway = $this->gateway;
		if ($this->dataframe) {
			$gateway->process_output()->with($this->dataframe)->execute();
		}
		return $this->dictionary;
	}
	// processes dataframe
	public function input($default_recover_state = NULL)
	{
		$gateway = $this->gateway;
		if ($this->dataframe)
			$gateway->process_input($default_recover_state)->with($this->dataframe)->execute();
		return $this;
	}
	// processes controlframe
	public function control_input($default_recover_state = NULL)
	{
		$gateway = $this->gateway;
		if ($this->controlframe)
			$gateway->process_input($default_recover_state)->with($this->controlframe)->execute();
		return $this;
	}
	public function validate()
	{
		$gateway = $this->gateway;
		if ($this->dataframe) {
			$gateway->process_validate()
				->with($this->dataframe)
				->errordictionaryindex($this->_errordictionaryindex)
				->errormessageindex($this->_errormessageindex)
				->execute();
		}
		return !$this->errorstate;
	}
	public function finishinput()
	{
		$gateway = $this->gateway;
		if ($this->dataframe)
			$gateway->process_finishinput()->with($this->dataframe)->execute();
		return $this;
	}
	public function recoverinput()
	{
		$gateway = $this->gateway;
		if ($this->dataframe)
			$gateway->process_recoverinput()->with($this->dataframe)->execute();
		return $this;
	}
	public function store()
	{
		$gateway = $this->gateway;
		if ($this->dataframe)
			$gateway->process_store()->with($this->dataframe)->execute();
		return $this;
	}
}
