<?php 

namespace Muster\Node;

use StdClass;
use DB, Session, Kohana;
use DataModel, System, Access, User, TableModel, Services;
use Muster\Node;

defined('SYSPATH') or die('No direct script access.');
/*
	** events:
	_on_set_query
	_on_set_datapool
	_on_get_dictionary
	_on_get_admindictionary
	
	$context properties used in list:
	
	@def control frame for the list
	@in _list_get_parameters
	@required true
	->controlframe 
	
	@def default orderby (column name) in absence of search
	@in _list_get_parameters
	@required false
	->default_orderby

	@def default orderdirection (column name) in absence of search
	@in _list_get_parameters
	@required false
	->default_orderdirection
	
	@def set of tokens required
	@in _set_permissions
	@required true
	->tokens, which has the following parameters:
		->premium 		= 'article_premium';
		->content 		= 'article_content';
		->hidden 		= 'article_hidden';
		->trash 		= 'article_trash';
		->unpublished 	= 'article_unpublished';
		->settings 		= 'article_settings';
	all these token properties are required
	
	@def redirect uri for redirect options (page change, perpage change
	@in _list_process_redirect_command, _list_get_dictionary, _list_set_session_data
	
	$def schema file name (without extension)
	@in _list_get_tablemodel
	@required true
	->tablemodel
	
	@def the binder scope for search - causes leftedge and root condition
	@in _list_get_tablemodel
	@required false
	->binderscope
	
	@def augments column list of master table
	@in _list_get_tablemodel
	@required false
	->selectarray
	
	@in _list_get_tablemodel
		@def the formal object type of the recordset being searched. 
			In the case of folders, where $context->foreignkeyvalue is set, the object type is
			considered to belong to the master, and any kind of child can be returned.
	@in _list_set_gateway_resources
		@def and 'objecttype' constraint is autmatically set for the objecttype
	@required required
	->objecttype
	
	@in _list_set_session_data
	->objectname
	
	@def not yet used but included for symmetry to record context
	->messagefile 
	
	@def  - for folder objecttype, the FolderID of parent instance records
		- for entry type, the ParentID of entry records
	@in _list_get_tablemodel
	@required false
	->foreignkeyvalue

	@def for folder, indicated if treetops constraint is being requested
	@in _list_get_tablemodel
	@required false
	->treetopsflag
	
	@def frame filename (no ext) for the call
	@in _list_set_gateway_resources
	@required true
	->dataframe
	
	@def constraints to add to $tablemodel->constraints ('index' => index,'value' => value)
	@in _list_set_gateway_resources
	@required false
	->constraints
	
	@def identities for master table
	@in _list_set_gateway_resources
	@required true
	->mastertableindex - index into $datapool->tables for Point::DATASOURCE_TABLEACTIVERECORD
	->mastertablename - not currently used, but included for symmetry
	->masterrecordindex - index into $datapool->activerecords for Point::DATASOURCE_ACTIVERECORD
	@required true for objecttype = folder,document
	@def containing folder list
	->parenttableindex - index into $datapool->tables for Point::DATASOURCE_TABLEACTIVERECORD
	->parenttablename - index into tablemodel tablename ($model->activerecord(...))
	->parentrecordindex - index into $datapool->activerecords for Point::DATASOURCE_ACTIVERECORD
	
	@def lookups in datapool->arrays, for example list of partials
	@in _list_set_gateway_resources
	@required false
	->datapoolarrays
	
*/
abstract class ListNode extends Node {
	public function __construct($controller)
	{
		# get engine
		$this->_datamodel = new DataModel($controller);
		parent::__construct($controller);
	}
	#----------------[ List ]----------------------
	public function generate() // $context = NULL)
	{
		$messages = $this->_messages = new StdClass;
		$messages->errors = array();
		$messages->warnings = array();
		$messages->messages = array();
		# save context
		$context = $this->_context;
		
		# set constraints for parameters
		$this->_set_constraints($context);
		
		# establish parameters
		$parameters = $this->_parameters = $this->_get_parameters($context);
		
		# redirect page change, perpage, or certain search requests
		if ($parameters->command) $this->_process_redirect_command($parameters,$context);
		
		# permissions
		$permissions = $this->_permissions = $this->_set_permissions($parameters,$context);
		
		# tablemodel
		$tablemodel = $this->_tablemodel = $this->_get_tablemodel($parameters,$permissions,$context);
		
		# boundaries
		$boundaries = $this->_boundaries = $this->_get_boundaries($tablemodel,$parameters);

		# finalize query
		$this->_finalize_query($parameters,$tablemodel,$boundaries);
		
		# gateway resources
		$this->_set_gateway_resources($parameters,$tablemodel,$context);

		# record change commands
		$this->_process_operation_command($parameters,$context);

		# get template dictionaries	
		$dictionary = $this->_get_dictionary($permissions,$parameters,$boundaries,$context);
		isset($context->get_admindictionary) or ($context->get_admindictionary = TRUE);
		if ($context->get_admindictionary) {
			$admindictionary = $this->_get_admindictionary($permissions,$parameters,$dictionary);
		} else {
			$admindictionary = NULL;
		}

		# assemble return data
		$data = compact('dictionary','admindictionary');
		return $data;
	}

	protected function _set_constraints($context)
	{
		if (property_exists($context,'constraints') and !empty($context->constraints)) {
			$constraints = $context->constraints;
			$datapool = $this->_datamodel->datapool;
			foreach ($constraints as $constraint) {
				$constraint = (object) $constraint;
				$datapool->constraints[$constraint->index] = $constraint->value;
			}
		}
	}

	/*	---------------------------------------------------------------------
		request parameter data could be 
		#commands
		first TRUE|FALSE
		previous TRUE|FALSE
		next TRUE|FALSE
		last TRUE|FALSE
		selected NULL|Search
		changeperpage NULL|Change
		#parameters
		search ''|<string>
		searchmethod  ''|<string>
		currentpage NULL|<integer>
		perpage NULL|<integer>
		order  ''|<string>
		direction  ''|desc|asc
		select ''|<string>
		authorid ''|<integer>
	*/
	protected function _get_parameters($context)
	{
		$parameters = new StdClass;
		$datamodel = $this->_datamodel;
		if ($this->_controller->is_request()) {
			$request_data = 
				$datamodel->controlframe($context->controlframe)
					->control_input()
					->get_parameters();
		} else {
			$request_data = array();
		}
		$parameters = (object) $request_data;
		// make sure standard parameters exist for _get_redirect_parameters
		empty($parameters->mode) and ($parameters->mode = NULL);
		empty($parameters->instanceid) and ($parameters->instanceid = NULL);
		empty($parameters->path) and ($parameters->path = NULL);
		empty($parameters->itempath) and ($parameters->itempath = NULL);
		empty($parameters->itemperpage) and ($parameters->itemperpage = NULL);
		empty($parameters->listview) and ($parameters->listview = NULL);
		
		empty($parameters->search) and ($parameters->search = NULL);
		empty($parameters->searchmethod) and ($parameters->searchmethod = NULL);
		empty($parameters->select) and ($parameters->select = NULL);
		property_exists($parameters,'currentpage') or ($parameters->currentpage = NULL);
		empty($parameters->orderby) and ($parameters->orderby = NULL);
		empty($parameters->perpage) and ($parameters->perpage = NULL);
		empty($parameters->direction) and ($parameters->direction = NULL);
		property_exists($parameters,'authorid') or ($parameters->authorid = NULL);
		empty($parameters->fsearch) and ($parameters->fsearch = NULL);
		empty($parameters->fsfield) and ($parameters->fsfield = NULL);
		empty($parameters->fswild) and ($parameters->fswild = 0);
		
		$parameters->articleid = isset($context->articleid)?$context->articleid:NULL;
		$control = 
			$datamodel->get_command_control();
		if ($control) {
			if (property_exists($control,'parameters')) {
				$parms = $control->parameters; // pass parameters->mode = 'admin' for new record command
				foreach ($parms as $key => $value) {
					$parameters->$key = $value;
				}
			}
			$parameters->command = $control->value;
		} else {
			$parameters->command = !empty($parameters->action)?$parameters->action:NULL;
		}
		// $system = System::instance();
		$parameters->premiumactive = FALSE; // ** DEFERRED (bool) $system->ArticlePremiumPrecedence;
		# collect and normalize parameters
		isset($context->get_requestdata) or ($context->get_requestdata = TRUE);
		$parameters->selecttext = '';
		if ((!empty($request_data)) and $context->get_requestdata) {
			isset($request_data['search']) and ($parameters->search = $request_data['search']);
			isset($request_data['searchmethod']) and ($parameters->searchmethod = $request_data['searchmethod']);
			
			isset($request_data['select']) and ($parameters->select = $request_data['select']);
			if (isset($parameters->select) and ($parameters->select == 'all')) $parameters->select = '';
			isset($request_data['authorid']) and ($parameters->selectauthorid = $request_data['authorid']);
			
			if ($parameters->select == 'requested')
				$parameters->selecttext = 'approvals requested';
			else
				$parameters->selecttext = $parameters->select;
			
			isset($request_data['orderby']) and ($parameters->orderby = $request_data['orderby']);
			isset($request_data['direction']) and  ($parameters->direction = $request_data['direction']);
			
			isset($request_data['currentpage']) and ($parameters->currentpage = $request_data['currentpage']);
			isset($request_data['perpage']) and ($parameters->perpage = $request_data['perpage']);
			
		} else {
			$parameters->search = NULL;
			$parameters->select = NULL;
			$parameters->selectauthorid = NULL;
			$parameters->currentpage = 1;
			$parameters->perpage = 30;
			$parameters->orderby = NULL;
		}
		!isset($parameters->selectauthorid) and ($parameters->selectauthorid = NULL);
		if (empty($context->secondary_orderby))
			$context->secondary_orderby = 'Title';
		if (empty($context->secondary_direction))
			$context->secondary_direction = 'ASC';
		if (empty($context->default_orderby))
			$context->default_orderby = 'ArticleDate';
		if (empty($context->default_direction)) {
			if ($context->default_orderby == 'ArticleDate')
				$context->default_direction = 'DESC';
			else
				$context->default_direction = 'ASC';
		}
		if (empty($parameters->search) and empty($parameters->fsearch)) {
			$parameters->searchmethod = NULL;
			// set orderby
			if (empty($parameters->orderby)) {
				$parameters->orderby = $context->default_orderby;
			}
			// set direction
			if ($parameters->orderby == 'EditDate') {
				$parameters->direction = 'DESC';
			} elseif ($parameters->orderby == 'Order') {
				$parameters->direction = 'ASC';
			}
			if (empty($parameters->direction)) {
				$parameters->direction = $context->default_direction;
			}
			$parameters->listordertext = '';
			// set listordertext from all possible orderby
			if ($parameters->orderby == 'ArticleDate') {
				$parameters->listordertext = 'date ';
				if ($parameters->direction == 'DESC') $parameters->listordertext .= '(desc)';
				elseif ($parameters->direction == 'ASC') $parameters->listordertext .= '(asc)';
				if (!((!empty($context->objecttype)) and ($context->objecttype == 'entry')))
					$parameters->listordertext .= ', title (asc)';
			} elseif ($parameters->orderby == 'Title') {
				$parameters->listordertext = 'title ';
				if ($parameters->direction == 'DESC') $parameters->listordertext .= '(desc)';
				elseif ($parameters->direction == 'ASC') $parameters->listordertext .= '(asc)';
			} elseif ($parameters->orderby == 'EditDate') {
				$parameters->listordertext = 'edit date (desc)';
			} elseif ($parameters->orderby == 'Order') {
				$parameters->listordertext = '&middot;custom order&middot; (if any)';
				if ($context->secondary_orderby == 'Title')
					$parameters->listordertext .= ', title ';
				if ($context->secondary_orderby == 'ArticleDate')
					$parameters->listordertext .= ', date ';
				if ($context->secondary_direction == 'DESC') $parameters->listordertext .= '(desc)';
				elseif ($context->secondary_direction == 'ASC') $parameters->listordertext .= '(asc)';
				if ($context->secondary_orderby == 'ArticleDate')
					$parameters->listordertext .= ', title (asc)';
			} else {
				$parameters->selecttext = '';
			}
				// extend for article date
		} elseif (!empty($parameters->search)) {
			$parameters->listordertext = 'relevance';
			$parameters->searchmethod = 'bm';
			$parameters->orderby = NULL;
			$parameters->direction = NULL;
		} else { // !empty $parameters->fsearch
			$parameters->orderby = 'ArticleDate';
			$parameters->direction = 'DESC';
			$parameters->listordertext = 'date (desc), title (asc)';
		}
		if ($parameters->perpage > 200) {
			$parameters->perpage = 200;
		}
		if ($parameters->perpage <= 2) {
			$parameters->perpage = 30;
		}
		return $parameters;
	}
	protected function _process_redirect_command($parameters,$context)
	{
		$datamodel = $this->_datamodel;
		$command = $datamodel->get_command();
		if (!$command) return;
//		var_dump($command); exit;
		switch ($command) {
			case 'first':
				$parameters->currentpage = 1;
				break;
			case 'previous':
				$parameters->currentpage--;
				break;
			case 'next':
				$parameters->currentpage++;
				break;
			case 'last':
				$parameters->currentpage = "max";
				break;
			case 'recentupdates':
				$parameters->orderby = 'EditDate';
				// pre-empt search parameters...
				$parameters->search = NULL;
				$parameters->searchmethod = NULL;
				$parameters->direction = 'DESC';
				break;
			case 'myarticles': // ??
				$user = $this->_user;
				if ($user->authenticated())
					$parameters->selectauthorid = $user->userid();
			case 'changeperpage':
			case 'searched': // search, can check $value for refinements
				$parameters->currentpage = 1; // reset for change
				break;
			default: return;
		}
		$this->_controller->request->redirect($context->redirect.'?' . 
			"search=" . urlencode($parameters->search) .
			"&m={$parameters->searchmethod}" .
			"&select=" . "{$parameters->select}" .
			"&orderby={$parameters->orderby}" .
			"&d={$parameters->direction}" .
			"&fsearch={$parameters->fsearch}" .
			"&fsfield={$parameters->fsfield}" .
			"&fswild={$parameters->fswild}" .
			"&authorid={$parameters->selectauthorid}" .
			"&listview={$parameters->listview}" .
			"&page={$parameters->currentpage}" .
			"&perpage={$parameters->perpage}" . 
			"&instanceid={$parameters->instanceid}" . 
			"&path={$parameters->path}" .
			"&itempath={$parameters->itempath}" .
			"&itemperpage={$parameters->itemperpage}");
	}
	protected function _set_permissions($parameters,$context)
	{
		$access = Access::instance();
		$user = $this->_user;
		$tokens = $context->tokens;
		$permissions = new StdClass;
		$permissions->cancreate = $access->permission($tokens->content,'C');
		$permissions->canread = $access->permission($tokens->content,'R');
		$permissions->canedit = $access->permission($tokens->content,'E');
		$permissions->cantrash = $access->permission($tokens->content,'D');
		if ($user->authenticated()) {
			$userdata = $user->user_data();
			// ** the following two and's should be at the specific cred permission level
			$permissions->createarticlesenabled = ($userdata->Permissions->article and $permissions->cancreate);
			$permissions->createblogsenabled = ($userdata->Permissions->blog and $permissions->cancreate); // assumes on blog item
			$permissions->createcommentsenabled = $userdata->Permissions->comment; // ** shouldn't be needed here
			$permissions->unmoderatedenabled = $userdata->Permissions->unmoderated;
		} else {
			$permissions->createarticlesenabled = FALSE;
			$permissions->createblogsenabled = FALSE;
			$permissions->createcommentsenabled = FALSE;
			$permissions->unmoderatedenabled = FALSE;
		}
		if ($permissions->canread) {
			$permissions->canreadhidden = $access->permission($tokens->hidden,'R');
			$permissions->canreadtrash = $access->permission($tokens->trash,'R');
			$permissions->canreadunpublished = $access->permission($tokens->unpublished,'R');
		} else {
			$permissions->canreadhidden = FALSE;
			$permissions->canreadtrash = FALSE;
			$permissions->canreadunpublished = FALSE;
		}
		$permissions->canadminister = ($access->permission($tokens->settings,'E') and $permissions->unmoderatedenabled);
		// search options for admin dictionary
		$subsetoption = $permissions->subsetoption = $access->permission($tokens->settings,'R');
		if ($subsetoption) {
			$permissions->unpublishedenabled = $access->permission($tokens->unpublished,'R');
			$permissions->hiddenenabled = $access->permission($tokens->hidden,'R');
			$permissions->trashenabled = $access->permission($tokens->trash,'R');
		} else {
			$permissions->unpublishedenabled = FALSE;
			$permissions->hiddenenabled = FALSE;
			$permissions->trashenabled = FALSE;
		}
		$permissions->favourites = $this->_user->authenticated();
		$permissions->directoriesavailable = $access->permission('directory_content','R');
		$permissions->inboxesavailable = $access->permission('inbox_content','R');
		return $permissions;
	}
	protected function _get_tablemodel($parameters,$permissions,$context)
	{
		# control objects
		$user = User::instance();
		$tablemodel = new StdClass;
		$model = $tablemodel->model = new TableModel($context->tablemodel);
		$mastertable = $tablemodel->mastertable = $model->table();
		$query = $tablemodel->query = $mastertable->query();
		# set binder boundaries
		if (!is_null($context->binderscope)) {
			$binder = DB::select('ID','LeftEdge','RightEdge','RootID')
				->from('Binders_View')
				->where('Code','=',$context->binderscope)
				->as_object()
				->prepare($model->db())
				->execute()->current();
			if (!$binder) return;
			$query->where('LeftEdge','BETWEEN',DB::expr($binder->LeftEdge . ' AND ' . $binder->RightEdge))
				->where('RootID','=',$binder->RootID);
		}
		# global settings
		# apply parameters to table
		$selectarray = // avoid loading Body field
			array('ID','Title','Caption','Description','Location','Hide','Trash','Locked','ObjectType','Frozen',
				'Image','Source','Author','AuthorID','ShowByline','PublicProfile','AuthorUserCode','ArticleDate','Approval',
				'AuthorNotesClosed','FollowupNotesClosed','BinderID','BinderTitle','LeftEdge','RightEdge','RootID');
		if (property_exists($context,'selectarray')) {
			$addselect = $context->selectarray;
			foreach ($addselect as $value) {
				if (!in_array($value,$selectarray))
					$selectarray[] = $value;
			}
		}
		$query->select_array($selectarray);
		# apply permissions
		$this->_set_query_constraints($query,$mastertable->tablename()); // in Node::
		# apply search clause
		if ($permissions->canread) {
			if (!empty($parameters->search)) {
				$mode = $tablemodel->mode = 'IN BOOLEAN MODE';
				$search = $model->db()->escape($parameters->search);
				$query->where_open();
				$query->where(
						DB::expr('MATCH(`Title`,`Caption`,`Description`,`Location`,`Body`)'),
						'', // no operator
						DB::expr('AGAINST(' . $search . ' ' . $mode . ')'));
				$query->where_close();
			} elseif ((!empty($parameters->fsearch)) and (!empty($parameters->fsfield))) {
				$fsfield = $parameters->fsfield;
				if (in_array($fsfield,array('Title','Caption','Location','Source','URL','FileName','Description','Body',))) {
					$query->where_open();
					if ($parameters->fswild) {
						$fsearch = $model->db()->escape('%'.$parameters->fsearch.'%');
						$query->where($fsfield,'LIKE',DB::expr($fsearch));
					} else {
						$fsearch = $model->db()->escape($parameters->fsearch);
						$query->where($fsfield,'LIKE',DB::expr($fsearch));
					}
					$query->where_close();
					$this->_set_orderby($query,$parameters);
				}
			} elseif (!empty($parameters->orderby)) {
					$this->_set_orderby($query,$parameters);
			}
			# add select conditions
			if ($select = $parameters->select) {
				if (in_array(
					$select,
					array(
						'published','unpublished','drafts','requested','declined',
						'hidden','trashed','opennotes','openfollowup','pending','unassigned',
					))) {
					$query->where_open();
					switch ($select) {
						case 'published':
							$query->where_open();
							$query->where('Approval','=','A');
							$query->or_where('Approval','=','P');
							$query->where_close();
							break;
						case 'pending':
							$query->where('Approval','=','P');
							break;
						case 'unpublished':
							$query->where_open();
							$query->where('Approval','<>','A');
							$query->where('Approval','<>','P');
							$query->where_close();
							break;
						case 'drafts':
							$query->where('Approval','=','D');
							break;
						case 'requested':
							$query->where_open();
							$query->where('Approval','=','R');
							$query->or_where('Approval','=','P');
							$query->where_close();
							break;
						case 'declined':
							$query->where('Approval','=','X');
							break;
						case 'hidden':
							$query->where('Hide','IS',DB::expr('TRUE'));
							break;
						case 'trashed':
							$query->where('Trash','IS',DB::expr('TRUE'));
							break;
						case 'opennotes':
							$query->where('AuthorNotesClosed','IS',DB::expr('FALSE'));
							break;
						case 'openfollowup':
							$query->where('FollowupNotesClosed','IS',DB::expr('FALSE'));
							break;
						case 'unassigned':
							$query->where_open();
							$query->where('ID','NOT IN',
								DB::expr('(SELECT `NodeID` from `Nodes_Instances` WHERE `NodeID` = `' . 
									$query->table()->tablename() . '`.`ID`)'));
							$query->where('isTreeTop','IS',DB::expr('FALSE'));
							$query->where_close();
							break;
					}
					$query->where_close();
				}
			}
			if (property_exists($parameters,'selectauthorid') and is_numeric($parameters->selectauthorid)) {
				$query->where_open()->where('AuthorID','=',$parameters->selectauthorid)->where_close();
			}
			if (property_exists($context,'objecttype')) { // ??
				$this->_on_set_query($query);
			}
			if (property_exists($context,'foreignkeyvalue') and is_numeric($context->foreignkeyvalue)) {
				$query->where_open()->where($context->foreignkeyname,'=',$context->foreignkeyvalue)->where_close();
			}
//			 var_dump($query->sql());
		}
		return $tablemodel;
	}
	protected function _set_orderby($query,$parameters)
	{
		$context = $this->_context;
		if ($parameters->orderby == 'Order') {
			$query->order_by(DB::expr('ISNULL(`Order`)'),$parameters->direction);
			$query->order_by($parameters->orderby,$parameters->direction);
			$query->order_by($context->secondary_orderby,$context->secondary_direction);
			if ($context->secondary_orderby == 'ArticleDate') {
				$query->order_by('Title','ASC');
			}
			$query->order_by('ID','ASC');
		} else {
			$query->order_by($parameters->orderby,$parameters->direction);
			if ($parameters->orderby == 'ArticleDate')
				$query->order_by('Title','ASC');
		}
	}
	protected function _on_set_query($query) {}
	
	protected function _get_boundaries($tablemodel,$parameters)
	{
		$mastertable = $tablemodel->mastertable;
		// get potential record count, for pagination and to set boundaries for limit and offset
		$recordcount = $mastertable->unlimited_count();
		$perpage = $parameters->perpage;
		
		// set $maxpage, normalize $page
		$maxpage = ceil($recordcount / $perpage);
		$page = $parameters->currentpage;
		if ($page == 'max') $page = $maxpage; // from command
		if ($page > $maxpage) $page = $maxpage;
		if ($page <= 0) $page = 1;
		if ($recordcount == 0) $page = 0;
		$parameters->currentpage = $page;
		// create and normalize $offset
		$offset = (($page * $perpage) - ($perpage));
		if ($offset < 0) $offset = 0;
		
		// set up $startrecord and $endrecord
		if ($recordcount == 0) $startrecord = 0;
		else $startrecord = $offset + 1;
		$endrecord = ($perpage * $page);
		if ($endrecord > $recordcount) $endrecord = $recordcount;
		
		$boundaries = new StdClass;
		$boundaries->recordcount = $recordcount;
		$boundaries->maxpage = $maxpage;
		$boundaries->offset = $offset;
		$boundaries->startrecord = $startrecord;
		$boundaries->endrecord = $endrecord;
		
		return $boundaries;
	}
	protected function _finalize_query($parameters,$tablemodel,$boundaries)
	{
		// sort by relevance
		$query = $tablemodel->query;
		if (!empty($parameters->search)) {
			$search = $tablemodel->model->db()->escape($parameters->search);
			$mode = $tablemodel->mode;
			$query->select(array(DB::expr('(MATCH(`Title`,`Caption`) AGAINST(' . $search . ' ' . $mode . '))'),'titlerelevance'));
			$query->select(array(DB::expr('(MATCH(`Title`,`Caption`,`Description`,`Location`) AGAINST(' . $search . ' ' . $mode . '))'),'descriptionrelevance'));
			$query->select(array(DB::expr('(MATCH(`Title`,`Caption`,`Description`,`Location`,`Body`) AGAINST(' . $search . ' ' . $mode . '))'),'bodyrelevance'));
			$query->order_by('titlerelevance','desc');
			$query->order_by('descriptionrelevance','desc');
			$query->order_by('bodyrelevance','desc');
			$query->order_by('ArticleDate','desc');
			$query->order_by('Title','asc');
		}
		// assert offset and limit
		$query->offset($boundaries->offset);
		$query->limit($parameters->perpage);
//		var_dump($query->sql());
	}
	protected function _set_gateway_resources($parameters,$tablemodel,$context)
	{
		$datamodel = $this->_datamodel;
		$mastertable = $tablemodel->mastertable;
		$masterrecord = $tablemodel->masterrecord = $mastertable->activerecord();
		$record = $masterrecord->current();
		if (!$record) {
			$db = $tablemodel->model->db();
			if ($err = $db->statement_error()) {
				if ((int)$err[0] != 0) {
					echo 'system error nodelist.php set_gateway_resources <br />';
					die(var_export($db->statement_error()));
				}
			}
		}
		$datapool = $datamodel->datapool;
		if (property_exists($context,'objecttype') and (!empty($context->objecttype))) {
			$datapool->constraints['objecttype'] = $context->objecttype;
		}
		if ($this->_permissions->canadminister and ($parameters->orderby == 'EditDate')) {
			$datapool->constraints['updatedata'] = TRUE;
		}
		if (!empty($context->list_layout))
			$datapool->variables['layouttype'] = $context->list_layout;
		# primary components
		$datamodel->dataframe($context->dataframe);
		$datapool->tables[$context->mastertableindex] = $mastertable;
		$datapool->activerecords[$context->masterrecordindex] = $masterrecord;
		
		if (property_exists($context,'datapoolarrays')) {
			foreach ($context->datapoolarrays as $index=>$value) {
				$datapool->arrays[$index] = $value;
			}
		}
		$this->_on_set_datapool($datapool);
//		var_dump($mastertable->query()->sql());
	}
	protected function _on_set_datapool($datapool) {} // stub
	
	protected function _process_operation_command($parameters,$context)
	{
		if ($parameters->command) {
			
			$result = $this->_respond($parameters->command); // returns Node::FLOW_CONTINUE or Node::FLOW_REDIRECT
			if ($result == Node::FLOW_REDIRECT) {
				$parms = $this->_get_redirect_parameters();
				$redirect = $context->redirect . $parms;
				$this->_controller->request->redirect($redirect);
			}
		} 
		$session = session::instance();
		$messages = $session->get_once('listmessages');
		if ($messages) {
			$index = $messages['index'];
			unset($messages['index']);
			$messages = (object) $messages;
			$this->_datamodel->datapool->variables[$index] = $messages;
		}
	}
	protected function _get_redirect_parameters()
	{
		$parameters = $this->_parameters;
		$parms = array(
			'mode' => $parameters->mode,
			'instanceid' => $parameters->instanceid,
			'path' => $parameters->path,
			'listview' => $parameters->listview,
			'search' => $parameters->search,
			'm' => $parameters->searchmethod,
			'select' => $parameters->select,
			'page' => $parameters->currentpage,
			'orderby' => $parameters->orderby,
			'perpage' => $parameters->perpage,
			'd' => $parameters->direction,
			'fsearch' => $parameters->fsearch,
			'fsfield' => $parameters->fsfield,
			'fswild' => $parameters->fswild,
			'authorid' => $parameters->authorid,
		);
		$p = array();
		foreach ($parms as $key => $value) {
			if ($value)
				$p[] = $key . '=' . $value;
		}
		if (empty($p)) return '';
		else return '?' . implode('&',$p);
	}
	protected function _respond($command)
	{
		$result = Node::FLOW_CONTINUE;
		switch ($command) {
			case 'actionadd':
				$result = $this->_process_action('add');
				break;
			case 'actionremove':
				$result = $this->_process_action('remove');
				break;
			case 'actionmove':
				$result = $this->_process_action('move');
				break;
			case 'actionpublish':
				$result = $this->_process_action_publish();
				break;
			default:
				$result = $this->_on_respond($command);
		}
		return $result;
	}
	protected function _on_respond($command)
	{
		return Node::FLOW_CONTINUE;
	}
	protected function _process_action_publish()
	{
		$result = Node::FLOW_CONTINUE;
		if (!($this->_permissions->canadminister))
			return $result;
		$parameters = $this->_parameters;
		$actionselections = $parameters->actionselections;
		$is_error = FALSE;
		$messages = $this->_messages;
		$error_messages = &$messages->errors;
		$success_messages = &$messages->messages;
		$warning_messages = &$messages->warnings;
		unset($messages);
		if (empty($actionselections)) {
			$is_error = TRUE;
			$warning_messages[] = array('warning' => 'no records were selected');
		}
		// check permissions
		if (!$is_error) {
			if (!$this->_permissions->canadminister) {
				$is_error = TRUE;
				$error_messages[] = array('error' => 'insufficient permissions for action');
			}
		}
		if (!$is_error) {
			$actionselections = array_keys($actionselections);
			$tablemodel = $this->_tablemodel;
			$db = $tablemodel->model->db();
			$db->lock_tables(array('Articles'));
			try {
				$operation = DB::update('Articles')
					->set(array('Approval'=>'A','ApproveUserID'=>$this->_user->userid(),'ApproveDate'=> date('Y-m-d')))
					->where('ID','IN',$actionselections)
					->where('ObjectType','NOT IN',array('inbox','directory'))
					->where('Approval','!=','A')
					->prepare($db)
					->execute();
			} catch(Exception $e) {
				$is_error = TRUE;
				 $error_messages[] = array('error' => $e->getMessage());
			}
			$db->unlock_tables();
			if (!$is_error) {
				if (!$operation->success()) {
					$error_info = $operation->error_info(); 
					if (!empty($error_info[2])) {
						$error_messages[] = array('error' => $error_info[2]);
					} else {
						$error_messages[] = array('error' => 'unable to process update query, please call tech support.');
					}
				} else {
					$success_messages[] = array('message' => $operation->row_count() . ' records have been published.');
				}
			}
		}
		if (!empty($error_messages) or !empty($success_messages) or !empty($warning_messages)) {
			$session = Session::instance();
			$messages = array(
				'index'=>'messagesection',
			);
			if (!empty($error_messages)) {
				$messages['errors'] = $error_messages;
			}
			if (!empty($success_messages)) {
				$messages['messages'] = $success_messages;
			}
			if (!empty($warning_messages)) {
				$messages['warnings'] = $warning_messages;
			}
			$session->set('listmessages',$messages);
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _process_action($actionname)
	{
		$result = Node::FLOW_CONTINUE;
		if (!($this->_permissions->canadminister))
			return $result;
		$parameters = $this->_parameters;
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$db = $tablemodel->model->db();
		$actionselections = $parameters->actionselections;
		$messages = $this->_messages;
		$error_messages = &$messages->errors;
		$success_messages = &$messages->messages;
		$warning_messages = &$messages->warnings;
		unset($messages);
		$is_error = FALSE;
		// check permissions
		if (($actionname == 'move') or ($actionname == 'add')) {
			switch ($actionname) {
				case 'add':
					$targetid = $parameters->actionaddtargetid;
					break;
				case 'move':
					$targetid = $parameters->actionmovetargetid;
					break;
			}
			if (!is_numeric($targetid)) {
				$is_error = TRUE;
				$error_messages[] = array('error' => 'invalid master record number');
			}
			if (!$is_error) {
				$editable = Services::check_article_permission($targetid);
				if (!$editable) {
					$is_error = TRUE;
					$error_messages[] = array('error' => 'insufficient permissions for action');
				}
			}
		}
		if ((!$is_error) and is_null($actionselections)) {
			$is_error = TRUE;
			$error_messages[] = array('error' => 'no items were selected');
		}
		if (!$is_error) {
			switch ($actionname) {
				case 'add':
					$targetid = $parameters->actionaddtargetid;
					if (empty($targetid) and !is_numeric($targetid)) {
						$is_error = TRUE;
						$error_messages[] = array('error' => 'no container target id for add');
					}
					break;
				case 'remove':
					if (property_exists($context,'foreignkeyvalue')) {
						$containerid = $context->foreignkeyvalue; // assume this is FolderID for Node_Instances
					} else {
						$is_error = TRUE;
						$error_messages[] = array('error' => 'no containerid id for remove');
					}
					break;
				case 'move':
					$targetid = $parameters->actionmovetargetid;
					if (empty($targetid) and !is_numeric($targetid)) {
						$is_error = TRUE;
						$error_messages[] = array('error' => 'no target container id for move');
					}
					if (!$is_error) {
						if (property_exists($context,'foreignkeyvalue')) {
							$containerid = $context->foreignkeyvalue; // assume this is FolderID for Node_Instances
						} else {
							$is_error = TRUE;
							$error_messages[] = array('error' => 'no containerid id for move');
						}
					}
					break;
				default:
					die('bad action command');
			}
		}
		if (!$is_error) { 
			$allowedcontainers = Kohana::$config->load('allowedcontainers')->getArrayCopy();
			$objecttypenames = Kohana::$config->load('objecttypenames')->getArrayCopy();
			$db->lock_tables(array('Nodes_Instances_View','Articles_Authors','Access_Tokens','Access_Role_Tokens'));
			// check validity of targetid and containerid
			if (($actionname == 'add') or ($actionname == 'move')) { // check targetid, get targetrecord
				$operation = DB::select('ID','ObjectType','Title','AuthorID')
					->from('Articles')
					->where('ID','=',$targetid)
					->as_object()
					->prepare($db)
					->execute();
				if ($operation->success()) {
					$targetrecord = $operation->current();
					if (!$targetrecord) {
						$is_error = TRUE;
						$error_messages[] = array('error'=>'target container record is not available');
					}
				} else {
					$is_error = TRUE;
					$error_info = $operation->error_info();
					$error_messages[] = array('error'=>'error getting target container record: ' . $error_info[2]);
				}
				if (!$is_error) {
					$objecttype = $targetrecord->ObjectType;
					if (!in_array($objecttype,array('folder','file','inbox'))) {
						$is_error = TRUE;
						$error_messages[] = array('error'=>'target container is wrong type: must be Topic, Library Folder, or Inbox');
					}
				}
				// check edit permissions for target container
				if (!$is_error) {
					switch ($objecttype) { // guaranteed to be one of the three from above test
						case 'folder':
							$tokensetname = 'topic';
							break;
						case 'file':
							$tokensetname = 'folder';
							break;
						case 'inbox':
							$tokensetname = 'inbox';
							break;
					}
					$tokenset = Services::get_token_set($tokensetname);
					$access = Access::instance();
					$editable = $access->permission($tokenset->content,'E',$targetrecord->AuthorID,$targetrecord->ID);
					if (!$editable) {
						$is_error = TRUE;
						$error_messages[] = array('error'=>'you do not have permissions required to edit target container');
					} else {
						$frozen = Services::dblookup('Articles','Frozen','ID',$targetrecord->ID);
						if ($frozen) {
							$is_error = TRUE;
							$error_messages[] = array('error'=>'target container is frozen, and cannot be changed');
						}
					}
				}
				if (!$is_error) {
					$success_messages[] = array('message'=>'Target container is ' . $objecttypenames[$objecttype] . ' [' . $targetrecord->Title . '] (' . $targetid . ')');
				}			
			}
			if ((!$is_error) and (($actionname == 'remove') or ($actionname == 'move'))) { // check containerid, get containerrecord
				$operation = DB::select('ID','ObjectType','Title','Frozen')
					->from('Articles')
					->where('ID','=',$containerid)
					->as_object()
					->prepare($db)
					->execute();
				if ($operation->success()) {
					$containerrecord = $operation->current();
					if (!$containerrecord) {
						$is_error = TRUE;
						$error_messages[] = array('error'=>'this container record is not available');
					}
				} else {
					$is_error = TRUE;
					$error_info = $operation->error_info();
					$error_messages[] = array('error'=>'error getting this container record: ' . $error_info[2]);
				}
				if (!$is_error) {
					$objecttype = $containerrecord->ObjectType;
					if (!in_array($objecttype,array('folder','file','inbox'))) {
						$is_error = TRUE;
						$error_messages[] = array('error'=>'container is wrong type: must be Topic, Library File, or Inbox');
					}
				}
				if (!$is_error) {
					$frozen = $containerrecord->Frozen;
					if ($frozen) {
						$is_error = TRUE;
						$error_messages[] = array('error'=>'container is frozen and cannot be changed');
					}
				}
			}
			// process items
			if (!$is_error) {
				try {
					$actionselections = array_keys($actionselections);
					$error_items = array(); // to inform remove not to process those
					foreach ($actionselections as $itemid) {
						if (($actionname == 'add') or ($actionname == 'move')) {
							$this->_addactionitem(
								$targetid,$itemid,
								$error_messages,$warning_messages,$success_messages,
								$error_items,$db,$objecttypenames,$allowedcontainers,$targetrecord
							);
						}
						if (($actionname == 'remove') or ($actionname == 'move')) {
							if (!in_array($itemid,$error_items)) {
								$this->_removeactionitem(
									$containerid,$itemid,
									$error_messages,$warning_messages,$success_messages,
									$db,$objecttypenames,$containerrecord
								);
							} else {
								$title = Services::dblookup('Articles','Title','ID',$itemid);
								// add warning
								$warning_messages[] = array('warning'=>'[' . $title . '] (' . $itemid . ') was not removed from this container');
							}
						}
					}
				} catch (Exception $e) {
					$error_messages[] = array('error' => $e->getMessage());
				}
			}
			$db->unlock_tables();
		}
		if (!empty($error_messages) or !empty($warning_messages) or !empty($success_messages)) {
			$session = session::instance();
			$messages = array();
			$messages = array(
				'index'=>'messagesection',
			);
			if (!empty($error_messages)) {
				$messages['errors'] = $error_messages;
			}
			if (!empty($warning_messages)) {
				$messages['warnings'] = $warning_messages;
			}
			if (!empty($success_messages)) {
				$messages['messages'] = $success_messages;
			}
			$session->set('listmessages',$messages);
		}
		$result = Node::FLOW_REDIRECT;
		return $result;
	}
	protected function _addactionitem(
		$targetid,$itemid,
		&$error_messages,&$warning_messages,&$success_messages,
		&$error_items,$db,
		$objecttypenames,$allowedcontainers,
		$targetrecord)
	{
		// processing
		# get item record
		$is_error = FALSE;
		$operation = DB::select('ID','ObjectType','Title')
			->from('Articles')
			->where('ID','=',$itemid)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$itemrecord = $operation->current();
			if (!$itemrecord) {
				$is_error = TRUE;
				$error_messages[] = array('error'=>'selected record (' . $itemid . ') was not available');
				$error_items[] = $itemid;
				return;
			}
		} else {
			$is_error = TRUE;
			$error_info = $operation->error_info();
			$error_messages[] = array('error'=>'error getting item record (' . $itemid . '): ' . $error_info[2]);
			$error_items[] = $itemid;
			return;
		}
		$objecttype = $itemrecord->ObjectType;
		$targetobjecttype = $targetrecord->ObjectType;
		$allowedcontainers = $allowedcontainers[$objecttype];
		if (!in_array($targetobjecttype,$allowedcontainers)) {
			$is_error = TRUE;
			$error_messages[] = 
				array('error'=>'target container cannot accept item type ' . 
					$objecttypenames[$objecttype] . 
					' [' . $itemrecord->Title . '] (' . $itemid . ')');
			$error_items[] = $itemid;
			return;
		}
		# process record
		$operation = DB::insert('Nodes_Instances',array('FolderID','NodeID'))
			->values(array($targetid,$itemid))
			->prepare()
			->execute();
		$result = $operation->success();
		if (!$result) {
			$error_info = $operation->error_info();
			if ($error_info[0] == '23000') {
				$warning_messages[] = 
					array('warning'=>'item ' . 
						$objecttypenames[$objecttype] . 
						' [' . $itemrecord->Title . '] (' . $itemid . ') was already in the target container');
			} else {
				$error_items[] = $itemid;
				$error_messages[] = 
					array('error'=>'error adding item ' . 
						$objecttypenames[$objecttype] . 
						' [' . $itemrecord->Title . '] (' . $itemid . ') to the container');
			}
			return;
		}
		$success_messages[] = 
			array('message'=>'item ' . 
				$objecttypenames[$objecttype] . 
				' [' . $itemrecord->Title . '] (' . $itemid . ') has been added to the target');
	}
	protected function _removeactionitem(
		$containerid,$itemid,
		&$error_messages,&$warning_messages,&$success_messages,
		$db,$objecttypenames,$containerrecord)
	{
		# get item record
		$is_error = FALSE;
		$operation = DB::select('ID','ObjectType','Title')
			->from('Articles')
			->where('ID','=',$itemid)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$itemrecord = $operation->current();
			if (!$itemrecord) {
				$is_error = TRUE;
				$error_messages[] = array('error'=>'selected record (' . $itemid . ') was not available for removal');
				$error_items[] = $itemid;
				return;
			}
		} else {
			$is_error = TRUE;
			$error_info = $operation->error_info();
			$error_messages[] = array('error'=>'error getting item record (' . $itemid . '): ' . $error_info[2]);
			$error_items[] = $itemid;
			return;
		}
		$objecttype = $itemrecord->ObjectType;
		
		$operation = db::delete('Nodes_Instances')
			->where('FolderID','=',$containerid)
			->where('NodeID','=',$itemid)
			->prepare($db)
			->execute();
		$result = $operation->success();
		if (!$result) {
			$error_info = $operation->error_info();
			$error_messages[] = array('error'=>'error removing item record (' . $itemid . '): ' . $error_info[2]);
			return;
		}
		$success_messages[] = 
			array('message'=>'item ' . 
				$objecttypenames[$objecttype] . 
				' [' . $itemrecord->Title . '] (' . $itemid . ') was removed from this container');
	}
	protected function _get_dictionary($permissions,$parameters,$boundaries,$context)
	{
		# combine main dictionary with main view
		$this->_datamodel->retrieve();
		$dictionary = $this->_datamodel->output();
		
		if (!empty($context->list_layout))
			$dictionary['list_layout'] = $context->list_layout;
		$dictionary['can_edit'] = $permissions->canedit;
		$dictionary['can_favourites'] = $permissions->favourites;
		$dictionary['can_administer'] = $permissions->canadminister;

		$dictionary['updatedata'] = ($permissions->canadminister and ($parameters->orderby == 'EditDate'));
		$dictionary['selectvalue'] = $parameters->select;
		$dictionary['selecttext'] = $parameters->selecttext;
		$dictionary['listordertext'] = $parameters->listordertext;
		$dictionary['authoridvalue'] = $parameters->selectauthorid;

		# common variables
		$dictionary['common'] = $this->_controller->_common;
		$dictionary['base'] = $this->_controller->_base;
		$dictionary['publicbase'] = System::instance()->PublicBase;
		$dictionary['uploadbase'] = System::instance()->UploadBase;

		# set up pagination
		$dictionary['is_customorder'] = !empty($parameters->customorder);
		$dictionary['recordcount'] = $boundaries->recordcount;
		$dictionary['currentstartrecord'] = $boundaries->startrecord;
		$dictionary['currentendrecord'] = $boundaries->endrecord;
		$maxpage = $dictionary['pagecount'] = $boundaries->maxpage;
		$page = $dictionary['currentpage'] = $parameters->currentpage;
		$dictionary['atfirstdisabled'] = ($page <= 1);
		$dictionary['ismultipage'] = ($maxpage > 1);
		$dictionary['atlastdisabled'] = ($page == $maxpage);

		# set up state for recycling
		$dictionary['searchvalue'] = $parameters->search;
		$dictionary['fsearchvalue'] = $parameters->fsearch;
		$dictionary['fsfield'] = $parameters->fsfield;
		$dictionary['searchmethod'] = $parameters->searchmethod;
		$dictionary['orderby'] = $parameters->orderby;
		$dictionary['direction'] = $parameters->direction;
		
		$perpage = $dictionary['perpage'] = $parameters->perpage;
		$perpageoptionsdata = 
			array("10"=>"10","15"=>"15","20"=>"20","30"=>"30","40"=>"40","60"=>"60","80"=>"80","100"=>"100","200"=>"200",);
		$perpageoptions = array();
		foreach ($perpageoptionsdata as $key=>$value) {
			$perpageoption = array('value' => $key,'caption' => $value);
			if ($perpage == $value) $perpageoption['selected'] = TRUE;
			$perpageoptions[] = (object) $perpageoption;
		}
		$dictionary['perpageoptions'] = $perpageoptions;
		$this->_on_get_dictionary($dictionary);
		return $dictionary;
	}
	protected function _on_get_dictionary(&$dictionary) {}
	
	// for complete lists, not content lists (handled by master record);
	protected function _get_admindictionary($permissions,$parameters,$dictionary)
	{
		# combine dictionary for sidebar with side view
		$admindictionary = array();
//		$admindictionary['can_create'] = $permissions->cancreate; // ** deprecated
		$admindictionary['can_add'] = $permissions->cancreate;
		$access = Access::instance();
		$admindictionary['can_administer'] = $permissions->canadminister;
		$admindictionary['can_edit'] = $permissions->canedit;
		$admindictionary['is_articlesenabled'] = $permissions->createarticlesenabled;
		$admindictionary['is_blogsenabled'] = $permissions->createblogsenabled;
		$admindictionary['is_commentsenabled'] = $permissions->createcommentsenabled; // shouldn't be needed
		$subsetoption = $permissions->subsetoption;
		if ($subsetoption) {
			$admindictionary['is_subsetoption'] = $permissions->subsetoption;
			$admindictionary['is_unpublishedenabled'] = $permissions->unpublishedenabled;
			$admindictionary['is_hiddenenabled'] = $permissions->hiddenenabled;
			$admindictionary['is_trashenabled'] = $permissions->trashenabled;
		}
		$searchdictindexes = 
			array('searchvalue','searchmethod','currentpage','perpage','orderby','direction','base','common',);
		foreach ($searchdictindexes as $searchdictindex) {
			$admindictionary[$searchdictindex] = $dictionary[$searchdictindex];
		}
		$admindictionary['is_directories_available'] = $permissions->directoriesavailable;
		$admindictionary['is_inboxes_available'] = $permissions->inboxesavailable;
		$this->_on_get_admindictionary($admindictionary);
		return $admindictionary;
	}
	protected function _on_get_admindictionary(&$admindictionary) {}
}