<?php 
// ** review logic $authorid == $userid whereever it occurs\
// ** set collaboratorid as parameter
// editsection narrative for input
namespace Muster\Node;

use StdClass;
use Kohana, Text, DB, Session, I18n, URL;
use DataModel, System, Access, User, TableModel, Point, Services, Template, Notification;
use 
	Muster\Node;

defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
/* 
** add checks against recursion for cross-linking
** update deleteforever for pages
** change parameters->mode to states->mode
** set constraints before command and parameters are collected
*/

/* 
	events:
	_on_set_states($states)
	_on_get_parameters($parameters)
	_on_set_permissions($permissions)
	_on_get_tokens($tokens)
	_on_set_datapool($datapool)
	_on_get_dictionary(&$dictionary)
	_on_get_admindictionary(&$admindictionary)
	_on_get_bookmark(&$bookmark)
	_on_get_redirect_parameters(&$parms)
	_on_respond($command)
	_on_set_update_locktables(&$locktables)
	_on_set_insert_locktables(&$locktables)
	_on_set_delete_locktables(&$locktables)
	// all the following after tables are locked, and before they are unlocked
	// parmdata should be extract-ed by caller
	_on_before_update($parmdata,&$error_info)
	_on_after_update($parmdata,&$error_info)
	_on_after_update_post($parmdata)
	_on_before_insert($parmdata,&$error_info)
	_on_insert_extension_error($parmdata,&$error_info)
	_on_after_insert($parmdata,&$error_info)
	_on_after_insert_post($parmdata) // after unlocktables()
	_on_before_delete($parmdata,&$error_info)
	_on_after_delete($parmdata,&$error_info)
	
	$this->_states
		// from record:
		->hidden = FALSE;
		->trash = FALSE;
		->published = FALSE;
		->frozen = FALSE;
		// from context
		->adminmode = FALSE;
		->premium = (bool) System::instance()->ArticlePremiumPrecedence;
		->provisionalpublish = (bool) System::instance()->AllowProvisionalPublish;
		->insertmode = FALSE;
		->locked = TRUE;
		->commentsclosed 
		->member (is logged in/ recognized as member)
		->editmode (true if editing)
		->previewmode

	$this->_parameters
		// (parameters from $datamodel->get_parameters())
		->command // from datamodel
		// (parameters from command object - like mode = 'admin' for add record)
		->articleid // from context
		->authorid // from record
		->bindersoptions
		
	$this->_tokens
		->names
			->premium
			->content
			->hidden
			->trash
			->unpublished (used for general case)
			->settings
		->content (resolved by $access->get_token(...))
		->settings (resolved like base)
	
	// *** add canprivatize and canfreeze and canlock and cancomment and canclosecomments and canadd
	$this->_permissions
		->canread
		->canreadhidden
		->canreadunpulished
		->canreadtrash
		->userallowarticles // *** canadd
		->canadminister // can edit record and associated lists
		->canapprove
		->canprivatize
		->canfreeze
		->canlock
		->cancomment
		->canclosecomments
		
	$this->_tablemodel
		->model
		->table
		->query
		->activerecord
		->datarecord
		->masterrecord
		->priorrecord

*/
abstract class RecordNode extends Node {
	/* global abstract data structures ($this->):
	
		# controls
		(_user) // from the environment
		_datamodel
		_context
		_states
		_parameters
		_permissions
		_tablemodel
		_tokens
		
	*/
	public function __construct($controller)
	{
		# get engine
		$this->_datamodel = new DataModel($controller);
		parent::__construct($controller);
	}
	// returns associative array $data, indexes 'dictionary', 'admindictionary'
	public function generate($context = NULL)
	{
		## collect control data
		# save context
		if ($context) $this->_context = $context;
		else $context = $this->_context;
		
		# set constraints for parameter fetch
		$this->_set_constraints($context);

		# initialize states
		$states = $this->_states = $this->_get_global_states(); 
		
		# establish parameters
		$parameters = $this->_parameters = $this->_get_parameters($states,$context);
		
		# respond to change mode or navigate record commands with redirect
		if ($parameters->command) { // change mode = plainview vs. administer
			$this->_process_redirect_command($parameters,$context);
		}
		# general access permissions
		$tokens = $this->_tokens = $this->_get_tokens($states,$context);
		$permissions = $this->_permissions = $this->_get_global_permissions($tokens);

		## collect data model...
		# tablemodel, based on general access
		$tablemodel = $this->_tablemodel = $this->_get_tablemodel($parameters,$states,$permissions,$context);

		## ... then make control adjustments based on current datarecord:
		# - assert record permissions, based on author or user tokens
		$this->_set_record_permissions($parameters,$permissions);

		# - set record states
		$this->_set_record_states($parameters,$permissions,$states,$tablemodel);

		## collect processing resources, and process request...
		# set gateway resources
		$this->_set_gateway_resources($parameters,$tablemodel,$context,$states);
		
		# respond to record change commands
		$this->_process_operation_command($parameters,$permissions,$states,$context);
		
		# set dictionaries
		$dictionary = $this->_get_dictionary($parameters,$permissions,$states,$tablemodel);
		$this->_dictionary = &$dictionary; // required for bookmark
		# get bookmarkdata
		isset($context->get_bookmark) or ($context->get_bookmark = TRUE);
		if ($context->get_bookmark) {
			$bookmark = $this->_get_bookmark();
		} else {
			$bookmark = NULL;
		}
		
		isset($context->get_admindictionary) or ($context->get_admindictionary = TRUE);
		if ($context->get_admindictionary) {
			$admindictionary = $this->_get_admindictionary($parameters,$permissions,$states, $dictionary,$tablemodel);
		} else {
			$admindictionary = NULL;
		}

		# return data object
		$data = compact('dictionary','admindictionary','bookmark');
		return $data;
	}
	
	protected function _set_constraints($context)
	{
		if (property_exists($context,'constraints') and !empty($context->constraints)) {
			$constraints = $context->constraints;
			$datapool = $this->_datamodel->datapool;
			foreach ($constraints as $constraint) {
				$constraint = (object) $constraint;
				$datapool->constraints[$constraint->index] = $constraint->value;
			}
		}
	}
	// initialize states; refined after record is fetched
	protected function _get_global_states()
	{
		$states = new StdClass; 
		$states->hidden = FALSE;
		$states->trash = FALSE;
		$states->published = FALSE;
		
		$states->adminmode = FALSE;
		$states->newcommentmode = FALSE;
		$states->premium = (bool) System::instance()->ArticlePremiumPrecedence;
		$states->provisionalpublish = (bool) System::instance()->AllowProvisionalPublish;
		$states->insertmode = FALSE;
		
		$states->frozen = FALSE;
		$states->locked = FALSE;
		$states->editmode = FALSE;
		$states->previewmode = FALSE;
		
		$states->commentsclosed = FALSE;
		$states->member = $this->_user->authenticated();
		
		return $states;
	}
	protected function _get_parameters($states,$context)
	{
		# initialize parameters
		$datamodel = $this->_datamodel;
		$datamodel
			->controlframe($context->controlframe)
			->control_input();
		$parameters = $this->_parameters = 
			(object) $datamodel->get_parameters(); // mode
		empty($parameters->perpage) and ($parameters->perpage=NULL);
		// initialize for subforms...
		empty($parameters->instanceid) and ($parameters->instanceid = '');
		empty($parameters->path) and ($parameters->path = '');
		$control = $datamodel->get_command_control();
		if ($control) {
			if (property_exists($control,'parameters')) {
				$parms = $control->parameters; // pass parameters->mode = 'admin' for new record command
				foreach ($parms as $key => $value) {
					$parameters->$key = $value;
				}
			}
			$parameters->command = $control->value;
		} else {
			$parameters->command = !empty($parameters->action)?$parameters->action:NULL;
		}
		$parameters->articleid = $context->articleid;
		$parameters->pagepath = $parameters->path;
		$this->_on_get_parameters($parameters);
		return $parameters;
	}
	protected function _on_get_parameters($parameters) {}
	
	protected function _process_redirect_command($parameters,$context)
	{
		$command = $parameters->command;
		// change mode
		switch ($command) {
			case 'administer':
				$parameters->mode = 'admin';
				break;
			case 'plainview':
				$parameters->mode = '';
				break;
			default:
				return;
		}
		$articleid = $parameters->articleid;
		$parms = $this->_get_redirect_parameters();
		$this->_controller->request->redirect($context->redirect . "/$articleid$parms");
	}
	protected function _get_tokens($states,$context)
	{
		$tokens = new StdClass;
		$this->_on_before_get_tokens($tokens);
		$tokens->names = $context->tokens;
		return $tokens;
	}
	protected function _on_before_get_tokens($tokens) {}
	// general access permissions
	protected function _get_global_permissions($tokens)
	{
		$access = Access::instance();
		$permissions = new StdClass;
		#collect role access permissions
		$permissions->canread = $access->permission($tokens->names->content,'R');
		$permissions->canreadhidden = $access->permission($tokens->names->hidden,'R');
		$permissions->canreadunpublished = $access->permission($tokens->names->unpublished,'R');
		$permissions->canreadtrash = $access->permission($tokens->names->trash,'R');
		if ($this->_user->authenticated()) {
			$permissions->cancomment = (bool) $this->_user->user_data()->Permissions->comment;
		} else {
			$permissions->cancomment = false;
		}
		$permissions->favourites = $this->_user->authenticated();
		
		return $permissions;
	}
	// ... based on general access permissions
	protected function _get_tablemodel($parameters,$states,$permissions,$context)
	{
		$tablemodel = $this->_tablemodel = new StdClass;
		
		$model = $tablemodel->model = new TableModel($context->tablemodel);
		$table = $tablemodel->table = $model->table();
		$query = $tablemodel->query = $table->query();
		$activerecord = $tablemodel->activerecord = $table->activerecord();
		
		$this->_set_query($parameters,$states,$permissions);
		
		$datarecord = $tablemodel->datarecord = $activerecord->current(); // current record object, or NULL
		if (!empty($datarecord))
			$tablemodel->priorrecord = clone $datarecord;
		else
			$tablemodel->priorrecord = NULL;
		if (!$datarecord) {
			$db = $model->db();
			if ($err = $db->statement_error()) {
				if ((int)$err[0] != 0) {
					echo 'system error noderecord.php get_tablemodel <br />';
					die(var_export($db->statement_error()));
				}
			}
		} else {
			$this->_redirect_mismatched_objecttype($datarecord);
		}
		$userid = $this->_user->userid();
		if ($datarecord) {
			$parameters->authorid = $datarecord->AuthorID;
		} else { // can only be new record (or no record, which doesn't matter)
			$parameters->authorid = $userid; // may be a new record
		}
		# collect scope options
		if ($parameters->mode == 'admin') {
			$documentsbinder = DB::select('ID','LeftEdge','RightEdge','RootID')
				->from('Binders_View')
				->where('Code','=',$context->binderscope)
				->as_object()
				->prepare($tablemodel->model->db())
				->execute()->current();
			$parameters->bindersoptions = NULL;
			if ($documentsbinder) { // ** only required if inserting
				$bindersoptions = DB::select('ID','Title')
					->from('Binders_View')
					->where('LeftEdge','BETWEEN',DB::expr($documentsbinder->LeftEdge.' AND '.$documentsbinder->RightEdge))
					->where('IsBox','IS',DB::expr('FALSE'))
					->where('RootID','=',$documentsbinder->RootID)
					->where('Closed','IS',DB::expr('FALSE'))
					->execute()->as_array('ID','Title');
				$parameters->bindersoptions = $bindersoptions;
			}
		} else {
			$parameters->bindersoptions = NULL;
		}
//		var_dump($query->sql()); exit;
		return $tablemodel;
	}
	protected function _set_query($parameters,$states,$permissions)
	{
		$articleid = $parameters->articleid;
		$query = $this->_tablemodel->query;
		$user = $this->_user;
		if (empty($articleid) and !is_numeric($articleid)) {
			$query->where('ID','IS',DB::expr('NULL')); // always blank
		} else {
			$query->where_open();
			$query->where('ID','=',$articleid); // the requested article
			$query->where_close();
			# apply permissions
			$this->_set_query_constraints($query,$this->_tablemodel->table->tablename());
		}
//			 var_dump($query->sql());
	}
	protected function _redirect_mismatched_objecttype($datarecord)
	{
		$context = $this->_context;
		$node = $this->_controller->request->param('name');
		$is_node = (!empty($node) and $node == 'node');
		// redirect if this is the wrong type of record
		if ((!empty($datarecord->ObjectType)) and (!empty($context->objecttype)) and 
			(($context->objecttype != $datarecord->ObjectType) or $is_node)) {
			$objecttype = $datarecord->ObjectType;
			$id = $datarecord->ID;
			switch ($objecttype) {
				case 'document':
					$redir = 'documents/'.$id;
					break;
				case 'entry':
					$blogid = Services::dblookup('Articles_Extensions','ParentID','ArticleID',$id);
					if (is_null($blogid)) goto defaultlabel;
					$blogcode = Services::dblookup('Articles_Extensions','BlogCode','ArticleID',$blogid);
					if (is_null($blogcode)) goto defaultlabel;
					$redir = 'blogs/'.$blogcode.'/'.$id;
					break;
				case 'page':
					$redir = 'pages/'.$id;
					break;
				case 'blog':
					$redir = 'blogs/'.$id;
					break;
				case 'folder':
					$redir = 'topics/'.$id;
					break;
				case 'picture':
					$redir = 'library/pictures/'.$id;
					break;
				case 'docfile':
					$redir = 'library/docfiles/'.$id;
					break;
				case 'link':
					$redir = 'library/links/'.$id;
					break;
				case 'comment':
					$redir = 'library/comments/'.$id;
					break;
				case 'file':
					$redir = 'library/folders/'.$id;
					break;
				case 'directory':
					$redir = 'library/directories/'.$id;
					break;
				case 'widget':
					$redir = 'library/widgets/'.$id;
					break;
				case 'inbox':
					$redir = 'library/inboxes/'.$id;
					break;
				case 'docextract':
					$redir = 'library/docextracts/'.$id;
					break;
				// pending
				case 'contact':
					$redir = 'library/contacts/'.$id;
					break;
				case 'workgroup':
					$redir = 'workgroups/'.$id;
					break;
				case 'message':
					$redir = 'messages/'.$id;
					break;
				default:
					defaultlabel:
					die('incorrect object type (' . $datarecord->ObjectType . ') - please report to technical support; hit back button'); // *** create graceful error message
			}
			$this->_controller->request->redirect($redir . URL::query());
		}
	}
	protected function _set_record_permissions($parameters,$permissions)
	{
		$tablemodel = $this->_tablemodel;
		
		# - get tokens for user or author, content and settings
		$tokens = $this->_get_record_user_tokens($parameters,$tablemodel);
		# set permissions
		// get userallowarticles etc.
		$user = $this->_user;
		$permissions->canread = $tokens->content->read;
		if ($user->authenticated()) {
			// basic permissions
			$userdata = $user->user_data();
			$permissions->userallowarticles = ($userdata->Permissions->article);
			$permissions->userallowblogs = ($userdata->Permissions->blog);
			$permissions->userallowcomments = ($userdata->Permissions->comment);
			$objecttype = $this->_context->objecttype;
			switch ($objecttype) {
				case 'blog':
				case 'entry':
					$allowindex = 'userallowblogs';
					break;
				case 'comment':
					$allowindex = 'userallowcomments';
					break;
				default:
					$allowindex = 'userallowarticles';
			}
			$permissions->userallowaccess = $permissions->$allowindex;
			$permissions->unmoderated = ($userdata->Permissions->unmoderated);
			$tokens = $this->_tokens;
			// edit permissions
			$permissions->cantrash = ($tokens->content->delete and $permissions->userallowaccess);
			$permissions->canedit = ($tokens->content->edit and $permissions->userallowaccess);
			$permissions->canadd = ($tokens->content->create and $permissions->userallowaccess);
			// owner settings
			$permissions->cansetmode = 
				(($tokens->content->edit or $tokens->settings->edit) and 
					$permissions->userallowaccess);
			$permissions->cansetstatus = ($tokens->settings->edit and $permissions->userallowaccess);
			$permissions->canprivatize = ($tokens->content->edit and $tokens->settings->edit and $permissions->userallowaccess);
			$permissions->canfreeze = ($tokens->content->edit and $tokens->settings->edit and $permissions->userallowaccess);
			$permissions->canlock = ($tokens->content->edit and $tokens->settings->edit and $permissions->userallowaccess);
			$permissions->canchangeauthor = Access::instance()->permission($this->_tokens->names->settings,'E') and $permissions->unmoderated;
			$permissions->canchange = Access::instance()->permission($this->_tokens->names->settings,'E') and $permissions->unmoderated;
			$permissions->caneditapprovenotes = Access::instance()->permission($this->_tokens->names->settings,'E') and $permissions->unmoderated;
			$permissions->canclosecomments = ($tokens->settings->edit and $permissions->userallowaccess);
			$permissions->canset = ($tokens->settings->edit and $permissions->userallowaccess);
			$permissions->canadminister = ($tokens->settings->edit and $permissions->userallowaccess);
			
			$permissions->candelete = ($tokens->settings->delete and $permissions->userallowaccess);
			// administrartion settings
			$permissions->canapprove = Access::instance()->permission($this->_tokens->names->settings,'E') and $permissions->unmoderated;
		} else {
			$permissions->userallowarticles = FALSE;
			$permissions->userallowcomments = FALSE;
			$permissions->userallowblogs = FALSE;
			$permissions->userallowaccess = FALSE;
			$permissions->unmoderated = FALSE;
			
			$permissions->cansetmode = FALSE;
			$permissions->cansetstatus = FALSE;
			$permissions->canprivatize = FALSE;
			$permissions->canfreeze = FALSE;
			$permissions->canlock = FALSE;
			$permissions->canchange = FALSE;
			$permissions->canchangeauthor = FALSE;
			$permissions->caneditapprovenotes = FALSE;
			$permissions->canclosecomments = FALSE;
			$permissions->canadminister = FALSE;
			$permissions->canapprove = FALSE;
			$permissions->cantrash = FALSE;
			$permissions->candelete = FALSE;
			$permissions->canset = FALSE;
			$permissions->canedit = FALSE;
			$permissions->canadd = FALSE;
		}
		$this->_on_set_permissions($permissions);
		// get individual permissions: content, and settings
	}
	
	protected function _on_set_permissions($permissions) {}
	
	// if user is author get author tokens, else get global tokens
	protected function _get_record_user_tokens($parameters,$tablemodel)
	{
		$userid = $this->_user->userid();
		$datarecord = $tablemodel->datarecord;
		if (!empty($datarecord))
			$articleid = $datarecord->ID;
		else
			$articleid = NULL;
		$authorid = $parameters->authorid;
		$tokens = $this->_tokens;
		$tokenslist = (array) $tokens->names;
		$access = Access::instance();
		foreach ($tokenslist as $tokenindex => $tokenname) {
			$tokens->$tokenindex = $access->get_token($tokenname,$authorid,$articleid);
		}
		return $tokens;
	}
	
	protected function _set_record_states($parameters,$permissions,$states,$tablemodel)
	{
		$datarecord = $tablemodel->datarecord;
		if ($datarecord) {
			$datarecord->Hide and ($states->hidden = TRUE);
			$datarecord->Trash and ($states->trash = TRUE);
			$datarecord->Frozen and ($states->frozen = TRUE);
			$datarecord->Locked and ($states->locked = TRUE);
			$states->commentsclosed = (bool) $datarecord->CommentsClosed;
			$states->published = (($datarecord->Approval == 'A') or ($datarecord->Approval == 'P'));
		}
		# finalize adminmode
		$states->adminmode = (($parameters->mode == 'admin') and ($permissions->canedit == TRUE));
		$this->_on_set_states($states);
	}
	protected function _on_set_states($states) {}
	
	protected function _set_gateway_resources($parameters,$tablemodel,$context,$states)
	{
		$datamodel = $this->_datamodel;
		$datapool = $datamodel->datapool;
		if (!property_exists($context,'objecttype')) {
			$articlerecord = $tablemodel->datarecord;
			if ($articlerecord) {
				$context->objecttype = $articlerecord->ObjectType;
				$datapool->constraints['objecttype'] = $context->objecttype;
			}
		} else {
			$datapool->constraints['objecttype'] = $context->objecttype;
		}
		# establish datamodel dataframe for subsequent actions
		$datamodel->dataframe($context->dataframe);
		if ($states->adminmode)
			$datapool->constraints['adminmode'] = TRUE;
		# set engine parameters for main content
		$datapool->variables['articleid'] = $parameters->articleid;
		$datapool->variables['binderoptions'] = $parameters->bindersoptions;
		$datapool->activerecords[$context->masterrecordindex] = $tablemodel->activerecord;
		if (property_exists($context,'datapoolarrays')) {
			foreach ($context->datapoolarrays as $index=>$value) {
				$datapool->arrays[$index] = $value;
			}
		}
		$this->_on_set_datapool($datapool);
	}
	protected function _on_set_datapool($datapool) {}
	
	// returns void
	protected function _process_operation_command($parameters,$permissions,$states,$context)
	{
		if ($parameters->command) {
			
			$result = $this->_respond($parameters->command); // returns Node::FLOW_CONTINUE or Node::FLOW_REDIRECT
			$articleid = $parameters->articleid; // might have been set by insert
			if ($result == Node::FLOW_REDIRECT) {
				$parms = $this->_get_redirect_parameters();
				$redirect = $context->redirect . "/$articleid$parms";
				$this->_controller->request->redirect($redirect);
			} else {
				if ($states->insertmode) {
					$parameters->mode = 'admin';
					$states->adminmode = TRUE;
				}
			}
		}
		$session = session::instance();
		$messages = $session->get_once('messages');
		if ($messages) {
			$index = $messages['index'];
			unset($messages['index']);
			$messages = (object) $messages;
			$this->_datamodel->datapool->variables[$index] = $messages;
		}
	}
	protected function _get_redirect_parameters()
	{
		$parameters = $this->_parameters;
		$parms = array();
		$parms['mode'] = $parameters->mode;
		$parms['instanceid'] = $parameters->instanceid;
		$parms['path'] = $parameters->path;
		$parms['perpage'] = $parameters->perpage;
		$this->_on_get_redirect_parameters($parms);
		$p = array();
		foreach ($parms as $key => $value) {
			if ($value)
				$p[] = $key . '=' . $value;
		}
		if (empty($p)) return '';
		$params = '?' . implode('&',$p);
		return $params;
	}
	// event
	protected function _on_get_redirect_parameters(&$parms)	{}
	// returns array $dictionary
	protected function _get_dictionary($parameters,$permissions,$states,$tablemodel)
	{
		$datarecord = $tablemodel->datarecord;
		# get parameterized values
		$datamodel = $this->_datamodel;
		$datamodel->retrieve();
		$dictionary = $datamodel->output();
		
		# standard values
		$dictionary['base'] = $this->_controller->_base;
		$dictionary['common'] = $this->_controller->_common;
		$dictionary['publicbase'] = System::instance()->PublicBase;
		$dictionary['uploadbase'] = System::instance()->UploadBase;
		# parameters
		if ($datarecord and !$states->insertmode) {
			$dictionary['approvaldate'] = $datarecord->ApproveDate;
			$dictionary['approvalusername'] = $datarecord->ApproverName;
			$dictionary['approvalusercode'] = $datarecord->ApproverUserCode;
		}
		if (isset($datarecord->Hide) and $datarecord->Hide) { // from states?
			$dictionary['visibility'] = 'private';
		} else {
			$dictionary['visibility'] = 'visible';
		}
		$dictionary['mode'] = $parameters->mode;
		$dictionary['articleid'] = $parameters->articleid; // so frame will know if an article was requested
		$dictionary['instanceid'] = $parameters->instanceid;
		$dictionary['path'] = $parameters->path;
		$dictionary['perpage'] = $parameters->perpage;
		
		# states
		$dictionary['is_record'] = (((bool) $datarecord) or $states->insertmode);
		if ($datarecord and !$states->insertmode) {
			if ($states->newcommentmode) {
				$dictionary['is_containerancillarydata'] = TRUE;
				$dictionary['is_articleancillarydata'] = TRUE;
				$dictionary['is_comments'] = TRUE;
				$dictionary['is_comment'] = TRUE;
				$dictionary['is_newcomment'] = TRUE;
			}
			$dictionary['is_approved'] = ($datarecord->Approval == 'A');
			$dictionary['is_pending'] = ($datarecord->Approval == 'P');
			$dictionary['is_published'] = (($datarecord->Approval == 'A') or ($datarecord->Approval == 'P'));
			$dictionary['default_order'] = $datarecord->DefaultOrder;
			$dictionary['default_direction'] = $datarecord->DefaultDirection;
			$dictionary['list_layout'] = $datarecord->DefaultFolio;
		} else {
			$dictionary['default_order'] = NULL;
			$dictionary['default_direction'] = NULL;
			$dictionary['list_layout'] = NULL;
		}
		$dictionary['is_moderated'] = !$permissions->unmoderated;
		$dictionary['is_commentsclosed'] = $states->commentsclosed;
		$dictionary['is_adminmode'] = $states->adminmode;
		$pageview = $this->_controller->pageview();
		$pageview->is_adminmode = $states->adminmode;

		$dictionary['is_insertmode'] = $states->insertmode;
		$dictionary['is_trash'] = $states->trash;
		$dictionary['is_locked'] = $states->locked;
		$dictionary['is_privatized'] = $states->hidden;
		$dictionary['is_frozen'] = $states->frozen;
		$dictionary['is_edit'] = $states->editmode;
		$dictionary['is_preview'] = $states->previewmode;
		$dictionary['is_display'] = !$states->editmode;
		
		// permissions
		$dictionary['can_add'] = $permissions->canadd;
		$dictionary['can_edit'] = $permissions->canedit;
		$dictionary['can_trash'] = $permissions->cantrash;
		$dictionary['can_delete'] = $permissions->candelete;

		// ** these should be collapsed to 'can_administer'
		$dictionary['can_administer'] = $permissions->canadminister;
		$dictionary['can_change'] = $permissions->canchange; // to change record type
		$dictionary['can_editapprovenotes'] = $permissions->caneditapprovenotes;
		$dictionary['can_editsettings'] = $permissions->canadminister;
		$dictionary['can_changeauthor'] = $permissions->canchangeauthor;

		$dictionary['can_comment'] = $permissions->userallowcomments;
		$dictionary['can_favourites'] = $permissions->favourites;
		
		# acquire content dictionary and combine with content template
		$this->_on_get_dictionary($dictionary);
		return $dictionary;
	}
	protected function _on_get_dictionary(&$dictionary) {}
	// ** datarecord data should come from parameters and states
	// returns array $admindictionary
	protected function _get_admindictionary($parameters, $permissions, $states, $dictionary, $tablemodel)
	{
		$datarecord = $tablemodel->datarecord;
		
		$articleid = $parameters->articleid;

		$admindictionary = array();
		# set sidebar dictionary
		# parameters
		$isrecord = (bool) $datarecord;
		if ($isrecord) {
			$admindictionary['emailtitle'] = rawurlencode($datarecord->Title); // ??
			$admindictionary['title'] = $datarecord->Title;
			$isprivatized = (isset($datarecord->Hide) and $datarecord->Hide);
			$isfrozen = (isset($datarecord->Frozen) and $datarecord->Frozen);
			$islocked = (isset($datarecord->Locked) and $datarecord->Locked);
			$iscommentsclosed = (isset($datarecord->CommentsClosed) and $datarecord->CommentsClosed);
		} else {
			$isprivatized = FALSE;
			$isfrozen = FALSE;
			$islocked = FALSE;
			$iscommentsclosed = TRUE;
		}
		if ($isrecord) {
			// trash
			if (isset($datarecord->Trash) and $datarecord->Trash) {
				$istrash = TRUE;
			} else {
				$istrash = FALSE;
			}
			// visibility
			if ($isprivatized) {
				$admindictionary['visibility'] = 'private';
			} else {
				$admindictionary['visibility'] = 'visible';
			}
			// statuscode
			if (isset($datarecord->Approval)) {
				$statuscode = $datarecord->Approval;
			} else {
				$statuscode = 'D';
			}
			// status
			switch ($statuscode) {
				case "D":
					$status = 'draft';
					break;
				case "R":
					$status = 'requested';
					break;
				case "A":
					if ($isprivatized)
						$status = 'publishable';
					else
						$status = 'published';
					break;
				case "X":
					$status = 'declined';
					break;
				case "P":
					$status = 'provisional';
					break;
				default:
					$status = 'draft';
			}
			$admindictionary['status'] = $status;
		}
		$admindictionary['base'] = $this->_controller->_base;
		$admindictionary['common'] = $this->_controller->_common;
		$admindictionary['website'] = System::instance()->Website;

		$admindictionary['mode'] = $parameters->mode;
		$admindictionary['articleid'] = $parameters->articleid; // so frame will know if an article was requested
		$admindictionary['instanceid'] = $parameters->instanceid;
		$admindictionary['perpage'] = $parameters->perpage;
		$admindictionary['path'] = $parameters->path;
		$admindictionary['pagepath'] = $parameters->pagepath;
		$admindictionary['id'] = $dictionary['id']; // $articleid;
		$admindictionary['createdate'] = $dictionary['createdate'];
		$admindictionary['createuserid'] = $dictionary['createuserid'];
		$admindictionary['editdate'] = $dictionary['editdate'];
		$admindictionary['edituserid'] = $dictionary['edituserid'];
		$admindictionary['recordversion'] = $dictionary['recordversion'];

		# permissions
		$admindictionary['can_setmode'] = $permissions->cansetmode;
		$admindictionary['can_setstatus'] = $permissions->cansetstatus;
		$admindictionary['can_privatize'] = $permissions->canprivatize;
		$admindictionary['can_administer'] = $permissions->canadminister;
		$admindictionary['can_edit'] = $permissions->canedit;
		$admindictionary['can_approve'] = $permissions->canapprove;;
		$admindictionary['can_freeze'] = $permissions->canfreeze;
		$admindictionary['can_lock'] = $permissions->canlock;
		$admindictionary['can_changeauthor'] = $permissions->canchangeauthor;
		$admindictionary['can_comment'] = $permissions->cancomment;
		$admindictionary['can_closecomments'] = $permissions->canclosecomments;
		$admindictionary['can_add'] = $permissions->canadd;
//		$admindictionary['can_article'] = $permissions->userallowarticles; // ** not used
//		$admindictionary['can_blog'] = $permissions->userallowblogs; // ** not used
		
		#states
		$admindictionary['is_record'] = $isrecord;
		$admindictionary['is_frozen'] = $isfrozen;
		$admindictionary['is_locked'] = $islocked;
		$admindictionary['is_privatized'] = $isprivatized;
		$admindictionary['is_insertmode'] = $states->insertmode;
		$admindictionary['is_adminmode'] = $states->adminmode;
		$admindictionary['is_moderated'] = !$permissions->unmoderated;
		$admindictionary['is_commentsclosed'] = $states->commentsclosed;
		$admindictionary['is_member'] = $states->member;
		if ($permissions->canadminister and $isrecord) {
			$admindictionary['is_approved'] = ($statuscode == "A");
			$admindictionary['is_requested'] = ($statuscode == "R");
			$admindictionary['is_draft'] = ($statuscode == "D");
			$admindictionary['is_declined'] = ($statuscode == "X");
			$admindictionary['is_pending'] = ($statuscode == "P");
			$admindictionary['is_trash'] = $istrash;
		}
		if ($datarecord and !$states->insertmode) {
			if ($states->newcommentmode) {
				$admindictionary['is_newcomment'] = TRUE;
			}
		}
		# assert content state
//		isset($admindictionary['status']) and ($dictionary['status'] = $admindictionary['status']);
//		isset($admindictionary['trash']) and ($dictionary['trash'] = $admindictionary['trash']);
//		isset($admindictionary['visibility']) and ($dictionary['visibility'] = $admindictionary['visibility']);
		$this->_on_get_admindictionary($admindictionary);
		return $admindictionary;
	}
	protected function _on_get_admindictionary(&$admindictionary) {}
	
	// ** listcaption from context? ** listcaption => listname **
	# ==========================================================================================
	# ===============================[ OPERATIONS RESPONSES ]===================================
	# ==========================================================================================
	// controller called from _record_process_operation_command above
	// #returns constant Node::FLOW_CONTINUE or Node::FLOW_REDIRECT
	protected function _respond($command)
	{
		$datamodel = $this->_datamodel;
		$recycle = Node::FLOW_CONTINUE; // continue with processing; false results in redirect
		try {
			switch ($command) {
				//  show/hide
				case 'hide':
					$recycle = $this->_record_hide($datamodel); // common
					break;
				case 'unhide':
					$recycle = $this->_record_unhide($datamodel); // common
					break;
				case 'lock':
					$recycle = $this->_record_lock($datamodel); // common *
					break;
				case 'unlock':
					$recycle = $this->_record_unlock($datamodel); // common *
					break;
				case 'lock':
					$recycle = $this->_record_lock($datamodel); // common *
					break;
				case 'unlock':
					$recycle = $this->_record_unlock($datamodel); // common *
					break;
				case 'opencomments':
					$recycle = $this->_record_opencomments($datamodel); // common *
					break;
				case 'closecomments':
					$recycle = $this->_record_closecomments($datamodel); // common *
					break;
				case 'freeze':
					$recycle = $this->_record_freeze($datamodel); // common *
					break;
				case 'unfreeze':
					$recycle = $this->_record_unfreeze($datamodel); // common *
					break;
				/*
				case 'seteveryone':
					$recycle = $this->_record_seteveryone($datamodel); // common *
					break;
				case 'setmembersonly':
					$recycle = $this->_record_setmembersonly($datamodel); // common *
					break;
				*/
				// status
				case 'draft':
					$recycle = $this->_record_draft($datamodel); // common
					break;
				case 'request':
					$recycle = $this->_record_request($datamodel); // common
					break;
				case 'publish':
					$recycle = $this->_record_publish($datamodel); // common
					break;
				case 'approve':
					$recycle = $this->_record_approve($datamodel); // common
					break;
				case 'decline':
					$recycle = $this->_record_decline($datamodel); // common
					break;
				// basic edit
				case 'editarticle':
					$recycle = $this->_record_edit($datamodel); // common
					break;
				case 'addarticle':
					$recycle = $this->_record_add($datamodel); // common
					break;
				case 'comment':
					$recycle = $this->_record_addcomment($datamodel); // common
					break;
				case 'trasharticle':
					$recycle = $this->_record_trash($datamodel); // common
					break;
				case 'untrasharticle':
					$recycle = $this->_record_untrash($datamodel); // common
					break;
				case 'savearticle':
					$recycle = $this->_record_save($datamodel); // common but forked: objecttype
					break;
				case 'deleteforeverarticle':
					$recycle = $this->_record_deleteforever($datamodel); // common but forked: objecttype
					break;
				case 'saveandedit':
					$recycle = $this->_record_saveandedit($datamodel);
					break;
				case 'previewarticle':
					$recycle = $this->_record_preview($datamodel);
					break;
				case 'cancelarticle':
					$recycle = $this->_record_cancel($datamodel);
					break;
				// related data
				case 'savenotes':
					$recycle = $this->_record_savenotes($datamodel);
					break;
				case 'changeauthor':
					$recycle = $this->_record_changeauthor($datamodel);
					break;
				case 'changebinder':
					$recycle = $this->_record_changebinder($datamodel);
					break;
				case 'saveapproval':
					$recycle = $this->_record_saveapproval($datamodel); // common -> ** s/b save adminnotes
					break;
				case 'savefollowup':
					$recycle = $this->_record_savefollowup($datamodel);
					break;
				case 'start':
					$recycle = $this->_record_start($datamodel);
					break;
				// custom 
				default: {
					$recycle = $this->_on_respond($command); // sub_type/class
				}
			}
		} catch(Exception $e) {
			echo 'MESSAGE: ' . $e->getMessage() . '<br />';
			echo 'ERROR CODE: ' . $e->getCode() . '<br />';
			echo 'FILE: ' . $e->getFile() . '<br />';
			echo 'LINE: ' . $e->getLine() . '<br />';
			die('unexpected error in noderecord.php. Copy error to tech support, then hit back key.');
		}
		return $recycle;
	}
	
	// stub
	protected function _on_respond($command)
	{ 
		return self::FLOW_CONTINUE; // don't repeat process
	}
	
	/* **
		To do:
		- "start" advanced search
		- tabs:
			- settings
			- notes
			- approval
			- followup
		- asterisk for open notes and open followup
		- send email to author for approve/deny
	*/
	protected function _record_hide($datamodel)
	{
		if (!$this->_permissions->canprivatize)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
//		$extensionrecord = $datamodel->datapool->records[$this->_context->extensionrecordindex] = new StdClass; ??
		$datamodel->input()->finishinput()->store();
		$this->_states->hidden = $datarecord->Hide = TRUE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_unhide($datamodel)
	{
		if (!$this->_permissions->canprivatize)
			return $this->_record_report_access_error($datamodel);
		if ($this->_states->locked)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->hidden = $datarecord->Hide = FALSE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_trash($datamodel)
	{
		if ($this->_states->frozen)
			return $this->_record_report_access_error($datamodel);
		if (!$this->_permissions->cantrash)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->trash = $datarecord->Trash = TRUE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_untrash($datamodel)
	{
		if (!$this->_permissions->cantrash)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->trash = $datarecord->Trash = FALSE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_draft($datamodel)
	{
		if (!$this->_permissions->cansetstatus)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$datarecord->Approval = 'D';
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_request($datamodel)
	{
		if (!$this->_permissions->cansetstatus)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$datarecord->Approval = 'R';
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$this->_on_after_record_request($datamodel);
		return Node::FLOW_REDIRECT;
	}
	protected function _on_after_record_request($datamodel)
	{
		$system = System::instance();
		$activerecord = $this->_tablemodel->activerecord;
		$activerecord->current();
		$approvalcode = $activerecord->Approval;
		if ($approvalcode == 'A') return;
		$recipients = $system->AdministratorEmailAddress;
		$date = date('d-M-yy');
		$selector = 'request';
		$record = $activerecord->current();
		$data = array(
			'username' => $activerecord->Author,
			'usercode' => $activerecord->AuthorUserCode,
			'memberurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'member/account/' . $activerecord->AuthorUserCode,
			'recordtype' => I18n::get($activerecord->ObjectType),
			'recordurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'node/' . $activerecord->ID,
		);
		list($successes,$failures) = Notification::email($recipients,$date,$selector,$data);
		if ($successes) {
			$this->_set_datapool_message($this->_datamodel->datapool,'request for moderation has been sent to an administrator','messages');
		} else {
			$this->_set_datapool_message($this->_datamodel->datapool,'request for moderation has failed','errors');
		}
		$messages = (array) $this->_get_datapool_messageblock($datamodel->datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
	}
	protected function _record_publish($datamodel)
	{
		if (!$this->_permissions->cansetstatus)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
//		$extensionrecord = $datamodel->datapool->records[$this->_context->extensionrecordindex] = new StdClass; ??
		$datamodel->input()->finishinput()->store();
		$datarecord->Approval = 'A';
		$datarecord->ApproveDate = $this->_tablemodel->table->db()->sql_datetime(time());
		$datarecord->ApproveUserID = $this->_user->userid();
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_approve($datamodel)
	{
		if (!$this->_permissions->cansetstatus)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$datarecord->Approval = 'A';
		$datarecord->ApproveDate = $this->_tablemodel->table->db()->sql_datetime(time());
		$datarecord->ApproveUserID = $this->_user->userid();
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$this->_on_after_record_approve($datamodel);
		return Node::FLOW_REDIRECT;
	}
	protected function _on_after_record_approve($datamodel)
	{
		$system = System::instance();
		$notify = (bool) $this->_parameters->notification;
		if (!$notify) return;
		$activerecord = $this->_tablemodel->activerecord;
		$activerecord->current();
		$recipients = $activerecord->AuthorID;
		$date = date('d-M-yy');
		$selector = 'requestapproved';
		$record = $activerecord->current();
		$data = array(
			'recordtype' => I18n::get($activerecord->ObjectType),
			'recordurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'node/' . $activerecord->ID,
		);
		list($successes,$failures) = Notification::email($recipients,$date,$selector,$data);
		if ($successes) {
			$this->_set_datapool_message($this->_datamodel->datapool,'request approval notification has been sent member','messages');
		} else {
			$this->_set_datapool_message($this->_datamodel->datapool,'request  approval notification has failed','errors');
		}
		$messages = (array) $this->_get_datapool_messageblock($datamodel->datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
	}
	protected function _record_decline($datamodel)
	{
		if (!$this->_permissions->cansetstatus)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$datarecord->Approval = 'X';
		$datarecord->ApproveDate = $this->_tablemodel->table->db()->sql_datetime(time());
		$datarecord->ApproveUserID = $this->_user->userid();
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$this->_on_after_record_decline($datamodel);
		return Node::FLOW_REDIRECT;
	}
	protected function _on_after_record_decline($datamodel)
	{
		$system = System::instance();
		$notify = (bool) $this->_parameters->notification;
		if (!$notify) return;
		$activerecord = $this->_tablemodel->activerecord;
		$activerecord->current();
		$recipients = $activerecord->AuthorID;
		$date = date('d-M-yy');
		$selector = 'requestdeclined';
		$record = $activerecord->current();
		$data = array(
			'recordtype' => I18n::get($activerecord->ObjectType),
			'recordurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'node/' . $activerecord->ID,
			'adminaddress' => $system->AdministratorEmailAddress,
		);
		list($successes,$failures) = Notification::email($recipients,$date,$selector,$data);
		if ($successes) {
			$this->_set_datapool_message($this->_datamodel->datapool,'request declined notification has been sent member','messages');
		} else {
			$this->_set_datapool_message($this->_datamodel->datapool,'request  declined notification has failed','errors');
		}
		$messages = (array) $this->_get_datapool_messageblock($datamodel->datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
	}
	protected function _record_edit($datamodel)
	{
		if (!$this->_tokens->content->edit)
			return $this->_record_report_access_error($datamodel);
		if ($this->_states->frozen)
			return $this->_record_report_access_error($datamodel);
		if ($this->_tablemodel->datarecord or $this->_states->insertmode) {
			$datamodel->datapool->constraints['edit'] = TRUE;
			$this->_states->editmode = TRUE;
		}
		return Node::FLOW_CONTINUE;
	}
	protected function _record_add($datamodel)
	{
		if (!$this->_permissions->canadd)
			return $this->_record_report_access_error($datamodel);
		if (!($this->_tablemodel->datarecord)) {
			$datamodel->datapool->constraints['edit'] = TRUE;
			$this->_states->editmode = TRUE;
			$datamodel->datapool->constraints['insertmode'] = TRUE;
			$this->_states->insertmode = TRUE;
			return Node::FLOW_CONTINUE;
		} else {
			return Node::FLOW_REDIRECT;
		}
	}
	protected function _record_addcomment($datamodel)
	{
		if (!$this->_permissions->userallowcomments)
			return $this->_record_report_access_error($datamodel);
		$commenttokenset = Services::get_objecttype_tokenset('comment');
		$permission = Access::instance($commenttokenset->content,'C');
		if (!$permission)
			return $this->_record_report_access_error($datamodel);
		if (($this->_tablemodel->datarecord)) { // existing master record
			$states = $this->_states;
			$states->insertmode = FALSE;
			$states->newcommentmode = TRUE;
			$states->adminmode = FALSE;
			$this->_parameters->mode = NULL;
			return Node::FLOW_CONTINUE;
		}
		return Node::FLOW_REDIRECT;
	}
	
	// ** modify for extensions: binder for blogs; extensions: blogs, clusters, folders; tree for folders
	/*
		- collect input, validate, and finishinput
		- if successful, store
		- direct to insert, or update
	*/
	protected function _record_save($datamodel)
	{
//		var_dump($this->_permissions);
		if (!$this->_permissions->canedit)
			return $this->_record_report_access_error($datamodel);
		$context = $this->_context;
		$datamodel->datapool->constraints['edit'] = TRUE;
		$states = $this->_states;
		$states->editmode = TRUE;
		$datamodel->datapool->variables['messagesection'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('messagesection')
			->errormessageindex($context->messagefile)
			->input()
			->validate();
		if (empty($datamodel->dataframe->pointlist['id']->value)) {
			$states->insertmode = TRUE;
			$datamodel->datapool->constraints['insertmode'] = TRUE;
		}
		$datamodel->finishinput();
		if (!$result) {
			$datamodel->recoverinput();
			return self::FLOW_CONTINUE;
		} else {
			unset($datamodel->datapool->variables['messagesection']);
			# store record
			$datarecord = $datamodel->datapool->records[$context->masterrecordindex] = new StdClass;
			if (!empty($this->_extensionflag)) {
				$extensionrecord = $datamodel->datapool->records[$context->extensionrecordindex] = new StdClass;
			}
			$datamodel->store();
			# save record
			$activerecord = $this->_tablemodel->activerecord;
			if ($states->insertmode) { // (is_null($datarecord->ID) and is_null($activerecord->AuthorID)) { // new record
				$result = $this->_record_insert();
				if (!$result) return self::FLOW_CONTINUE;
			} else { // existing record
				$result = $this->_record_update();
				if (!$result) return self::FLOW_CONTINUE;
			}
			// refresh current record
			$this->_tablemodel->datarecord = $activerecord->current();
			$datapool = $datamodel->datapool;
			$this->_set_datapool_message(
				$datapool,
				Kohana::message($this->_context->messagefile,'save'),
				'messages'
			);
			$messages = (array) $this->_get_datapool_messageblock($datapool);
			$messages['index'] = 'messagesection';
			$session = session::instance();
			$session->set('messages',$messages);
		}
		return self::FLOW_REDIRECT;
	}
	// @returns boolean success
	protected function _record_update()
	{
		$context = $this->_context;
		$datamodel = $this->_datamodel;
		$tablemodel = $this->_tablemodel;
		// keep prior record for comparison
		$priorrecord = $tablemodel->priorrecord;
		$parms = array('priorrecord');
		$model = $tablemodel->model;
		$db = $model->db();
		$activerecord = $model->activerecord();
		$masterrecord = $datamodel->datapool->records[$context->masterrecordindex];
		$parms = array_merge($parms,array('activerecord','masterrecord'));
		$activerecord->table()->auto_lock(FALSE);
		if (!empty($this->_extensionflag)) {
			$extensionactiverecord = $model->activerecord($context->extensiontablename);
			$extensionrecord = $datamodel->datapool->records[$context->extensionrecordindex];
			$parms = array_merge($parms,array('extensionactiverecord','extensionrecord'));
			$extensionactiverecord->table()->auto_lock(FALSE);
		}
		$locktables = array(
			$model->table()->tablename(),
			'Nodes_Instances',
			'Articles_Authors',
			'Access_Tokens',
			'Access_Role_Tokens'
		);
		$this->_on_set_update_locktables($locktables);
		$db->lock_tables($locktables); // includes extention
		$parmdata = compact($parms); // priorrecord; master active and new record; extension active and new record
		$error_info = array();
		$result = $this->_on_before_update($parmdata,&$error_info);
		if ($result and !empty($this->_extensionflag)) {
			$insertrequired = FALSE;
			if (!$extensionactiverecord->current()) {
				$insertrequired = TRUE;
			}
			if ($insertrequired) {
				$result = $extensionactiverecord->insert((array) $extensionrecord)->save();
			} else {
				$result = $extensionactiverecord->update((array) $extensionrecord)->save();
			}
			if (!$result) {
				$error_info = $extensionactiverecord->error_info();
				if ($error_info[0] == '23000') {
					$message = $error_info[2];
					if (strpos($message,"key 'RepositoryID_FileName'") !== FALSE) {
						$error_info[2] = 'A record already exists for this Filename/Directory combination';
					} elseif (strpos($message,"key 'BlogCode'") !== FALSE) {
						$error_info[2] = 'This Blog Code already exists';
					} elseif (strpos($message,"key 'FolderCode'") !== FALSE) {
						$error_info[2] = 'This Topic Code already exists';
					}
				}
				$db->unlock_tables();
				$this->_record_report_dberror($datamodel,$error_info);
				return FALSE;
			} 
		}
		if ($result) {
			$result = $activerecord->update((array) $masterrecord)->save();
			if (!$result) {
				$error_info = $activerecord->error_info();
				$db->unlock_tables();
				$this->_record_report_dberror($datamodel,$error_info);
				return FALSE;
			} 
		}
		if ($result) {
			$parmdata = compact($parms); // up to date data
			$result = $this->_on_after_update($parmdata,$error_info);
		}
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$error_info);
			return FALSE;
		}
		$parmdata = compact($parms); // up to date data
		$this->_on_after_update_post($parmdata);
		return TRUE;
	}
	protected function _on_set_update_locktables(&$locktables) {}
	
	protected function _on_before_update($recorddata,&$error_info) // for sub-classes
	{
		return TRUE;
	}
	protected function _on_after_update($recorddata,&$error_info) // for sub-classes
	{
		return TRUE;
	}
	protected function _on_after_update_post($recorddata) // for sub-classes
	{
		return TRUE;
	}
	// returns boolean success
	protected function _record_insert()
	{
		$context = $this->_context;
		$parameters = $this->_parameters;
		$datamodel = $this->_datamodel;
		$tablemodel = $this->_tablemodel;
		$model = $tablemodel->model;
		$db = $model->db();
		$activerecord = $model->activerecord();
		$masterrecord = $datamodel->datapool->records[$context->masterrecordindex];
		# initialize
		$masterrecord->AuthorID = $this->_user->userid();
		$masterrecord->ObjectType = $context->objecttype;
		$parms = array('activerecord','masterrecord');
		$activerecord->table()->auto_lock(FALSE);
		if (!empty($this->_extensionflag)) {
			$extensionactiverecord = $model->activerecord($context->extensiontablename);
			$extensionrecord = $datamodel->datapool->records[$context->extensionrecordindex];
			$parms = array_merge($parms,array('extensionactiverecord','extensionrecord'));
			$extensionactiverecord->table()->auto_lock(FALSE);
		}
		$locktables = array(
			$model->table()->tablename(),
			'Nodes_Instances',
			'Articles_Authors',
			'Access_Tokens',
			'Access_Role_Tokens'
		);
		$this->_on_set_insert_locktables($locktables);
		$db->lock_tables($locktables); // includes extention
		$parmdata = compact($parms); // priorrecord; master active and new record; extension active and new record
		$error_info = array();
		$result = $this->_on_before_insert($parmdata,$error_info);
		// straightforward insert for non-extension record
		if ($result) {
			$result = $activerecord->insert((array) $masterrecord)->save();
			if (!$result) {
				$this->_record_report_dberror($datamodel,$activerecord->error_info());
				$db->unlock_tables();
				return FALSE; // failure
			}
			$articleid = $activerecord->insert_id();
			$parameters->articleid = $articleid;
			$this->_tablemodel->table->preset()->query()->where('ID','=',$articleid); // for insert extension
			if (!empty($this->_extensionflag)) {
				$result = $extensionactiverecord->insert((array) $extensionrecord)->save();
				if (!$result) {
					$error_info = $extensionactiverecord->error_info();
					if ($error_info[0] == '23000') {
						$message = $error_info[2];
						if (strpos($message,"key 'RepositoryID_FileName'") !== FALSE) {
							$error_info[2] = 'A record already exists for this Filename/Directory combination';
						} elseif (strpos($message,"key 'BlogCode'") !== FALSE) {
							$error_info[2] = 'This BlogCode already exists';
						} elseif (strpos($message,"key 'FolderCode'") !== FALSE) {
							$error_info[2] = 'This FolderCode already exists';
						}
					}
					$activerecord->delete();
					$parmdata = compact($parms); // up to date data
					$this->_on_insert_extension_error($parmdata,$error_info);
					$this->_record_report_dberror($datamodel,$error_info);
					$db->unlock_tables();
					return FALSE; // failure
				}
				$this->_tablemodel->table->preset()->query()->where('ID','=',$articleid); // for after insert
			}
			$parmdata = compact($parms); // up to date data
			$result = $this->_on_after_insert($parmdata,$error_info);
		}
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$error_info);
			return FALSE;
		}
		// uncouple from existing navigator...
		$parameters->instanceid = '';
		$parameters->path = '';
		$parmdata = compact($parms); // up to date data
		$this->_on_after_insert_post($parmdata);
		
		// must be successful to get here
		$this->_states->insertmode = FALSE;
		$datamodel->datapool->constraints['insertmode'] = FALSE;
		
		return TRUE; // success
	}
	protected function _on_set_insert_locktables(&$locktables) {}
	// returns boolean success
	protected function _on_before_insert($recorddata,&$error_info) // for sub-classes
	{
		return TRUE;
	}
	protected function _on_insert_extension_error($recorddata,&$error_info) {}
	protected function _on_after_insert($recorddata,&$error_info) // for sub-classes
	{
		return TRUE;
	}
	protected function _on_after_insert_post($recorddata) // for sub-classes
	{
		return TRUE;
	}
	// returns cosntant Node::FLOW_REDIRECT or Node::FLOW_CONTINUE
	protected function _record_deleteforever($datamodel)
	{
		if (!$this->_permissions->candelete)
			return $this->_record_report_access_error($datamodel);
		$user = $this->_user;
		# check if user is authenitcated
		$datarecord = $this->_tablemodel->activerecord->current();
		$isrecord = (bool) $datarecord;
		$isauthenticated = $user->authenticated();
		$result = TRUE;
		if ($isrecord and $isauthenticated) {
			$access = Access::instance();
			$userid = $user->userid();
			$authorid = $datarecord->AuthorID;
			$tokens = $this->_tokens;
			if ($userid == $authorid) {
				$compareid = $userid;
				$permission = $access->permission($tokens->names->trash,'D',$compareid);
			} else {
				$permission = $access->permission($tokens->names->trash,'D');
			}
			$trash = $datarecord->Trash;
			$id = $datarecord->ID;
			if ($trash and $permission and (!is_null($id))) { // proceed with deletion
				$db = $this->_tablemodel->model->db();
				try {
					$context = $this->_context;
					$error_info = array();
					$result = $this->_record_delete($db,$datarecord,$error_info);
				} catch (Exception $e) {
					$db->unlock_tables();
					$result = FALSE;
					$error_info = array('muster','muster error',$e->getMessage());
					// setup error_info
				}
				if (!$result) {
					$index = $error_info[0];
					$error_message = Kohana::message($this->_context->messagefile,$index);
					if (!$error_message) $error_message = $error_info[2];
					$this->_set_datapool_message(
						$datamodel->datapool,
						$error_message,
						'errors'
					);
					return self::FLOW_CONTINUE;
				}
			}
		}
		return self::FLOW_REDIRECT;
	}
	// returns boolean success
	protected function _record_delete($db,$datarecord,&$error_info)
	{
		$context = $this->_context;
		/* for blog,
		* - check that there are no blog entries (must be deleted first)
		* for page
		* - check that there are no child pages (must be deleted first)
		*/
		$recordid = $datarecord->ID;
		if (is_null($recordid)) {
			$result = FALSE;
			$error_info[0] = 'INTEGRITY';
			$error_info[1] = 'DELETE';
			$error_info[2] = 'SYSTEM ERROR: NO RECORDID FOR DELETE';
			return $result;
		}
		$locktables = array( // 8 delete operations
			'Articles',
			'Articles_Extensions',
			'Articles_Authors',
			'Articles_Favourites',
			'Articles_Attachments', // x 2
			'Nodes_Instances', // x 2
			'Access_Tokens',
			'Access_Role_Tokens',
		);
		$this->_on_set_delete_locktables($locktables);
		$db->lock_tables($locktables);
		$result = TRUE;
		$errors = array();
		$record = 
			DB::select('ID')
			->from('Articles_Extensions')
			->where('CommentArticleID','=',$recordid)
			->as_object()
			->prepare()
			->execute()
			->current();
		if ($record) {
			$result = FALSE;
			$errors[0] = 'INTEGRITY';
			$errors[1] = 'DELETE';
			$errors[2] = 'Comments exist for this record';
		}
		$parms = array('db','datarecord');
		if ($result) {
			$parmdata = compact($parms);
			$result = $this->_on_before_delete($parmdata,$error_info);
		}
		if ($result) {  // 1
			$operation = DB::delete('Nodes_Instances')
				->where('FolderID','=',$recordid)
				->prepare()
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) {  // 2
			$operation = DB::delete('Nodes_Instances') // delete folders instances
				->where('NodeID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) { // 3
			$operation = DB::delete('Articles_Authors') // delete folders instances
				->where('ArticleID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) { // 4
			$operation = DB::delete('Articles_Extensions') // delete folders
				->where('ArticleID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) { // 5
			$operation = DB::delete('Articles_Attachments') // delete article comments
				->where('ArticleID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) { // 6
			$operation = DB::delete('Articles_Attachments') // delete article comments
				->where('AttachmentID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) { // 7
			$operation = DB::delete('Articles_Favourites') // delete article comments
				->where('ArticleID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) { // 8
			$operation = DB::delete('Articles') // delete article record
				->where('ID','=',$recordid)
				->prepare($db)
				->execute();
			$errors = $operation->error_info();
			$result = $operation->success();
		}
		if ($result) {
			$parmdata = compact($parms);
			$result = $this->_on_after_delete($parmdata,$error_info);
		}
		$db->unlock_tables();
		if (!empty($errors)) {
			foreach ($errors as $key => $value)
			{
				$error_info[$key] = $value;
			}
		}
		return $result; // success
	}
	protected function _on_set_delete_locktables(&$locktables) {}
	
	protected function _on_before_delete($parmdata,&$error_info)
	{
		return TRUE;
	}
	protected function _on_after_delete($parmdata,&$error_info)
	{
		return TRUE;
	}
	// returns void
	protected function _record_saveandedit($datamodel)
	{
		if (!$this->_permissions->canedit)
			return $this->_record_report_access_error($datamodel);
		if ($this->_record_save($datamodel) == Node::FLOW_CONTINUE) {
			return Node::FLOW_CONTINUE; // problem
		} else {
			(!$this->_states->insertmode) or ($this->_states->insertmode = FALSE);
			$datamodel->dataframe->reset(); // re-collect data for edit
			$this->_set_datapool_message(
				$datamodel->datapool,
				Kohana::message($this->_context->messagefile,'saveandedit'),
				'messages'
			);
			return $this->_record_edit($datamodel);
		}
	}
	protected function _record_preview($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = TRUE;
		$this->_states->editmode = TRUE;
		$datamodel->datapool->constraints['preview'] = TRUE;
		$this->_states->previewmode = TRUE;
		$datamodel->input(Point::RECOVERSTATE_RECYCLE);
		if (empty($datamodel->dataframe->pointlist['id']->value))
			$this->_states->insertmode = TRUE;
		$this->_set_datapool_message(
			$datamodel->datapool,
			Kohana::message($this->_context->messagefile,'preview'),
			'messages'
		);
		return Node::FLOW_CONTINUE;
	}
	protected function _record_cancel($datamodel)
	{
		$parameters = $this->_parameters;
		if ($parameters->articleid) {
			$datapool = $datamodel->datapool;
			$this->_set_datapool_message(
				$datapool,
				Kohana::message($this->_context->messagefile,'cancel'),
				'messages'
			);
			$messages = (array) $this->_get_datapool_messageblock($datapool);
			$messages['index'] = 'messagesection';
			$session = session::instance();
			$session->set('messages',$messages);
		}
		return Node::FLOW_REDIRECT; // done
	}
	protected function _record_savenotes($datamodel)
	{
		if (!$this->_permissions->canedit)
			return $this->_record_report_access_error($datamodel);
		$datamodel->datapool->constraints['editsection'] = 'author';
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$datapool = $datamodel->datapool;
		$this->_set_datapool_message(
			$datapool,
			Kohana::message($this->_context->messagefile,'savenotes'),
			'messages'
		);
		$messages = (array) $this->_get_datapool_messageblock($datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
	protected function _record_changeauthor($datamodel)
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		$datamodel->datapool->constraints['changeauthor'] = TRUE;
		$datamodel->datapool->constraints['editsection'] = 'author';
		$datamodel->input()->finishinput();
		$activerecord = $this->_tablemodel->activerecord;
		$newauthorcode = $datamodel->dataframe->pointlist['authorsection']->pointlist['newauthorcode']->value;
		if (!$newauthorcode) {
			$this->_set_datapool_message(
				$datamodel->datapool,
				Kohana::message($this->_context->messagefile,'nonewauthorcode'),
				'errors'
			);
			return Node::FLOW_CONTINUE;
		} else {
			$newauthorid = services::dblookup('Access_Users','ID','Code',
				$newauthorcode,$this->_tablemodel->table->db());
			if (is_null($newauthorid)) {
				$this->_set_datapool_message(
					$datamodel->datapool,
					Kohana::message($this->_context->messagefile,'unrecognizedauthorcode'),
					'errors'
				);
				return Node::FLOW_CONTINUE;
			}
		}
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->store();
		$datarecord->AuthorID = $newauthorid;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$datapool = $datamodel->datapool;
		$this->_set_datapool_message(
			$datapool,
			Kohana::message($this->_context->messagefile,'newauthorsaved'),
			'messages'
		);
		$messages = (array) $this->_get_datapool_messageblock($datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
	protected function _record_saveapproval($datamodel)
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		$datamodel->datapool->constraints['editsection'] = 'approval';
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$datapool = $datamodel->datapool;
		$this->_set_datapool_message(
			$datapool,
			Kohana::message($this->_context->messagefile,'saveapprovals'),
			'messages'
		);
		$messages = (array) $this->_get_datapool_messageblock($datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
	
	protected function _record_savefollowup($datamodel)
	{
		if (!$this->_permissions->canedit)
			return $this->_record_report_access_error($datamodel);
		$datamodel->datapool->constraints['editsection'] = 'followup';
		$datamodel->datapool->variables['messagesection'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('messagesection')
			->errormessageindex($this->_context->messagefile)
			->input()
			->validate();
		if (!$result) {
			$datamodel->finishinput();
			return Node::FLOW_CONTINUE;
		}
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->finishinput()->store();
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		$datapool = $datamodel->datapool;
		$this->_set_datapool_message(
			$datapool,
			Kohana::message($this->_context->messagefile,'savefollowup'),
			'messages'
		);
		$messages = (array) $this->_get_datapool_messageblock($datapool);
		$messages['index'] = 'messagesection';
		$session = session::instance();
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
	protected function _record_lock($datamodel)
	{
		if (!$this->_permissions->canlock)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->locked = $datarecord->Locked = TRUE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_unlock($datamodel)
	{
		if (!$this->_permissions->canlock)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->locked = $datarecord->Locked = FALSE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_opencomments($datamodel)
	{
		if (!$this->_permissions->canclosecomments)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->commentsclosed = $datarecord->CommentsClosed = FALSE;
		$activerecord = $this->_tablemodel->activerecord;
		$result = $activerecord->update((array) $datarecord)->save();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_closecomments($datamodel)
	{
		if (!$this->_permissions->canclosecomments)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->commentsclosed = $datarecord->CommentsClosed = TRUE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlocktables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_freeze($datamodel)
	{
		if (!$this->_permissions->canfreeze)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->frozen = $datarecord->Frozen = TRUE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	protected function _record_unfreeze($datamodel)
	{
		if (!$this->_permissions->canfreeze)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $datamodel->datapool->records[$this->_context->masterrecordindex] = new StdClass;
		$datamodel->input()->finishinput()->store();
		$this->_states->frozen = $datarecord->Frozen = FALSE;
		$activerecord = $this->_tablemodel->activerecord;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$result = $activerecord->update((array) $datarecord)->save();
		$db->unlock_tables();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activerecord->error_info());
			return Node::FLOW_CONTINUE; // recycle
		}
		return Node::FLOW_REDIRECT;
	}
	// ** deferred - collaboaration
	/*
	protected function _record_seteveryone($datamodel)
	{
		echo 'seteveryone';
		return Node::FLOW_REDIRECT;
	}
	protected function _record_setmembersonly($datamodel)
	{
		echo 'setmembersonly';
		return Node::FLOW_REDIRECT;
	}
	*/
	protected function _record_changebinder($datamodel)
	{
		echo 'changebinder';
		return Node::FLOW_REDIRECT;
	}
	# -----------------------------------------------------------------------------------#
	# ================================[ Bookmark ]=======================================#
	# -----------------------------------------------------------------------------------#
	protected function _get_bookmark()
	{
		$bookmark = array();
		if ($this->_states->insertmode) return $bookmark;
		
		$parameters = $this->_parameters;
		$datarecord = $this->_tablemodel->datarecord;
		if ($datarecord) {
			$objecttype = $datarecord->ObjectType;
		} else {
			$objecttype = NULL;
		}
		if (!empty($parameters->path) or (!empty($parameters->instanceid))) {
			$retval = $this->_get_instance_bookmark($bookmark);
			if (!$retval) {
				$parameters->path = '';
				$parameters->instanceid = '';
				$bookmark = (array) $bookmark;
				$warningindex = $bookmark['warningindex'];
				$bookmark = (object) array(
					'warningmessage' =>
					'Warning: The Navigator is unavailable for this bookmark because there\'s an unrecognized value in the bookmark\'s path or instanceid in the url; record relationships have most likely been changed by the website custodians. Such changes are legitimate, and this is not a system error '
					. '(' . $warningindex . ').'
				);
				return $bookmark;
			}
		} elseif ($objecttype == 'entry') {
			$this->_get_blogpost_bookmark($bookmark);
		}

		// otherwise: blog post, or info page
		
		$this->_on_get_bookmark($bookmark);
		// in any case increment path
		if (in_array($objecttype,array('folder','file','inbox','directory'))) {
			$dictionary = &$this->_dictionary;
			$parameters->pagepath = $dictionary['pagepath'] = $parameters->path;
			$this->_append_instance_path();
			$dictionary['path'] = $parameters->path;
		}
		return $bookmark;
	}
	protected function _on_get_bookmark(&$bookmark) {}

	protected function _append_instance_path()
	{
		$parameters = $this->_parameters;
		$path = empty($parameters->path)?NULL:$parameters->path;
		$path = empty($path)?array():explode(',',$path);
		$pathitem = $parameters->articleid . '-' . $parameters->instanceid;
		$endvalue = end($path);
		if ($endvalue !== $pathitem) {
			$path[] = $pathitem;
		}
		$path = implode(',',$path);
		$parameters->path = $path;
	}
	# ---------------------------[ Instance Bookmark ]--------------------------------#
	protected function _get_instance_bookmark(&$bookmark)
	{
		# get shareed data
		$datarecord = $this->_tablemodel->datarecord;
		if (empty($datarecord)) {
			$bookmark['warningindex'] = 'new record';
			return FALSE;
		}
		$data = new StdClass;
		$data->secondary_order = 'Title';
		$data->secondary_direction = 'ASC';
		$data->referencedata = new StdClass;
		$parameters = $data->parameters = $this->_parameters;
		$data->db = $this->_tablemodel->model->db();
		$data->partial_lookup = Kohana::$config->load('recordpartials')->getArrayCopy();
		$data->commonfields = array('ID','Title','ObjectType','ParentID','BlogCode','OrderText','isTreeTop');
		$bookmark = new StdClass;
		$bookmark->perpage = $parameters->perpage;
		$data->bookmark = $bookmark;
		# get instance data
		$instanceid = $data->parameters->instanceid;
		if ($instanceid) { // Nodes_Instances
			$operation = DB::select('*')
				->from('Nodes_Instances')
				->where('ID','=',$instanceid)
				->as_object()
				->prepare($data->db)
				->execute();
			if ($operation->success()) {
				$instancerecord = $operation->current();
			} else {
				$bookmark->warningindex = 'fetch failure';
				return FALSE;
			}
			if (!$instancerecord) {
				$bookmark->warningindex = 'no record';
				return FALSE;
			}
			$data->instancerecord = $instancerecord;
			$data->tablename = 'Nodes_Instances_View';
			$data->commonfields[] = 'InstanceID';
			$data->containerid = $instancerecord->FolderID;
			$data->comparefield = 'InstanceID';
			$data->comparevalue = $instanceid;
			$bookmark->guide = $instancerecord->InstanceDescription;
			$path = $parameters->path;
			if (!$path) { // instanceid without accompanying path; add partnered path
				$path = array();
				$path[] = $data->containerid .'-';
				$parameters->path = implode(',',$path);
			}
		} else { // Nodes_View
			$data->tablename = 'Nodes_View';
			$instancerecord = $data->instancerecord = $this->_tablemodel->datarecord;
			if (!$instancerecord) {
				$bookmark->warningindex = 'no instance';
				return FALSE;
			}
			$data->containerid = $instancerecord->ID;
			$data->comparefield = 'ID';
			$data->comparevalue = $instancerecord->ID;
			$bookmark->guide = $instancerecord->OrderGuide;
		}
		# collect resources
		# set base data
		$dictionary = &$this->_dictionary;
		$dictionary['ordertext'] = $instancerecord->OrderText?$instancerecord->OrderText:$instancerecord->Order;
		$dictionary['instancebookmark'] = TRUE;
		# get subject and ancestors
		if (!$this->_get_instance_subject($data)) {
			$bookmark->warningindex = 'no subject';
			return FALSE;
		}
		# get next
		$this->_get_instance_next($data);
		# get previous
		$this->_get_instance_previous($data);
		$referencedata = $data->referencedata;
		$previousnumber = $referencedata->previouscount - $referencedata->previousoffset;
		$bookmark->recordnumber = $previousnumber + 1;
		$bookmark->recordcount = 
			($previousnumber) + ($referencedata->nextcount - $referencedata->nextoffset) + 1;
			
		$pageview = $this->_controller->pageview();
		$pageview->itemnumber = $bookmark->recordnumber;
		$pageview->itemid = $parameters->articleid;
		if (property_exists($bookmark->subject,'id')) {
			$pageview->containerid = $bookmark->subject->id;
		} else {
			$pageview->containerid = NULL;
		}
		$pageview->itempath = $parameters->path;
		$pageview->itemperpage = $parameters->perpage;
		$pageview->bookmark = $bookmark;
		$pageview->adminmode = $this->_states->adminmode;
		
		return TRUE;
	}
	protected function _get_instance_subject($data) // and ancestors
	{
		if (in_array($data->parameters->path,array('treetops-','ftreetops-','itreetops-','directories-'))) {
			return $this->_get_treetop_subject($data);
		}
		$subject = new StdClass;
		$operation = DB::select_array(array('ID','Title','ObjectType','isTreeTop','DefaultOrder','DefaultDirection'))
			->from('Nodes_View')
			->where('ID','=',$data->containerid)
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$subjectrecord = $operation->current();
		} else return FALSE;
		if (!$subjectrecord) return FALSE;
		$data->secondary_order = $subjectrecord->DefaultOrder;
		$data->secondary_direction = $subjectrecord->DefaultDirection;
		if (empty($data->secondary_order)) { // masks error, but prevents crash
			$data->secondary_order = 'Title';
			$data->secondary_direction = 'ASC';
		}
		$subject->id = $subjectrecord->ID;
		$subject->title = $subjectrecord->Title;
		$subject->istreetop = (bool) $subjectrecord->isTreeTop;
		$subject->bookmark_subject = $data->partial_lookup['masterpartials'][$subjectrecord->ObjectType];
		$subject->subjectname = I18n::get($subjectrecord->ObjectType);
		$subject->bm_flag = TRUE;
		# exploit path data
		$path = $data->parameters->path;
		if (!empty($path)) {
			$path = explode(',',$path);
			list($subjectarticleid,$subjectinstanceid) = explode('-',end($path));
			($subjectarticleid == '') and ($subjectarticleid = NULL);
			# verify correct synchronization of path and subject
			if (empty($subjectarticleid) or ($subject->id == $subjectarticleid)) {
				# set subject path and instanceid
				$subject->instanceid = $subjectinstanceid;
				array_pop($path);
				if (!empty($path))
					$subject->path = implode(',',$path);
				# collect ancestors
				$ancestors = $this->_get_instance_ancestors($path,$data);
				if ($ancestors === FALSE) return FALSE;
				$data->bookmark->ancestors = $ancestors;
			} else {
				return FALSE;
			}
		}
		$data->bookmark->subject = $subject;
		return TRUE;
	}
	protected function _get_treetop_subject($data)
	{
		$subject = new StdClass;
		$ancestor = $data->parameters->path;
		if ($ancestor == 'directories-')
			$subject->subjectname = I18n::get('directories');
		else
			$subject->subjectname = I18n::get('treetops');
		switch ($ancestor) {
			case 'treetops-':
				$ancestorid = 'treetop';
				break;
			case 'ftreetops-':
				$ancestorid = 'ftreetop';
				break;
			case 'itreetops-':
				$ancestorid = 'itreetop';
				break;
			case 'directories-':
				$ancestorid = 'directories';
				break;
		}
		$subject->bookmark_subject = $data->partial_lookup['masterpartials'][$ancestorid];
		$data->bookmark->subject = $subject;
		return TRUE;
	}
	protected function _get_instance_ancestors($path,$data)
	{
		$ancestors = array();
		while ($ancestordata = array_pop($path)) {
			list($ancestorarticleid,$ancestorinstanceid) = explode('-',$ancestordata);
			if (in_array($ancestorarticleid,array('treetops','ftreetops','itreetops','directories'))) {
				switch ($ancestorarticleid) {
					case 'treetops':
						$ancestorid = 'treetop';
						break;
					case 'ftreetops':
						$ancestorid = 'ftreetop';
						break;
					case 'itreetops':
						$ancestorid = 'itreetop';
						break;
					case 'directories':
						$ancestorid = 'directories';
						break;
				}
				$ancestor = new StdClass;
				$ancestor->bookmark_ancestor = $data->partial_lookup['masterpartials'][$ancestorid];
			} else {
				$operation = DB::select_array(array('ID','Title','ObjectType','isTreeTop'))
					->from('Nodes_View')
					->where('ID','=',$ancestorarticleid)
					->as_object()
					->prepare($data->db)
					->execute();
				if ($operation->success()) {
					$ancestorrecord = $operation->current();
				} else {
					return FALSE;
				}
				if (!$ancestorrecord) {
					return FALSE;
				}
				$ancestor = new StdClass;
				$ancestor->id = $ancestorrecord->ID;
				$ancestor->title = $ancestorrecord->Title;
				$ancestor->istreetop = (bool) $ancestorrecord->isTreeTop;
				$ancestor->bookmark_ancestor = $data->partial_lookup['masterpartials'][$ancestorrecord->ObjectType];
				$ancestor->instanceid = $ancestorinstanceid;
				$ancestor->bm_flag = TRUE;
			}
			$ancestor->path = implode(',',$path);
			$ancestors[] = $ancestor;
		}
		$ancestors = array_reverse($ancestors);
		return $ancestors;
	}
	protected function _get_instance_next($data)
	{
		$next = new StdClass;
		$query = DB::select_array($data->commonfields)
			->from($data->tablename);
		$this->_set_query_constraints($query,$data->tablename);
		# forward sort
		$query->order_by(DB::expr('ISNULL(`Order`)'),'ASC');
		$query->order_by('Order','ASC');
		$query->order_by($data->secondary_order,$data->secondary_direction);
		if ($data->secondary_order == 'ArticleDate')
			$query->order_by('Title','ASC'); // tertiary order
		if (!$data->parameters->instanceid) { // it's Node_View
			$objecttype = $data->instancerecord->ObjectType;
			if ($objecttype != 'directory') {
				$query->where('isTreeTop','IS',DB::expr('TRUE'));
				$query->where('ObjectType','=',$data->instancerecord->ObjectType);
				$query->order_by('ID','ASC'); // quaternary
			} else {
				$query->where('ObjectType','=',$data->instancerecord->ObjectType);
				$query->order_by('ID','ASC'); // quaternary
			}
		} else { // it's Nodes_Instances_View
			$query->where('FolderID','=',$data->containerid);
			$query->order_by('NodeID','ASC');
		}
		# try to get close
		if (!is_null($data->instancerecord->Order)) {
			$query->where_open()->where('Order','>=',$data->instancerecord->Order)
				->or_where('Order','IS',DB::expr('NULL'))->where_close();
		} else {
			if ($data->secondary_direction == 'ASC')
				$operator = '>=';
			else
				$operator = '<=';
			if ($data->secondary_order == 'Title') {
				$query->where('Order','IS',DB::expr('NULL'))
					->and_where('Title',$operator,$this->_tablemodel->datarecord->Title);
			} else { // order = ArticleDate
				$query->where('Order','IS',DB::expr('NULL'))
					->and_where('ArticleDate',$operator,$this->_tablemodel->datarecord->ArticleDate);
			}
		}
		$operation = $query
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$nextrecord = $operation->current();
		} else {
			return;
		}
		# find reference record
		$offset = 0;
		while ($nextrecord and ($nextrecord->{$data->comparefield} != $data->comparevalue)) { // anchor - current record
			$nextrecord = $operation->fetch();
			$offset++;
		}
		// record next offset
		# get next record from reference record
		if ($nextrecord) {
			$nextrecord = $operation->fetch(); // next
			$offset++;
		}
		$data->referencedata->nextoffset = $offset;
		# get next count
		$count = 0;
		$query->clear_select();
		$query->select(DB::expr('count(*) AS Count'));
		$operation = $query
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$countrecord = $operation->current();
			$count = $countrecord->Count;
		}
		$data->referencedata->nextcount = $count;
		// record previous count
		if ($nextrecord) {
			$next->id = $nextrecord->ID;
			$next->title = $nextrecord->Title;
			if ($data->parameters->instanceid)
				$next->instanceid = $nextrecord->InstanceID;
			$next->ordertext = $nextrecord->OrderText;
			$next->istreetop = (bool) $nextrecord->isTreeTop;
			$next->path = $data->parameters->path;
			$objecttype = $nextrecord->ObjectType;
			$next->objectname = I18n::get($objecttype);
			switch ($objecttype) {
				case 'entry':
					$next->entryparentcode = Services::dblookup('Articles_Extensions','BlogCode','ArticleID',$nextrecord->ParentID);
					break;
				case 'blog':
					$next->commoncode = $nextrecord->BlogCode;
					break;
			}
			$next->bookmark_next = $data->partial_lookup['masterpartials'][$objecttype];
			$data->bookmark->next = $next;
		}
	}
	protected function _get_instance_previous($data)
	{
		$previous = new StdClass;
		$query = DB::select_array($data->commonfields)
			->from($data->tablename);
		$this->_set_query_constraints($query,$data->tablename);
		# backward sort
		$query->order_by(DB::expr('ISNULL(`Order`)'),'DESC');
		$query->order_by('Order','DESC');
		$direction = ($data->secondary_direction=='ASC')?'DESC':'ASC'; // opposite
		$query->order_by($data->secondary_order,$direction);
		if ($data->secondary_order == 'ArticleDate')
			$query->order_by('Title','DESC');
		if (!$data->parameters->instanceid) { // node_view
			$objecttype = $data->instancerecord->ObjectType;
			if ($objecttype != 'directory') {
				$query->order_by('ID','DESC');
				$query->where('isTreeTop','IS',DB::expr('TRUE'));
				$query->where('ObjectType','=',$data->instancerecord->ObjectType);
			} else {
				$query->order_by('ID','DESC');
				$query->where('ObjectType','=',$data->instancerecord->ObjectType);
			}
		} else { // nodes_view
			$query->order_by('NodeID','DESC');
			$query->where('FolderID','=',$data->containerid);
		}
		# try to get close
		if (!is_null($data->instancerecord->Order)) {
			$query->where_open()->where('Order','<=',$data->instancerecord->Order) // previous has to have an order value
			->where_close();
		} else {
			$query->where_open();
			$query->where_open()
				->where('Order','IS',DB::expr('NULL'));
			if ($data->secondary_direction == 'ASC')
				$operator = '<=';
			else
				$operator = '>=';
			if ($data->secondary_order == 'Title') {
				$query->and_where('Title',$operator,$this->_tablemodel->datarecord->Title);
			} else {
				$query->and_where('ArticleDate',$operator,$this->_tablemodel->datarecord->ArticleDate);
			}
			$query->where_close()
				->or_where('Order','IS NOT',DB::expr('NULL'))
			->where_close();
		}
		$operation = $query
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$previousrecord = $operation->current();
		} else {
//			var_dump($operation->error_info());
			return;
		}
		# find reference record
		$offset = 0;
		while ($previousrecord and ($previousrecord->{$data->comparefield} != $data->comparevalue)) { // anchor = current record
			$previousrecord = $operation->fetch();
			$offset++;
		}
		# get previous record from reference record
		if ($previousrecord) {
			$previousrecord = $operation->fetch(); // previous
			$offset++;
		}
		// record previous offset
		$data->referencedata->previousoffset = $offset;
		# get previous count
		$count = 0;
		$query->clear_select();
		$query->select(DB::expr('count(*) AS Count'));
		$operation = $query
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$countrecord = $operation->current();
			$count = $countrecord->Count;
		}
		$data->referencedata->previouscount = $count;
		if ($previousrecord) {
			$previous->id = $previousrecord->ID;
			$previous->title = $previousrecord->Title;
			if ($data->parameters->instanceid)
				$previous->instanceid = $previousrecord->InstanceID;
			$previous->ordertext = $previousrecord->OrderText;
			$previous->istreetop = (bool) $previousrecord->isTreeTop;
			$previous->path = $data->parameters->path;
			$objecttype = $previousrecord->ObjectType;
			$previous->objectname = I18n::get($objecttype);
			switch ($objecttype) {
				case 'entry':
					$previous->entryparentcode = Services::dblookup('Articles_Extensions','BlogCode','ArticleID',$previousrecord->ParentID);
					break;
				case 'blog':
					$previous->commoncode = $previousrecord->BlogCode;
					break;
			}
			$previous->bookmark_previous = $data->partial_lookup['masterpartials'][$objecttype];
			$data->bookmark->previous = $previous;
		}
	}
	# ---------------------------[ Blogpost Bookmark ]--------------------------------#
	
	protected function _get_blogpost_bookmark(&$bookmark)
	{
		$datarecord = $this->_tablemodel->datarecord;
		if (!$datarecord) return;
		$parameters = $this->_parameters;
		$db = $this->_tablemodel->model->db();
		$bookmark = new stdClass;
		$bookmark->perpage = $parameters->perpage;
//		$bookmark->value = 'hello'; // ** debug
		$dictionary = &$this->_dictionary;
		$dictionary['blogpostbookmark'] = TRUE;
		// get blog data: commoncode, title
		$blog = new StdClass;
		$blogid = $datarecord->ParentID;
		$query = DB::select('BlogCode','Title')
			->from('Nodes_View')
			->where('ID','=',$blogid);
		$operation = $query
			->as_object()
			->prepare($db)
			->execute();
		if (!$operation->success()) return;
		$record = $operation->current();
		if (!$record) return;
		$data = new StdClass;
		$data->db = $db;
		$data->blogcode = $blog->commoncode = $record->BlogCode;
		$blog->title = $record->Title;
		$blog->date = date('d-M-yy',strtotime($datarecord->ArticleDate));
		$bookmark->blog = $blog;
		$data->tablename = 'Nodes_View';
		$data->blogid = $blogid;
		$data->commonfields = array('ID','Title','ArticleDate');
		// get previous data: entryparentcode, id, title
		$data->bookmark = $bookmark;
		$this->_get_blogpost_previous($data);
		$this->_get_blogpost_next($data);
	}
	
	protected function _get_blogpost_previous($data)
	{
		$datarecord = $this->_tablemodel->datarecord;
		$previous = new StdClass;
		$query = DB::select_array($data->commonfields)
			->from($data->tablename);
		$this->_set_query_constraints($query,$data->tablename);
		# backward sort
		$query->order_by('ArticleDate','DESC')
			->order_by('ID','DESC')
			->where('ObjectType','=','entry')
			->where('ParentID','=',$data->blogid)
			->where('ArticleDate','<=',$datarecord->ArticleDate);
		$operation = $query
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$previousrecord = $operation->current();
		} else {
//			var_dump($operation->error_info());
			return;
		}
		# find reference record
		while ($previousrecord and ($previousrecord->ID != $datarecord->ID)) { // anchor = current record
			$previousrecord = $operation->fetch();
		}
		# get previous record from reference record
		if ($previousrecord) {
			$previousrecord = $operation->fetch(); // previous
		}
		if ($previousrecord) {
			$previous->entryparentcode = $data->blogcode;
			$previous->id = $previousrecord->ID;
			$previous->title = $previousrecord->Title;
			$previous->date = date('d-M-yy',strtotime($previousrecord->ArticleDate));
			$data->bookmark->previous = $previous;
		}
	}
	
	protected function _get_blogpost_next($data)
	{
		$datarecord = $this->_tablemodel->datarecord;
		$next = new StdClass;
		$query = DB::select_array($data->commonfields)
			->from($data->tablename);
		$this->_set_query_constraints($query,$data->tablename);
		# backward sort
		$query->order_by('ArticleDate','ASC')
			->order_by('ID','ASC')
			->where('ObjectType','=','entry')
			->where('ParentID','=',$data->blogid)
			->where('ArticleDate','>=',$datarecord->ArticleDate);
		$operation = $query
			->as_object()
			->prepare($data->db)
			->execute();
		if ($operation->success()) {
			$nextrecord = $operation->current();
		} else {
//			var_dump($operation->error_info());
			return;
		}
		# find reference record
		while ($nextrecord and ($nextrecord->ID != $datarecord->ID)) { // anchor = current record
			$nextrecord = $operation->fetch();
		}
		# get previous record from reference record
		if ($nextrecord) {
			$nextrecord = $operation->fetch(); // previous
		}
		if ($nextrecord) {
			$next->entryparentcode = $data->blogcode;
			$next->id = $nextrecord->ID;
			$next->title = $nextrecord->Title;
			$next->date = date('d-M-yy',strtotime($nextrecord->ArticleDate));
			$data->bookmark->next = $next;
		}
	}
	
	# ---------------------------[ Start Reading ]--------------------------------#
	protected function _record_start($datamodel)
	{
		$datarecord = $this->_tablemodel->datarecord;
		if (!$datarecord) return Node::FLOW_CONTINUE;
		$objecttype = $datarecord->ObjectType;
		switch ($objecttype) {
			case 'inbox':
			case 'file':
			case 'directory':
			case 'folder':
				$this->_start_folder($datamodel);
				break;
		}
		return Node::FLOW_CONTINUE;
	}
	// ** this could live in the folder subtype code 
	protected function _start_folder($datamodel)
	{
		$tablemodel = $this->_tablemodel;
		$datarecord = $tablemodel->datarecord;
		$secondary_order = $datarecord->DefaultOrder;
		$secondary_direction = $datarecord->DefaultDirection;
		if (empty($secondary_order)) { // mask error to prevent crash
			$secondary_order = 'Title';
			$secondary_direction = 'ASC';
		}
		$folderid = $datarecord->ID;
		$db = $tablemodel->model->db();
		# get first record
		$tablename = 'Nodes_Instances_View';
		$query = DB::select_array(array('ID','Title','InstanceID','ObjectType','ParentID','BlogCode','OrderText','ParentID'))
			->from($tablename);
		$this->_set_query_constraints($query,$tablename);
		$query->order_by(DB::expr('ISNULL(`Order`)'),'ASC');
		$query->order_by('Order','ASC');
		$query->order_by($secondary_order,$secondary_direction);
		if ($secondary_order == 'ArticleDate') {
			$query->order_by('Title','ASC');
		}
		$query->order_by('NodeID','ASC');
		$query->where('FolderID','=',$folderid);
		$operation = $query
			->offset(0)
			->limit(1)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$firstrecord = $operation->current();
		} else return;
		if (!$firstrecord) return;
		$objecttype = $firstrecord->ObjectType;
		$instanceid = $this->_parameters->instanceid = $firstrecord->InstanceID;
		$articleid = $firstrecord->ID;
		$parms = $this->_get_redirect_parameters();
		switch($objecttype) {
			case 'document':
				$redirect = 'documents';
				break;
			case 'folder':
				$redirect = 'topics';
				break;
			case 'file':
				$redirect = 'library/files';
				break;
			case 'directory':
				$redirect = 'library/directories';
				break;
			case 'inbox':
				$redirect = 'library/inboxes';
				break;
			case 'entry':
				$redirect = 'blogs/';
				$parentcode = Services::dblookup('Articles_Extensions','BlogCode','ArticleID',$firstrecord->ParentID,$db);
				if (!$parentcode) return;
				$redirect .= $parentcode;
				break;
			case 'link':
				$redirect = 'library/links';
				break;
			case 'widget':
				$redirect = 'library/widgets';
				break;
			case 'docfile':
				$redirect = 'library/docfiles';
				break;
			case 'docextract':
				$redirect = 'library/docextracts';
				break;
			case 'picture':
				$redirect = 'library/pictures';
				break;
			default: return;
		}
		$redirect .= "/$articleid$parms";
		$this->_controller->request->redirect($redirect);
	}
}