<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Muster;

use Kohana;

defined('SYSPATH') or die('No direct script access.');

class Point {
	#======[ factory selectors ]======#
	// datapoint metadata
	const META = 'Meta';
	// datasets: 
	const FRAME = 'Frame';
	const BLOCK = 'Block';
	const BLOCKLIST = 'Blocklist';
	const BLOCKSELECT = 'Blockselect';
	const LISTITEM = 'Listitem'; // ** not implemented
	// frame types
	const DATAFRAME = 'Dataframe';
	const CONTROLFRAME = 'Controlframe';
	const CONTAINERFRAME = 'Containerframe'; // ** not implemented
	// datapoint implementation
	const DATA = 'Data'; 
	const CONTROL = 'Control';
	const PARAMETER = 'Parameter'; // should be implemented
	const CONTAINER = 'Container';
	#======[ data sources ]======#
	// datapool sources for value
	// if caching is on, the records found in cache->*
	const DATASOURCE_ARRAY = 1; // datapool->arrays[]
	const DATASOURCE_VARIABLE = 2; // datapool->variables[]
	const DATASOURCE_RECORD = 3; // datapool->records[]
	const DATASOURCE_RECORD_JSON = 4; // complex field value in json array structure
	const DATASOURCE_ACTIVERECORD_JSON = 5; // complex field value in json array structure
	const DATASOURCE_RECORDSET = 6; // for table edit processing
	// other sources for value
	const DATASOURCE_DEFAULT = 7; // read the default for the source
	const DATASOURCE_FUNCTION = 8; // custom callbacks, reserved
	const DATASOURCE_MATCH = 9; // specified comparison yields value
	const DATASOURCE_LOOKUP_TABLE = 10; // gets entire table for select or button field
	const DATASOURCE_SELF = 11; // from 'source' property
	const DATASOURCE_VALUE = 12; // from 'value' property
	const DATASOURCE_OBJECT = 13;
	const DATASOURCE_NOT_PERMITTED = 14;
	const DATASOURCE_PERMITTED = 15;
	const DATASOURCE_INPUT = 16; // from object's 'input' instructions
	const DATASOURCE_BLOCKVALUE = 17;
	const DATASOURCE_CONDITIONAL = 18;
	const DATASOURCE_ACTIVERECORD = 19; // retrieve, so far (June '11)
	const DATASOURCE_TABLEACTIVERECORD = 20;
	const DATASOURCE_ARRAYS = 21;
	const DATASOURCE_LOOKUP_PARTIAL = 22;
	const DATASOURCE_LOOKUP_LAYOUT_PARTIAL = 23;
	const DATASOURCE_ISFAVOURITE = 24;
	const DATASOURCE_ISCOMMENTS = 25;
	const DATASOURCE_ISATTACHMENTS = 26;
	const DATASOURCE_ISNARRATIVE = 27;
	const DATASOURCE_ISANCILLARYDATA = 28;
	const DATASOURCE_CONTENTCOUNT = 29;
	const DATASOURCE_LOOKUP_LAYOUT_IMAGEDATA = 30;
	const DATASOURCE_LOOKUP_RECORD_IMAGEDATA = 31;
	const DATASOURCE_CALCULATE_RECORDSET_DELIMITER = 32;
	#======[ data targets ]======#
	const DATATARGET_RECORD = 1;
	const DATATARGET_RECORD_JSON = 2;
	const DATATARGET_RECORDSET = 3;
	#======[ datatypes ]======#
	const DATATYPE_STRING = 1;
	const DATATYPE_INTEGER = 2;
	const DATATYPE_FLOAT = 3;
	const DATATYPE_DATE = 4;
	const DATATYPE_DATETIME = 5;
	const DATATYPE_BOOLEAN = 6;
	const DATATYPE_OBJECT = 7;
	const DATATYPE_ARRAY = 8;
	const DATATYPE_JSON = 9;
	const DATATYPE_MIXED = 10; // leave it alone
	#======[ filters ]======#
	// filter names have following 'arguments' property if required
	const FILTER_CHECKED = 1;
	const FILTER_SELECTED = 2;
	const FILTER_LOOKUP = 3; // table lookup
	const FILTER_COMPARE = 4;
	const FILTER_DISABLED = 5;
	const FILTER_HASHPASSWORD = 6;
	const FILTER_MARKUP = 7;
	const FILTER_PURIFYTRUSTED = 8;
	#======[ value error ]======#
	// for validate
	const ERRORSTATE_NOERROR = 0;
	const ERRORSTATE_ERROR = 1;
	const RECOVERSTATE_NORECOVER = 0;
	const RECOVERSTATE_RECOVER = 1;
	const RECOVERSTATE_RECYCLE = 1; // alias of recover
	const INPUT_RECYCLE = 1; // alias of recover
	#==============================
	// general factory function
	public static function factory($point_type)
	{
		$classname = 'Muster\\Point\\' . $point_type;
		return new $classname;
	}
	#frame types. The type controls the processing branch for gateway
	public static function dataframe() // data frame
	{
		return self::factory(self::FRAME)->type(self::DATAFRAME);
	}
	// alias for dataframe()
	public static function dictionary() // metadata for any point
	{
		return self::dataframe();
	}
	public static function commandframe() // command frame
	{
		return self::factory(self::FRAME)->type(self::CONTROLFRAME);
	}
	// alias for commandframe
	public static function commandfilter() // metadata for any point
	{
		return self::commandframe();
	}
	public static function containerframe() // container frame ** not implemented
	{
		return self::factory(self::FRAME)->type(self::CONTAINERFRAME);
	}
	# core point types:
	public static function meta($metadata) // metadata for any point
	{
		return self::factory(self::META)->metadata($metadata);
	}
	// alias for meta()
	public static function variable($metadata) // metadata for any point
	{
		return self::meta($metadata);
	}
	// alias for meta()
	public static function command($metadata)
	{
		return self::meta($metadata);
	}
	// alias for meta()
	public static function parameter($metadata) // metadata for any point
	{
		return self::meta($metadata);
	}
	public static function data($value = NULL,$metapoint = NULL) // general data point
	{
		$data = self::factory(self::DATA);
		if (!is_null($value)) $data->value($value);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		return $data;
	}
	public static function get_command($value = NULL,$metapoint = NULL) // command point
	{
		$data = self::factory(self::CONTROL);
		$data->value($value);
		if (!is_null($metapoint)) {
			$data->metapoint = $metapoint;
			if (property_exists($metapoint,'parameters'))
				$data->parameters = $metapoint->parameters;
		}
		return $data;
	}
	public static function get_parameter($value = NULL,$metapoint = NULL) // command point
	{
		$data = self::factory(self::PARAMETER);
		$data->value($value);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		return $data;
	}
	public static function container($value = NULL,$metapoint = NULL) // container point
	{
		$data = self::factory(self::CONTAINER);
		if (!is_null($value)) $data->value($value);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		return $data;
	}
	# data frame block types
	// the atomic grouping structure
	public static function block($metapoint = NULL)
	{
		$data = self::factory(self::BLOCK);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		return $data;
	}
	// alias for block()
	public static function section($metapoint = NULL)
	{
		return self::block($metapoint);
	}
	// lists of structures
	public static function blocklist($metapoint = NULL)
	{
		$data = self::factory(self::BLOCKLIST);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		return $data;
	}
	// alias for blocklist()
	public static function sectionlist($metapoint = NULL)
	{
		return self::blocklist($metapoint);
	}
	public static function listitem($metapoint = NULL,$pointlist = NULL)
	{
		$data = self::factory(self::LISTITEM);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		if (!is_null($pointlist)) $data->pointlist = $pointlist;
		return $data;
	}
	// special case: the select (dropdown/radio button) datapoint and support groupings
	public static function blockselect($metapoint = NULL,$value = NULL)
	{
		$data = self::factory(self::BLOCKSELECT);
		if (!is_null($metapoint)) $data->metapoint = $metapoint;
		if (!is_null($value)) $data->value = $value;
		return $data;
	}
	// alias for blockselect()
	public static function selectoptions($metapoint = NULL,$value = NULL)
	{
		return self::blockselect($metapoint,$value);
	}
	// alias for blockselect()
	public static function optionsection($metapoint = NULL,$value = NULL)
	{
		return self::blockselect($metapoint,$value);
	}
	# property accessors //** Generator::get_dictionary is a duplicate of this
	public static function get_frame($filename)
	{
		$file = Kohana::find_file('frames',$filename);
		if ($file)
			return Kohana::load($file);
		else
			return NULL;
	}
}