<?php 

namespace Muster;

use DB, Database, Kohana;

defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
/**
 * services for classes in the bechmann module
 *
 * @package	bechmann
 * @author		Henrik Bechmann
 * @copyright	Henrik Bechmann, 2008, 2009 Toronto, Canada
 * @version 	July 2009
 * @license		BSD
 */
class Services {

	protected static $_tokensets;
	protected static $_objecttypetokensetmap;
	/**
	* verify that the listed properties of an object exist
	*
	* @param	object
	* @param	list array of property names
	* @return	TRUE if all properties exist
	* @return	list array of missing properties if some properties do not exist
	*/
	public static function verify_properties($object,$propertylist) 
	{
		$missing = array();
		foreach ($propertylist as $property) 
		{
			if (!property_exists($object,$property))
			{
				$missing[] = $property;
			}
		}
		return $missing
			? $missing
			: TRUE;
	}

	/**
	* assert properties for an object, with default values, for those properties that do not exist
	*
	* @param	object
	* @param	associative array of properties to assert: [propertyname] => value
	* @return	associative array of properties to assert, with current values
	*/
	public static function assert_properties($object,$properties) 
	{
		if (! is_object($object)) 
		{
			$object = (object) $object;
		}
		foreach ($properties as $property => $default) 
		{
		if (!property_exists($object,$property))
			{
				$object->$property=$default;
			}
			else
			{
				$properties[$property]=$object->$property;
			}
		}
		return $properties;
	}
	
	public static function debug($variable,$message = NULL,$die = FALSE) 
	{
		echo '<pre>' . $message;
		print_r($variable);
		echo '</pre>';
		if ($die) die();
	}
	public static function dbrecordlookup($table,array $lookupfieldnames,$keyname,$keyvalue,$database = NULL)
	{
		$operation = DB::select_array($lookupfieldnames)
			->from($table)
			->where($keyname,'=',$keyvalue)
			->as_object()
			->prepare($database)
			->execute();
		if (!$operation->success())
			die(var_dump($operation->error_info()));
		$record = $operation->fetch();
		if ($record) 
			return $record;
		else
			return NULL;
	}
	public static function dblookup($table,$lookupfieldname,$keyname,$keyvalue,$database = NULL)
	{
		$operation = DB::select($lookupfieldname)
			->from($table)
			->where($keyname,'=',$keyvalue)
			->as_object()
			->prepare($database)
			->execute();
		if (!$operation->success())
			die(var_dump($operation->error_info()));
		$record = $operation->fetch();
		if ($record) 
			return $record->$lookupfieldname;
		else
			return NULL;
	}
	// find alternatekey from ID (if $lookup is_int) or vice versa.
	public static function lookup($table,$lookup,$alternatekeyname = 'Code',$primarykeyname = 'ID',$db = NULL)
	{
		$db = Database::instance();
		$findalternatekey = is_numeric($lookup);
		if ($findalternatekey)
		{
			$retval = @DB::
				select($alternatekeyname)
				->from($table)
				->where($primarykeyname,"=",$lookup)
				->as_object()
				->execute($db)
				->current()
				->$alternatekeyname;
		}
		else
		{
			$retval = @DB::
				select($primarykeyname)
				->from($table)
				->where($alternatekeyname,"=",$lookup)
				->as_object()
				->execute($db)
				->current()
				->$primarykeyname;
		}
		return $retval;
	}
	
	public static function get_sql_datetime($phpdate)
	{
		return date( 'Y-m-d H:i:s', $phpdate );
	}
	
	public static function get_php_datetime($mysqldate)
	{
		return strtotime($mysqldate);	
	}
	public static function strdate_to_mysql($strdate)
	{
		if (!is_null($strdate))
		{
			$phpdate = Services::get_php_datetime($strdate);
			$strdate = Services::get_sql_datetime($phpdate);//date('Y-m-d',$phpdate);
		}
		return $strdate;
	}
	
	public static function trimnotnull($string)
	{
		if (is_null($string)) return $string;
		$trim = trim($string);
		if ($trim == '') $trim = NULL;
		return $trim;
	}

	public static function validate_identifier_string($value)
	{
		$retval = TRUE;
		if (!is_null($value))
		{
			{
				$valid_code = (bool) preg_match('/^[A-Za-z][\w\-]*$/',$value);
				if (!$valid_code) $retval = FALSE;
			}
		}
		return $retval;
	}
	public static function validate_htmlid_string($value)
	{
		$retval = TRUE;
		if (!is_null($value))
		{
			{
				$valid_code = (bool) preg_match('/^[A-Za-z][\w\-:_.]*$/',$value);
				if (!$valid_code) $retval = FALSE;
			}
		}
		return $retval;
	}
	public static function validate_tag_code($value)
	{
		$retval = TRUE;
		if (!is_null($value))
		{
			{
				$valid_code = (bool) preg_match('/^[A-Za-z][\w]*$/',$value);
				if (!$valid_code) $retval = FALSE;
			}
		}
		return $retval;
	}
	public static function validate_picture_id($value) {
		$objecttype = self::dblookup('Articles','ObjectType','ID',$value);
		$retval = ($objecttype and ($objecttype == 'picture'));
		return $retval;
	}
	public static function get_object_property($object,$index,$default=NULL)
	{
		if (property_exists($object,$index)) return $object->$index;
		return $default;
	}
	public static function get_array_value($array,$index,$default=NULL)
	{
		if (!isset($array[$index]))
			return $default;
		else
			return $array[$index];
	}
	public static function get_value($var,$default=NULL)
	{
		if ((!isset($var)) or is_null($var))
			return $default;
		else
			return $var;
	}
	public static function make_query_string($queryparts)
	{
		if (empty($queryparts)) return '';
		$arr = array();
		foreach ($queryparts as $key => $value)
		{
			$arr[] = $key.'='.$value;
		}
		return '?' . implode('&',$arr);
	}
	public static function add_class($class,$newclass)
	{
		$classes = explode(" ",$class);
		if (!in_array($newclass,$classes)) $classes[] = $newclass;
		$class = implode(" ",$classes);
		return $class;
	}
	public static function message($file,$path,$substitutions = NULL,$default = NULL)
	{
		$message = Kohana::message($file,$path);
		if ((!$message) and $default)
			$message = $default;
		if ($message and $substitutions) {
			if (is_array($substitutions))
				$message = strtr($message,$substitutions);
		}
		return $message;
	}
	public static function validate_filetype($filename, $allowed)
	{
		if (empty($filename)) return FALSE;
		$info = pathinfo($filename);
		if (empty($info['extension'])) return FALSE;
		$ext = strtolower($info['extension']);
		$name = $info['filename'];
		if (empty($name)) return FALSE;
		return in_array($ext, $allowed);
	}
	public static function get_token_set($setname)
	{
		if (empty(self::$_tokensets))
			self::$_tokensets = Kohana::$config->load('tokensets')->getArrayCopy();
		$tokensets = self::$_tokensets;
		$tokens = $tokensets[$setname];
		return (object) $tokens;
	}
	public static function get_objecttype_tokenset($objecttype)
	{
		if (empty(self::$_objecttypetokensetmap))
			self::$_objecttypetokensetmap = Kohana::$config->load('objecttypetokensetmap')->getArrayCopy();
		$objecttypetokensetmap = self::$_objecttypetokensetmap;
		if (empty($objecttypetokensetmap[$objecttype]))
			return NULL;
		$setname = $objecttypetokensetmap[$objecttype];
		return self::get_token_set($setname);
	}
	// alias of check...
	public static function get_article_permission($articleid,$requires = 'E',$token = 'content') {
		return self::check_article_permission($articleid,$requires,$token);
	}
	// get article permission including for recordowner and record collaborator (default requirement = 'E')
	public static function check_article_permission($articleid,$requires = 'E',$token = 'content')
	{
		$record = self::dbrecordlookup('Articles',array('AuthorID','ObjectType'),'ID',$articleid);
		if (!$record) return FALSE;
		$ownerid = $record->AuthorID;
		$objecttype = $record->ObjectType;
		if (is_numeric($ownerid) and (!empty($objecttype)) and is_numeric($articleid)) {
			$tokenset = self::get_objecttype_tokenset($objecttype);
			$access = Access::instance();
			$result = $access->permission($tokenset->$token,$requires,$ownerid,$articleid);
			return $result;
		} else {
			return FALSE;
		}
	}
} // End Services
