<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Muster\Tablemodel;

use Tablemodel;

defined('SYSPATH') or die('No direct script access.');
/*
 file: abstractindex.php
 package: Tablemodel
 category: ORM (Object Relational Manager)
 date: August 2010
* abstract class for Foreignkey and Recordindex
 update_observer must be set by subclass
*/
abstract class AbstractIndex implements Observer, Observed
{
	protected $_indexdef;
	protected $_column_values; // associative array of values initialized on creation
	//references
	protected $_observers = array();
	
	public function __construct($parameters)
	{
		$this->_indexdef = $parameters->indexdef;
		foreach ($this->_indexdef->column_names() as $columnname)
		{
			$this->_column_values[$columnname] = NULL;
		}
	}
	public function destroy() // allow garbage collection
	{
		$this->_observers = NULL;
		$this->_indexdef = NULL;
	}
	// observer methods...
	public function attach_observer(Observer $observer) // TableModelObserved method
	{
		if (! in_array($observer, $this->_observers))
		{
			$this->_observers[] = $observer;
		}
	}
	public function detach_observer(Observer $observer) // TableModelObserved method
	{
		$pos = array_search($observer,$this->_observers);
		if (!($pos === FALSE))
		{
			array_splice($this->_observers,$pos,1);
		}
	}
	public function notify_observers() // TableModelObserved method; notify CursorRelations
	{
		foreach ($this->_observers as $observer)
		{
			$observer->update_observer($this);
		}
	}
	public function observer_name() // Tablemodel_Observer method
	{
		return $this->_indexdef->name();
	}
	public function debug_observer_list() // Tablemodel_Observed method
	{
		return Tablemodel::debug_observer_list($this->_observers);
	}
	// property accessors...
	public function column_values()
	{
		return $this->_column_values;
	}
	public function set_column_value($column_name,$column_value)
	{
		$this->_column_values[$column_name] = $column_value;
	}
	public function indexdef()
	{
		return $this->_indexdef;
	}
}
