<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Muster\Tablemodel;

use Kohana, Kohana_Exception;

defined('SYSPATH') or die('No direct script access.');
/**
 file: indexdef.php
 package: Tablemodel
 category: ORM (Object Relational Manager)
 date: August 2010
* class: TableIndexdef
* table index definition
* contains back link to table
*/
class Indexdef
{
	protected $_tabledef; // the table object BACK LINK
	protected $_name;
	protected $_index_type; //TableModel::UNIQUE | TableModel::INDEX
	protected $_column_names; // list array of index column names
	
	public function __construct($parameters) 
	{
		// set properties
		$this->_tabledef = $parameters->tabledef;
		$this->_name = $parameters->name;
		$this->_index_type = $parameters->index_type;

		$this->_validate_columns($parameters);
		// set index column properties
		$this->_column_names = $parameters->columns;
	}
	
	private function _validate_columns($parameters)
	{
		$debug_level = $this->_tabledef->model()->debug_level();
		if ($debug_level > 0) 
		{
			// check for existence of index columns in table
			$columnspecs = $this->_tabledef->columnspecs();
			foreach ($parameters->columns as $column_name) 
			{
				if (! isset($columnspecs[$column_name]))
				{
					throw new Kohana_Exception
						(
							Kohana::message('tablemodel','indexdef.misnamed_resource'),
								array(':column'=>$column_name,
								':index'=>$this->_name,
								':table'=>$this->_tabledef->tablealias())
						);
				}
			}
		}
	}
	// property accessors...
	public function tabledef()
	{
		return $this->_tabledef;
	}
	public function name()
	{
		return $this->_name;
	}
	public function index_type()
	{
		return $this->_index_type;
	}
	public function column_names()
	{
		return $this->_column_names;
	}
}
