<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Muster\Tablemodel;

defined('SYSPATH') or die('No direct script access.');
/**
 file: recordset.php
 package: Tablemodel
 category: ORM (Object Relational Manager)
 date: August 2010
 
 ** This is incomplete ** HB
 
* class: Recordset
  mandate: Act as staging area of batch import and export of table records, through Activerecord
  todo: add recurse($path = NULL)

  fetch($keyfield = NULL,$valuefield = NULL) // clears, calls as_array of table, returns the array
  export($keyfield = NULL, $valuefield = NULL,$clear = FALSE) // returns array
  import($recordset) // first clears cache, builder
  post()
  delete()
  clear() // builder method - clears existing internal array
  get_distinct_for($tablename);
  table()
  where_conditions($where_conditions_array)
  validation($validation)
*/
class Recordset
{
	protected $_data;
	protected $_db;
	protected $_table;
	protected $_target_tablename;
	protected $_validation;
	protected $_where_conditions;
	
	public function __construct($parameters)
	{
		$this->_db = $parameters->db;
		$this->_table = $parameters->table;
	}
	
	// get distinct records of the current recordset table (leaf) 
	// 	for all descendent records of the parameter table record (root)
	public function get_distinct_for($root_tablename)
	{
		$found = FALSE;
		// get a list of ancestor tables including the requested parent_tablename
		$root_candidate = $this->_table;
		$table_queue = array($root_candidate);
		while ($root_candidate = $root_candidate->parent()) // guarantees at least two tables in list
		{
			$table_queue[] = $root_candidate;
			if ($found = ($root_candidate->name() == $root_tablename))
				break;
			$root_candidate->requery();
		}
		if (!$found)
			return FALSE;
		$table_queue = array_reverse($table_queue); // ascending list from root table = 0
		$this->_unique_data = array(); // prepare receptacle for unique rows
		$this->_unique_target = array_pop($table_queue); // the leaf table for unique records
		$activerecord = array_shift($table_queue)->activerecord(); // leave the activerecord from the root table
		
		$this->collect_records($table_queue); // iterated through activerecords for children to get list of leaf
		
		$array = array_values($this->_unique_data); // lose the index values that were used to combine duplicates
		$this->_unique_target = NULL; // reset
		$this->_unique_data = NULL; // reset
		return $array;
	}
	// for get_distinct_for
	protected function collect_records($queue)
	{
		if (!empty($queue))
		{
			$activerecord = array_shift($queue)->activerecord();
			while ($activerecord->fetch())
				$this->collect_records($queue);
		}
		else
		{
			$keyfield = $this->_unique_target->tabledef()->primary_key_column_name();
			$fetches = $this->_unique_target->recordset()->fetch($keyfield);
			foreach ($fetches as $key => $fetch)
				$this->_unique_data[$key] = $fetch;
		}
	}
	
	public function clear()
	{
		$this->_data = NULL;
		return $this;
	}
	
	public function destroy()
	{
		$this->_db = NULL;
		$this->_table = NULL;
		$this->_data = NULL;
	}
	
	public function load($keyfield = NULL,$valuefield = NULL)
	{
		$this->_data = $this->_table->as_array($keyfield,$valuefield);
		return $this;
	}

	public function fetch($keyfield = NULL,$valuefield = NULL)
	{
		return $this->load()->export($keyfield,$valuefield);
	}
	
	public function export($keyfield = NULL, $valuefield = NULL, $clear = FALSE) // returns array
	{
		$rows = $this->_data; // array of objects
		if (!is_array($rows)) return $rows;
		$results = array();

		if ($keyfield === NULL AND $valuefield === NULL) // returns indexed array of rows
		{
			$results = $rows;
		}
		elseif ($keyfield === NULL) // $valuefield = column name used for value of indexed array
		// returns indexed array of single values
		{
			foreach ($rows as $row)
			{
				$results[] = $row->$valuefield;
			}
		}
		elseif ($valuefield === NULL) // $keyfield = column name used for associative index
		// returns dictionary of rows
		{
			foreach ($rows as $row)
			{
				$results[$row->$keyfield] = $row;
			}
		}
		else // $keyfield used as column name for associative key, $valuefield used for record value
		// returns dictionary of single values
		{
			foreach ($rows as $row)
			{
				$results[$row->$keyfield] = $row->$valuefield;
			}
		}
		if ($clear) $this->_data = NULL;
		return $results;
	}
	public function import($recordset)
	{
		$this->_data = $recordset;
		return $this;
	}
	public function validation($validation)
	{
		$this->_validation = $validation;
		return $this;
	}
	
	public function where_conditions($where_conditions_array)
	{
		$this->_where_conditions = $where_conditions_array;
		return $this;
	}
	
	public function table()
	{
		return $this->_table;
	}
  
	# datarecords is an array of object records; key is used to cross reference errors
	# returns 2-item array of cross referenced postedrecords and error data
	public function post($datarecords = NULL)
	{
		$datarecords or ($datarecords = $this->_data);
		$table = $this->table();
		$keycolumnname = $table->tabledef()->primary_key_column_name();
		$activerecord = $table->activerecord();
		$errordata = array();
		$postedrecords = array();
		$whereconditions = $this->_where_conditions;
		// set aside table->query
		$currentquery = clone $table->query();
		// if $validation, set aside current activerecord validation
		if ($this->_validation)
		{
			$currentvalidation = $table->activerecord()->validation();
			$table->activerecord()->validation($this->_validation);
		}
		$query = $table->query();
		foreach ($datarecords as $key => $datarecord)
		{
			// clear table and query
			$table->reset()->query()->reset();
			if ($whereconditions)
			{
				foreach($whereconditions as $condition)
				{
					list($col,$op,$val) = $condition;
					$query->where($col,$op,$val);
				}
			}
			$id = NULL;
			property_exists($datarecord,$keycolumnname) and ($id = $datarecord->$keycolumnname);
			if (!is_null($id)) // update
			{
				$table->query()->where($keycolumnname,'=',$id);
				$record = $activerecord->current();
				if ($record)
				{
					foreach ($datarecord as $name => $value)
					{
						if ($activerecord->$name !== $value)
							$activerecord->$name = $value;
					}
					if (!$activerecord->save())
					{
						$errordata[$key] = $activerecord->error_info();
						$postedrecords[$key] = NULL;
					}
					else
					{
						$postedrecords[$key] = $activerecord->current();
					}
				}
				else
				{
					$errordata[$key] = array('posting_error',0,'record not found for update');
					$postedrecords[$key] = NULL;
				}
			}
			else // insert
			{
				$activerecord->insert();
				foreach ($datarecord as $name => $value)
				{
					$activerecord->$name = $value;
				}
				if (!$activerecord->save())
				{
					$errordata[$key] = $activerecord->error_info();
					$postedrecords[$key] = NULL;
				}
				else
				{
					$record = $this->activerecord()->current();
					if ($record)
						$postedrecords[$key] = $record;
					else
						$postedrecords[$key] = NULL;
				}
			}
		}
		// restore table query and activerecord->validation
		$table->set_query($currentquery);
		$this->_validation and $table->activerecord()->validation($currentvalidation);
		// reset table
		$table->reset();
		return array($errordata,$postedrecords);
	}
	# returns 2-item array of cross referenced postedrecords and error data
	public function delete($datarecords = NULL)
	{
		$datarecords or ($datarecords = $this->_data);
		$table = $this->table();
		$keycolumnname = $table->tabledef()->primary_key_column_name();
		$activerecord = $table->activerecord();
		$errordata = array();
		$deletedrecords = array();
		// set aside table->query
		$currentquery = clone $table->query();
		foreach ($datarecords as $key => $record)
		{
			$id = NULL;
			property_exists($datarecord,$keycolumnname) and ($id = $datarecord->$keycolumnname);
			if (!is_null($id))
			{
				$table->query()->reset()->where($keycolumnname,'=',$id);
				if ($activerecord->current()) 
				{
					if ($activerecord->delete())
					{
						$deletedrecords[$key] = $record;
					}
					else
					{
						$errordata[$key] = $activerecord->error_info();
					}
				}
				else
				{
					$errordata[$key] = 'record not found for delete';
				}
			}
			else
			{
				$deletedrecords[$key] = $record; // new records cannot be deleted but are legitimate
			}
		}
		$table->set_query($currentquery);
		// reset table
		$table->reset();
		return array($errordata,$deletedrecords);
	}
}
