<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Muster\Tablemodel;

use Tablemodel;

defined('SYSPATH') or die('No direct script access.');

/**
 file: relation.php
 package: Tablemodel
 category: ORM (Object Relational Manager)
 date: August 2010
* class: Recordrelation
* observes local Recordindex
* observed by Foreignkey ($singleton)
* instance of relationdef
contains foreignkey and recordindex components, and the ability to map values from recordindex key change to foreign key values
API:
*/
class Relation implements Observer, Observed
{
	protected $_relationdef;
	//references
	protected $_observers = array(); // Foreignkey
	protected $_recordindex; // local table
	protected $_foreignkey; // remote
	protected $_column_map; // local to remote
	
	public function __construct($parameters)
	{
		// assign properties
		$this->_relationdef = $parameters->relationdef;
		$this->_recordindex = $parameters->recordindex; // from local table
		$this->_foreignkey = $parameters->foreignkey; // from foreign table
		// make a dictionary: record index names used for dictionary keys, index key names are dictionary values
		$this->_column_map = array_combine( 
			$this->_recordindex->indexdef()->column_names(),
			$this->_foreignkey->indexdef()->column_names());
		// cross-link relation observer and observed
		$this->_recordindex->attach_observer($this); // Recordrelation observes Recordindex
		$this->attach_observer($this->_foreignkey); // Foreignkey observes Recordrelation
	}
	public function destroy()
	{
		$this->_relationdef = NULL;
		$this->_observers = NULL;
		$this->_recordindex = NULL;
		$this->_foreignkey = NULL;
	}
	// observer methods...
	// update Recordrelation
	public function update_observer(Observed $recordindex) // Observer method
	{
		$remote_column_values = $this->_foreignkey->column_values();
		$column_values = $this->_recordindex->column_values();
		$column_map = $this->_column_map;
		$change = FALSE;
		foreach ($column_map as $source_column => $target_column)
		{
			if (($remote_column_values[$target_column] !== $column_values[$source_column]) or 
					(is_null($remote_column_values[$target_column]) and is_null($column_values[$source_column]))) // for unique indexes that allow NULL
			{
				$this->_foreignkey
					->set_column_value($target_column,$column_values[$source_column]);
				$change = TRUE;
			}
		}
		if ($change) $this->notify_observers();
	}
	public function attach_observer(Observer $foreignkey) // Observed method
	{
		if (! in_array($foreignkey, $this->_observers))
		{
			$this->_observers[] = $foreignkey;
		}
	}
	public function detach_observer(Observer $foreignkey) // Observed method
	{
		$pos = array_search($foreignkey,$this->_observers);
		if (!($pos === FALSE))
		{
			array_splice($this->_observers,$pos,1);
		}
	}
	public function notify_observers() // Observed method; notify tables
	{
		foreach ($this->_observers as $observer)
		{
			$observer->update_observer($this);
		}
	}
	public function debug_observer_list() // Tablemodel_Observed method
	{
		return Tablemodel::debug_observer_list($this->_observers);
	}
	public function observer_name() // Observer method
	{
		return $this->_relationdef->name();
	}
	// property accessor
	public function foreignkey()
	{
		return $this->_foreignkey;
	}
	public function recordindex()
	{
		return $this->_recordindex;
	}
	public function relationdef()
	{
		return $this->_relationdef;
	}
}
