<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Muster\User;

use StdClass, Exception;
use Kohana, Session, URL, Text, DB;
use TableModel, Services, System, Template, JsonField;
use User;

defined('SYSPATH') or die('No direct script access.');
/*
- generate email for register
- generate email for confirm
- generate email for generate_password
- generate email for update profile
- generate email for remove_user

- generalize email for user interactions

- test delete_expired_pending for multiple expired pending records

API:
	register($data) // sets $this->_insert_id
	subscribe($data, $email = FALSE) // sets $this->_insert_id
	confirm($confirmationtoken, $email = FALSE) 
	update_profile($data)
	generate_password($usercode, $emailaddress, $password = FALSE, $email = FALSE)
	new_password()
	remove_user($username, $password = FALSE) // sets $this->_insert_id
	** process_membership($usercode,$decision);
*/
class Account {

	public static $instance; // last instance created

	const REGISTER_MISSING_DATA = 1;
	const REGISTER_INCOMPLETE_DATA = 2;
	const REGISTER_SUCCESS_DB = 0; // error_info captured
	const REGISTER_FAILURE_DB = 3; // error_info captured
	const REGISTER_DUPLICATE_DB = 4; // error info captured
	const REGISTER_PENDING = 'Pending';
	const REGISTER_REQUESTED = 'Requested';
	const REGISTER_ACTIVE = 'Active';
	
	const UPDATE_SUCCESS_DB = 0; // error_info captured
	const UPDATE_MISSING_ID = 5;
	const UPDATE_FAILURE_DB = 6; // error_info captured
	
	const SUBSCRIBE_SUCCESS_DB = 0; // error_info captured
	const SUBSCRIBE_FAILURE_REGISTER_DB = 7; // error_info captured
	const SUBSCRIBE_FAILURE_PENDING_DB = 8; // error_info captured
	const SUBSCRIBE_SEND_EMAIL = TRUE;
	const SUBSCRIBE_NO_EMAIL = FALSE;

	const CONFIRMATION_SUCCESS = 0;
	const CONFIRMATION_FAILURE_PENDING = 9;
	const CONFIRMATION_FAILURE_SYSTEM = 10;
	const CONFIRMATION_FAILURE_EXPIRED = 11;
	const CONFIRMATION_FAILURE_STATUS_DB = 12; // error_info captured
	const CONFIRMATION_SEND_EMAIL = TRUE;
	const CONFIRMATION_NO_EMAIL = FALSE;
	
	const REMOVE_USER_NO_PASSWORD = FALSE; // for remove_user
	const REMOVE_USER_SUCCESS_DB = 0; // error_info captured
	const REMOVE_USER_NOT_FOUND = 13;
	const REMOVE_USER_WRONG_PASSWORD = 14;
	const REMOVE_USER_FAILURE_DB = 15; // error_info captured

	const GENERATE_PASSWORD_SUCCESS = 0;
	const GENERATE_PASSWORD_INCOMPLETE_DATA = 16;
	const GENERATE_PASSWORD_USER_NOT_FOUND = 17;
	const GENERATE_PASSWORD_USER_WRONG_PASSWORD = 18;
	const GENERATE_PASSWORD_USER_WRONG_EMAIL = 19;
	const GENERATE_PASSWORD_SAVE_FAILURE_DB = 20; // error_info captured
	
	const NEWPASSWORD_INCOMPLETE_DATA = 21;
	const NEWPASSWORD_NO_USER_RECORD = 22;
	const NEWPASSWORD_WRONG_DATA = 23;
	const NEWPASSWORD_FAILURE_DB = 24;
	const NEWPASSWORD_SUCCESS = 25;
	const NEWPASSWORD_TOKENRECORD_EXPIRED = 26;
	const NEWPASSWORD_FAILURE_SYSTEM = 27;
	const NEWPASSWORD_NO_CONFIRMATION_USER_RECORD = 28;
	const NEWPASSWORD_CONFIRMATION_SUCCESS = 29;
	
	protected $_userid;
	protected $_error_messages = array
		(
			0 => 'Operation was successful',
			self::REGISTER_MISSING_DATA => 'Missing data for registration',
			self::REGISTER_INCOMPLETE_DATA => 'Incomplete data for registration',
			self::REGISTER_FAILURE_DB => 'System error: Save registration data failed',
			self::REGISTER_DUPLICATE_DB => 'User code aleady exists',
	
			self::UPDATE_FAILURE_DB => 'Update record failed',
			self::UPDATE_MISSING_ID => 'Update missing user ID',

			self::SUBSCRIBE_FAILURE_REGISTER_DB => 'Subscription application registration failed',
			self::SUBSCRIBE_FAILURE_PENDING_DB => 'Subscription application pending record failed',

			self::CONFIRMATION_FAILURE_PENDING => 'Confirmation failure: Pending User record not found',
			self::CONFIRMATION_FAILURE_SYSTEM => 'System error: UserID not found with confirmation token',
			self::CONFIRMATION_FAILURE_EXPIRED => 'Confirmation period has expired; registration record has been deleted.',
			self::CONFIRMATION_FAILURE_STATUS_DB => 'Confirmation failure: Unable to update user status',

			self::REMOVE_USER_FAILURE_DB => 'Unable to remove user',
			self::REMOVE_USER_NOT_FOUND => 'User not found for remove user',
			self::REMOVE_USER_WRONG_PASSWORD => 'Wrong password for remove user',
			
			self::GENERATE_PASSWORD_USER_NOT_FOUND => 'User not found for new password',
			self::GENERATE_PASSWORD_USER_WRONG_PASSWORD => 'Generate password wrong password',
			self::GENERATE_PASSWORD_SAVE_FAILURE_DB => 'New user password not saved',
			self::GENERATE_PASSWORD_INCOMPLETE_DATA => 'both correct userid and email address are required',
			
			self::NEWPASSWORD_INCOMPLETE_DATA => 'Incomplete data for new password pending record',
			self::NEWPASSWORD_NO_USER_RECORD => 'No user record for new password pending',
			self::NEWPASSWORD_WRONG_DATA => 'Wrong data for new password pending record',
			self::NEWPASSWORD_FAILURE_DB => 'Failure posting new password pending record',
			self::NEWPASSWORD_SUCCESS => 'Successful posting of new password pending record',
			self::NEWPASSWORD_TOKENRECORD_EXPIRED => 'Token for new password has expired. Try again',
			self::NEWPASSWORD_FAILURE_SYSTEM => 'System error: no userid recovered with token',
			self::NEWPASSWORD_NO_CONFIRMATION_USER_RECORD => 'No user record for the new password token. This is a permanent failure',
			self::NEWPASSWORD_CONFIRMATION_SUCCESS => 'Password confirmation has been successfully completed',
			
		);

	// setup...
	protected $_pending_lifetime; // from config, in seconds
	protected $_now; // from clock
	protected $_table_model;
	protected $_db;
	protected $_system_properties; //??
	// operation
	protected $_newpassword;
	protected $_passwordtoken;
	protected $_error_code;
	protected $_error_info;
	protected $_user;
	
	public function __construct($parameters = array())
	{
		$this->_user = User::instance();
		$parameters = (array) Services::assert_properties
		(
			$parameters,
			array(
				'mastertable' => 'users'
			)
		);
		$config = Kohana::$config->load('userauth');
		$this->_pending_lifetime = $config['pendinglifetime'];
		$this->_session = Session::instance(); // needed?
		$this->_now = time(); // needed?
		$model = $this->_table_model = new TableModel('accountschema',$parameters);
		$this->_db = $model->db();
		self::$instance = $this;
	}
	
	public static function instance()
	{
		return self::$instance;
	}
	
	public static function verify_password($password) // for validation
	{
		$instance = self::$instance;
		if (!$instance) return FALSE;
		$model = $instance->model();
		if (!$model) return FALSE;
		$cursor = $model->activerecord();
		if (!$cursor->has_current()) return FALSE;
		$userpassword = $cursor->Password;
		if ($userpassword != User::instance()->hash_password($password,$userpassword))
			return FALSE;
		else
			return TRUE;
	}
	# account management
	// $active can be User_Account::REGISTER_ACTIVE, User_Account::REGISTER_PENDING, User_Account::REGISTER_VERIFIED
	public function register($data, $active = self::REGISTER_ACTIVE, $email = FALSE) 
	{
		$datafields = array
		(
			'usercode' => 'Code',
			'password' => 'Password',
			'email' => 'Email',
			'name' => 'Name',
			'city' => 'City',
			'memo' => 'Interests',
		);
		$data = (object) $data;
		$result = Services::verify_properties
			(
				$data,
				array
				(
					'usercode',
					'password',
					'email'
				)
			);
		if (!($result === TRUE))
		{
			$this->_error_code = self::REGISTER_MISSING_DATA;
			$this->_error_info[0] = 'missing';
			$this->_error_info[1] = $this->_error_messages[self::REGISTER_MISSING_DATA];
			return FALSE;
		}
		if ((!$data->usercode) or (!$data->password) or (!$data->email))
		{
			$this->_error_code = self::REGISTER_INCOMPLETE_DATA;
			$this->_error_info[0] = 'incomplete';
			$this->_error_info[1] = $this->_error_messages[self::REGISTER_INCOMPLETE_DATA];
			return FALSE;
		}
		Services::assert_properties
		(
			$data,
			array
			(
				'name' => NULL,
				'city' => NULL,
				'memo' => NULL,
			)
		);
		$record_data = new stdClass;
		foreach ($datafields as $index => $fieldname) {
			$record_data->$fieldname = $data->$index;
		}
		$record_data->Password = $this->_user->hash_password($record_data->Password);
		$status = $active;
		$record_data->StatusID = Services::lookup('Access_Lookup_User_Status',$status);
		$permissions = new StdClass;
		$defaults = (object) Kohana::$config->load('accountdata');
		
		$permissions->name = $defaults->nameprivacy;
		$permissions->city = $defaults->cityprivacy;
		$permissions->email = $defaults->emailprivacy;
		$permissions->birthdate = $defaults->birthdateprivacy;
		$permissions->gender = $defaults->genderprivacy;
		$permissions->interests = $defaults->interestsprivacy;
		$permissions->image = $defaults->imageprivacy;
		$permissions->detailprofile = $defaults->detailprofileprivacy;
		
		$permissions->comment = (bool) $defaults->commentpermission;
		$permissions->blog = (bool) $defaults->blogpermission;
		$permissions->multiblog = (bool) $defaults->multiblogpermission;
		$permissions->article = (bool) $defaults->articlepermission;
		$permissions->unmoderated = (bool) $defaults->unmoderated;
		
		$record_data->Permissions = json_encode($permissions);
		$record_data->RoleID = System::instance()->DefaultRoleID;
		$record_data->PublicProfile = $defaults->profileprivacy;
		
		$record_data->ReleaseDetailProfile = (bool) $defaults->detailprofilerelease;
		
		$subjectuser = $this->_table_model->activerecord();
		$this->_error_info = NULL;
		try {
			 $result = $subjectuser->insert($record_data)->save();
			 if (!$result) throw new Exception();
		} catch (Exception $e) {
			$this->_error_info = $subjectuser->error_info();
			if ($this->_error_info[1] == 1062) {// duplicate
				$this->_error_code = self::REGISTER_DUPLICATE_DB;
			} else {
				$this->_error_code = self::REGISTER_FAILURE_DB;
			}
			$this->_insert_id = NULL;
			return FALSE;
		}
		$this->_insert_id = $subjectuser->insert_id();
		$subjectuser->table()->reset()->query()->where('ID','=',$this->_insert_id); // anticipate use by subscribe
		$subjectuser->current();
		$this->_error_code = self::REGISTER_SUCCESS_DB;
		return TRUE;
	}

	public function subscribe( // for membership
		$data, 
		$email = FALSE) // User_Account::SUBSCRIBE_SEND_EMAIL or User_Account::SUBSCRIBE_NO_EMAIL
	{
		// Do garbage collection of pending records
		$this->delete_expired_pending();
		$this->delete_passwords_pending(); // subscribe visited more frequently than new password
		
		if (!$this->register($data, self::REGISTER_PENDING))
		{
//			register() will have set error data
			return FALSE;
		}
		$newuserid = $this->_insert_id;
		$pendingtoken = $this->create_pending_token();
		$postingdata = array
		(
			'UserID' => $newuserid,
			'ConfirmationToken' => $pendingtoken,
			'Created' => time(),
			'Expires' => (time() + $this->_pending_lifetime)
		);
		$model = $this->_table_model;
		$this->_error_info = NULL;
		$subjectrecord = $model->activerecord();
		try {
			$pendingrecord = $model->activerecord('pending');
			if (!$pendingrecord->insert($postingdata)->save())
				throw new Exception();
		} catch (Exception $e) {
			$this->_error_info = $pendingrecord->error_info();
			$subjectrecord()->current()->delete(); // give up on the subscription
			$this->_insert_id = NULL;
			$this->_error_code = self::SUBSCRIBE_FAILURE_PENDING_DB;
			return FALSE;
		}
		$this->_insert_id = $newuserid;
		$this->_error_code = self::SUBSCRIBE_SUCCESS_DB;
		
		if (!$email) return TRUE;
		// send mail for confirmation
		$config = Kohana::$config->load('registeremails');
		$to      = $subjectrecord->Name . ' <' . $subjectrecord->Email . '>';
		$subject = $config['confirm']['subject'];
		$headers = $config['headers'];
		
		$dictionary = array(
			'username' => $subjectrecord->Name,
			'website' => System::instance()->Website,
			'usercode' => $subjectrecord->Code,
			'queryemail' => System::instance()->ReturnEmailAddress,
			'confirmurl' => $_SERVER['HTTP_HOST'] . URL::base() . 'member/confirm/',
			'confirmtoken' => $pendingtoken,
		);
 
		$emailview = Template::view(array('filename' => 'email_registerconfirm','ext'=>'txt'),$dictionary);
 
		$result = mail($to, $subject, $emailview, $headers);
		return TRUE;
	}

	// take into account moderation option set by system
	public function confirm(
		$confirmationtoken, 
		$email = self::CONFIRMATION_NO_EMAIL, 
		$moderatoremail = self::CONFIRMATION_NO_EMAIL) // can be self::CONFIRMATION_SEND_EMAIL or User_Account::CONFIRMATION_NO_EMAIL
	{
		$resultset = DB::select()
			->from('Access_Users_Pending')
			->where('ConfirmationToken','=',$confirmationtoken)
			->as_object()
			->execute($this->_db);
		$row = $resultset->current();
		if (!$row)
		{
			$this->_error_code = self::CONFIRMATION_FAILURE_PENDING;
			return FALSE;
		}
		$userid = $row->UserID;
		if ($row->Expires < time())
		{
			$this->_error_code = self::CONFIRMATION_FAILURE_EXPIRED;
			$model = $this->_table_model;
			$model->table()->reset()->query()->where('ID','=',$userid);
			$model->activerecord()->load()->delete();
			$model->table()->reset();
			return FALSE;
		}
		if (!$userid)
		{
			$this->_error_code = self::CONFIRMATION_FAILURE_SYSTEM;
			return FALSE;
		}
		$moderated = (bool) System::instance()->ModerateSubscription;
		if ($moderated) $statuscode = 'Requested';
		else $statuscode = 'Active';
		$statusid = Services::lookup('Access_Lookup_User_Status',$statuscode);
		$model = $this->_table_model;
		$model->table()->reset()->query()->where('ID','=',$userid);
		$this->_error_info = NULL;
		if (!$model->activerecord()->load()->update(array('StatusID'=>$statusid))->save())
		{
			$this->_error_info = $subjectuser->activerecord()->error_info();
			$this->_error_code = self::CONFIRMATION_FAILURE_STATUS_DB;
			return FALSE;
		}
		$model->activerecord('pending')->load()->delete();
		$this->_error_code = self::CONFIRMATION_SUCCESS;
		if (!$email) return TRUE;
		$subjectrecord = $model->activerecord();
		// send mail for confirmation
		$config = Kohana::$config->load('registeremails');
		$to      = $subjectrecord->Name . ' <' . $subjectrecord->Email . '>';
		$subject = $config['complete']['subject'];
		$headers = $config['headers'];

		$dictionary = array(
			'username' => $subjectrecord->Name,
			'website' => System::instance()->Website,
			'usercode' => $subjectrecord->Code,
			'queryemail' => System::instance()->ReturnEmailAddress,
			'moderated' => $moderated,
			'loginurl' => $_SERVER['HTTP_HOST'] . URL::base() . 'member/login',
		);
 
		$emailview = Template::view(array('filename' => 'email_registercomplete','ext'=>'txt'),$dictionary);
		
		$result = mail($to, $subject, $emailview, $headers);
		
		if ($moderated and $moderatoremail) {
			$to      = '<' . System::instance()->AdministratorEmailAddress . '>';
			$subject = $config['notify']['subject'];
			$headers = $config['headers'];
			
			$dictionary = array(
				'username' => $subjectrecord->Name,
				'website' => System::instance()->Website,
				'usercode' => $subjectrecord->Code,
				'interests' => $subjectrecord->Interests,
				'adminoptionsurl' => $_SERVER['HTTP_HOST'] . URL::base() . 'member/account/' . $subjectrecord->Code . '#administration',
			);
			$emailview = Template::view(array('filename' => 'email_registernotify','ext'=>'txt'),$dictionary);
			
			$result = mail($to, $subject, $emailview, $headers);
		}
		return TRUE;
	}
	
	protected function create_pending_token()
	{
		while (TRUE)
		{
			// Create a random token
			$token = Text::random('alnum', 32);

			// Make sure the token does not already exist
			if (DB::select('ID')
				->where('ConfirmationToken','=', $token)
				->from('Access_Users_Pending')
				->execute($this->_db)->count() === 0)
			{
				// A unique token has been found
				return $token;
			}
		}
	}

	protected function delete_expired_pending()
	{
		// Delete all expired pending records
		$resultset = DB::select('UserID')
			->from('Access_Users_Pending')
			->where('Expires','<', $this->_now)
			->as_object()
			->execute($this->_db);
		while($row = $resultset->fetch())
		{
			DB::delete('Access_Users')->where('ID','=',$row->UserID)->execute($this->_db);
		}
		
		return $this;
	}
	
	public function petition_for_password($data,// for membership
		$email = FALSE) // User_Account::SUBSCRIBE_SEND_EMAIL or User_Account::SUBSCRIBE_NO_EMAIL
	{
		$this->delete_passwords_pending();
		if ( (!isset($data['usercode'])) or !isset($data['email'])) {
			$this->_error_code = self::NEWPASSWORD_INCOMPLETE_DATA;
			return FALSE;
		}
		$subjectrecord = $this->model()->activerecord();
		if (!$subjectrecord->has_current()) {
			$this->_error_code = self::NEWPASSWORD_NO_USER_RECORD;
			return FALSE;
		}
		if (($subjectrecord->Code != $data['usercode']) or ($subjectrecord->Email != $data['email'])) {
			$this->_error_code = self::NEWPASSWORD_WRONG_DATA;
			return FALSE;
		}
		$userid = $this->model()->activerecord()->ID;
		// Do garbage collection of pending records
		
		$passwordtoken = $this->create_password_token();
		$postingcolumns = array
		(
			'UserID',
			'ConfirmationToken',
			'Created',
			'Expires',
		);
		
		$postingdata = array
		(
			$userid,
			$passwordtoken,
			time(),
			(time() + $this->_pending_lifetime),
		);
		
		$result = DB::insert('Access_Users_Pending_Password',$postingcolumns)
			->values($postingdata)->execute($this->_db);
		if (!$result) {
			$this->_error_code = self::NEWPASSWORD_FAILURE_DB;
			return FALSE;
		}
		$this->_error_code = self::NEWPASSWORD_SUCCESS;
		if (!$email) return TRUE;
		// send mail for confirmation
		$config = Kohana::$config->load('registeremails');
		$to      = $subjectrecord->Name . ' <' . $subjectrecord->Email . '>';
		$subject = $config['password']['subject'];
		$headers = $config['headers'];
		
		$dictionary = array(
			'username' => $subjectrecord->Name,
			'website' => System::instance()->Website,
			'usercode' => $subjectrecord->Code,
			'queryemail' => System::instance()->ReturnEmailAddress,
			'confirmurl' => $_SERVER['HTTP_HOST'] . URL::base() . 'member/confirmpassword/',
			'confirmtoken' => $passwordtoken,
		);
 
		$emailview = Template::view(array('filename' => 'email_registernewpasswordconfirm','ext'=>'txt'),$dictionary);
 
		$result = mail($to, $subject, $emailview, $headers);
		return TRUE;
	}
	protected function delete_passwords_pending()
	{
		// Delete all expired pending records
		$resultset = DB::delete('Access_Users_Pending_Password')
			->where('Expires','<', $this->_now)
			->execute($this->_db);
		return $this;
	}
	protected function create_password_token()
	{
		while (TRUE)
		{
			// Create a random token
			$token = Text::random('alnum', 32);

			// Make sure the token does not already exist
			if (DB::select('ID')
				->where('ConfirmationToken','=', $token)
				->from('Access_Users_Pending_Password')
				->execute($this->_db)->count() === 0)
			{
				// A unique token has been found
				return $token;
			}
		}
	}
	public function confirm_password_request(
		$confirmationtoken, 
		$email = self::CONFIRMATION_NO_EMAIL) // can be self::CONFIRMATION_SEND_EMAIL or User_Account::CONFIRMATION_NO_EMAIL
	{
		$resultset = DB::select()
			->from('Access_Users_Pending_Password')
			->where('ConfirmationToken','=',$confirmationtoken)
			->as_object()
			->execute($this->_db);
		$row = $resultset->current();
		if (!$row)
		{
			$this->_error_code = self::NEWPASSWORD_TOKENRECORD_EXPIRED;
			return FALSE;
		}
		$userid = $row->UserID;
		// set user record for account
		if ($row->Expires < time())
		{
			DB::delete('Access_Users_Pending_Password')
				->where('ConfirmationToken','=',$confirmationtoken)
				->execute($this->_db);
			$this->_error_code = self::NEWPASSWORD_TOKENRECORD_EXPIRED;
			return FALSE;
		}
		if (!$userid)
		{
			$this->_error_code = self::NEWPASSWORD_FAILURE_SYSTEM;
			return FALSE;
		}
		DB::delete('Access_Users_Pending_Password')
			->where('ConfirmationToken','=',$confirmationtoken)
			->execute($this->_db);
		
		$useraccount = $this;
		$table = $useraccount->model()->table();
		$table->reset()->query()->where('ID','=',$userid);
		$subjectrecord = $table->activerecord();
		$subjectrecord->load();
		$success = (bool) $subjectrecord->current();
		if (!$success) {
			$this->_error_code = self::NEWPASSWORD_NO_CONFIRMATION_USER_RECORD;
			return FALSE;
		}
		$result = $this->generate_password($subjectrecord->Code,$subjectrecord->Email);
		if ($result) {
			$this->_error_code = self::NEWPASSWORD_CONFIRMATION_SUCCESS;
		}
		if (!$email) return TRUE;
		$password = $this->_newpassword;
		// send password email to user
		$config = Kohana::$config->load('registeremails');
		$to      = $subjectrecord->Name . ' <' . $subjectrecord->Email . '>';
		$subject = $config['completepassword']['subject'];
		$headers = $config['headers'];

		$dictionary = array(
			'username' => $subjectrecord->Name,
			'website' => System::instance()->Website,
			'usercode' => $subjectrecord->Code,
			'queryemail' => System::instance()->ReturnEmailAddress,
			'password' => $password,
			'loginurl' => $_SERVER['HTTP_HOST'] . URL::base() . 'member/login',
		);
 
		$emailview = Template::view(array('filename' => 'email_registercompletepassword','ext'=>'txt'),$dictionary);
		
		$result = mail($to, $subject, $emailview, $headers);
		
		return TRUE;
	}
	
	public function generate_password($usercode, $emailaddress, $password = FALSE)
	{
		$this->_newpassword = NULL;
		
		if (empty($usercode) or empty($emailaddress))
		{
			$this->_error_code = self::GENERATE_PASSWORD_INCOMPLETE_DATA;
			return FALSE;
		}

		$subjectuser = $this->model();
		$subjectuser->query()->reset()->where('Code','=',$usercode)->execute();
		$subjectcursor = $subjectuser->activerecord();
		if (!$subjectcursor->load()->has_record())
		{
			$this->_error_code = self::GENERATE_PASSWORD_USER_NOT_FOUND;
			return FALSE;
		}
		if ((!$subjectcursor->Email == $emailaddress))
		{
			$this->_error_code = self::GENERATE_PASSWORD_INCOMPLETE_DATA;
			return FALSE;
		}
		if (!($password === FALSE)) // get and check password
		{
			if (!($subjectcursor->Password == $this->_user->hash_password($password,$subjectcursor->Password)))
			{
				$this->_error_code = self::GENERATE_PASSWORD_USER_WRONG_PASSWORD;
				return FALSE;
			}
		}
		$newpassword = $this->get_unique_code(10);
		$hashedpassword = $this->_user->hash_password($newpassword);
		$this->_error_info = NULL;
		if (!$subjectcursor->update(array('Password' => $hashedpassword))->save())
		{
			$this->_error_info = $subjectcursor->error_info();
			$this->_error_code = self::GENERATE_PASSWORD_SAVE_FAILURE_DB;
			return FALSE;
		}
		$this->_newpassword = $newpassword;
		$this->_error_code = self::GENERATE_PASSWORD_SUCCESS;
		return TRUE;
	}
	
	protected function get_unique_code($length = NULL)
	{	
		$code = md5(uniqid(rand(), true));
		if ($length) 
		{
			return substr($code, 0, $length);
		}
		else
		{
			return $code;
		}
	}	
	
	public function remove_user($username, $password = FALSE, $email = FALSE)
	{
		$subjectuser = $this->get_table_model();
		$subjectuser->query()->reset()->where('Code','=',$username)->execute();
		$subjectcursor = $subjectuser->activerecord();
		if (!$subjectcursor->set_current()->has_record())
		{
			$this->_error_code = self::REMOVE_USER_NOT_FOUND;
			return FALSE;
		}
		if (!($password === FALSE)) // get and check password
		{
			if (!($subjectcursor->Password == $this->_user->hash_password($password,$subjectcursor->Password)))
			{
				$this->_error_code = self::REMOVE_USER_WRONG_PASSWORD;
				return FALSE;
			}
		}
		if (!$subjectcursor->delete())
		{
			$this->_insert_id = NULL;
			$this->_error_code = self::REMOVE_USER_FAILURE_DB;
			return FALSE;
		}
		$this->_insert_id = $subjectuser->recordset()->dbupdate_resultset()->insert_id();
		$this->_error_code = self::REMOVE_USER_SUCCESS_DB;
		return TRUE;
	}

	public function new_password()
	{
		return $this->_newpassword;
	}
	public function password_token()
	{
		return $this->_passwordtoken;
	}
	public function error_code()
	{
		return $this->_error_code;
	}
	public function error_message($status_code = NULL)
	{
		if ($status_code === NULL)
		{
			$status_code = $this->_error_code;
		}
		return $this->_error_messages[$status_code];
	}
	public function model()
	{
		return $this->_table_model;
	}
	// *** identical to account rank; should be superclass
	public function rank()
	{
		$owneraccount = $this->model()->activerecord();
		if (!$owneraccount->has_current()) return NULL;
		
		$ownerroleid = $owneraccount->RoleID;
		if (is_null($ownerroleid)) $ownerroleid = System::instance()->DefaultRoleID;
		if (!is_null($ownerroleid)) {
			$record = DB::select('Rank')
				->from('Access_Group_Roles')
				->where('ID','=',$ownerroleid)
				->as_object()
				->execute($this->_db)
				->current();
			if ($record) {
				$rank = (integer) $record->Rank;
			} else {
				$rank = 0;
			}
		} else {
			$rank = 0;
		}
		return $rank;
	}
	public function administered_rank()
	{	
		$rank = NULL;
		$owneraccountid = NULL;
		$user = User::instance();
		if ($user->authenticated()) {
			$useraccountid = $user->userid();
			$useraccount = $user->model()->activerecord();
			$owneraccount = $this->model()->activerecord();
			$owneraccountid = $owneraccount->ID;
		}
		if (!is_null($owneraccountid)) {
			$ownerrank = $this->rank();
			$userrank = $user->rank();
			if (!($ownerrank and $userrank)) return 0;
			$maxrank = System::instance()->MaxRank;
			if ($userrank == $maxrank) return $maxrank;
			if ($useraccountid != $owneraccountid) return --$userrank;
			return $userrank;
		}
		if (is_null($rank)) $rank = 0;
		return $rank;
	}
	public function account_data()
	{
		$system = System::instance();
		$cursor = $this->model()->activerecord();
		if ($cursor->has_current())
		{
			$record = clone $cursor->current();
			unset (
				$record->Password,
				$record->MembershipExpires,
				$record->Expiry
			);
			if ($record->Permissions) {
				$record->Permissions = new JsonField(json_decode($record->Permissions));
			}
			if (!$system->MultipleGroups) {
				$roleid = $record->RoleID;
				if (is_null($roleid)) {
					$record->RoleID = $system->DefaultRoleID;
					$record->GroupID = $system->DefaultGroupID;
				} else {
					$record->GroupID = Services::dblookup(
						'Access_Group_Roles','GroupID','ID',$roleid);
				}
			}
		}
		else
		{
			$record = new StdClass;
			$record->ID = NULL;
		}
		return $record;
	}
	public function error_info()
	{
		return $this->_error_info;
	}
}