<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana, DB;
use Muster\Node\RecordNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class BlogRecord extends RecordNode {
	public function __construct($controller)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context();
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass();
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'blog';
		# tablemodel
		$context->mastertablename = 'master';
		$context->extensiontablename = 'extension';
		# recordset
		$context->objecttype = 'blog';
		$context->binderscope = 'Blogs';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		$context->constraints = array(
			array(
				'index' => 'containerslist',
				'value' => 'notrequired',
			)
		);
		# node
		$context->redirect =  'blogs/record';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		
		return $context;
	}
	protected function _on_respond($command)
	{
//		var_dump($command); exit;
		$result = Node::FLOW_CONTINUE;
		switch ($command) {
			case 'startposts':
				$result = $this->_start_posts();
				break;
		}
		return $result;
	}
	protected function _start_posts()
	{
		$db = $this->_tablemodel->model->db();
		$datarecord = $this->_tablemodel->datarecord;
		if (!$datarecord) return Node::FLOW_CONTINUE;
		$blogid = $datarecord->ID;
		$blogcode = $datarecord->BlogCode;
		# get first record
		$tablename = 'Nodes_View';
		$query = DB::select_array(array('ID','Title','ObjectType','ParentID','BlogCode','OrderText','ParentID'))
			->from($tablename);
		$this->_set_query_constraints($query,$tablename);
		$query->order_by('ArticleDate','DESC');
		$query->order_by('ID','DESC');
		$query->where('ObjectType','=','entry');
		$query->where('ParentID','=',$blogid);
		$operation = $query
			->offset(0)
			->limit(1)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$firstrecord = $operation->current();
		} else {
			var_dump($operation->error_info()); exit;
			return Node::FLOW_CONTINUE;
		}
		if (!$firstrecord) return Node::FLOW_CONTINUE;
		$articleid = $firstrecord->ID;
		$redirect = 'blogs/' . $blogcode;
		$this->_controller->request->redirect($redirect . "/$articleid");
		return Node::FLOW_CONTINUE;
	}
	protected function _on_set_delete_locktables(&$locktables)
	{
		$locktables[] = 'Access_Users';
	}
	// check for existing blog entries for the binder...
	protected function _on_before_delete($parmdata,&$error_info)
	{
		extract($parmdata);
		$operation = DB::select('ArticleID')
			->from('Articles_Extensions')
			->where('ParentID','=',$datarecord->ID)
			->as_object()
			->prepare($db)
			->execute();
		$result = $operation->success();
		if ($result) {
			$rowcount = $operation->row_count();
			if ($rowcount != 0) {
				$error_info[0] = 'delete blog';
				$error_info[1] = 'blog entries';
				$error_info[2] = 'Blog entries exist for this blog - cannot delete';
				$result = FALSE;
			}
		} else {
			foreach ($operation->error_info() as $key => $value) {
				$error_info[$key] = $value;
			}
		}
		if ($result) {
			$blogid = $datarecord->ID;
			$operation = DB::update('Access_Users')
				->set(array('BlogID' => DB::expr('NULL')))
				->where('BlogID','=',$blogid)
				->prepare()
				->execute();
			if ($operation->success()) {
				$result = TRUE;
			} else {
				$error_info = $operation->error_info();
				$result = FALSE;
			}
		}
		return $result;
	}
}