<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Kohana, DB, Session;
use Services;
use Muster\Node\RecordNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

// note that this extends recordnode, with collaboratorlist a dependent of the master
class CollaboratorList extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	// the master table is an article (Node), 
	// and the collaborators are taken as a side effect 
	// of the access to the dependent table.
	protected function _get_context($selector)
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->dataframe = 'collaboratordata';
		$context->controlframe = 'collaboratorcontrols';
		$context->messagefile = 'collaborators';
		# tablemodel
		$context->mastertablename = 'master';
		$context->collaboratortablename = 'collaborators';
		# recordset
//		$context->selectarray = array('EmptyBody','EditDate','EditUserID');
		$context->binderscope = NULL; // any article record
		#gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->collaboratortableindex = 'collaboratortable';
		$context->collaboratorrecordindex = 'collaboratorrecord';
		# node
		$context->get_admindictionary = FALSE;
		$context->redirect = 'sublists/collaborators/admin';
		# permissions
		return $context;
	}
	protected function _on_before_get_tokens($tokens) // pick up tokens for master record
	{
		$context = $this->_context;
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		$dictionary['articleid'] = $this->_context->articleid;
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->tables[$context->collaboratortableindex] = 
			$tablemodel->model->table($context->collaboratortablename);
		$activerecord = $datapool->activerecords[$context->collaboratorrecordindex] = 
			$tablemodel->model->activerecord($context->collaboratortablename);
	}
	protected function _on_respond($command)
	{ 
		$datamodel = $this->_datamodel;
		$result = Node::FLOW_CONTINUE;
//		$result = Node::FLOW_REDIRECT;
		switch ($command) {
			case 'addcollaborator':
				$result = $this->_addcollaborator($datamodel); // folder
				break;
			case 'removecollaborators':
				$result = $this->_removecollaborators($datamodel); // folder
				break;
		}
		return $result;
	}
	protected function _addcollaborator($datamodel)
	{
		if (!$this->_user->authenticated()) return FLOW_CONTINUE;
		$permissions = $this->_permissions;
		if (!($permissions->canadminister and $permissions->canedit))
			return $this->_record_report_access_error($datamodel);
		$context = $this->_context;
		$parameters = $this->_parameters;
		$db = $this->_tablemodel->model->db();
		$datapool = $datamodel->datapool;
		$articleid = $context->articleid;
		if (empty($articleid) and !is_numeric($articleid))
			die('unexpected missing articleid in collaboratorlist.php addcollaborator');
		$collaboratorcode = $parameters->newcollaborator;
		if (empty($collaboratorcode)) {
			$error_info = 'unspecified collaborator code';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		if ($context->frozen) {
			$error_info = 'the master record has frozen editing';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		// editing rights flow from master record
		$canadd = Services::check_article_permission($articleid,'E','settings');
		if ($canadd)
			$canadd = ($canadd and $this->_user->user_data()->Permissions->article);
		if (!$canadd) {
			$error_info = 'insufficient permissions for editing';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		$db->lock_tables(array('Nodes_View','Articles_Authors','Access_Users','Access_Group_Roles'));
		$collaboratorrecord = Services::dbrecordlookup('Access_Users',array('Name','ID','RoleID'),'Code',$collaboratorcode,$db);
		if ($collaboratorrecord) {
			$collaboratorid = $collaboratorrecord->ID;
			$collaborator_roleid = $collaboratorrecord->RoleID;
			$username = $collaboratorrecord->Name;
		} else {
			$error_info = 'unknown collaborator code';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		$collaborator_rank = Services::dblookup('Access_Group_Roles','Rank','ID',$collaborator_roleid);
		if ($collaborator_rank <= 1) {
			$error_info = $collaboratorcode . ' is not authorized to collaborate';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		$operation = DB::insert('Articles_Authors',array('ArticleID','UserID'))
			->values(array($articleid,$collaboratorid))
			->prepare()
			->execute();
		$result = $operation->success();
		if (!$result) {
			$error_info = $operation->error_info();
			$this->_set_datapool_error(compact('datapool','error_info'));
			$db->unlock_tables();
			return Node::FLOW_CONTINUE; // recycle
		}
		$db->unlock_tables();
		$session = Session::instance();
		$messages = array(
			'messages' => array(
				array('message'=> '[' . $username . '] has been attached to this article')
			),
			'index'=>'messagesection',
		);
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
	protected function _removecollaborators($datamodel)
	{
		if (!$this->_user->authenticated()) return FLOW_CONTINUE;
		$permissions = $this->_permissions;
		if (!($permissions->canadminister and $permissions->canedit))
			return $this->_record_report_access_error($datamodel);
		$parameters = $this->_parameters;
		$datapool = $datamodel->datapool;
		$removearray = $parameters->collaboratorinstances;
		$context = $this->_context;
		if ($context->frozen) {
			$error_info = 'the master record has frozen editing';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		$canremove = Services::check_article_permission($this->_context->articleid,'E','settings');
		if ($canremove)
			$canremove = ($canremove and $this->_user->user_data()->Permissions->article);
		if (!$canremove) {
			$error_info = 'insufficient permissions for editing';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		if (!empty($removearray)) {
			$removearray = array_keys($removearray);
			$operation = db::delete('Articles_Authors')
				->where('ID','IN',$removearray)
				->prepare()
				->execute();
			$result = $operation->success();
			if (!$result) {
				$error_info = $operation->error_info();
				$this->_set_datapool_error(compact('datapool','error_info'));
				return Node::FLOW_CONTINUE;
			}
			$session = session::instance();
			$messages = array(
				'messages' => array(
					array('message'=> 'Collaborators were removed')
				),
				'index'=>'messagesection',
			);
			$session->set('messages',$messages);
			return Node::FLOW_REDIRECT;
		} else {
			$error_info = 'no collaborators were selected for removal';
			$this->_set_datapool_error(compact('datapool','error_info'));
			return Node::FLOW_CONTINUE;
		}
		return Node::FLOW_REDIRECT;
	}
}