<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Kohana;
use Services;
use Muster\Node\ListNode;

defined('SYSPATH') or die('No direct script access.');

class CommentList extends ListNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'nodecontrols';
		$context->dataframe = 'nodedata';
		$context->messagefile = 'article';
		# tablemodel
		$context->mastertablename = 'master';
		$context->commentarticletablename = 'commentarticle';
		# recordset
		$context->objecttype = 'comment';
		$context->selectarray = array('ObjectType','Body','EditDate','EditUserID','CommentArticleID');
		$context->binderscope = 'Comments';
		$context->foreignkeyname = 'CommentArticleID';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->commentarticletableindex = 'commentarticletable';
		$context->commentarticlerecordindex = 'commentarticlerecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		$context->constraints = array(
			array(
				'index' => 'containerslist',
				'value' => 'notrequired',
			),
			array(
				'index' => 'commentarticles',
				'value' => 'required',
			),
			array(
				'index' => 'showbody',
				'value' => TRUE,
			),
		);
		$context->get_admindictionary = FALSE;
		# node
		$context->redirect = 'subitems/comments';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->tables[$context->commentarticletableindex] = 
			$tablemodel->model->table($context->commentarticletablename);
		$datapool->activerecords[$context->commentarticlerecordindex] = 
			$tablemodel->model->activerecord($context->commentarticletablename);
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		if ($this->_user->authenticated()) {
			$dictionary['can_comment'] = (bool) $this->_user->user_data()->Permissions->comment;
		} else {
			$dictionary['can_comment'] = FALSE;
		}
		$articleid = $dictionary['articleid'] = $this->_context->articleid; // parentid record
		if (!$articleid) {
			$dictionary['can_comment'] = FALSE;
			$dictionary['is_commentsclosed'] = TRUE;
		} else {
			$dictionary['is_commentsclosed'] = (bool) Services::dblookup('Articles','CommentsClosed','ID',$articleid);
		}
	}
}