<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Kohana, I18N;
use Notification, Services, System;
use Muster\Node\RecordNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class CommentRecord extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'commentcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'article';
		# tablemodel
		$context->mastertablename = 'master';
		$context->extensiontablename = 'extension';
		$context->commentarticletablename = 'commentarticle';
		# recordsset
		$context->objecttype = 'comment';
		$context->binderscope = 'Comments';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->commentarticletableindex = 'commentarticletable';
		$context->commentarticlerecordindex = 'commentarticlerecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		$context->constraints = array(
			array(
				'index' => 'containerslist',
				'value' => 'notrequired',
			)
		);
		# node
		$context->redirect =  'subitems/comment';
		$context->get_bookmark = FALSE;
		$context->get_admindictionary = FALSE;
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	protected function _on_get_parameters($parameters)
	{
		$context = $this->_context;
		$parameters->mode = 'admin';
		property_exists($context,'parentid') and ($parameters->parentid = $context->parentid);
	}
	protected function _on_set_states($states)
	{
		$states->adminmode = TRUE;
		$parameters = $this->_parameters;
		if (empty($parameters->parentid) or (!is_numeric($parameters->parentid)))
			return;
		$states->commentsclosed = (bool) Services::dblookup('Articles','CommentsClosed','ID',$parameters->parentid);
	}
	protected function _on_set_datapool($datapool)
	{
		$tablemodel = $this->_tablemodel;
		$context = $this->_context;
		$containertable = $datapool->tables[$context->commentarticletableindex] = 
			$tablemodel->model->table($context->commentarticletablename);
		$datapool->activerecords[$context->commentarticlerecordindex] = 
			$tablemodel->model->activerecord($context->commentarticletablename);
	}
	protected function _on_get_redirect_parameters(&$parms)
	{
		$parms['parentid'] = $this->_parameters->parentid;
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		$states = $this->_states;
		if (empty($this->_parameters->parentid) or (!is_numeric($this->_parameters->parentid))) {
			var_dump($this->_parameters->parentid);
			die('unexpected null $this->_parameters->parentid in commentrecord.php');
		}
		if ($states->insertmode) {
			$dictionary['commentarticleid'] = $this->_parameters->parentid;
		}
		$dictionary['parentid'] = $this->_parameters->parentid;
	}
	protected function _on_before_insert($parmdata,&$error_info)
	{
		$db = $this->_tablemodel->model->db();
		extract($parmdata);
		if (empty($extensionrecord->CommentArticleID) or (!is_numeric($extensionrecord->CommentArticleID)))
			die('unexpected null CommentArticleID in commentrecord.php');
		$permissions = $this->_permissions;
		$states = $this->_states;
		if (!$permissions->userallowcomments) {
			$error_info[0] = 'commentrecord';
			$error_info[1] = 'nocomments';
			$error_info[2] = 'User is not allowed comments';
			return FALSE;
		}
		if (!$permissions->unmoderated) {
			if ($states->provisionalpublish) {
				$masterrecord->Approval = 'P';
			} else {
				$masterrecord->Approval = 'D';
			}
		} else {
			$masterrecord->Approval = 'A'; // ** make conditional on permissions
		}
		$binderid = Services::dblookup('Binders','ID','Code','Comments',$db);
		$masterrecord->BinderID = $binderid;
		$masterrecord->ShowByline = TRUE;
		
		return TRUE;
	}
	protected function _on_after_insert_post($parmdata)
	{
		$system = System::instance();
		if (!$system->Notification) return;
		extract($parmdata); // activerecord, masterrecord, extensionactiverecord, extenstionrecord
		$activerecord->current();
		$approvalcode = $activerecord->Approval;
		if ($approvalcode == 'A') return;
		if ($approvalcode == 'P')
			$qualifier = 'which has been provisionally published';
		else 
			$qualifier = '';
		$recipients = $system->AdministratorEmailAddress;
		$date = date('d-M-yy');
		$selector = 'request';
		$record = $activerecord->current();
		$data = array(
			'username' => $activerecord->Author,
			'usercode' => $activerecord->AuthorUserCode,
			'memberurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'member/account/' . $activerecord->AuthorUserCode,
			'recordtype' => I18n::get($activerecord->ObjectType),
			'recordurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'node/' . $activerecord->ID,
			'requestqualifier' => $qualifier,
		);
		list($successes,$failures) = Notification::email($recipients,$date,$selector,$data);
		if ($successes)
			$this->_set_datapool_message($this->_datamodel->datapool,'request for moderation has been sent to an administrator','messages');
		if ($approvalcode == 'P')
			$this->_set_datapool_message($this->_datamodel->datapool,'the comment has been provisionally published','messages');
	}
	protected function _on_before_update($parmdata,&$error_info)
	{
		$permissions = $this->_permissions;
		if (!$permissions->userallowcomments) {
			$error_info[0] = 'commentrecord';
			$error_info[1] = 'nocomments';
			$error_info[2] = 'User is not allowed comments';
			return FALSE;
		}
		extract($parmdata); // activerecord, masterrecord, extensionactiverecord, extenstionrecord, priorrecord
		if ((!$permissions->unmoderated) and ($activerecord->Approval == 'A')) {
			if ($this->_states->provisionalpublish) {
				$masterrecord->Approval = 'P';
			} else {
				$masterrecord->Approval = 'D';
			}
		}
		return TRUE;
	}
	protected function _on_after_update_post($parmdata)
	{
		$system = System::instance();
		if (!$system->Notification) return;
		extract($parmdata); // activerecord, masterrecord, extensionactiverecord, extenstionrecord, priorrecord
		$activerecord->current();
		$approvalcode = $activerecord->Approval;
		if ($approvalcode == 'A') return;
		if ($priorrecord->Approval != 'A') return;
		if ($approvalcode == 'P')
			$qualifier = 'which has been provisionally published';
		else 
			$qualifier = '';
		$recipients = $system->AdministratorEmailAddress;
		$date = date('d-M-yy');
		$selector = 'editrequest';
		$record = $activerecord->current();
		$data = array(
			'username' => $activerecord->Author,
			'usercode' => $activerecord->AuthorUserCode,
			'memberurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'member/account/' . $activerecord->AuthorUserCode,
			'recordtype' => I18n::get($activerecord->ObjectType),
			'recordurl' => 'http://'. System::instance()->Website . $this->_controller->_base . 'node/' . $activerecord->ID,
			'requestqualifier' => $qualifier,
		);
		list($successes,$failures) = Notification::email($recipients,$date,$selector,$data);
		if ($successes)
			$this->_set_datapool_message($this->_datamodel->datapool,'request for edit moderation has been sent to an administrator','messages');
		if ($approvalcode == 'P')
			$this->_set_datapool_message($this->_datamodel->datapool,'the comment has been provisionally published','messages');
	}
	protected function _on_respond($command)
	{ 
		$result = parent::_on_respond($command);
		$datamodel = $this->_datamodel;
		switch ($command) {
			case 'newcomment':
				$result = $this->_newcomment($datamodel);
				break;
		}
		return $result;
	}
	protected function _newcomment($datamodel)
	{
		if (!$this->_permissions->userallowcomments)
			return $this->_record_report_access_error($datamodel);
		if (!($this->_tablemodel->datarecord)) {
			$datamodel->datapool->constraints['edit'] = TRUE;
			$this->_states->editmode = TRUE;
			$datamodel->datapool->constraints['insertmode'] = TRUE;
			$this->_states->insertmode = TRUE;
			return Node::FLOW_CONTINUE;
		} else {
			return Node::FLOW_REDIRECT;
		}
	}
}