<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
//** s/b hierarchy like pages - HB
namespace Base\NodeType;

use StdClass;
use Kohana, DB, Session;
use System,TableModel,Filer,Services,User,Access;
use Muster\Node\ListNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class DirectoryList extends ListNode {
	public function __construct($controller,$selector)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context($selector)
	{
		$context = new StdClass();
		# common
		// recordset
		$context->objecttype = 'directory';
		$context->default_orderby = 'Title';
		$context->default_orderdirection = 'ASC';
		// gateway
		$context->controlframe = 'nodecontrols';
		$context->messagefile = 'file'; // not used
		# differences
		switch ($selector) {
			case 'list': {
				// tablemodel
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->selectarray = array('ObjectType','EmptyBody','DefaultFolio','EditDate','EditUserID');
				$context->binderscope = 'Directories';
				// gateway
				$context->dataframe = 'nodedata';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				# node
				$context->redirect = 'library/directories';
				// permissions
				$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
			}
			break;
			case 'contents': {
				// tablemodel
				$context->tablemodel = 'nodeinstanceschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->selectarray = array('ObjectType','InstanceID','NodeID','EmptyBody','isTreeTop','BlogCode','ParentID','FolderCode','Approval','Order','OrderText','DividerAbove','DividerBelow','InstanceDescription','URL','Source','FileName','Directory');
				$context->binderscope = NULL; // mixed
				$context->foreignkeyname = 'FolderID';
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->constraints = array(
					array('index' => 'contentdisplay','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// redirect extended by controller
				$context->tokens = Services::get_token_set('mixed');
			}
			break;
			case 'binders': {
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
//				$context->blogtablename = 'blog';
				// recordset
				$context->selectarray = array('NodeID','EmptyBody','ObjectType','BlogCode','ParentID','FolderCode');
				$context->binderscope = NULL;
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->constraints = array(
					array('index' => 'contentdisplay','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// redirect extended by controller
				$context->tokens = Services::get_token_set('mixed');
			}
		}
		# node
		return $context;
	}
	// over_rides
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
	}
	protected function _on_respond($command)
	{
		$result = Node::FLOW_CONTINUE;
		switch ($command) {
			case 'actionimport':
				return $this->_process_import_files();
			break;
			case 'actiongenerate':
				return $this->_process_generate_files();
			break;
			case 'actionupload':
				return $this->_process_upload_files();
			break;
			case 'actionshoworphans':
				return $this->_process_show_orphans();
			break;
			case 'startdirectory':
				$result = $this->_start_directory();
			break;
		}
		return $result;
	}
	protected function _start_directory()
	{
		$db = $this->_tablemodel->model->db();
		# get first record
		$tablename = 'Nodes_View';
		$query = DB::select_array(array('ID','Title','ObjectType','ParentID','BlogCode','OrderText','ParentID'))
			->from($tablename);
		$this->_set_query_constraints($query,$tablename);
		$query->order_by('Title','ASC');
		$query->order_by('ID','ASC');
		$query->where('ObjectType','=','directory');
		$operation = $query
			->offset(0)
			->limit(1)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$firstrecord = $operation->current();
		} else {
			var_dump($operation->error_info()); exit;
			return;
		}
		if (!$firstrecord) return;
		$articleid = $firstrecord->ID;
		$parms = '?path=' . $this->_parameters->path;
		$redirect = 'library/directories';
		$this->_controller->request->redirect($redirect . "/$articleid$parms");
		return Node::FLOW_REDIRECT;
	}
	protected function _process_import_files()
	{
		$tokens = Services::get_objecttype_tokenset('directory');
		$access = Access::instance();
		if (!$access->permission($tokens->bulkimports,'C'))
			return $this->_record_report_access_error($datamodel);
		$context = $this->_context;
		$containerid = $context->foreignkeyvalue;
		$tablemodel = $this->_tablemodel;
		$model = $tablemodel->model;
		$errors = array();
		$messages = array();
		$db = $model->db();
		$db->lock_tables(array('Nodes_View','Nodes_Instances'));
		$files = array();
		$newcount = 0;
		$candidatecount = 0;
		try {
			$is_error = FALSE;
			$operation = DB::select('*')
				->from('Nodes_View')
				->where('ID','=',$containerid)
				->as_object()
				->prepare($db)
				->execute();
			if (!$operation->success()) {
				$error_info = $operation->error_info();
				$errors[] = array('error'=> $error_info[2]);
				$is_error = TRUE;
			} else {
				$directory_record = $operation->fetch();
				if (!$directory_record)  {
					$errors[] = array('error'=>'directory record not found');
					$is_error = TRUE;
				}
			}
			if (!$is_error) {
				$path = $directory_record->Directory;
				$repositoryid = $directory_record->RepositoryID;
				$system = System::instance();
				$public = $system->PublicBase;
				$upload = $system->UploadBase;
				$path = $public.$upload.$path;
				$filesraw = scandir($path); // get list of disk files/directories
				foreach ($filesraw as $file) {// filter for files
					if (is_file($path.$file)) {
						$files[] = $file;
					}
				}
				if (!empty($files)) { // process files
					// get list of valid file types
					$filerconfig = (object) Kohana::$config->load('filer');
					$filetypes = $filerconfig->types;
					// get tablemodel for file records
					$filemodel = new TableModel('nodeschema');
					$filetable = $filemodel->table();
					$extensiontable = $filemodel->table('extension');
					$filetable->auto_lock(FALSE);
					$fileactive = $filetable->activerecord();
					$extensionactive = $extensiontable->activerecord();
					$extensiontable->auto_lock(FALSE);
					// get binder options for new records
					$picturebinderid = Services::dblookup('Binders','ID','Code','Pictures');
					$docfilebinderid = Services::dblookup('Binders','ID','Code','DocFiles');
					// get userid
					$authorid = User::instance()->userid();
					foreach ($files as $file) { // process each file
						$filename = pathinfo($file,PATHINFO_FILENAME);
						if (strpos($filename,' ')!== FALSE) {
							$errors[] = array('error'=>'['.$file.'] contains a space and cannot be imported.');
							continue;
						}
						$filetype = $this->_get_file_type($file,$filetypes);
						if ($filetype === Filer::FILETYPE_IMAGE) {
							// filter out versions
							if (preg_match('/[.](largest|larger|large|medium|small|smaller|smallest|thumbnail)$/',$filename))
								$filetype = FALSE;
						}
						if ($filetype !== FALSE) {
							$candidatecount++;
							$filetable->preset()->query()
								->where('RepositoryID','=',$repositoryid)
								->where('FileName','=',$file);
							$record = $fileactive->current();
							if (!$record) { // insert new file record
								$newcount ++;
								// collect data for new record
								$filedate = date('Y-m-d',filemtime($path.$file));
								if ($filetype === Filer::FILETYPE_IMAGE) {
									$objecttype = 'picture';
									$binderid = $picturebinderid;
								} elseif ($filetype === Filer::FILETYPE_XDOC) {
									$objecttype = 'docfile';
									$binderid = $docfilebinderid;
								} else 
									die('unknown filetype in directorylist.php _process_import_files');
								$this->_create_file_record(
									$filedate,$file,$objecttype,$binderid,$repositoryid,$containerid,$authorid,
									$filemodel,$errors,$messages
								);
							}
						}
					}
				}
			}
		} catch (Exception $e) {
			$errors[] = array('error','System error: ' . $e->getMessage());
		}
		$db->unlock_tables();
		$count = count($files);
		$messages[] = array('message'=>
			$count . ' files processed; ' . 
			$candidatecount . ' files considered; ' . 
			$newcount . ' file records added.');
		$messagesection = compact('errors','messages');
		if (!empty($errors)) {
			$this->_datamodel->datapool->variables['messagesection'] = (object) $messagesection;
			return Node::FLOW_CONTINUE;
		} else {
			$messagesection['index'] = 'messagesection';
			Session::instance()->set('listmessages',$messagesection);
			return Node::FLOW_REDIRECT;
		}
	}
	protected function _create_file_record(
		$filedate,$file,$objecttype,$binderid,$repositoryid,$containerid,$authorid,
		$model,&$errors,&$messages)
	{
		// collect data for new record
		$db = $model->db();
		$filetable = $model->table();
		$fileactive = $model->activerecord();
		$extensionactive = $model->activerecord('extension');
		# 1. insert main Articles record
		$result = $fileactive->insert(array(
			'Title'=>$file,
			'ArticleDate'=>$filedate,
			'ObjectType'=>$objecttype,
			'BinderID'=>$binderid,
			'AuthorID'=> $authorid,
		))->save();
		# 2. insert Articles_Extensions record
		if (!$result) {
			$error_info = $fileactive->error_info();
			$errors[] = array('error'=>'record add failure for ' . $file . ': ' . $error_info[2]);
		} else {
			$newrecordid = $fileactive->insert_id();
			$filetable->preset()->query()->where('ID','=',$newrecordid);
			if ($filerecord = $fileactive->current()) {
				$result = $extensionactive->insert(array(
					'RepositoryID' => $repositoryid,
					'FileName' => $file,
				))->save();
			}
			if (!$result) {
				$error_info = $extensionactive->error_info();
				$errors[] = array('error'=>'extension add failure for ' . $file . ': ' . $error_info[2]);
			}
		}
		# 3. insert Nodes_Instances record
		if ($result) {
			$operation = DB::insert('Nodes_Instances',array('FolderID','NodeID'))
				->values(array($containerid,$newrecordid))
				->prepare($db)
				->execute();
			if (!$operation->success()) {
				$error_info = $operation->error_info();
				$errors[] = array('error'=>'instance add failure for ' . $file . ': ' . $error_info[2]);
				$result = FALSE;
			}
		}
		return $result;
	}
	protected function _get_file_type($filename,$filetypes)
	{
		$extension = pathinfo($filename,PATHINFO_EXTENSION);
		if (empty($extension)) return FALSE;
		$extension = strtolower($extension);
		foreach ($filetypes as $type) {
			if (in_array($extension,$type['extensions']))
				return $type['type_code'];
		}
		return FALSE;
	}
	protected function _process_generate_files()
	{
		$tokens = Services::get_objecttype_tokenset('directory');
		$access = Access::instance();
		if (!$access->permission($tokens->content,'E'))
			return $this->_record_report_access_error($datamodel);
		$parameters = $this->_parameters;
		$actionselections = $parameters->actionselections;
		if (!empty($actionselections))
			$actionselections = array_keys($actionselections);
		else 
			$actionselections = array();
		$regenerateversions = $parameters->regenerateversions;
		$system = System::instance();
		$public = $system->PublicBase;
		$upload = $system->UploadBase;
		$tablemodel = $this->_tablemodel;
		$model = $tablemodel->model;
		$db = $model->db();
		$filer = new Filer();
		$errors = array();
		$messages = array();
		$operation = DB::select_array(array('ID','Directory','FileName','ObjectType'))
			->from('Nodes_View')
			->where('ID','=',':selection')
			->as_object()
			->prepare($db);
		$messages[] = array('message' => count($actionselections) . ' records selected for processing.');
		$processed = 0;
		foreach ($actionselections as $selectionid) {
			$operation->execute(array(':selection'=>$selectionid));
			$record = $operation->fetch();
			if ($record) {
				$directory = $record->Directory;
				$filename = $record->FileName;
				$filespec = $public.$upload.$directory.$filename;
				$is_file = is_file($filespec);
				$do_versions = FALSE;
				if ($is_file) {
					$is_image = ($filer->get_filetype($filespec) === Filer::FILETYPE_IMAGE);
					if ($is_image) {
						if (!$regenerateversions) {
							$fileinfo = pathinfo($filespec);
							$proxyspec = $fileinfo['dirname'].'/'.$fileinfo['filename'].'.thumbnail.'.$fileinfo['extension'];
							$is_proxyfile = is_file($proxyspec);
							$do_versions = !$is_proxyfile;
						} else {
							$do_versions = TRUE;
						}
					}
				} else {
					$errors[] = array('error'=>'file not found for ' . $filename);
				}
				if ($do_versions) {
					$processed++;
					$filer->create_versions($filespec)->go();
					$filer_errors = $filer->errors();
					if (!empty($filer_errors)) {
						foreach ($filer_errors as $filer_error)
							$errors[] = array('error'=>$filer_error);
					}
					$filer_messages = $filer->messages();
					if (!empty($filer_messages)) {
						foreach ($filer_messages as $filer_message)
							$messages[] = array('message'=>$filename . ': ' . $filer_message);
					}
				}
			} else {
				$errors[] = array('error','record not found - generate picture versions - for record ' . $selectionid);
			}
		}
		$messages[] = array('message'=>$processed . ' files were processed');
		// package messages
		$messagesection = compact('errors','messages');
		if (!empty($errors)) {
			$this->_datamodel->datapool->variables['messagesection'] = (object) $messagesection;
			return Node::FLOW_CONTINUE;
		} else {
			$messagesection['index'] = 'messagesection';
			Session::instance()->set('listmessages',$messagesection);
			return Node::FLOW_REDIRECT;
		}
	}
	protected function _process_upload_files()
	{
		$tokens = Services::get_objecttype_tokenset('directory');
		$access = Access::instance();
		if (!$access->permission($tokens->bulkuploads,'C'))
			return $this->_record_report_access_error($datamodel);
		$filesdata = array();
		$files = end($_FILES);
		foreach ($files as $key => $valuearray) { // discreet arrays for each file
			foreach ($valuearray as $valuekey => $value)
			$filesdata[$valuekey][$key] = $value;
		}
		// collect resources
		$messages = array();
		$warnings = array();
		$errors = array();
		$context = $this->_context;
		$containerid = $context->foreignkeyvalue;
		$authorid = User::instance()->userid();
		$picturebinderid = Services::dblookup('Binders','ID','Code','Pictures');
		$docfilebinderid = Services::dblookup('Binders','ID','Code','DocFiles');
		$tablemodel = $this->_tablemodel;
		$model = $tablemodel->model;
		$db = $model->db();
		$parameters = $this->_parameters;
		$replace_existing = (bool) $parameters->uploadreplaceexisting; // * s/b set to boolean on datamodel processing
		$generate_versions = (bool) $parameters->uploadgenerateversions;
		$no_generate_records = (bool) $parameters->uploadnogeneraterecords;
		// find directory record
		$operation = DB::select('*') // not locking tables to accommodate long operation; unlikely to be interfered with.
			->from('Nodes_View')
			->where('ID','=',$containerid)
			->as_object()
			->prepare($db)
			->execute();
		$is_error = FALSE;
		if (!$operation->success()) {
			$error_info = $operation->error_info();
			$errors[] = array('error' => $error_info[2]);
			$is_error = TRUE;
		} else {
			$directory_record = $operation->fetch();
			if (!$directory_record) {
				$errors[] = array('error' => 'container record not found');
				$is_error = TRUE;
			}
		}
		if (!$is_error) { // get directory and repository data
			$directory = $directory_record->Directory;
			$repositoryid = $directory_record->RepositoryID;
		} else {
			$messagesection = compact('errors');
			$this->_datamodel->datapool->variables['messagesection'] = (object) $messagesection;
			return Node::FLOW_CONTINUE;
		}
		// for creation of file records...
		$filemodel = new TableModel('nodeschema');
		foreach ($filesdata as $filedata) { // process each import
			$filespec = NULL;
			$filetype = NULL;
			list($result,$filespec,$filetype) = 
				$this->_do_upload(
					$filedata, $directory, $replace_existing, $generate_versions,
					$messages, $errors, $warnings
				);
			if ($result) {
				$filedate = date('Y-m-d',filemtime($filespec));
				$file = pathinfo($filespec,PATHINFO_BASENAME);
				if ($filetype === Filer::FILETYPE_IMAGE) {
					$objecttype = 'picture';
					$binderid = $picturebinderid;
				} elseif ($filetype === Filer::FILETYPE_XDOC) {
					$objecttype = 'docfile';
					$binderid = $docfilebinderid;
				} else 
					die('unknown filetype in directorylist.php _process_import_files');
				if (!$no_generate_records) {
					$this->_create_file_record(
						$filedate,$file,$objecttype,$binderid,$repositoryid,$containerid,$authorid,
						$filemodel,$errors,$messages
					);
				} else {
					$messages[] = array('message'=>'->no record generated for '. $file);
				}
			}
		}
		$messages[] = array('message'=>count($filesdata) . ' files processed');
		$messagesection = compact('errors','messages');
		if (!empty($errors)) {
			$this->_datamodel->datapool->variables['messagesection'] = (object) $messagesection;
			return Node::FLOW_CONTINUE;
		} else {
			$messagesection['index'] = 'messagesection';
			Session::instance()->set('listmessages',$messagesection);
			return Node::FLOW_REDIRECT;
		}
	}
	protected function _do_upload($filedata, $directory, $replace_existing, $generate_versions, &$messages, &$errors, &$warnings)
	{
		$result = TRUE;
		$filespec = NULL;
		$filetype = NULL;
		$system = System::instance();
		$filer = Filer::instance();
		$result = $filer
			->reset()
			->auto_convert(TRUE) // automatically replace space etc. in filename with underscore
			->import($filedata)
			->targetbase(System::instance()->UploadBase)
			->to($directory)
			->replace_existing($replace_existing)
			->go();
		// return error data
		if ($result) {
			$filespec = $filer->filespec();
			$filetype = $filer->filetype();
			if (($filetype === Filer::FILETYPE_IMAGE) and $generate_versions) {
				$filer->create_versions($filespec)->go();
				$versionmessages = $filer->messages();
				$versionerrors = $filer->errors();
				if (!empty($versionerrors)) {
					foreach ($versionerrors as $error) {
						$errors[] = array('error' => $error);
					}
				}
				if (!empty($versionmessages)) {
					foreach ($versionmessages as $message) {
						$messages[] = array('message' => $message);
					}
				}
			}
		}
		if (!$result) {
			$filererrors = $filer->errors();
			foreach ($filererrors as $error) {
				$errors[] = array('error' => $error);
			}
		} else {
			$basename = pathinfo($filespec,PATHINFO_BASENAME);
			$messages[] = array('message' => $directory.$basename . ' was uploaded');
		}
		return array($result,$filespec,$filetype);
	}
	protected function _process_show_orphans()
	{
		$tokens = Services::get_objecttype_tokenset('directory');
		$access = Access::instance();
		if (!$access->permission($tokens->content,'R'))
			return $this->_record_report_access_error($datamodel);
		$warnings = array();
		$messages = array();
		$context = $this->_context;
		$containerid = $context->foreignkeyvalue;
		$system = System::instance();
		$public = $system->PublicBase;
		$upload = $system->UploadBase;
		$db = $this->_tablemodel->model->db();
		$operation = DB::select_array(array('NodeID','Directory','FileName')) // not locking tables to accommodate long operation; unlikely to be interfered with.
			->from('Nodes_Instances_View')
			->where('FolderID','=',$containerid)
			->as_object()
			->limit(1000)
			->prepare($db)
			->execute();
		$count = 0;
		$warningcount = 0;
		if ($operation->success()) {
			$path = $public.$upload;
			while ($record = $operation->fetch()) {
				$count++;
				$directory = $record->Directory;
				$filename = $record->FileName;
				$id = $record->NodeID;
				$filespec = $path.$directory.$filename;
				if (!is_file($filespec)) {
					$warnings[] = array('warning'=>$filename . ' ['.$id . '] not found');
					$warningcount++;
				}
			}
		}
		$messages[] = array('message'=>$count . ' records processed; ' . $warningcount . ' warnings generated');
		$messagesection = compact('warnings','messages');
		$messagesection['index'] = 'messagesection';
		Session::instance()->set('listmessages',$messagesection);
		return Node::FLOW_REDIRECT;
	}
}