<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
// ** s/b hierarchy like pages - HB
namespace Base\NodeType;

use StdClass;
use Services;
use Kohana,DB;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class DirectoryRecord extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context($selector)
	{
		$context = new StdClass();
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'folder';
		# table
		$context->mastertablename = 'master';
		$context->parenttablename = 'containers';
		# recordset
		$context->objecttype = 'directory';
		$context->binderscope = 'Directories';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->parenttableindex = 'containerstable';
		$context->parentrecordindex = 'containersrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		# selector
		if ($selector == 'setup') {
			$context->extensiontablename = 'extension';
			$context->extensiontableindex = 'extensiontable';
			$context->extensionrecordindex = 'extensionrecord';
			$context->redirect = 'library/directories/setup';
		} else {
			$context->redirect = 'library/directories';
		}
		return $context;
	}
	
	protected function _on_set_datapool($datapool)
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$containertable = $datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	// check for existing file entries for the directory...
	protected function _on_before_delete($parmdata,&$error_info)
	{
		extract($parmdata);
		$operation = DB::select('FolderID')
			->from('Nodes_Instances')
			->where('FolderID','=',$datarecord->ID)
			->as_object()
			->limit(1)
			->prepare($db)
			->execute();
		$result = $operation->success();
		if ($result) {
			$rowcount = $operation->row_count();
			if ($rowcount != 0) {
				$error_info[0] = 'delete directory';
				$error_info[1] = 'files';
				$error_info[2] = 'Files exist for this directory - cannot delete';
				$result = FALSE;
			}
		} else {
			foreach ($operation->error_info() as $key => $value) {
				$error_info[$key] = $value;
			}
		}
		return $result;
	}
}
