<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

// ** s/b part of directory hierarchy - HB

namespace Base\NodeType;

use StdClass;
use System, Services, Node, Filer;
use Kohana, Session, DB;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class DocFileRecord extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'article';
		# tablemodel
		$context->mastertablename = 'master';
		$context->containertablename = 'containers';
		$context->extensiontablename = 'extension';
		# recordsset
		$context->objecttype = 'docfile';
		$context->binderscope = 'DocFiles';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->containertableindex = 'containerstable';
		$context->containerrecordindex = 'containersrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# node
		$context->redirect =  'library/docfiles';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	// presumes success; otherwise reconcile -- HB
	protected function _on_after_update($params,&$error_info)
	{
		extract($params);
		if ($priorrecord->RepositoryID !== $activerecord->RepositoryID) {
			$priordirectoryrecordid = NULL;
			$activedirectoryrecordid = NULL;
			$prior_repositoryid = $priorrecord->RepositoryID;
			$active_repositoryid = $activerecord->RepositoryID;
			$db = $this->_tablemodel->model->db();
			// get prior directory record id
			$operation = DB::select('ID')
				->from('Nodes_View')
				->where('ObjectType','=','directory')
				->where('RepositoryID','=',$prior_repositoryid)
				->as_object()
				->prepare($db)
				->execute();
			if ($operation->success()) {
				$record = $operation->fetch();
				if ($record) {
					$priordirectoryrecordid = $record->ID;
				}
			}
			// get active directory record id
			$operation = DB::select('ID')
				->from('Nodes_View')
				->where('ObjectType','=','directory')
				->where('RepositoryID','=',$active_repositoryid)
				->as_object()
				->prepare($db)
				->execute();
			if ($operation->success()) {
				$record = $operation->fetch();
				if ($record) {
					$activedirectoryrecordid = $record->ID;
				}
			}
			$recordid = $activerecord->ID;
			if (!is_numeric($recordid)) return TRUE;
			if (is_numeric($priordirectoryrecordid)) {
				$operation = DB::delete('Nodes_Instances')
					->where('FolderID','=',$priordirectoryrecordid)
					->where('NodeID','=',$recordid)
					->prepare($db)
					->execute();
			}
			if (is_numeric($activedirectoryrecordid)) {
				$operation = $operation = DB::insert('Nodes_Instances',array('FolderID','NodeID'))
					->values(array($activedirectoryrecordid,$recordid))
					->prepare($db)
					->execute();
			}
		}
		return TRUE;
	}
	protected function _on_after_insert($params,&$error_info)
	{
		extract($params);
		$record = $activerecord->current();
		if (!$record) return TRUE;
		$db = $this->_tablemodel->model->db();
		$activedirectoryrecordid = NULL;
		$active_repositoryid = $activerecord->RepositoryID;
		// get active directory record id
		$operation = DB::select('ID')
			->from('Nodes_View')
			->where('ObjectType','=','directory')
			->where('RepositoryID','=',$active_repositoryid)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$record = $operation->fetch();
			if ($record) {
				$activedirectoryrecordid = $record->ID;
			}
		}
		$recordid = $activerecord->ID;
		if (!is_numeric($recordid)) return TRUE;
		if (is_numeric($activedirectoryrecordid)) {
			$operation = $operation = DB::insert('Nodes_Instances',array('FolderID','NodeID'))
				->values(array($activedirectoryrecordid,$recordid))
				->prepare($db)
				->execute();
		}
		return TRUE;
	}
	protected function _on_set_datapool($datapool)
	{
		$tablemodel = $this->_tablemodel;
		$context = $this->_context;
		$containertable = $datapool->tables[$context->containertableindex] = 
			$tablemodel->model->table($context->containertablename);
		$datapool->activerecords[$context->containerrecordindex] = 
			$tablemodel->model->activerecord($context->containertablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		$datarecord = $this->_tablemodel->datarecord;
		if (empty($datarecord)) {
			$dictionary['file_exists'] = FALSE;
			return;
		}
		$filename = $datarecord->FileName;
		$fileext = strtolower(pathinfo($filename,PATHINFO_EXTENSION));
		$dictionary['fileextension'] = '.' . $fileext;
		$dictionary['is_pdf'] = ($fileext == 'pdf');
		$directory = $datarecord->Directory;
		$base = $this->_controller->_base;
		$publicbase = System::instance()->PublicBase;
		$uploadbase = System::instance()->UploadBase;
		$filespec = $publicbase.$uploadbase.$directory.$filename;
		$exists = $dictionary['file_exists'] = is_file($filespec);
		if ($exists) {
			$filespec = $dictionary['filespec'] = $base.$filespec;
		}
	}
	protected function _on_respond($command)
	{
		$flow = self::FLOW_CONTINUE; // don't repeat process
		switch ($command) {
			case 'uploaddocfile':
				$flow = $this->_upload_docfile();
				break;
			case 'changearticle':
				$flow = $this->_change_article();
				break;
		}
		
		return $flow;
	}
	protected function _upload_docfile()
	{
		$tokens = $this->_tokens;
		if (!$tokens->import->create)
			return $this->_record_report_access_error($datamodel);
		$retval = Node::FLOW_REDIRECT;
		$datamodel = $this->_datamodel;
		$post = &$datamodel->datapool->arrays['post'];
		$parameters = $this->_parameters;
		$files = $datamodel->datapool->arrays['files'];
		$errors = array();
		$result = TRUE;
		if (!empty($files['file']['name'])) {
			$sourcefilename = $files['file']['name'];
		} else {
			$errors[] = array('error'=>'No source file to upload');
			$result = FALSE;
		}
		if ($result) {
			$targetfilename = $post['filename'];
			(empty($targetfilename)) and ($targetfilename = $sourcefilename);
			$targetfilename = strtr($targetfilename,' ','_');
			$post['filename'] = $targetfilename;
			$repositoryid = $post['repositoryid'];
			($repositoryid == '') and ($repositoryid = NULL);
		}
		if ($this->_record_save($datamodel) == Node::FLOW_CONTINUE) {
			$modelvars = &$this->_datamodel->datapool->variables;
			$messagesection = isset($modelvars['messagesection'])?$modelvars['messagesection']:array();
			$messagesection->warnings[]['warning'] = 'file was not uploaded due to field error';
			$modelvars['messagesection'] = $messagesection;
			return Node::FLOW_CONTINUE; // problem
		} else {
			(!$this->_states->insertmode) or ($this->_states->insertmode = FALSE);
		}
		$messagesection = $this->_messages; // ** HB - error: _messages currently not initialized
		$messages = $messagesection['messages'];
		if ($result) {
			if (!is_null($repositoryid)) {
				$subdirectory = Services::dblookup('System_Repositories','Directory','ID',$repositoryid);
				if (is_null($subdirectory)) {
					$result = FALSE;
					$errors[] = array('error' => 'invalid Directory');
				}
			} else {
				$subdirectory = NULL;
			}
			$replace_existing = (bool) $parameters->uploadreplaceexisting;
			// now have key data: $files (source),$targetfilename,$subdirectory,$replace_existing
			if ($result) {
				$this->_do_upload($files, $targetfilename, $subdirectory, $replace_existing, $messages, $errors);
			}
		}
		$datamodel = $this->_datamodel;
		$datamodel->dataframe->reset(); // re-collect data for edit
		unset($messagesection['index']);
		$messagesection['messages'] = $messages;
		if (!empty($errors)) $messagesection['errors'] = $errors;
		 // get rid of previous messageset
		$session = Session::instance();
		$messages = $session->get_once('messages');
		$datamodel->datapool->variables['messagesection'] = (object) $messagesection;
		return $this->_record_edit($datamodel);
	}
	protected function _do_upload($files, $targetfilename, $subdirectory, $replace_existing, &$messages, &$errors)
	{
		$result = TRUE;
		$filer = Filer::instance();
		$filedata = end($files); // only one
		$result = $filer
			->import($filedata)
			->as_docfile()
			->targetbase(System::instance()->UploadBase)
			->to($subdirectory)
			->as_name($targetfilename)
			->replace_existing($replace_existing)
			->go();
		// return error data
		if (!$result) {
			$filererrors = $filer->errors();
			foreach ($filererrors as $error) {
				$errors[] = array('error' => $error);
			}
		} else {
			$messages[] = array('message' => $targetfilename . ' was uploaded');
		}
		return $result;
	}
	protected function _change_article()
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		$flow = self::FLOW_REDIRECT;
		$parameters = $this->_parameters;
		$changetarget = $parameters->changetarget;
		$model = $this->_tablemodel->model;
		$activerecord = $model->activerecord();
		$extensionrecord = $model->activerecord('extension');
		switch ($changetarget) {
			case 'document':
				$binderid = Services::dblookup('Binders','ID','Code','DocumentsBinder');
				break;
			case 'docextract':
				$binderid = Services::dblookup('Binders','ID','Code','DocExtracts');
				break;
			case 'link':
				$binderid = Services::dblookup('Binders','ID','Code','Links');
				$activerecord->Source = NULL;
				break;
			default:
				return $flow;
		}
		if (is_null($binderid)) return $flow;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$extensionrecord->FileName = NULL;
		$extensionrecord->RepositoryID = NULL;
		$extensionrecord->save();
		$activerecord->ObjectType = $changetarget;
		$activerecord->BinderID = $binderid;
		$activerecord->save();
		$db->unlock_tables();
		return $flow;
	}
}