<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class DocumentRecord extends RecordNode {

	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'article';
		# tablemodel
		$context->mastertablename = 'master';
		$context->containertablename = 'containers';
		$context->extensiontablename = 'extension';
		# recordsset
		$context->objecttype = 'document';
		$context->binderscope = 'Documents';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->containertableindex = 'containerstable';
		$context->containerrecordindex = 'containersrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# node
		$context->redirect =  'documents';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	protected function _on_set_datapool($datapool)
	{
		$tablemodel = $this->_tablemodel;
		$context = $this->_context;
		$containertable = $datapool->tables[$context->containertableindex] = 
			$tablemodel->model->table($context->containertablename);
		$datapool->activerecords[$context->containerrecordindex] = 
			$tablemodel->model->activerecord($context->containertablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	protected function _on_respond($command)
	{
		$flow = self::FLOW_CONTINUE; // don't repeat process
		switch ($command) {
			case 'changearticle':
				$flow = $this->_change_article();
				break;
		}
		
		return $flow;
	}
	protected function _change_article()
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		$flow = self::FLOW_REDIRECT;
		$parameters = $this->_parameters;
		$changetarget = $parameters->changetarget;
		$model = $this->_tablemodel->model;
		$activerecord = $model->activerecord();
		switch ($changetarget) {
			case 'folder':
				$binderid = Services::dblookup('Binders','ID','Code','TopicalFolders');
				break;
			case 'docfile':
				$binderid = Services::dblookup('Binders','ID','Code','DocFiles');
				break;
			case 'docextract':
				$binderid = Services::dblookup('Binders','ID','Code','DocExtracts');
				break;
			case 'link':
				$binderid = Services::dblookup('Binders','ID','Code','Links');
				$activerecord->Source = NULL;
				break;
			default:
				return $flow;
		}
		if (is_null($binderid)) return $flow;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$activerecord->ObjectType = $changetarget;
		$activerecord->BinderID = $binderid;
		$activerecord->save();
		$db->unlock_tables();
		return $flow;
	}
}