<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

/*
	doesn't have context->datapoolarrays
*/

namespace Base\NodeType;

use StdClass;
use Services, Access;
use Kohana, DB;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class EntryRecord extends RecordNode {
	public function __construct($controller)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context();
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'blog';
		# tablemodel
		$context->mastertablename = 'master';
		$context->parenttablename = 'containers';
		$context->extensiontablename = 'extension';
		$context->blogtablename = 'blog';
		# recordset
		$context->objecttype = 'entry';
		$context->binderscope = 'BlogItems';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->parenttableindex = 'containerstable';
		$context->parentrecordindex = 'containersrecord';
		$context->blogtableindex = 'blogtable';
		$context->blogrecordindex = 'blogrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);

		return $context;
	}
	protected function _on_set_permissions($permissions) 
	{
		// inherit blog permissions
		$parentid = $this->_context->parentid;
		$user = $this->_user;
		if ($user->authenticated()) {
			if ($this->_user->user_data()->Permissions->blog) {
				if (!$permissions->canadd) {
					$create_permission = Services::get_article_permission($parentid,'C');
					$permissions->canadd = ($permissions->canadd or $create_permission);
				}
				if (!$permissions->canedit) {
					$edit_permission = Services::get_article_permission($parentid,'E');
					$permissions->canedit = ($permissions->canedit or $edit_permission);
				}
				if (!$permissions->cantrash) {
					$trash_permission = Services::get_article_permission($parentid,'D');
					$permissions->cantrash = ($permissions->cantrash or $trash_permission);
				}
				if (!$permissions->candelete) {
					$delete_permission = Services::get_article_permission($parentid,'D','settings');
					$permissions->candelete = ($permissions->candelete or $delete_permission);
				}
			}
		}
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$containertable = $datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		$datapool->activerecords[$context->blogrecordindex] = 
			$tablemodel->model->activerecord($context->blogtablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	// set parentid of new blog post
	protected function _on_before_insert($parmdata,&$error_info)
	{
		extract($parmdata);
		$context = $this->_context;
		if (!is_null($context->parentid)) {
			$extensionrecord->ParentID = $context->parentid;
		} else {
			$error_info[0] = 'entry error';
			$error_info[1] = 'no parent id';
			$error_info[2] = 'system error: parentid not found for entry';
			return FALSE;
		}
		return TRUE;
	}
	// show current blog for new blog post record
	protected function _on_get_dictionary(&$dictionary)
	{
		$states = $this->_states;
		if ($states->insertmode) { // get blog record data
			$blogid = $this->_context->parentid;
			$record = DB::select('ID','Title','BlogCode')
				->from('Nodes_View')
				->where('ID','=',$blogid)
				->as_object()
				->execute()
				->current();
			if ($record) {
				$blogrecord = $dictionary['narrativesection']->blogrecord;
				$blogrecord->id = $record->ID;
				$blogrecord->title = $record->Title;
				$blogrecord->blogcode = $record->BlogCode;
			}
		}
	}
}