<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana;
use Node,DB;
use Muster\Node\ListNode;

defined('SYSPATH') or die('No direct script access.');

class Favourites extends ListNode {
	public function __construct($controller)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context();
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'favouritesschema';
		$context->controlframe = 'nodecontrols';
		$context->dataframe = 'nodedata';
		$context->messagefile = 'favourites'; // not used
		# tablemodel
		$context->mastertablename = 'master';
		$context->blogtablename = 'blog';
		$context->parenttablename = 'containers';
		$context->pageparenttablename = 'pageparent';
		$context->commentarticletablename = 'commentarticle';
		# recordset
		$context->selectarray = array('ObjectType','ParentID','PageParentID','EmptyBody','isTreeTop','DefaultFolio','EditDate','EditUserID','BlogCode','FolderCode','Body','CommentArticleID','Source','URL','FileName','Directory','Parameters');
		$context->binderscope = NULL;
		$context->foreignkeyname = 'UserID';
		$context->default_orderby = 'Title';
		$context->default_orderdirection = 'ASC';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->parenttableindex = 'containerstable';
		$context->parentrecordindex = 'containersrecord';
		$context->blogtableindex = 'blogtable';
		$context->blogrecordindex = 'blogrecord';
		$context->pageparenttableindex = 'pageparenttable';
		$context->pageparentrecordindex = 'pageparentrecord';
		$context->commentarticletableindex = 'commentarticletable';
		$context->commentarticlerecordindex = 'commentarticlerecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		$context->constraints = array(
			array(
				'index' => 'showblogparent',
				'value' => TRUE,
			),
			array(
				'index' => 'showpageparent',
				'value' => TRUE,
			),
			array(
				'index' => 'showbody',
				'value' => TRUE,
			),
			array(
				'index' => 'commentarticles',
				'value' => 'required',
			),
			array(
				'index' => 'node',
				'value' => 'bookmarks',
			),
		);
		# node
		$context->redirect = 'favourites';
		# permissions
		$context->tokens = Services::get_token_set('mixed');
		return $context;
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		$datapool->activerecords[$context->blogrecordindex] = 
			$tablemodel->model->activerecord($context->blogtablename);
		$datapool->activerecords[$context->pageparentrecordindex] = 
			$tablemodel->model->activerecord($context->pageparenttablename);
		$datapool->tables[$context->commentarticletableindex] = 
			$tablemodel->model->table($context->commentarticletablename);
		$datapool->activerecords[$context->commentarticlerecordindex] = 
			$tablemodel->model->activerecord($context->commentarticletablename);
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		$dictionary['loading_bookmarks'] = TRUE;
	}
	protected function _on_respond($command)
	{
		$result = Node::FLOW_CONTINUE;
		switch ($command) {
			case 'clearbookmarks':
				$result = $this->_clearbookmarks();
				break;
		}
		return $result;
	}
	protected function _clearbookmarks()
	{
		$userid = $this->_user->userid();
		$db = $this->_tablemodel->model->db();
		if (is_null($userid)) Node::FLOW_REDIRECT;
		DB::delete('Articles_Favourites')
			->where('UserID','=',$userid)
			->prepare($db)
			->execute();
		return Node::FLOW_REDIRECT;
	}
}