<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class FileRecord extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context($selector)
	{
		$context = new StdClass();
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'folder';
		# table
		$context->mastertablename = 'master';
		$context->parenttablename = 'containers';
		# recordset
		$context->objecttype = 'file';
		$context->binderscope = 'LibraryFiles';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->parenttableindex = 'containerstable';
		$context->parentrecordindex = 'containersrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		# selector
		if ($selector == 'setup') {
			$context->extensiontablename = 'extension';
			$context->extensiontableindex = 'extensiontable';
			$context->extensionrecordindex = 'extensionrecord';
			$context->redirect = 'library/files/setup';
		} else {
			$context->redirect = 'library/files';
		}
		return $context;
	}
	
	protected function _on_set_datapool($datapool)
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$containertable = $datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	protected function _on_respond($command)
	{
		$flow = self::FLOW_CONTINUE; // don't repeat process
		switch ($command) {
			case 'changearticle':
				$flow = $this->_change_article();
				break;
		}
		
		return $flow;
	}
	protected function _change_article()
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		$flow = self::FLOW_REDIRECT;
		$parameters = $this->_parameters;
		$changetarget = $parameters->changetarget;
		$model = $this->_tablemodel->model;
		$activerecord = $model->activerecord();
		$id = $activerecord->ID;
		switch ($changetarget) {
			case 'folder':
				$binderid = Services::dblookup('Binders','ID','Code','TopicalFolders');
				break;
			default:
				return $flow;
		}
		if (is_null($binderid)) return $flow;
		$table = $activerecord->table();
		$table->auto_lock(FALSE);
		$db = $table->tabledef()->model()->db();
		$db->lock_tables(array($table->tablename(),'Articles_Authors'));
		$activerecord->ObjectType = $changetarget;
		$activerecord->BinderID = $binderid;
		$activerecord->save();
		$db->unlock_tables();
		return $flow;
	}
}
