<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana, DB, Session;
use Muster\Node\ListNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class FolderList extends ListNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context($selector)
	{
		$context = new StdClass;
		// common...
		// recordset
		$context->objecttype = 'folder';
		$context->default_orderby = 'Title';
		$context->default_orderdirection = 'ASC';
		// gateway
		$context->controlframe = 'nodecontrols';
		$context->messagefile = 'folder';
		
		// differences...
		switch ($selector) {
			case 'list': {
				// tablemodel
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->selectarray = array('ObjectType','EmptyBody','isTreeTop','DefaultFolio','EditDate','EditUserID','FolderCode',);
				$context->binderscope = 'TopicalFolders';
				//gateway
				$context->dataframe = 'nodedata';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// node
				$context->redirect = 'topics';
				$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
				break;
			}
			case 'contents': {
				// tablemodel
				$context->tablemodel = 'nodeinstanceschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				$context->blogtablename = 'blog';
				// recordset
				$context->selectarray = array('ObjectType','InstanceID','NodeID','EmptyBody','isTreeTop','BlogCode','ParentID','FolderCode','Approval','Order','OrderText','DividerAbove','DividerBelow','InstanceDescription','Source','URL','FileName','Directory');
				$context->binderscope = NULL; // mixed
				$context->foreignkeyname = 'FolderID';
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->blogtableindex = 'blogtable';
				$context->blogrecordindex = 'blogrecord';
				$context->constraints = array(
					array('index' => 'contentdisplay','value'=>TRUE),
					array('index' => 'showblogparent','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// redirect extended by controller
				$context->tokens = Services::get_token_set('mixed');
				break;
			}
			case 'treetops': {
				// tablemodel
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->treetopsflag = TRUE;
				$context->selectarray = array('ObjectType','EmptyBody','isTreeTop','DefaultFolio','FolderCode','Order','OrderText','OrderTextRaw','OrderGuide','DividerAbove','DividerBelow',);
				$context->binderscope = 'TopicalFolders';
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->constraints = array(
					array('index' => 'treetop','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// session
				$context->redirect = 'topics/treetops';
				$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
				break;
			}
			case 'treetopsadmin': {
				// tablemodel
				$context->controlframe = 'contentadmincontrols';
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->treetopsflag = TRUE;
				$context->selectarray = array('ObjectType','EmptyBody','isTreeTop','DefaultFolio','FolderCode','Order','OrderText','OrderTextRaw','OrderGuide','DividerAbove','DividerBelow',);
				$context->binderscope = 'TopicalFolders';
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->constraints = array(
					array('index' => 'treetopadmin','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// session
				$context->redirect = 'topics/treetops/admin';
				$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
				break;
			}
			case 'binders': {
				// model
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'articles';
				$context->parenttablename = 'parents';
//				$context->blogtablename = 'blog';
				// recordset
				$context->selectarray = array('InstanceID','NodeID','EmptyBody','ObjectType','BlogCode','ParentID','FolderCode');
				$context->binderscope = NULL;
				// gateway
				$context->dataframe = 'articlesdata';
				$context->parenttableindex = 'parentstable';
				$context->parentrecordindex = 'parentsrecord';
				$context->mastertableindex = 'articlestable';
				$context->masterrecordindex = 'articlesrecord';
//				$context->blogtableindex = 'blogtable';
//				$context->blogrecordindex = 'blogrecord';
				$context->constraints = array(
					array('index' => 'contentdisplay','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				$context->tokens = Services::get_token_set('mixed');
				// session
				// redirect extended by controller
				break;
			}
		}
		return $context;
	}
	protected function _on_set_query($query)
	{
		$context = $this->_context;
		if (!empty($context->treetopsflag)) {
			$query->where_open()->where('isTreeTop','IS',DB::expr('TRUE'))->where_close();
		}
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		if (property_exists($context,'blogrecordindex')) {
			$datapool->activerecords[$context->blogrecordindex] = 
				$tablemodel->model->activerecord($context->blogtablename);
		}
	}
	protected function _on_respond($command)
	{
		$result = Node::FLOW_CONTINUE;
		switch ($command) {
			case 'starttreetop':
				$result = $this->_start_treetop();
				break;
			case 'savecontents':
				$result = $this->_savetreetops();
				break;
			case 'renumber':
				$result = $this->_renumbertreetops();
				break;
		}
		return $result;
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		$parameters = $this->_parameters;
		if ((!empty($parameters->listview)) and ($parameters->listview == 'narrative')) {
			$dictionary['listview'] = 'narrative';
			$dictionary['is_narrativeview'] = TRUE;
		}
	}
	protected function _start_treetop()
	{
		$db = $this->_tablemodel->model->db();
		# get first record
		$tablename = 'Nodes_View';
		$query = DB::select_array(array('ID','Title','ObjectType','ParentID','BlogCode','OrderText','ParentID'))
			->from($tablename);
		$this->_set_query_constraints($query,$tablename);
		$query->order_by(DB::expr('ISNULL(`Order`)'),'ASC');
		$query->order_by('Order','ASC');
		$query->order_by('Title','ASC');
		$query->order_by('ID','ASC');
		$query->where('isTreeTop','IS',DB::expr('TRUE'));
		$query->where('ObjectType','=','folder');
		$operation = $query
			->offset(0)
			->limit(1)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$firstrecord = $operation->current();
		} else {
			var_dump($operation->error_info()); exit;
			return;
		}
		if (!$firstrecord) return;
		$articleid = $firstrecord->ID;
		$parms = '?path=' . $this->_parameters->path;
		$redirect = 'topics';
		$this->_controller->request->redirect($redirect . "/$articleid$parms");
		return Node::FLOW_REDIRECT;
	}
	protected function _savetreetops()
	{
		$permissions = $this->_permissions;
		if (!$permissions->canedit)
			return $this->_record_report_access_error($datamodel);
		$parameters = $this->_parameters;
		$is_narrativeview = ($parameters->listview == 'narrative');
		$recordset = $parameters->recordset;
		$db = $this->_tablemodel->model->db();
		$errors = array();
		$warnings = array();
		$posted = 'records ';
		$db->lock_tables(array('Nodes_View'));
		$is_errors = FALSE;
		foreach ($recordset as $record) {
			unset($record['InstanceID']);
			$nodeid = $record['NodeID'];
			if (!is_numeric($nodeid)) continue; // being careful
			unset($record['NodeID']);
			foreach ($record as &$value) {
				$value = trim($value);
				if ($value === '') $value = NULL;
			}
			unset($value);
			if (!$is_narrativeview) { // order data
				$record['DividerAbove'] = empty($record['DividerAbove'])?FALSE:TRUE;
				$record['DividerBelow'] = empty($record['DividerBelow'])?FALSE:TRUE;
				if (!is_null($record['Order']) and !is_numeric($record['Order'])) {
					$record['Order'] = 0;
					$warnings[] = array('warning' => 'Order for [' . $nodeid . '] was not numeric - changed to "0"');
				}
				if (strlen($record['OrderText']) > 60) {
					$record['OrderText'] = substr($record['OrderText'],0,60);
					$warnings[] = array('warning' => 'Order Text for [' . $nodeid . '] was truncated to 60 characters');
				}
			} else { // narrative data
				$record['OrderGuide'] = $record['InstanceDescription'];
				unset($record['InstanceDescription']);
				if (strlen($record['OrderGuide']) > 255) {
					$record['OrderGuide'] = substr($record['OrderGuide'],0,255);
					$warnings[] = array('warning' => 'Narrative for [' . $nodeid . '] was truncated to 255 characters');
				}
			}
			$operation = DB::update('Articles_Extensions')
				->set($record)
				->where('ArticleID','=',$nodeid)
				->prepare($db)
				->execute();
			$success = $operation->success();
			if (!$success) {
				$is_errors = TRUE;
				$err = $operation->error_info(); $err = $err[2];
				$message = '[' . $nodeid . '] ' . $err;
				$errors[] = array('error' => $message);
			} else {
//				$posted .= '[' . $nodeid . '] ';
			}
		} // end of foreach $record
		$db->unlock_tables();
		if (!empty($posted))
			$messages = array(array('message' => $posted . 'were updated'));
		else
			$messages = array();
		if ($is_errors) {
			$messagesection = (object) array(
				'messages' => $messages,
				'errors' => $errors,
				'warnings' => $warnings,
			);
			$datamodel->datapool->variables['messagesection'] = $messagesection;
			return Node::FLOW_CONTINUE;
		} else { // redirect
			$session = Session::instance();
			$messagearray = array(
				'messages' => $messages,
				'warnings' => $warnings,
				'index'=>'messagesection',
			);
			$session->set('listmessages',$messagearray);
			return Node::FLOW_REDIRECT;
		}
	}
	protected function _renumbertreetops()
	{
		$permissions = $this->_permissions;
		if (!$permissions->canedit)
			return $this->_record_report_access_error($datamodel);
		$tablemodel = $this->_tablemodel;
		$context = $this->_context;
		$db = $tablemodel->model->db();
		$db->lock_tables(array('Nodes_View'));
		try {
			$operation = DB::select('ID')
				->from('Nodes_View')
				->where('IsTreeTop','IS',DB::expr('TRUE'))
				->where('ObjectType','=','folder')
				->where('Order','IS',DB::expr('NOT NULL'))
				->order_by('Order','DESC')
				->order_by('Title','DESC')
				->as_object()
				->prepare($db)
				->execute();
			if (!$operation->success()) {
				$db->unlock_tables();
				$messagesection = (object) array(
					'errors' => array(array('error' => 'failure to find records for sort')),
				);
				$datamodel->datapool->variables['messagesection'] = $messagesection;
				return Node::FLOW_CONTINUE;
			}
			$count = $operation->row_count();
			while ($record = $operation->fetch()) {
				if (!is_null($record->ID)) {
					$update = DB::update('Articles_Extensions')
						->set(array('Order'=>$count))
						->where('ArticleID','=',$record->ID)
						->prepare($db)
						->execute();
					if (!$update->success()) {
						$db->unlock_tables();
						$messagesection = (object) array(
							'errors' => array(array('error' => 'unexpected failure to update record order: treetops')),
						);
						$datamodel->datapool->variables['messagesection'] = $messagesection;
						return Node::FLOW_CONTINUE;
					}
				}
				$count--;
			}
		} catch (Exception $e) {}
		$db->unlock_tables();
		$session = Session::instance();
		$messagearray = array(
			'messages' => array(array('message'=>'items have been renumbered')),
			'index'=>'messagesection',
		);
		$session->set('listmessages',$messagearray);
		return Node::FLOW_REDIRECT;
	}
}