<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana, DB;
use Muster\Node\ListNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class InboxList extends ListNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context($selector)
	{
		$context = new StdClass();
		// common
		// recordset
		$context->objecttype = 'inbox';
		$context->default_orderby = 'Title';
		$context->default_orderdirection = 'ASC';
		// gateway
		$context->controlframe = 'nodecontrols';
		$context->messagefile = 'inbox'; // not used

		// security
		switch ($selector) {
			case 'list': {
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->selectarray = array('ObjectType','EmptyBody','isTreeTop','DefaultFolio','EditDate','EditUserID');
				$context->binderscope = 'InBoxes';
				// gateway
				$context->dataframe = 'nodedata';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// node
				$context->redirect = 'library/inboxes';
				$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
			}
			break;
			case 'contents': {
				// tablemodel
				$context->tablemodel = 'nodeinstanceschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				$context->blogtablename = 'blog';
				// recordset
				$context->selectarray = array('ObjectType','InstanceID','NodeID','EmptyBody','isTreeTop','BlogCode','ParentID','FolderCode','Approval','Order','OrderText','DividerAbove','DividerBelow','InstanceDescription','Source','URL','FileName','Directory');
				$context->binderscope = NULL;
				$context->foreignkeyname = 'FolderID';
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->blogtableindex = 'blogtable';
				$context->blogrecordindex = 'blogrecord';
				$context->constraints = array(
					array('index' => 'contentdisplay','value'=>TRUE),
					array('index' => 'showblogparent','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// redirect extended by controller
				$context->tokens = Services::get_token_set('mixed');
			}
			break;
			case 'treetops': {
				// tablemodel
				$context->tablemodel = 'nodeschema';
				$context->mastertablename = 'master';
				$context->parenttablename = 'containers';
				// recordset
				$context->treetopsflag = TRUE;
				$context->selectarray = array('ObjectType','EmptyBody','isTreeTop','DefaultFolio','FolderCode');
				$context->binderscope = 'InBoxes';
				// gateway
				$context->dataframe = 'nodedata';
				$context->parenttableindex = 'containerstable';
				$context->parentrecordindex = 'containersrecord';
				$context->mastertableindex = 'mastertable';
				$context->masterrecordindex = 'masterrecord';
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// session
				$context->redirect = 'library/inboxes/treetops';
				$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
			}
			break;
			case 'binders': {
				// model
				$context->tablemodel = 'nodeinstancesschema';
				$context->mastertablename = 'articles';
				$context->parenttablename = 'parents';
//				$context->blogtablename = 'blog';
				// recordset
				$context->selectarray = array('InstanceID','NodeID','EmptyBody','ObjectType','BlogCode','ParentID','FolderCode');
				$context->binderscope = NULL;
				// gateway
				$context->dataframe = 'articlesdata';
				$context->parenttableindex = 'parentstable';
				$context->parentrecordindex = 'parentsrecord';
				$context->mastertableindex = 'articlestable';
				$context->masterrecordindex = 'articlesrecord';
//				$context->blogtableindex = 'blogtable';
//				$context->blogrecordindex = 'blogrecord';
				$context->constraints = array(
					array('index' => 'contentdisplay','value'=>TRUE),
				);
				$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
				// session
				// redirect extended by controller
				$context->tokens = Services::get_token_set('mixed');
			}
			break;
		}
		return $context;
	}
	protected function _on_set_query($query)
	{
		$context = $this->_context;
		if (!empty($context->treetopsflag)) {
			$query->where_open()->where('isTreeTop','IS',DB::expr('TRUE'))->where_close();
		}
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		if (property_exists($context,'blogrecordindex')) {
			$datapool->activerecords[$context->blogrecordindex] = 
				$tablemodel->model->activerecord($context->blogtablename);
		}
	}
	protected function _on_respond($command)
	{
		$result = Node::FLOW_CONTINUE;
		switch ($command) {
			case 'starttreetop':
				$result = $this->_start_treetop();
				break;
		}
		return $result;
	}
	protected function _start_treetop()
	{
		$db = $this->_tablemodel->model->db();
		# get first record
		$tablename = 'Nodes_View';
		$query = DB::select_array(array('ID','Title','ObjectType','ParentID','BlogCode','OrderText','ParentID'))
			->from($tablename);
		$this->_set_query_constraints($query,$tablename);
		$query->order_by(DB::expr('ISNULL(`Order`)'),'ASC');
		$query->order_by('Order','ASC');
		$query->order_by('Title','ASC');
		$query->order_by('ID','ASC');
		$query->where('isTreeTop','IS',DB::expr('TRUE'));
		$query->where('ObjectType','=','inbox');
		$operation = $query
			->offset(0)
			->limit(1)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$firstrecord = $operation->current();
		} else {
			var_dump($operation->error_info()); exit;
			return;
		}
		if (!$firstrecord) return;
		$articleid = $firstrecord->ID;
		$parms = '?path=' . $this->_parameters->path;
		$redirect = 'library/inboxes';
		$this->_controller->request->redirect($redirect . "/$articleid$parms");
	}
}