<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana, DB;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class InboxRecord extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller,$selector);
	}
	protected function _get_context($selector)
	{
		$context = new StdClass();
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'folder';
		# tablemodel
		$context->mastertablename = 'master';
		$context->parenttablename = 'containers';
		# recordset
		$context->objecttype = 'inbox';
		$context->binderscope = 'InBoxes';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->parenttableindex = 'containerstable';
		$context->parentrecordindex = 'containersrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		# selector
		if ($selector == 'setup') {
			$context->extensiontablename = 'extension';
			$context->extensiontableindex = 'extensiontable';
			$context->extensionrecordindex = 'extensionrecord';
			$context->redirect = 'library/inboxes/setup';
		} else {
			$context->redirect = 'library/inboxes';
		}
		return $context;
	}
	// @returns void
	protected function _on_set_datapool($datapool)
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$containertable = $datapool->tables[$context->parenttableindex] = 
			$tablemodel->model->table($context->parenttablename);
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	protected function _on_before_insert($parmdata,&$error_info)
	{
		extract($parmdata);
		$masterrecord->Approval = 'A';
		$masterrecord->Hide = TRUE;
		$masterrecord->Locked = TRUE;
		$masterrecord->CommentsClosed = TRUE;
		$masterrecord->ShowByline = TRUE;
		$states = $this->_states;
		$states->hidden = TRUE;
		$states->published = TRUE;
		$states->locked = TRUE;
		$states->commentsclosed = TRUE;
		return TRUE;
	}
	protected function _on_set_delete_locktables(&$locktables)
	{
		$locktables[] = 'Access_Users';
	}
	protected function _on_before_delete($parmdata,&$error_info)
	{
		extract($parmdata);
		$inboxid = $datarecord->ID;
		$operation = DB::update('Access_Users')
			->set(array('InboxID' => DB::expr('NULL')))
			->where('InboxID','=',$inboxid)
			->prepare()
			->execute();
		if ($operation->success()) {
			return TRUE;
		} else {
			$error_info = $operation->error_info();
			return FALSE;
		}
	}
}