<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Kohana, DB, Session;
use Services;
use Muster\Node\ListNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class ItemsList extends ListNode {
	public function __construct($controller,$selector = NULL,$subcontext = NULL) // ** $subcontext here is a hack - HB
	{
		$this->_context = $this->_get_context($selector,$subcontext);
		parent::__construct($controller);
	}
	protected function _get_context($selector,$subcontext = NULL)
	{
		$context = new StdClass;
		#resources
		if ($selector == 'contents') {
			$context->tablemodel = 'itemslistschema';
			$context->foreignkeyname = 'FolderID';
			$context->binderscope = NULL;
			$context->selectarray = array('InstanceID','Order','isTreeTop');
		} elseif ($selector == 'treetops') {
			$itempath = $subcontext->itempath;
			switch ($itempath) {
				case 'treetops-':
					$context->binderscope = 'TopicalFolders';
					$context->objecttype = 'folder';
					$context->treetopsflag = TRUE;
					break;
				case 'ftreetops-':
					$context->binderscope = 'LibraryFiles';
					$context->objecttype = 'file';
					$context->treetopsflag = TRUE;
					break;
				case 'itreetops-':
					$context->binderscope = 'InBoxes';
					$context->objecttype = 'inbox';
					$context->treetopsflag = TRUE;
					break;
				case 'directories-':
					$context->binderscope = 'Directories';
					$context->objecttype = 'directory';
					break;
			}
			$context->tablemodel = 'itemslistnodeschema';
			$context->selectarray = array('Order','isTreeTop');
		}
		$context->dataframe = 'itemslistdata';
		$context->controlframe = 'itemslistcontrols';
		$context->messagefile = 'itemslist';
		# tablemodel
		$context->mastertablename = 'master';
		# recordset
		$context->default_orderby = 'Order';
		# gateway
		$context->mastertableindex = 'itemslisttable';
		$context->masterrecordindex = 'itemslistrecord';
		# node
		$context->get_admindictionary = FALSE;
//		$context->get_requestdata = FALSE;
//		$context->redirect = 'sublists/contents/admin';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# security
		$context->tokens = Services::get_token_set('mixed');
		return $context;
	}
	protected function _on_set_datapool($datapool)
	{
		$datapool->variables['itempath'] = $this->_parameters->itempath;
	}
	protected function _on_set_query($query)
	{
		$context = $this->_context;
		if (!empty($context->treetopsflag)) {
			$query->where_open()->where('isTreeTop','IS',DB::expr('TRUE'))->where_close();
		}
	}
}