<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use Services;
use Kohana, DB;
use Muster\Node\ListNode;

defined('SYSPATH') or die('No direct script access.');

class PageList extends ListNode {
	public function __construct($controller)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context();
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'nodecontrols';
		$context->dataframe = 'nodedata';
		$context->messagefile = 'page';
		# tablemodel
		$context->mastertablename = 'master';
		$context->pageparenttablename = 'pageparent';
		# recordset
		$context->objecttype = 'page';
		$context->selectarray = array('ObjectType','EditDate','EditUserID','Locked','URL','PageParentID','PageCode');
		$context->binderscope = 'Pages';
		$context->foreignkeyname = 'ParentID';
		$context->default_orderby = 'Title';
		$context->default_orderdirection = 'ASC';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->pageparenttableindex = 'pageparenttable';
		$context->pageparentrecordindex = 'pageparentrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		$context->constraints = array(
			array(
				'index' => 'showpageparent',
				'value' => TRUE,
			),
			array(
				'index' => 'containerslist',
				'value' => 'notrequired',
			),
		);
		# node
		$context->redirect = 'pages/list';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$table = $datapool->tables[$context->pageparenttableindex] = 
			$tablemodel->model->table($context->pageparenttablename);
		$activerecord = $datapool->activerecords[$context->pageparentrecordindex] = 
			$tablemodel->model->activerecord($context->pageparenttablename);
	}
	protected function _on_set_query($query)
	{
		$context = $this->_context;
		if (!empty($context->branchflag)) {
			$query->where_open()->where('PageParentID','IS',DB::expr('NULL'))->where_close();
		}
	}
}