<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

// ** s/b part of directory hierarchy - HB
namespace Base\NodeType;

use StdClass;
use Kohana, DB, Session;
use Services;
use Muster\Node\RecordNode;
use Base\Node;

defined('SYSPATH') or die('No direct script access.');

class PageRecord extends RecordNode {
	const PAGE_BRANCH = 0;
	const PAGE_SIBLING = 1;
	const PAGE_CHILD = 2;
	protected $_page_add_type;
	
	public function __construct($controller)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context();
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'page';
		# tablemodel
		$context->mastertablename = 'master';
		$context->extensiontablename = 'extension';
		$context->parenttablename = 'pageparent';
		$context->childtablename = 'pagechildren';
		# recordset
		$context->objecttype = 'page';
		$context->binderscope = 'Pages';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->parenttableindex = 'pageparenttable';
		$context->parentrecordindex = 'pageparentrecord';
		$context->childtableindex = 'pagechildtable';
		$context->childrecordindex = 'pagechildrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		$context->constraints = array(
			array(
				'index' => 'containerslist',
				'value' => 'notrequired',
			)
		);
		# node
		$context->redirect =  'pages/record';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	protected function _on_set_datapool($datapool) 
	{
		$context = $this->_context;
		$tablemodel = $this->_tablemodel;
		$datapool->activerecords[$context->parentrecordindex] = 
			$tablemodel->model->activerecord($context->parenttablename);
		$datapool->tables[$context->childtableindex] = 
			$tablemodel->model->table($context->childtablename);
		$datapool->activerecords[$context->childrecordindex] = 
			$tablemodel->model->activerecord($context->childtablename);
	}
	protected function _on_respond($command)
	{
		$datamodel = $this->_datamodel;
		$recycle = Node::FLOW_CONTINUE; // continue with processing; false results in redirect
		switch ($command) {
			case 'addpagebranch':
				$recycle = $this->_page_addbranch($datamodel);
				break;
			case 'addpagesibling':
				$recycle = $this->_page_addsibling($datamodel);
				break;
			case 'addpagechild':
				$recycle = $this->_page_addchild($datamodel);
				break;
			case 'changepageparentid':
				$recycle = $this->_page_changeparent($datamodel);
				break;
			case 'clearpageparentid':
				$recycle = $this->_page_clearparent($datamodel);
				break;
		}
		return $recycle;
	}
	protected function _on_before_delete($parmdata,&$error_info)
	{
		extract($parmdata);
		$result = TRUE;
		$record = DB::select('ID')
			->from('Articles_Extensions')
			->where('PageParentID','=',$datarecord->ID)
			->as_object()
			->prepare()
			->execute()
			->current();
		if ($record) {
			$error_info[0] = 'page error';
			$error_info[1] = 'has children';
			$error_info[2] = 'Cannot delete page that has children';
			$result = FALSE;
		}
		return $result; // success
	}
	protected function _page_addbranch($datamodel)
	{
		$this->_page_add_type = self::PAGE_BRANCH;
		return $this->_record_add($datamodel);
	}
	protected function _page_addsibling($datamodel)
	{
		$this->_page_add_type = self::PAGE_SIBLING;
		return $this->_record_add($datamodel);
	}
	protected function _page_addchild($datamodel)
	{
		$this->_page_add_type = self::PAGE_CHILD;
		return $this->_record_add($datamodel);
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		if (!$this->_states->insertmode) return;
		// get pageparentid
		if (!is_null($this->_page_add_type)) { // first time
			$add_type = $this->_page_add_type;
			if ($add_type === self::PAGE_BRANCH) return; // nothing to do
			$parameters = $this->_parameters;
			if ($add_type === self::PAGE_SIBLING) {
				$pageparentid = $parameters->pageparentid;
			} else { // child
				$pageparentid = $parameters->pageid;
			}
			$dictionary['narrativesection']->pageparentid = $pageparentid;
		} else { // already set
			$pageparentid = $dictionary['narrativesection']->pageparentid;
		}
		$dictionary['pageparentid'] = $pageparentid;
			
		if (empty($pageparentid) and !is_numeric($pageperentid)) return;
		$pageparentid = (int) $pageparentid;
		// ... otherwise, show parent record details
		$record = DB::select('*')
			->from('Nodes_View')
			->where('ID','=',$pageparentid)
			->as_object()
			->prepare()
			->execute()
			->current();
		if ($record) {
			$parentrecord = new StdClass;
			$parentrecord->id = $record->ID;
			$parentrecord->title = $record->Title;
			$parentrecord->code = $record->PageCode;
			$dictionary['narrativesection']->pageparentrecord = $parentrecord;
		}
	}
	protected function _page_changeparent($datamodel)
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		if (!$this->_tokens->settings->edit)
			return $this->_record_report_access_error($datamodel);
		$newpageparentid = $this->_parameters->newpageparentid;
		if (empty($newpageparentid) and !is_numeric($newpageparentid)) return; // allow 0 (zero)
		$context = $this->_context;
		if (!is_numeric($newpageparentid)) { // must be number
			($error_message = Kohana::message($context->messagefile,'nonewpageparentid')) or 
				($error_message = 'valid page parentid required');
			$datamodel->datapool->variables['errorsection'] = 
				(object) array(
					'errors' => array(
						array('error'=>$error_message),
					)
				);
				return Node::FLOW_CONTINUE;
		} else { // must be legitimate
			$db = $this->_tablemodel->table->db();
			$db->lock_tables(array('Nodes_View')); // locks extension as well as article
			$objecttype = services::dblookup('Nodes_View','ObjectType','ID',
				$newpageparentid,$db);
			if (empty($objecttype) or ($objecttype != 'page')) {
				$db->unlock_tables();
				($error_message = Kohana::message($context->messagefile,'unrecognizedpageid')) or
				($error_message = 'not a valid page id');
				$datamodel->datapool->variables['errorsection'] = 
					(object) array(
						'errors' => array(
							array('error'=>$error_message),
						)
					);
				$db->unlock_tables();
				return Node::FLOW_CONTINUE;
			}
			
		}
		// ok to update pageparentid
		$extensionrecord = new StdClass;
		$extensionrecord->PageParentID = $newpageparentid;
		$activeextensionrecord = $this->_tablemodel->model->activerecord($context->extensiontablename);
		$activeextensionrecord->current();
		$result = $activeextensionrecord->update((array) $extensionrecord)->save();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activeextensionrecord->error_info(),'authorerrors');
			$db->unlock_tables();
			return Node::FLOW_CONTINUE; // recycle
		}
		$db->unlock_tables();
		$this->_tablemodel->model->table()->reset();
		$this->_tablemodel->datarecord = $this->_tablemodel->model->activerecord()->current();
		($message = Kohana::message($this->_context->messagefile,'something')) or 
			($message = 'new pageparentid saved');
		$session = Session::instance();
		$messages = array(
			'messages' => array(
				array('message'=> $message)
			),
			'index'=>'messagesection',
		);
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
	protected function _page_clearparent($datamodel)
	{
		if (!$this->_permissions->canadminister)
			return $this->_record_report_access_error($datamodel);
		if (!$this->_tokens->settings->edit)
			return $this->_record_report_access_error($datamodel);
		$context = $this->_context;
		$extensionrecord = new StdClass;
		$extensionrecord->PageParentID = NULL;
		$activeextensionrecord = $this->_tablemodel->model->activerecord($context->extensiontablename);
		$activeextensionrecord->current();
		$result = $activeextensionrecord->update((array) $extensionrecord)->save();
		if (!$result) {
			$this->_record_report_dberror($datamodel,$activeextensionrecord->error_info(),'authorerrors');
			return Node::FLOW_CONTINUE; // recycle
		}
		$this->_tablemodel->model->table()->reset();
		$datarecord = $this->_tablemodel->datarecord = $this->_tablemodel->model->activerecord()->current();
		($message = Kohana::message($this->_context->messagefile,'something')) or 
			($message = 'parentid cleared');
		$session = Session::instance();
		$messages = array(
			'messages' => array(
				array('message'=> $message)
			),
			'index'=>'messagesection',
		);
		$session->set('messages',$messages);
		return Node::FLOW_REDIRECT;
	}
}