<?php 
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net

namespace Base\NodeType;

use StdClass;
use System, Services, Node, Filer;
use Kohana, Session, DB;
use Muster\Node\RecordNode;

defined('SYSPATH') or die('No direct script access.');

class PictureRecord extends RecordNode {
	public function __construct($controller,$selector = NULL)
	{
		$this->_extensionflag = TRUE;
		$this->_context = $this->_get_context($selector);
		parent::__construct($controller);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->tablemodel = 'nodeschema';
		$context->controlframe = 'noderecordcontrols';
		$context->dataframe = 'noderecorddata';
		$context->messagefile = 'article';
		# tablemodel
		$context->mastertablename = 'master';
		$context->containertablename = 'containers';
		$context->extensiontablename = 'extension';
		# recordsset
		$context->objecttype = 'picture';
		$context->binderscope = 'Pictures';
		# gateway
		$context->mastertableindex = 'mastertable';
		$context->masterrecordindex = 'masterrecord';
		$context->containertableindex = 'containerstable';
		$context->containerrecordindex = 'containersrecord';
		$context->extensiontableindex = 'extensiontable';
		$context->extensionrecordindex = 'extensionrecord';
		$context->datapoolarrays = Kohana::$config->load('recordpartials')->getArrayCopy();
		# node
		$context->redirect =  'library/pictures';
		# permissions
		$context->tokens = Services::get_objecttype_tokenset($context->objecttype);
		return $context;
	}
	// presumes success; otherwise reconcile -- HB
	protected function _on_after_update($params,&$error_info)
	{
		extract($params);
		if ($priorrecord->RepositoryID !== $activerecord->RepositoryID) {
			$priordirectoryrecordid = NULL;
			$activedirectoryrecordid = NULL;
			$prior_repositoryid = $priorrecord->RepositoryID;
			$active_repositoryid = $activerecord->RepositoryID;
			$db = $this->_tablemodel->model->db();
			// get prior directory record id
			$operation = DB::select('ID')
				->from('Nodes_View')
				->where('ObjectType','=','directory')
				->where('RepositoryID','=',$prior_repositoryid)
				->as_object()
				->prepare($db)
				->execute();
			if ($operation->success()) {
				$record = $operation->fetch();
				if ($record) {
					$priordirectoryrecordid = $record->ID;
				}
			}
			// get active directory record id
			$operation = DB::select('ID')
				->from('Nodes_View')
				->where('ObjectType','=','directory')
				->where('RepositoryID','=',$active_repositoryid)
				->as_object()
				->prepare($db)
				->execute();
			if ($operation->success()) {
				$record = $operation->fetch();
				if ($record) {
					$activedirectoryrecordid = $record->ID;
				}
			}
			$recordid = $activerecord->ID;
			if (!is_numeric($recordid)) return TRUE;
			if (is_numeric($priordirectoryrecordid)) {
				$operation = DB::delete('Nodes_Instances')
					->where('FolderID','=',$priordirectoryrecordid)
					->where('NodeID','=',$recordid)
					->prepare($db)
					->execute();
			}
			if (is_numeric($activedirectoryrecordid)) {
				$operation = $operation = DB::insert('Nodes_Instances',array('FolderID','NodeID'))
					->values(array($activedirectoryrecordid,$recordid))
					->prepare($db)
					->execute();
			}
		}
		return TRUE;
	}
	protected function _on_after_insert($params,&$error_info)
	{
		extract($params);
		$record = $activerecord->current();
		if (!$record) return TRUE;
		$db = $this->_tablemodel->model->db();
		$activedirectoryrecordid = NULL;
		$active_repositoryid = $activerecord->RepositoryID;
		// get active directory record id
		$operation = DB::select('ID')
			->from('Nodes_View')
			->where('ObjectType','=','directory')
			->where('RepositoryID','=',$active_repositoryid)
			->as_object()
			->prepare($db)
			->execute();
		if ($operation->success()) {
			$record = $operation->fetch();
			if ($record) {
				$activedirectoryrecordid = $record->ID;
			}
		}
		$recordid = $activerecord->ID;
		if (!is_numeric($recordid)) return TRUE;
		if (is_numeric($activedirectoryrecordid)) {
			$operation = $operation = DB::insert('Nodes_Instances',array('FolderID','NodeID'))
				->values(array($activedirectoryrecordid,$recordid))
				->prepare($db)
				->execute();
		}
		return TRUE;
	}
	protected function _on_set_datapool($datapool)
	{
		$tablemodel = $this->_tablemodel;
		$context = $this->_context;
		$containertable = $datapool->tables[$context->containertableindex] = 
			$tablemodel->model->table($context->containertablename);
		$datapool->activerecords[$context->containerrecordindex] = 
			$tablemodel->model->activerecord($context->containertablename);
		// assert permissions
		$query = $containertable->query();
		$tablename = $containertable->tablename();
		$this->_record_set_folder_query($query,$tablename);
	}
	protected function _on_get_dictionary(&$dictionary)
	{
		$datarecord = $this->_tablemodel->datarecord;
		if (empty($datarecord)) {
			$dictionary['file_exists'] = FALSE;
			return;
		}
		$filename = $datarecord->FileName;
		$directory = $datarecord->Directory;
		$base = $this->_controller->_base;
		$publicbase = System::instance()->PublicBase;
		$uploadbase = System::instance()->UploadBase;
		$filespec = $publicbase.$uploadbase.$directory.$filename;
		$exists = $dictionary['file_exists'] = is_file($filespec);
		if ($exists) {
			$dictionary['filespec'] = $filespec;
			$dictionary['original_version'] = $filespec;
			$pathinfo = pathinfo($filespec);
			$leftpart = $pathinfo['dirname'].'/'.$pathinfo['filename'].'.';
			$rightpart = '.'.$pathinfo['extension'];
			$system = System::instance();
			$versions = (array) $system->PictureSizes;
			$defaultversion = $system->PictureSizeDefault;
			$currentversion = 'original';
			foreach ($versions as $key => $value) {
				if (is_null($value)) continue;
				$filespec = $leftpart.$key.$rightpart;
				if (is_file($filespec)) {
					if ($defaultversion == $key) {
						$currentversion = $key;
						$dictionary['filespec'] = $filespec;
					}
					$dictionary[$key.'_exists'] = TRUE;
					$dictionary[$key.'_version'] = $filespec;
				}
			}
			$dictionary['currentversion'] = $currentversion;
			$dictionary[$currentversion.'_selected'] = TRUE;
		}
	}
	protected function _on_respond($command)
	{
		$flow = self::FLOW_CONTINUE; // don't repeat process
		switch ($command) {
			case 'uploadpicture':
				$flow = $this->_upload_picture();
				break;
			case 'recreateversions':
				$flow = $this->_recreate_versions();
				break;
		}
		
		return $flow;
	}
	protected function _recreate_versions()
	{
		$tokens = $this->_tokens;
		if (!$tokens->import->edit)
			return $this->_record_report_access_error($datamodel);
		$retval = Node::FLOW_REDIRECT;
		$datamodel = $this->_datamodel;
		if ($this->_record_save($datamodel) == Node::FLOW_CONTINUE) {
			return Node::FLOW_CONTINUE;
		} else {
			(!$this->_states->insertmode) or ($this->_states->insertmode = FALSE);
		}
		$messagesection = $this->_messages;
		$filererrors = NULL;
		$filermessages = NULL;
		$this->_create_versions($filererrors,$filermessages);
		
		$datamodel->dataframe->reset(); // re-collect data for edit
		unset($messagesection['index']);
		if (!empty($filermessages)) {
			if (!empty($messagesection['messages']))
				$messages = $messagesection['messages'];
			else
				$messages = array();
			foreach ($filermessages as $message) {
				$messages[] = array('message'=>$message);
			}
			$messagesection['messages'] = $messages;
		}
		if (!empty($filererrors)) {
			if (!empty($messagesection['errors']))
				$errors = $messagesection['errors'];
			else
				$errors = array();
			foreach ($filererrors as $error)
				$errors[] = array('error'=>$error);
			$messagesection['errors'] = $errors;
		}
		 // get rid of previous messageset
		$session = Session::instance();
		$messages = $session->get_once('messages');
		$datamodel->datapool->variables['messagesection'] = (object) $messagesection;
		return $this->_record_edit($datamodel);
	}
	protected function _create_versions(&$errors,&$messages) {
		$tokens = $this->_tokens;
		if (!$tokens->import->edit)
			return $this->_record_report_access_error($datamodel);
		$datarecord = $this->_tablemodel->datarecord;
		$system = System::instance();
		$filespec = $system->PublicBase.$system->UploadBase.$datarecord->Directory.$datarecord->FileName;
		if (!is_file($filespec)) {
			$errors = array('File does not exist for version creation');
			return;
		}
		$filer = Filer::instance();
		try {
		$filer->create_versions($filespec)->go();
		} catch (Exception $e) {
			var_dump($e->getMessage());
		}
		$errors = $filer->errors();
		$messages = $filer->messages();
	}
	
	protected function _upload_picture()
	{
		$tokens = $this->_tokens;
		if (!$tokens->import->create)
			return $this->_record_report_access_error($datamodel);
		$retval = Node::FLOW_REDIRECT;
		$datamodel = $this->_datamodel;
		$post = &$datamodel->datapool->arrays['post'];
		$parameters = $this->_parameters;
		$uploadpictureversion = $parameters->uploadpictureversion;
		$files = $datamodel->datapool->arrays['files'];
		$errors = array();
		$result = TRUE;
		if (!empty($files['file']['name'])) {
			$sourcefilename = $files['file']['name'];
		} else {
			$errors[] = array('error'=>'No source file to upload');
			$result = FALSE;
		}
		if ($result) {
			$targetfilename = $post['filename'];
			if ($uploadpictureversion != 'original') {
				if (empty($targetfilename)) {
					$errors[] = array('error'=>'Original file must exist to upload custom version');
					$result = FALSE;
				}
			}
			if ($result) {
				(empty($targetfilename)) and ($targetfilename = $sourcefilename);
				$targetfilename = strtr($targetfilename,' ','_');
				$post['filename'] = $targetfilename;
				$repositoryid = $post['repositoryid'];
				($repositoryid == '') and ($repositoryid = NULL);
			}
		}
		if ($this->_record_save($datamodel) == Node::FLOW_CONTINUE) {
			$modelvars = &$this->_datamodel->datapool->variables;
			$messagesection = isset($modelvars['messagesection'])?$modelvars['messagesection']:array();
			$messagesection->warnings[]['warning'] = 'file was not uploaded due to field error';
			$modelvars['messagesection'] = $messagesection;
			return Node::FLOW_CONTINUE;
		} else {
			(!$this->_states->insertmode) or ($this->_states->insertmode = FALSE);
		}
		$messagesection = $this->_messages; // ** HB - error: _messages currently not initialized
		$messages = $messagesection['messages'];
		if ($result) {
			if (!is_null($repositoryid)) {
				$subdirectory = Services::dblookup('System_Repositories','Directory','ID',$repositoryid);
				if (is_null($subdirectory)) {
					$result = FALSE;
					$errors[] = array('error' => 'invalid Directory');
				}
			} else {
				$subdirectory = NULL;
			}
			$replace_existing = (bool) $parameters->uploadreplaceexisting;
			// now have key data: $files (source),$targetfilename,$subdirectory,$replace_existing
			if ($result) {
				$this->_do_upload($files, $targetfilename, $subdirectory, $replace_existing, $messages, $errors, $uploadpictureversion);
			}
		}
		$datamodel = $this->_datamodel;
		$datamodel->dataframe->reset(); // re-collect data for edit
		unset($messagesection['index']);
		$messagesection['messages'] = $messages;
		if (!empty($errors)) $messagesection['errors'] = $errors;
		 // get rid of previous messageset
		$session = Session::instance();
		$messages = $session->get_once('messages');
		$datamodel->datapool->variables['messagesection'] = (object) $messagesection;
		return $this->_record_edit($datamodel);
	}
	protected function _do_upload($files, $targetfilename, $subdirectory, $replace_existing, &$messages, &$errors, $uploadpictureversion)
	{
		$result = TRUE;
		$system = System::instance();
		$filer = Filer::instance();
		$filedata = end($files); // only one
		$filespec = $system->PublicBase.$system->UploadBase.$subdirectory.$targetfilename;
		if ((!empty($uploadpictureversion)) and ($uploadpictureversion != 'original')) {
			$tablemodel = $this->_tablemodel;
			$priorrecord = $tablemodel->priorrecord;
			$datarecord = $tablemodel->datarecord;
			if (empty($priorrecord)) {
				$errors[] = array('error' => 'Must create original on first upload');
				return FALSE;
			}
			if (($priorrecord->RepositoryID != $datarecord->RepositoryID) or 
				($priorrecord->FileName != $datarecord->FileName)) {
				$errors[] = array('error' => 'Cannot change target file name for custom version upload');
				return FALSE;
			}
			if (!is_file($filespec)) {
				$errors[] = array('error' => 'Original file must exist before uploading a custom version');
				return FALSE;
			}
			$pathinfo = pathinfo($targetfilename);
			$targetfilename = $pathinfo['filename'].'.'.$uploadpictureversion.'.'.$pathinfo['extension'];
			$replace_existing = TRUE;
		}
		$result = $filer
			->import($filedata)
			->as_image()
			->targetbase(System::instance()->UploadBase)
			->to($subdirectory)
			->as_name($targetfilename)
			->replace_existing($replace_existing)
			->go();
		// return error data
		if ($result and ($uploadpictureversion == 'original')) {
			$filer->create_versions($filespec)->go();
			$versionmessages = $filer->messages();
			$versionerrors = $filer->errors();
			if (!empty($versionerrors)) {
				foreach ($versionerrors as $error) {
					$errors[] = array('error' => $error);
				}
			}
			if (!empty($versionmessages)) {
				foreach ($versionmessages as $message) {
					$messages[] = array('message' => $message);
				}
			}
		}
		if (!$result) {
			$filererrors = $filer->errors();
			foreach ($filererrors as $error) {
				$errors[] = array('error' => $error);
			}
		} else {
			$messages[] = array('message' => $targetfilename . ' was uploaded');
		}
		return $result;
	}
}