<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
class Controller_Base Extends Controller {
	# set child by controller
	// base url
	public $_base; // ** should not have underscore prefix because it is public...
	public $_common; // ** same...
	// general template
	protected $_template;
	// dictionary for frameview
	protected $_dictionary;
	// view of general (pagee) template
	protected $_pageview;
	# set by base
	// the current controller action requested by the user
	protected $_action;
	// current request controller
	protected $_controller;
	// current session object ??
	protected $_session;
	// user object
	protected $_user;
	// muster access filter instance
	protected $_access;
	# can be overridden by controller
	// error controller
	protected $_error_controller = 'error';
	// default parser, invoked on command from simplewiki_instance()
	protected $_simplewiki_instance;
	
	public function __construct(Request $request, Response $response)
	{
		parent::__construct($request, $response);
		$this->_action = $this->request->action();
		$this->_controller = $this->request->controller();
		$shutdown = System::instance()->Shutdown;
		if ($shutdown) {
			if ($this->_controller != $this->_error_controller) {
				$request->redirect($this->_error_controller . '/shutdown');
			}
		}
		$config = Kohana::$config->load('basecontroller');
		foreach ($config as $key => $value) { // commonly template, token, hierarchy, dictionary (base, headertitle)
			$key = '_' . $key;
			$this->$key = $value;
		}
		$this->_base = Url::base();
		$this->_session = Session::instance();
		$user = $this->_user = User::instance();
		$dictionary = $this->_dictionary;
		if ($user->authenticated()) {
			$userdata = $user->user_data();
			$dictionary['custodiancode']= $userdata->Code;
			$dictionary['myblogcode'] = $userdata->BlogCode;
			$dictionary['myinboxcode'] = $userdata->InboxID;
		}
		$access = $this->_access = Access::instance();
		$dictionary['authenticated']=$user->authenticated();
		$dictionary['base'] = $this->_base;
		$dictionary['systemsidebar'] = $this->_systemsidebar;
		$dictionary['manage'] = ($access->permission('manage','R') or $access->permission('system','R'));
		$dictionary['manageapprovals'] = $access->permission('manage_approvals','R');
		$dictionary['managemembers'] = $access->permission('manage_members','R');
		$dictionary['managebinders'] = $access->permission('manage_binders','R');
		$dictionary['webpages'] = $access->permission('page_content','R');
		$dictionary['system'] = $access->permission('system','R');
		$this->_dictionary = $dictionary;
		$access->create_readpermission_tables();
	}
	
	// controller has opportunity to change controlling settings before processing.
	public function before()
	{	
		if ($this->_template)
		{
			$template = $this->_pageview = Template::view(array($this->_template),$this->_dictionary);
			$footerview = $this->_footerview = Template::view(array('template_footer'),array('base'=>$this->_base));
			$system = System::instance();
			$footerview->owner = $system->Owner;
			$footerview->copyright = $system->Copyright;
			$footerview->ownerwebsite = $system->OwnerWebsite;
			$template->footer = $footerview;
		}
	}
	public function show_resources()
	{
		if ( ! defined('MUSTER_TIME'))
		{
			define('MUSTER_TIME', microtime(TRUE) - KOHANA_START_TIME);
		}

		/**
		 * Define the memory usage at the start of the application, used for profiling.
		 */
		if ( ! defined('MUSTER_TOTALMEMORY'))
		{
			define('MUSTER_TOTALMEMORY', memory_get_usage());
		}
		if ( ! defined('MUSTER_MEMORY'))
		{
			define('MUSTER_MEMORY', memory_get_usage() - KOHANA_START_MEMORY);
		}
		$this->_footerview->muster_memory = number_format(MUSTER_MEMORY);
		$this->_footerview->muster_totalmemory = number_format(MUSTER_TOTALMEMORY);
		$this->_footerview->muster_time = number_format(MUSTER_TIME,3);
	}
	// some parameters have been passed that need to be processed
	protected function _is_request()
	{
		return !(empty($_GET) and empty($_POST) and empty($_FILES));
	}
	public function is_request() // for node
	{
		return $this->_is_request();
	}
	public function pageview() {
		return $this->_pageview;
	}
}
