<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
/*
	Notes:
	use reverse routing
*/
class Controller_Binders Extends Controller_Site {
	protected $_treemanager; // tree manager instance
	public function action_index()
	{
		$this->request->redirect('binders/tree');
	}
	
	public function before()
	{
		// first modify settings, notably _navigation_block and _navigation_item
		
		parent::before();
	}
# ---------------------------------------------------------------------------------	
# ===================================[ List ]======================================
# ---------------------------------------------------------------------------------	
	
	public function action_list()
	{
		# get permissions and parameters
		$permissions = $this->_permissions = $this->_list_get_permissions();
		// redirect out if insufficient permission
		if (!$permissions->canread) {
			$this->request->redirect('error/binderslist');
		}
		
		# get engine
		$this->_datamodel = new DataModel($this);
		
		# establish query parameters, including command response to change parameters
		$parameters = $this->_parameters = $this->_list_get_parameters();
		if ($parameters->command) $this->_list_process_redirect_command($parameters);
		
		# set database components
		$tablemodel = $this->_tablemodel = $this->_list_get_tablemodel($parameters);
		
		# collect page boundaries
		$boundaries = $this->_boundaries = $this->_list_get_boundaries($tablemodel,$parameters);
		$parameters->currentpage = $boundaries->page; // updated to normalized

		# fincalize query
		$this->_list_finalize_query_spec($tablemodel,$parameters,$boundaries);
		
		# collect session data and set up gateway resources
		$this->_list_set_gateway_resources($parameters,$boundaries);
		
		# get dictionaries
		$dictionary = $this->_list_get_dictionary($permissions);
		$optionsdictionary = $this->_list_get_optionsdictionary($permissions,$parameters);

		# session data
		$this->_list_set_session_data($parameters,$boundaries,$dictionary); // dictionary queried for record list

		# combine dictionaries with templates
		$listview = Template::view(array('binder_list'),$dictionary);
		$optionsview = Template::view(array('binder_list_sidebar'),$optionsdictionary);

		# combine views
		$pageview = $this->_pageview;
		$pageview->title = 'Binders - List';
		$pageview->centercolumn = $listview;
		$pageview->leftcolumn = $optionsview;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _list_get_permissions()
	{
		$permissions = new StdClass();
		$access = Access::instance();
		$permissions->canread = $access->permission('manage_binders','R');
		$permissions->canedit = $access->permission('manage_binders','E');
		$permissions->cancreate = $access->permission('manage_binders','C');
		return $permissions;
	}
	protected function _list_get_parameters()
	{
		$parameters = new StdClass();
		$parameters->sessionid = text::random('alnum', 32); // for gateway session variable
		$datamodel = $this->_datamodel;
		if ($this->_is_request()) {
			$request_data = $datamodel->controlframe('binderslistcontrols')
					->control_input()
					->get_parameters();
		} else {
			$request_data = array();
		}
		$parameters->command = $datamodel->get_command();
		# collect and normalize parameters
		if (!empty($request_data)) {
			$parameters->search = $request_data['search'];
			$parameters->searchmethod = $request_data['searchmethod'];
			$parameters->orderby = $request_data['orderby'];
			$parameters->direction = $request_data['direction'];
			
			$parameters->currentpage = $request_data['currentpage'];
			$parameters->perpage = $request_data['perpage'];
		} else {
			$parameters->search = NULL;
			$parameters->currentpage = 1;
			$parameters->perpage = 20;
		}
		if (empty($parameters->search)) { // setup default order
			$parameters->searchmethod = NULL;
			$parameters->orderby = 'Title';
			$parameters->direction = 'asc';
		} else { // suppress default order
			$parameters->searchmethod = 'bm';
			$parameters->orderby = NULL;
			$parameters->direction = NULL;
		}
		if ($parameters->perpage > 100) {
			$parameters->perpage = 100;
		}
		if ($parameters->perpage <= 20) {
			$parameters->perpage = 20;
		}
		return $parameters;
	}
	protected function _list_process_redirect_command($parameters)
	{
		$datamodel = $this->_datamodel;
		$command = $datamodel->get_command();
		if (!$command) return;
		switch ($command) {
			case 'first':
				$parameters->currentpage = 1;
				break;
			case 'previous':
				$parameters->currentpage--;
				break;
			case 'next':
				$parameters->currentpage++;
				break;
			case 'last':
				$parameters->currentpage = 'max';
				break;
			case 'changeperpage':
			case 'searched': // search, can check $value for refinements
				$parameters->currentpage = 1; // reset for change
				break;
			default: return;
		}
		$this->request->redirect('binders/list?' . 
			((isset($parameters->search) and $parameters->search)?'search=' . urlencode($parameters->search):'') .
			((isset($parameters->searchmethod) and $parameters->searchmethod)?"&m=".$parameters->searchmethod:'') .
			((isset($parameters->orderby) and $parameters->orderby)?"&orderby=". $parameters->orderby:'') .
			((isset($parameters->direction) and $parameters->direction)?"&d=".$parameters->direction:'') .
			"&page={$parameters->currentpage}" .
			"&perpage={$parameters->perpage}");
	}
	protected function _list_get_tablemodel($parameters)
	{
		$tablemodel = new StdClass();
		# control objects
		$model = $tablemodel->model = new TableModel('binderschema');
		$binderstable = $tablemodel->binderstable = $model->table();
		
		# assemble base query
		$query = $tablemodel->query = $binderstable->query();
		$query->select('*'); // select all columns
		
		if (!empty($parameters->search)) {
			$mode = 'IN BOOLEAN MODE';
			$search = $model->db()->escape($parameters->search);
			$query->where_open();
			$query->where(
					DB::expr('MATCH(`Title`,`Caption`,`Description`)'),
					'', // no operator
					DB::expr('AGAINST(' . $search . ' ' . $mode . ')'));
			$query->where_close();
		} else {
			if ($parameters->orderby)
				$query->order_by($parameters->orderby,$parameters->direction);
		}
		return $tablemodel;
	}
	protected function _list_get_boundaries($tablemodel,$parameters)
	{
		$binderstable = $tablemodel->binderstable;
		// get potential record count, for pagination and to set boundaries for limit and offset
		$recordcount = $binderstable->unlimited_count();
		$perpage = $parameters->perpage;
		
		// set $maxpage, normalize $page
		$maxpage =  ceil($recordcount / $perpage);
		$page = $parameters->currentpage;
		if ($page == 'max') $page = $maxpage; // from command
		if ($page > $maxpage) $page = $maxpage;
		if ($page <= 0) $page = 1;
		if ($recordcount == 0) $page = 0;
		
		// create and normalize $offset
		$offset = (($page * $perpage) - ($perpage));
		if ($offset < 0) $offset = 0;
		
		// set up $startrecord and $endrecord
		if ($recordcount == 0) $startrecord = 0;
		else $startrecord = $offset + 1;
		$endrecord = ($perpage * $page);
		if ($endrecord > $recordcount) $endrecord = $recordcount;
		
		$boundaries = new StdClass();
		$boundaries->recordcount = $recordcount;
		$boundaries->offset = $offset;
		$boundaries->page = $page;
		$boundaries->maxpage = $maxpage;
		$boundaries->startrecord = $startrecord;
		$boundaries->endrecord = $endrecord;
		
		return $boundaries;
	}
	protected function _list_finalize_query_spec($tablemodel,$parameters,$boundaries)
	{
		# assert search relevance and precedence
		// this is deferred to this point to avoid issues with open query count collected in list boundaries
		$query = $tablemodel->query;
		$model = $tablemodel->model;
		$search = $parameters->search;
		if (!empty($search)) {
			$mode = 'IN BOOLEAN MODE';
			$search = $model->db()->escape($search);
			$query->select(array(DB::expr('(MATCH(`Title`,`Caption`) AGAINST(' . $search . ' ' . $mode . '))'),'titlerelevance'));
			$query->select(array(DB::expr('(MATCH(`Title`,`Caption`,`Description`) AGAINST(' . $search . ' ' . $mode . '))'),'descrelevance'));
			$query->order_by('titlerelevance','desc')->order_by('descrelevance','desc');
		}

		# assert query limits
		$query->offset($boundaries->offset);
		$query->limit($parameters->perpage);
	}
	protected function _list_set_gateway_resources($parameters,$boundaries)
	{
		$datamodel = $this->_datamodel;
		$binderstable = $this->_tablemodel->model->table();
		$bindersrecord = $binderstable->activerecord();
		
		# primary components
		$datamodel->dataframe('binderslistdata');
		$datamodel->datapool->tables['binderstable'] = $binderstable;
		$datamodel->datapool->activerecords['bindersrecord'] = $bindersrecord;
		
		$variables = &$datamodel->datapool->variables;
		
		# common variables
		$variables['base'] = $this->_base;
		$variables['listing'] = (bool) $boundaries->recordcount;

		# set up pagination
		$variables['recordcount'] = $boundaries->recordcount;
		$variables['perpageoptions'] = 
			array("20"=>"20","30"=>"30","40"=>"40","60"=>"60","80"=>"80","100"=>"100",);
		$variables['perpage'] = $parameters->perpage;
		$page = $variables['currentpage'] = $parameters->currentpage;
		$variables['currentstartrecord'] = $boundaries->startrecord;
		$variables['currentendrecord'] = $boundaries->endrecord;
		$maxpage = $variables['pagecount'] = $boundaries->maxpage;
		$variables['multiplepages'] = ($maxpage > 1);
		$variables['atfirstdisabled'] = ($page == 1);
		$variables['atlastdisabled'] = ($page == $maxpage);
		
		$variables['sessionid'] = $parameters->sessionid;

		# set up state for recycling
		isset($parameters->search) and ($variables['searchvalue'] = $parameters->search);
		isset($parameters->searchmethod) and ($variables['searchmethod'] = $parameters->searchmethod);
		isset($parameters->orderby) and ($variables['orderby'] = $parameters->orderby);
		isset($parameters->direction) and ($variables['direction'] = $parameters->direction);
		$variables['currentpage'] =	$parameters->currentpage;
		$variables['perpage'] = $parameters->perpage; // redundant, also set up for pagination
	}
	protected function _list_get_dictionary($permissions)
	{
		$dictionary = $this->_datamodel->retrieve()->output();
		$dictionary['canedit'] = $permissions->canedit;
		$dictionary['base'] = $this->_base;
		$dictionary['common'] = $this->_common;
		return $dictionary;
	}
	protected function _list_get_optionsdictionary($permissions,$parameters)
	{
		// create sidebar dictionary
		$optionsdictionary = array();
		$optionsdictionary['base'] = $this->_base;
		$optionsdictionary['common'] = $this->_common;
		$optionsdictionary['listclass'] = 'highlight';
		$optionsdictionary['sessionid'] = $parameters->sessionid;
		$optionsdictionary['cancreate'] = $permissions->cancreate;
		return $optionsdictionary;
	}
	protected function _list_set_session_data($parameters,$boundaries,$dictionary)
	{
		$returnquery = 
			(isset($parameters->search)?'search=' . urlencode($parameters->search):'') .
			(isset($parameters->search)?"&m=".$parameters->searchmethod:'') .
			(isset($parameters->orderby)?"&orderby=". $parameters->orderby:'') .
			(isset($parameters->direction)?"&d=".$parameters->direction:'') .
			"&page={$parameters->currentpage}" .
			"&perpage={$parameters->perpage}";
			
		$spec = array();
			isset($parameters->search) and ($spec[] = "search = {$parameters->search}");
			isset($parameters->orderby) and ($spec[] = "orderby = {$parameters->orderby}");
			isset($parameters->direction) and ($spec[] = "direction = {$parameters->direction}");
			$spec[] = "page = {$parameters->currentpage}";
			$spec[] = "perpage = {$parameters->perpage}";
		$spec = implode("; ",$spec);
		$sessionarray = array(
			'sessionid' => $parameters->sessionid,
			'sessiontype' => 'searchlist',
			'currentpage' => $parameters->currentpage,
			'pagecount' => $boundaries->maxpage,
			'returnurl' => "binders/list",
			'returnquery' => '?' . $returnquery,
			'spec' => $spec,
		);
		$recordset = isset($dictionary['records'])
			?$dictionary['records']
			:NULL;
		if ($recordset) {
			$recordlist = array();
			foreach ($recordset as $record) {
				$recordlist[] = $record['id'];
			}
			$sessionarray['recordlist'] = $recordlist;
			session::instance()->set($sessionarray['sessionid'],$sessionarray);
		}
	}
# ---------------------------------------------------------------------------------	
# ====================================[ Tree ] ====================================
# ---------------------------------------------------------------------------------	

	public function action_tree()
	{
		# get permissions
		$permissions = $this->_permissions = $this->_tree_get_permissions();
		if (!$permissions->canread) {
			$this->request->redirect('error/binderstree');
		}
		
		# get engine
		$this->_datamodel = new DataModel($this);
		
		# establish parameters
		$parameters = $this->_parameters = $this->_tree_get_parameters();
		
		# handle any redirect
		if ($parameters->command) $this->_tree_process_redirect_command($parameters);

		# establish tablemodel
		$tablemodel = $this->_tablemodel = $this->_tree_get_tablemodel($parameters);

		# operate on the data
		if ($parameters->command) $this->_tree_process_operation_command($parameters);
		
		# collect boundaries
		$boundaries = $this->_boundaries = $this->_tree_get_boundaries($tablemodel,$parameters);
		$parameters->currentpage = $boundaries->page;

		# finalize data query
		$this->_tree_finalize_query_spec($tablemodel,$parameters,$boundaries);

		# set up gateway resources
		$this->_tree_set_gateway_resources($tablemodel,$parameters,$boundaries);
		
		# get dictionaries
		$dictionary = $this->_tree_get_dictionary($permissions);
		$optionsdictionary = $this->_tree_get_optionsdictionary($permissions,$parameters);

		#set session data
		$this->_tree_set_session_data($parameters,$boundaries,$dictionary); // dictionary queried for record list

		# combine dictionaries with templates
		$listview = Template::view(array('binder_tree'),$dictionary);
		$optionsview = Template::view(array('binder_tree_sidebar'),$optionsdictionary);

		# combine templates
		$pageview = $this->_pageview;
		$pageview->title = 'Binders - Tree';
		$pageview->centercolumn = $listview;
		$pageview->leftcolumn = $optionsview;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _tree_get_permissions()
	{
		$permissions = new StdClass();
		$access = Access::instance();
		$permissions->canread = $access->permission('manage_binders','R');
		$permissions->canedit = $access->permission('manage_binders','E');
		$permissions->cancreate = $access->permission('manage_binders','C');
		return $permissions;
	}
	protected function _tree_get_parameters()
	{
		$parameters = new StdClass();
		$parameters->binderid = $this->request->param('id');
		$parameters->layout = 'flat'; // ** parameterize
		$parameters->sessionid = text::random('alnum', 32); // for session record
		$datamodel = $this->_datamodel;
		if ($this->_is_request()) {
			$request_data = $datamodel->controlframe('binderstreecontrols')
				->control_input()
				->get_parameters();
		} else {
			$request_data = array();
		}
		$parameters->command = $datamodel->get_command();
		# collect and normalize parameters
		if (!empty($request_data)) {
			isset($request_data['search']) and ($parameters->search = $request_data['search']);
			isset($request_data['searchmethod']) and ($parameters->searchmethod = $request_data['searchmethod']);
			isset($request_data['orderby']) and ($parameters->orderby = $request_data['orderby']);
			isset($request_data['direction']) and ($parameters->direction = $request_data['direction']);
			
			$parameters->currentpage = $request_data['currentpage'];
			$parameters->perpage = $request_data['perpage'];
		} else {
			$parameters->search = NULL;
			$parameters->currentpage = 1;
			$parameters->perpage = 20;
		}
		if (empty($parameters->search)) {
			$parameters->searchmethod = NULL;
			$parameters->orderby = 'Title';
			$parameters->direction = 'asc';
		} else {
			$parameters->searchmethod = 'bm';
			$parameters->orderby = NULL;
			$parameters->direction = NULL;
		}
		if ($parameters->perpage > 100) {
			$parameters->perpage = 100;
		}
		if ($parameters->perpage <= 20) {
			$parameters->perpage = 20;
		}
		return $parameters;
	}
	protected function _tree_process_redirect_command($parameters)
	{
		$command = $parameters->command;
		if (!$command) return;
		switch ($command) {
			case 'first':
				$parameters->currentpage = 1;
				break;
			case 'previous':
				$parameters->currentpage--;
				break;
			case 'next':
				$parameters->currentpage++;
				break;
			case 'last':
				$parameters->currentpage = "max";
				break;
			case 'changeperpage':
			case 'searched': // search, can check $value for refinements
				$parameters->currentpage = 1; // reset for change
				break;
			default: return;
		}
		$this->request->redirect("binders/tree?" . 
			((isset($parameters->search) and $parameters->search)?'search=' . urlencode($parameters->search):'') .
			((isset($parameters->searchmethod) and $parameters->searchmethod)?"&m=".$parameters->searchmethod:'') .
			((isset($parameters->orderby) and $parameters->orderby)?"&orderby=". $parameters->orderby:'') .
			((isset($parameters->direction) and $parameters->direction)?"&d=".$parameters->direction:'') .
			"&page={$parameters->currentpage}" .
			"&perpage={$parameters->perpage}");
	}
	protected function _tree_get_tablemodel($parameters)
	{
		$tablemodel = new StdClass();
		$model = $tablemodel->model = new TableModel('binderschema');

		#----------------[ set database components ] -------------------
		$binderstable = $tablemodel->binderstable = $model->table();
		
		# assemble base query
		$query = $tablemodel->query = $binderstable->query();
		$query->select('*'); // select all columns
		
		$query->order_by('RootTitle')->order_by('LeftEdge');
		
		if ($parameters->binderid) {
			$result = DB::select('NodeID','LeftEdge','RightEdge','ParentID','RootID')
				->from('Binders_Tree_View')
				->where('NodeID','=',$parameters->binderid)
				->as_object()
				->prepare($model->db())
				->execute();
			$rootrecord = $result->fetch();
			if ($rootrecord) {
				if ($parameters->layout =='expand') {
					$leftedge = $rootrecord->LeftEdge;
					$rightedge = $rootrecord->RightEdge;
					$rootid = $rootrecord->RootID;
					$query
						->where('RootID','=',$rootid)
						->where('LeftEdge','BETWEEN',DB::expr($leftedge.' AND '.$rightedge));
				} else {
					$query
						->where('ID','=',$rootrecord->NodeID)
						->or_where('ParentID','=',$rootrecord->NodeID);
				}
			}
		} else {
			if ($parameters->layout == 'flat') {
				$query->where('ParentID','IS',DB::expr('NULL'));
			}
		}
		return $tablemodel;
	}
	protected function _tree_process_operation_command($parameters)
	{
		$command = $parameters->command;
		if (!$command) return;
		switch ($command) {
			case 'initialize':
				$result = $this->_tree_initialize();
				return $result;
				break;
			default: return NULL;
		}
	}
	protected function _tree_initialize()
	{
		$tree = TreeManager::binder_tree();
		$tree->initialize();
		return FALSE;
	}
	protected function _tree_get_boundaries($tablemodel,$parameters)
	{
		$binderstable = $tablemodel->binderstable;
		// get potential record count, for pagination and to set boundaries for limit and offset
		$recordcount = $binderstable->unlimited_count();
		$perpage = $parameters->perpage;
		
		// set $maxpage, normalize $page
		$maxpage = ceil($recordcount / $perpage);
		$page = $parameters->currentpage;
		if ($page == 'max') $page = $maxpage; // from command
		if ($page > $maxpage) $page = $maxpage;
		if ($page <= 0) $page = 1;
		if ($recordcount == 0) $page = 0;
		
		// create and normalize $offset
		$offset = (($page * $perpage) - ($perpage));
		if ($offset < 0) $offset = 0;
		
		// set up $startrecord and $endrecord
		if ($recordcount == 0) $startrecord = 0;
		else $startrecord = $offset + 1;
		$endrecord = ($perpage * $page);
		if ($endrecord > $recordcount) $endrecord = $recordcount;
		
		$boundaries = new StdClass();
		$boundaries->recordcount = $recordcount;
		$boundaries->page = $page;
		$boundaries->maxpage = $maxpage;
		$boundaries->offset = $offset;
		$boundaries->startrecord = $startrecord;
		$boundaries->endrecord = $endrecord;
		
		return $boundaries;
	}
	protected function _tree_finalize_query_spec($tablemodel,$parameters,$boundaries)
	{
		# assert query limits
		$query = $tablemodel->query;
		$query->offset($boundaries->offset);
		$query->limit($parameters->perpage);
	}
	protected function _tree_set_gateway_resources($tablemodel,$parameters,$boundaries)
	{
		$datamodel = $this->_datamodel;
		$binderstable = $tablemodel->model->table();
		$bindersrecord = $binderstable->activerecord();
		
		# primary components
		$datamodel->dataframe("binderstreedata");
		$datamodel->datapool->tables['binderstable'] = $binderstable;
		$datamodel->datapool->activerecords['bindersrecord'] = $bindersrecord;
		$variables = &$datamodel->datapool->variables;
		#path
		$firstrecord = $bindersrecord->current();
		if ($firstrecord) {
			$tree = TreeManager::binder_tree();
			$path = $tree->get_path($firstrecord->ID);
			$variables['breadcrumbs'] = $path;
			$variables['isbreadcrumbs'] = !empty($parameters->binderid);
		}
		# common variables
		$variables['base'] = $this->_base;
		$variables['listing'] = (bool) $boundaries->recordcount;

		# set up pagination
		$variables['recordcount'] = $boundaries->recordcount;
		$variables['perpageoptions'] = 
			array("20"=>"20","30"=>"30","40"=>"40","60"=>"60","80"=>"80","100"=>"100",);
		$variables['perpage'] = $parameters->perpage;
		$page = $variables['currentpage'] = $parameters->currentpage;
		$variables['currentstartrecord'] = $boundaries->startrecord;
		$variables['currentendrecord'] = $boundaries->endrecord;
		$maxpage = $variables['pagecount'] = $boundaries->maxpage;
		$variables['multiplepages'] = ($maxpage > 1);
		$variables['atfirstdisabled'] = ($page == 1);
		$variables['atlastdisabled'] = ($page == $maxpage);
		
		$variables['sessionid'] = $parameters->sessionid;

		# set up state for recycling
		isset($parameters->search) and ($variables['searchvalue'] = $parameters->search);
		isset($parameters->searchmethod) and ($variables['searchmethod'] = $parameters->searchmethod);
		isset($parameters->orderby) and ($variables['orderby'] = $parameters->orderby);
		isset($parameters->direction) and ($variables['direction'] = $parameters->direction);
		$variables['currentpage'] =	$parameters->currentpage;
		$variables['perpage'] = $parameters->perpage; // redundant, also set up for pagination
	}
	protected function _tree_get_dictionary($permissions)
	{
		$dictionary = $this->_datamodel->retrieve()->output();
		$dictionary['canedit'] = $permissions->canedit;
		$dictionary['base'] = $this->_base;
		$dictionary['common'] = $this->_common;
		
		return $dictionary;
	}
	protected function _tree_get_optionsdictionary($permissions,$parameters)
	{
		$optionsdictionary = array();
		$optionsdictionary['base'] = $this->_base;
		$optionsdictionary['common'] = $this->_common;
		$optionsdictionary['treeclass'] = 'highlight';
		$optionsdictionary['sessionid'] = $parameters->sessionid;
		$optionsdictionary['cancreate'] = $permissions->cancreate;
		
		return $optionsdictionary;
	}
	protected function _tree_set_session_data($parameters,$boundaries,$dictionary)
	{
		$returnquery = 
			(isset($parameters->search)?'search=' . urlencode($parameters->search):'') .
			(isset($parameters->search)?"&m=".$parameters->searchmethod:'') .
			(isset($parameters->orderby)?"&orderby=". $parameters->orderby:'') .
			(isset($parameters->direction)?"&d=".$parameters->direction:'') .
			"&page={$parameters->currentpage}" .
			"&perpage={$parameters->perpage}";
			
		$spec = array();
			isset($parameters->search) and ($spec[] = "search = {$parameters->search}");
			isset($parameters->orderby) and ($spec[] = "orderby = {$parameters->orderby}");
			isset($parameters->direction) and ($spec[] = "direction = {$parameters->direction}");
			$spec[] = "page = {$parameters->currentpage}";
			$spec[] = "perpage = {$parameters->perpage}";
		$spec = implode("; ",$spec);
		$sessionarray = array(
			'sessionid' => $parameters->sessionid,
			'sessiontype' => 'searchlist',
			'currentpage' => $parameters->currentpage,
			'pagecount' => $boundaries->maxpage,
			'returnurl' => 'binders/tree/' . $parameters->binderid,
			'returnquery' => '?' . $returnquery,
			'spec' => $spec,
		);
		$recordset = isset($dictionary['records'])
			?$dictionary['records']
			:NULL;
		if ($recordset) {
			$recordlist = array();
			foreach ($recordset as $record) {
				$recordlist[] = $record['id'];
			}
			$sessionarray['recordlist'] = $recordlist;
			session::instance()->set($sessionarray['sessionid'],$sessionarray);
		}
	}
# ---------------------------------------------------------------------------------	
# ================================[ Contents ]=====================================
# ---------------------------------------------------------------------------------	
/* Can be:
current:
document - article
entry - article
page - container
blog - container
topics - container

planned:
items - container
addresses - container
item - library
address - addresses
folio - modified container

*/
/*
	public function action_contents()
	{
		$binderid = $this->request->param('id');
		$binder = DB::select('ID','Code','SourceType','ObjectType')
			->from('Binders')
			->where('ID','=',$binderid)
			->as_object()
			->prepare()
			->execute();
		if (!$binder->success()) return;
		$binder_record = $binder->current();
		if (!$binder_record) return;
		$binderid = $binder_record->ID;
		$bindercode = $binder_record->Code;
		$content_type = $binder_record->ObjectType;
//		$context = $this->_contents_get_context($binder_record);
		$node = Base\Node::folder_list($this,'binders');
		$subcontext = new StdClass();
		$subcontext->listbranch = 'binders/contents/' . $binderid;
		$subcontext->recordbranch = 'binders/contents/' . $binderid;
		$subcontext->binderscope = $bindercode;
		$node->extend_context($subcontext);
		$context = $node->context();
		$data = $node->generate();
		$dictionary = $data['dictionary'];
		$searchdictionary = $data['admindictionary'];
		
		$content_type = $binder_record->ObjectType;
		$template = 'contents_list';
		$view = Template::view(array($template),$dictionary); // ** s/b binder_contents?
		
		$tree = TreeManager::binder_tree();
		$branchid = $this->request->param('id');
		$path = $tree->get_path($branchid);
		$dictionary['base'] = $this->_base;
		if (!empty($path)) {
			$dictionary['isbreadcrumbs'] = !empty($branchid);
			$dictionary['breadcrumbs'] = $path;
			$dictionary['listcontents'] = $view;
			$dictionary['targettitle'] = $path[count($path) - 1]['title'];
		}
		$viewwrapper = Template::view(array('binder_contents_wrapper'),$dictionary); // ** combine with binder_contents?

		$searchdictionary['listbranch'] = $context->listbranch;
		$searchdictionary['recordbranch'] = $context->recordbranch;
		$searchview = Template::view(array('document_list_sidebar'),$searchdictionary); // ** s/b binder_contents_sidebar

		# combine templates
		$pageview = $this->_pageview;
		$pageview->centercolumn = $viewwrapper;
		$pageview->leftcolumn = $searchview;
		$pageview->title = 'Binder - Contents';
		$this->show_resources();
		$this->response->body($pageview);
	}
*/
# ---------------------------------------------------------------------------------	
# =================================[ Record ]======================================
# ---------------------------------------------------------------------------------	

	public function action_record()
	{
		# get permissions, and screen usage
		$permissions = $this->_permissions = $this->_record_get_permissions();
		if (!$permissions->canread) {
			$this->request->redirect('error/bindersrecord');
		}
		
		# get engine
		$this->_datamodel = new DataModel($this);
		
		# get parameters
		$parameters = $this->_parameters = $this->_record_get_parameters();

		#get sessiondata
		$sessiondata = $this->_sessiondata = $this->_record_get_sessiondata($parameters);
		
		# process any redirect
		$this->_record_process_redirect_command($parameters,$sessiondata);
		
		# initialize states
		$states = $this->_states = $this->_record_get_states();

		# assemble tablemodel
		$tablemodel = $this->_tablemodel = $this->_record_get_tablemodel($permissions,$states,$parameters);
		
		# set gateway resources
		$this->_record_set_gateway_resources($parameters,$tablemodel);
		
		# respond to record change commands
		$this->_record_process_operation_command($parameters);
		
		# acquire data dictionaries
		$dictionary = $this->_record_get_dictionary($permissions,$states,$sessiondata,$tablemodel);
//		var_dump($dictionary);
		$optionsdictionary = $this->_record_get_optionsdictionary($permissions,$parameters);

		# generate templates
		$listview = Template::view(array('binder_record'),$dictionary);
		$optionsview = Template::view(array('binder_record_sidebar'),$optionsdictionary);
		
		# combine templates
		$pageview = $this->_pageview;
		$pageview->title = 'Binders - Record';
		$pageview->centercolumn = $listview;
		$pageview->leftcolumn = $optionsview;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _record_get_permissions()
	{
		$access = Access::instance();
		$permissions = $this->_permissions = new StdClass();
		$basetoken = 'manage_binders';
		$permissions->canread = $access->permission($basetoken,'R');
		$permissions->canedit = $access->permission($basetoken,'E');
		$permissions->candelete = $access->permission($basetoken,'D');
		$permissions->cancreate = $access->permission($basetoken,'C');
		return $permissions;
	}
	protected function _record_get_parameters()
	{
		# initialize parameters
		$datamodel = $this->_datamodel;
		$datamodel
			->controlframe('bindersrecordcontrols')
			->control_input(); // anticipate parameters
		$parameters =  (object) $datamodel->get_parameters(); // sessionid
		$parameters->command = $datamodel->get_command();
		$parameters->recordid = $this->request->param('id');
		return $parameters;
	}
	protected function _record_get_sessiondata($parameters)
	{
		$recordid = $parameters->recordid;
		$sessionid = $parameters->sessionid;
		$sessiondata = new StdClass();
		$sessiondata->data = NULL;
		if ($sessionid) {
			$session = session::instance();
			$sessiondata->data = $session->get($sessionid);
			if ($sessiondata->data) {
				$recordlist = $sessiondata->data['recordlist'];
				$itemindex = array_search($recordid,$recordlist);
				$itemcount = count($recordlist);
				if ($itemindex === FALSE) {
					$itemindex = NULL;
				}
				$sessiondata->itemindex = $itemindex;
				$sessiondata->itemcount = $itemcount;
			}
		}
		$sessiondata->sessionid = $sessionid;
		$sessiondata->dictionary = NULL; // default
		return $sessiondata;
	}
	protected function _record_process_redirect_command($parameters,$sessiondata)
	{
		$sessionid = $sessiondata->sessionid;
		if (!($sessionid and $sessiondata->data)) return;
		
		$recordlist = $sessiondata->data['recordlist'];
		$itemindex = $sessiondata->itemindex;
		$itemcount = $sessiondata->itemcount;
		$command = $parameters->command;
		if ($command) {
			$pointer = $itemindex;
			if (is_null($pointer)) $pointer = -1;
			$id = '';
			switch ($command) {
				case 'first':
					$id = $recordlist[0];
					break;
				case 'previous':
					--$pointer;
					if ($pointer < 0) $pointer = 0;
					$id = $recordlist[$pointer];
					break;
				case 'next':
					++$pointer;
					if ($pointer >= $itemcount) $pointer = ($itemcount-1);
					$id = $recordlist[$pointer];
					break;
				case 'last':
					$id = $recordlist[$itemcount-1];
					break;
				default: return;
			}
			if ($id) {
				if ($sessionid)
					$this->request->redirect("binders/record/$id?sessionid=$sessionid");
				else
					$this->request->redirect("binders/record/$id");
			}
		}
	}
	protected function _record_get_states()
	{
		$states = new StdClass(); 
		$states->insertmode = FALSE;
		return $states;
	}
	protected function _record_get_tablemodel($permissions,$states,$parameters)
	{
		# initialize data model
		$tablemodel = new StdClass();
		$model = $this->_model = // for treemanager ??? *** use _db instead
			$tablemodel->model = new TableModel('binderschema');
		$table = $tablemodel->table = $model->table();
		$query = $tablemodel->query = $table->query();
		$tablemodel->activerecord = $table->activerecord();
		$tablemodel->treeactiverecord = $table->activerecord('tree');
		$recordid = $parameters->recordid;
		if ((!$permissions->canread) or empty($recordid)) {
			$query->where('ID','IS',DB::expr('NULL')); // always blank
		} else { // can read
			$query->where('ID','=',$recordid); // $recordid could be null, yielding blank record
		}
		$tablemodel->datarecord = $tablemodel->activerecord->current(); // current record object, or NULL
		
		return $tablemodel;
	}
	protected function _record_set_gateway_resources($parameters,$tablemodel)
	{
		$datamodel = $this->_datamodel;
		$datamodel->dataframe('bindersrecorddata');
		
		$variables = &$datamodel->datapool->variables;
		$variables['recordid'] = $parameters->recordid;
		$variables['sessionid'] = $parameters->sessionid;
		$variables['base'] = $this->_base;
		
		$activerecords = &$datamodel->datapool->activerecords;
		$activerecords['bindersrecord'] = $tablemodel->activerecord;
		$activerecords['treerecord'] = $tablemodel->treeactiverecord;
	}
	// ===================[ operation command start ]=======================
	protected function _record_process_operation_command($parameters)
	{
		if ($parameters->command) {
			
			$recycle_whiteboard = $this->_record_respond($parameters->command);
			$recordid = $parameters->recordid; // might have been set by insert
			if (!$recycle_whiteboard) {
				$sessionid = $parameters->sessionid;
				if ($sessionid) {
					$this->request->redirect("binders/record/$recordid?sessionid=$sessionid");
				} else {
					$this->request->redirect("binders/record/$recordid");
				}
			}
		} else {
			$session = session::instance();
			$messages = $session->get_once('messages');
			if ($messages) {
				$index = $messages['index'];
				unset($messages['index']);
				$messages = (object) $messages;
				$this->_datamodel->datapool->variables[$index] = $messages;
			}
		}
	}

	# command controller
	protected function _record_respond($command)
	{
		$datamodel = $this->_datamodel;
		$result = TRUE; // continue with processing; false results in redirect
		switch ($command) {
			case 'add':
				$result = $this->_record_add($datamodel);
				break;
			case 'delete':
				$result = $this->_record_delete($datamodel);
				break;
			case 'save':
				$result = $this->_record_save($datamodel);
				break;
			case 'cancel':
				$result = $this->_record_cancel($datamodel);
				break;
		}
		return $result;
	}
	# add
	protected function _record_add($datamodel) {
		if (!$this->_permissions->cancreate)
			return FALSE;
		if (!($this->_tablemodel->datarecord)) {
			$datamodel->datapool->constraints['edit'] = TRUE;
			$this->_states->insertmode = TRUE;
			return TRUE;
		} else {
			return FALSE;
		}
	}
	# delete
	// *** must check integrity of existing relations before allowing delete
	protected function _record_delete($datamodel) {
		if (!$this->_permissions->candelete)
			return FALSE;
		$activerecord = $this->_tablemodel->activerecord;
		$db = $this->_tablemodel->model->db();
		$this->_tablemodel->model->table()->auto_lock(FALSE);
		$this->_tablemodel->model->table('tree')->auto_lock(FALSE);
		$db->lock_tables(array('Binders_View','Binders_Tree_View')); // a bit redundant, but mirrors table model
		// ---------------[ delete records ]---------------
		$datarecord = $activerecord->current();
		if ($datarecord and ($id = $datarecord->ID)) {
			$tree = TreeManager::binder_tree();
			$treeactiverecord = $this->_tablemodel->treeactiverecord;
			$treenode = $treeactiverecord->current();
			$branch = NULL;
			if ($treenode) {
				$branch = $tree->get_branch($datarecord->ID);
				$result = $tree->delete_node($treenode);
				if (!$result) $error_info = $this->_tablemodel->error_info = 
					$tree->error_info();
			}
			if ($branch) {
				foreach ($branch as $binder) {
					$binderid = $binder->NodeID;
					$operation = DB::delete('Binders')
						->where('ID','=',$binderid)
						->prepare($db)
						->execute();
				}
			} else {
				$activerecord->delete();
			}
			$this->_record_remove_sessionlist_item($id);
		}
		// ----------------[ end of delete records ]-------------------
		$db->unlock_tables();
		$this->_tablemodel->model->table()->auto_lock(TRUE);
		$this->_tablemodel->model->table('tree')->auto_lock(TRUE);
		$activerecord->table()->preset();
		$activerecord->current();
//		return TRUE;  // for debug
		return FALSE;
	}
	
	// -------------------------[ start of save ]-----------------------------
	// *** before allowing save of changed content type, must check existing integrity
	// change of isbox, sourcetype, objecttype, uri
	protected function _record_save($datamodel) {
		if (!$this->_permissions->canedit)
			return FALSE;
		$datamodel->datapool->variables['errorsection'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('errorsection')
			->errormessageindex('binders')
			->input()
			->validate();
		$datamodel->finishinput();
		if (empty($datamodel->dataframe->pointlist['id']->value)) {
			$this->_states->insertmode = TRUE;
		}
		if (!$result) {
			$datamodel->recoverinput();
			return TRUE;
		} else {
			unset($datamodel->datapool->variables['errorsection']); // use report_dberror instead
			// first validation successful - continue processing
			$result = $this->_record_process_save($datamodel);
		}
//		return true; // debug
		return !$result; // recycle
	}
	protected function _record_process_save($datamodel)
	{
		// first continue preprocessing
		$result = $this->_record_preprocess_save($datamodel);
		$error_info = $this->_tablemodel->error_info;
		if ($result) {
			// save data after successful preprocessing
			$result = $this->_record_process_update($datamodel);
			$error_info = $this->_tablemodel->error_info;
		}
		if (!$result) {
			$datamodel->recoverinput();
			$this->_record_report_dberror($datamodel,$error_info);
		} else {
			$session = session::instance();
			$messages = array(
				'messages' => array(
					array('message'=>Kohana::message('binders','save'))
				),
				'index'=>'messagesection',
			);
			$session->set('messages',$messages);
		}
		return $result;
	}
	protected function _record_preprocess_save($datamodel)
	{
		$result = TRUE;
		// test for existence of parentcode
		$this->_tablemodel->error_info = NULL;
		$db = $this->_tablemodel->model->db();
		if (!empty($datamodel->dataframe->pointlist['parentcode']->value)) {
			$parentcode = $datamodel->dataframe->pointlist['parentcode']->value;
			$record = DB::select('Code','ID')
				->from('Binders')
				->where('Code','=',$parentcode)
				->as_object()
				->execute($db)->current();
			if (!$record) {
				$result = FALSE;
				$error_info = $this->_tablemodel->error_info = 
					array(0=>'parentcode',2=>'Binder specified in ParentCode is not found');
				return $result;
			}
		}
		$binderrecord = $this->_tablemodel->binderrecord = 
			$datamodel->datapool->records['bindersrecord'] = new StdClass();
		$treerecord = $this->_tablemodel->treerecord = 
			$datamodel->datapool->records['treerecord'] = new StdClass();
		$datamodel->store();
		if (!$binderrecord->isBox) { // check content type and dependent data
			// *** check integrity before allowing change of sourcetype
			if (empty($binderrecord->SourceType)) {
				$result = FALSE;
				$error_info = $this->_tablemodel->error_info = 
					array(0=>'sourcetype',2=>'Content type is required if binder is not a box');
			}
			if ($result) {
				if ($binderrecord->SourceType == 'object') {
					$objecttype = $binderrecord->ObjectType;
					if (!in_array($objecttype,
						array('document','entry','item','address','page','folio','blog','topics','items','addresses','mixed',))) {
						$result = FALSE;
						$error_info = $this->_tablemodel->error_info = 
							array(0=>'objecttype',2=>'Internal type is required when Content type is Internal');
					}
				} else {
					$binderrecord->ObjectType = NULL;
				}
			}
			if ($result) {
				if ($binderrecord->SourceType == 'uri') {
/*					$result = (bool) (!empty($binderrecord->SourceURI));
					if (!$result) {
						$error_info = $this->_tablemodel->error_info = 
							array(0=>'uri',2=>'Repository path is required when Content type is Repository');
					}
*/
				} else {
					$binderrecord->SourceURI = NULL;
				}
			}
		} else {
			$binderrecord->SourceType = NULL;
			$binderrecord->ObjectType = NULL;
			$binderrecord->SourceURI = NULL;
		}
		return $result;
	}
	protected function _record_process_update($datamodel)
	{
		$result = TRUE;
		$activerecord = $this->_tablemodel->activerecord;
		$binderrecord = $this->_tablemodel->binderrecord;
		$treerecord = $this->_tablemodel->treerecord;
		$this->_tablemodel->model->table()->auto_lock(FALSE);
		$this->_tablemodel->model->table('tree')->auto_lock(FALSE);
		$db = $this->_tablemodel->model->db();
		$db->lock_tables(array('Binders_View')); // locks all base tables
		# update binder record
		if (is_null($binderrecord->ID)) { // new record
			$result = $activerecord->insert((array) $binderrecord)->save();
			if ($result) {
				$recordid = $this->_parameters->recordid = $activerecord->insert_id();
				$treerecord->NodeID = $recordid;
				$table = $this->_tablemodel->table;
				$table->preset()->query()->where('ID','=',$recordid); // for insert saveandedit
				$this->_states->insertmode = FALSE;
				$this->_record_update_sessionlist($recordid);
			} else {
				$error_info = $this->_tablemodel->error_info = 
					$activerecord->error_info();
			}
		} else { // existing record
			$result = (bool) $activerecord->update((array) $binderrecord)->save();
			if (!$result) $error_info = $this->_tablemodel->error_info = 
				$activerecord->error_info();
		}
		# update tree record
		if ($result) {
			$this->_tablemodel->datarecord = $activerecord->current();
			$tree = TreeManager::binder_tree();
			$treeactiverecord = $this->_tablemodel->treeactiverecord;
			$treedatarecord = $treeactiverecord->current();
			if ($treedatarecord) { // update
				$oldsortvalues = new StdClass();
				$oldsortvalues->Title = $binderrecord->Title;
				$oldsortvalues->Date = $binderrecord->Date;
				$oldsortvalues->SiblingOrder = $treerecord->SiblingOrder;
				$result = $tree->update_node($treerecord,$oldsortvalues);
				if (!$result) {
					$error_info = $this->_tablemodel->error_info = $tree->error_info();
				}
			} else { // insert
				$result = $tree->insert_node($treerecord);
				if (!$result) $error_info = $this->_tablemodel->error_info = 
					$tree->error_info();
			}
			if ($result) {
				$recordid = $this->_parameters->recordid;
				$this->_tablemodel->table->preset()->query()->where('ID','=',$recordid); // refresh after dependent table update
			}
		}
		$db->unlock_tables();
		$this->_tablemodel->model->table()->auto_lock(TRUE);
		$this->_tablemodel->model->table('tree')->auto_lock(TRUE);
		return $result;
	}
	protected function _record_update_sessionlist($recordid)
	{
		$sessionid = $this->_parameters->sessionid;
		if (!$sessionid) return;
		$session = session::instance();
		$listsession = $session->get($sessionid);
		if ($listsession) {
			$recordlist = $listsession['recordlist'];
			$recordlist[] = $recordid;
			$listsession['recordlist'] = $recordlist;
			$session->set($sessionid,$listsession);
			$this->_record_get_sessiondata($this->_parameters);
		}
	}
	protected function _record_remove_sessionlist_item($recordid)
	{
		$sessionid = $this->_parameters->sessionid;
		if (!$sessionid) return;
		$session = session::instance();
		$listsession = $session->get($sessionid);
		if ($listsession) {
			$recordlist = $listsession['recordlist'];
			$offset = array_search($recordid,$recordlist);
			if ($offset !== FALSE) {
				array_splice($recordlist,$offset,1);
				$listsession['recordlist'] = $recordlist;
				$session->set($sessionid,$listsession);
				$this->_record_get_sessiondata($this->_parameters);
			}
		}
	}
	// -------------------------[ end of save ]-----------------------------
	protected function _record_cancel($datamodel) {
		return FALSE;
	}
	// =================[ operation command end ]============================
	
	protected function _record_get_dictionary($permissions,$states,$sessiondata,$tablemodel)
	{
		$dictionary = array();
		$dictionary['base'] = $this->_base;
		$dictionary['common'] = $this->_common;
		$dictionary['canedit'] = $permissions->canedit;
		$dictionary['candelete'] = $permissions->candelete;
		$dictionary['publicbase'] = System::instance()->PublicBase;
		$dictionary['repositorybase'] = System::instance()->RepositoryBase;
		$dictionary['record'] = (((bool) $tablemodel->datarecord) or ($states->insertmode));
		$this->_record_set_session_dictionary($sessiondata);
		$is_sessiondata = $dictionary['sessiondata'] = (bool) $sessiondata->dictionary;
		if ($is_sessiondata) {
			$dictionary = $dictionary + $sessiondata->dictionary;
		}
		#path
		$record = $tablemodel->activerecord->current();
		if ($record) {
			$tree = TreeManager::binder_tree();
			$path = $tree->get_path($record->ID);
			$dictionary['breadcrumbs'] = $path;
			$dictionary['isbreadcrumbs'] = TRUE;
		}
		$dictionary = $this->_datamodel->retrieve()->output() + $dictionary;
		return $dictionary;
	}
	protected function _record_set_session_dictionary($sessiondata)
	{
		$sessionid = $sessiondata->sessionid;
		if (!($sessionid and $sessiondata->data)) return;
		
		$itemindex = $sessiondata->itemindex;
		$itemcount = $sessiondata->itemcount;
		$sessiontype = $sessiondata->data['sessiontype'];
		switch ($sessiontype) {
			case 'searchlist':
				$listcaption = 'search';
				break;
			default: $listcaption = '';
		}
		$data = array();
		if (is_null($itemindex)) {
			$itemnumber = '?';
		} else {
			$itemnumber = $itemindex + 1;
		}
		$data['itemnumber'] = $itemnumber;
		$data['currentpage'] = $sessiondata->data['currentpage'];
		$data['pagecount'] = $sessiondata->data['pagecount'];
		$data['itemcount'] = $itemcount;
		$data['atfirstdisabled'] = ($itemnumber === 1);
		$data['atlastdisabled'] = ($itemnumber === $itemcount);
		$data['listtype'] = $listcaption;
		$data['returnurl'] = $sessiondata->data['returnurl'];
		$data['returnquery'] = $sessiondata->data['returnquery'];
		$data['spec'] = $sessiondata->data['spec'];
		$sessiondata->dictionary = $data;
	}
	protected function _record_get_optionsdictionary($permissions,$parameters)
	{
		$optionsdictionary = array();
		$optionsdictionary['base'] = $this->_base;
		$optionsdictionary['common'] = $this->_common;
		$optionsdictionary['cancreate'] = $permissions->cancreate;
		$optionsdictionary['sessionid'] = $parameters->sessionid;
		return $optionsdictionary;
	}
	# ===============================[ UTILITIES ]=================================
	protected function _record_report_dberror($datamodel,$error_info,$errorsection = 'errorsection')
	{
		if (!$error_info) return;
//		var_dump($error_info);
		$index = $error_info[0];
		$error_message = Kohana::message('binders',$index);
		if (!$error_message) $error_message = $error_info[2];
		$datamodel->datapool->variables[$errorsection] = 
			(object) array(
				'errors' => array(
					array('error'=>$error_message),
				)
			);
	}
}
