<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
/*
*/
class Controller_GetJson Extends Controller_Site {
	public function action_index()
	{
		$this->_protect();
		$get = $this->request->query();
		$selector = $get['selector'];
//		var_dump($get); exit();
		switch ($selector) {
			case 'setfavourite':
				$data = $this->_set_favourite($get);
			break;
			case 'unsetfavourite':
				$data = $this->_unset_favourite($get);
			break;
			case 'add':
				$data = $this->_add_action($get);
				break;
			case 'move':
				$data = $this->_move_action($get);
				break;
			default:
				die('unrecognized selector');
		}
		if (isset($data['error'])) {
			// do nothing for now // *** review
		} else {
			$this->response->body(json_encode($data));
		}
	}
	protected function _protect()
	{
		if (!$this->request->is_ajax())
			die('must be an ajax request');
		$user = $this->_user;
		$authenticated = $user->authenticated();
		if (!$authenticated) {
			die('must be logged in to access get json');
		}
		// *** permission token may also be 'library_content'
		$authorized = Access::instance()->permission('article_content','R');
		if (!$authorized) {
			die('unauthorized use of access to json');
		}
	}
	protected function _add_action($get)
	{
		$data = array();
		$targetid = $get['data'];
		$objecttypenames = Kohana::$config->load('objecttypenames')->getArrayCopy();
		$targetobjecttype = Services::dblookup('Articles','ObjectType','ID',$targetid);
		if (is_null($targetobjecttype)) {
			$data['continue'] = FALSE;
			$data['errorText'] = 'Unknown target container id';
			return $data;
		}
		if (!in_array($targetobjecttype,array('folder','file','inbox'))) {
			$data['continue'] = FALSE;
			$data['errorText'] = 'Wrong target container type (must be Topic, File, or Inbox)';
			return $data;
		}
		$data['continue'] = TRUE;
		$data['objectName'] = $objecttypenames[$targetobjecttype];
		$data['objectTitle'] = Services::dblookup('Articles','Title','ID',$targetid);
		return $data;
	}
	protected function _move_action($get)
	{
		$data = array();
		$targetid = $get['data'];
		$objecttypenames = Kohana::$config->load('objecttypenames')->getArrayCopy();
		$targetobjecttype = Services::dblookup('Articles','ObjectType','ID',$targetid);
		if (is_null($targetobjecttype)) {
			$data['continue'] = FALSE;
			$data['errorText'] = 'Unknown target container id';
			return $data;
		}
		if (!in_array($targetobjecttype,array('folder','file','inbox'))) {
			$data['continue'] = FALSE;
			$data['errorText'] = 'Wrong target container type (must be Topic, File, or Inbox)';
			return $data;
		}
		$data['continue'] = TRUE;
		$data['objectName'] = $objecttypenames[$targetobjecttype];
		$data['objectTitle'] = Services::dblookup('Articles','Title','ID',$targetid);
		return $data;
	}
	protected function _set_favourite($get)
	{
		$indata = explode('|',$get['data']);
		$index = isset($indata[0])?$indata[0]:NULL;
		$instance = isset($indata[1])?$indata[1]:NULL;
		$path = isset($indata[2])?$indata[2]:NULL;
		$userid = User::instance()->userid();
		$db = Database::instance();
		$db->lock_tables(array('Articles','Articles_Favourites'));
		$record = NULL;
		try {
			$record = DB::select('ID')
				->from('Articles')
				->where('ID','=',$index)
				->as_object()
				->prepare($db)
				->execute()
				->current();
		} catch(Exception $e) {
			// nothing
		}
		$db->unlock_tables();
		if (!empty($instance) or is_numeric($instance) or !empty($path)) {
			$parms = array('instanceid'=>$instance,'path'=>$path);
			$parms = json_encode($parms);
			if ($parms === FALSE) $parms = NULL;
		} else {
			$parms = NULL;
		}
		if ($record) {
			$result = DB::insert('Articles_Favourites')
				->columns(array('UserID','ArticleID','Parameters'))
				->values(array($userid,$index,$parms))
				->prepare()
				->execute();
			$sibling = 'favourite';
			$data = array(
				'sibling' => $sibling,
			);
		} else {
			$data = array('error','record not found');
		}
		return $data;
	}
	protected function _unset_favourite($get)
	{
		$index = $get['data'];
		$userid = User::instance()->userid();
		$db = Database::instance();
		$db->lock_tables(array('Articles','Articles_Favourites'));
		try {
			DB::delete('Articles_Favourites')
				->where('UserID','=',$userid)
				->where('ArticleID','=',$index)
				->prepare()
				->execute();
		} catch(Exception $e) {
			// nothing
		}
		$db->unlock_tables();
		$sibling = 'notfavourite';
		$data = array(
			'sibling' => $sibling,
		);
		return $data;
	}
}
