<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
class Controller_Manage extends Controller_Site {
	public function action_index()
	{
		$this->request->redirect('manage/members');
	}
	public function action_members()
	{
		$access = Access::instance();
		$permission = $access->permission('manage_members','R');
		if (!$permission)
			$this->request->redirect('error/managemembers-disallowed');
		
		// collect data objects
		$context = $this->_context = $this->_get_context();
		$datamodel = $this->_datamodel = New DataModel($this);
		$parameters = $this->_parameters = $this->_get_parameters($datamodel,$context);
		
		$this->_process_redirect_command($parameters,$context);

		$datamodel->dataframe($context->dataframe); // set dataframe
		
		// collect tablemodel objects
		$tablemodel = new TableModel($context->tablemodel);
		$mastertable = $tablemodel->table();
		$query = $mastertable->query();
		// set query base
		$query->select('Image','Code','Name','City','Email','RoleID','StatusID','Logins','CreateDate')
			->order_by('Code')
			->where('StatusID','<>',1); // <> Pending
		// set query select
		if ((!empty($parameters->fsearch)) and (!empty($parameters->fsfield))) {
			$fsfield = $parameters->fsfield;
			if (in_array($fsfield,array('Code','Name','City','Email','StatusID',))) {
				$query->where_open();
				if ($parameters->fswild == 1) {
					$fsearch = $tablemodel->db()->escape('%'.$parameters->fsearch.'%');
					$query->where($fsfield,'LIKE',DB::expr($fsearch));
				} elseif ($parameters->fswild == 0) {
					$fsearch = $tablemodel->db()->escape($parameters->fsearch);
					$query->where($fsfield,'LIKE',DB::expr($fsearch));
				} elseif ($parameters->fswild == 2) {
					$fsearch = $parameters->fsearch;
					if (!is_numeric($fsearch)) $fsearch = -1; // no entries like this
					else $fsearch = (int) $fsearch;
					$query->where($fsfield,'=',$fsearch);
				}
				$query->where_close();
			}
		}
		// set boundaries
		$this->_set_boundaries($mastertable,$parameters);
		$query->offset($parameters->offset);
		$query->limit($parameters->perpage);
		
		// get record data
		$datamodel->datapool->recordsets[$context->recordsetindex] = 
			$mastertable->recordset()->fetch();
			
		// get dictionary	
		$dictionary = $this->_get_dictionary($datamodel,$parameters);
		
		// assemble templates
		$listview = Template::view(array('members'),$dictionary);
		$sideview = Template::view(array('members_sidebar'),$dictionary);
		
		$pageview = $this->_pageview;
		$pageview->title = 'Manage Members';
		$pageview->centercolumn = $listview;
		$pageview->leftcolumn = $sideview;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _get_context()
	{
		$context = new StdClass;
		# resources
		$context->controlframe = 'managememberscontrols';
		$context->dataframe = 'managemembersdata';
		# model
		$context->tablemodel = 'accountschema';
		# gateway
		$context->recordsetindex = 'useraccounts';
		# node
		$context->redirect = 'manage/members';
		return $context;
	}
	protected function _get_parameters($datamodel,$context)
	{
		$parameters = new StdClass;
		if ($this->is_request()) {
			$request_data = 
				$datamodel->controlframe($context->controlframe)
					->control_input()
					->get_parameters();
		} else {
			$request_data = array();
		}
		$parameters = (object) $request_data;
		// make sure standard parameters exist for _get_redirect_parameters
		property_exists($parameters,'currentpage') or ($parameters->currentpage = NULL);
		empty($parameters->perpage) and ($parameters->perpage = NULL);
		empty($parameters->fsearch) and ($parameters->fsearch = NULL);
		empty($parameters->fsfield) and ($parameters->fsfield = NULL);
		empty($parameters->fswild) and ($parameters->fswild = 0);
		
		$control = 
			$datamodel->get_command_control();
		if ($control) {
			if (property_exists($control,'parameters')) {
				$parms = $control->parameters; // pass parameters->mode = 'admin' for new record command
				foreach ($parms as $key => $value) {
					$parameters->$key = $value;
				}
			}
			$parameters->command = $control->value;
		} else {
			$parameters->command = NULL;
		}
		// normalize page boundaries data
		if (!empty($request_data)) {
			isset($request_data['currentpage']) and ($parameters->currentpage = $request_data['currentpage']);
			isset($request_data['perpage']) and ($parameters->perpage = $request_data['perpage']);
		} else {
			$parameters->currentpage = 1;
			$parameters->perpage = 30;
		}
		if ($parameters->perpage > 200) {
			$parameters->perpage = 200;
		}
		if ($parameters->perpage <= 2) {
			$parameters->perpage = 30;
		}
		return $parameters;
	}
	protected function _process_redirect_command($parameters,$context)
	{
		$command = $parameters->command;
		if (!$command) return;
//		var_dump($command); exit;
		switch ($command) {
			case 'first':
				$parameters->currentpage = 1;
				break;
			case 'previous':
				$parameters->currentpage--;
				break;
			case 'next':
				$parameters->currentpage++;
				break;
			case 'last':
				$parameters->currentpage = "max";
				break;
			case 'changeperpage':
				$parameters->currentpage = 1; // reset for change
				break;
			default: return;
		}
		$this->request->redirect($context->redirect.'?' . 
			"&fsearch={$parameters->fsearch}" .
			"&fsfield={$parameters->fsfield}" .
			"&fswild={$parameters->fswild}" .
			"&page={$parameters->currentpage}" .
			"&perpage={$parameters->perpage}");
	}
	protected function _set_boundaries($mastertable,$parameters)
	{
		// get potential record count, for pagination and to set boundaries for limit and offset
		$recordcount = $mastertable->unlimited_count();
		$perpage = $parameters->perpage;
		
		// set $maxpage, normalize $page
		$maxpage = ceil($recordcount / $perpage);
		$page = $parameters->currentpage;
		if ($page == 'max') $page = $maxpage; // from command
		if ($page > $maxpage) $page = $maxpage;
		if ($page <= 0) $page = 1;
		if ($recordcount == 0) $page = 0;
		$parameters->currentpage = $page;
		
		// create and normalize $offset
		$offset = (($page * $perpage) - ($perpage));
		if ($offset < 0) $offset = 0;
		
		// set up $startrecord and $endrecord
		if ($recordcount == 0) $startrecord = 0;
		else $startrecord = $offset + 1;
		$endrecord = ($perpage * $page);
		if ($endrecord > $recordcount) $endrecord = $recordcount;
		
		// collect parameters
		$parameters->recordcount = $recordcount;
		$parameters->maxpage = $maxpage;
		$parameters->offset = $offset;
		$parameters->startrecord = $startrecord;
		$parameters->endrecord = $endrecord;
		
	}
	protected function _get_dictionary($datamodel,$parameters)
	{
		$dictionary = $datamodel->retrieve()->output();
		$dictionary['base'] = $this->_base;
		$dictionary['fsearchvalue'] = $parameters->fsearch;
		$dictionary['fsfield'] = $parameters->fsfield;
		$dictionary['fswild'] = $parameters->fswild;
		$dictionary['currentpage'] = $parameters->currentpage;
		$dictionary['pagecount'] = $parameters->maxpage;
		$dictionary['currentstartrecord'] = $parameters->startrecord;
		$dictionary['currentendrecord'] = $parameters->endrecord;
		$dictionary['recordcount'] = $parameters->recordcount;
		$dictionary['atfirstdisabled'] = ($parameters->currentpage <= 1);
		$dictionary['ismultipage'] = ($parameters->maxpage > 1);
		$dictionary['atlastdisabled'] = ($parameters->currentpage == $parameters->maxpage);
		$dictionary['perpage'] = $parameters->perpage;
		$perpage = $parameters->perpage;
		$perpageoptionsdata = 
			array("10"=>"10","15"=>"15","20"=>"20","30"=>"30","40"=>"40","60"=>"60","80"=>"80","100"=>"100","200"=>"200",);
		$perpageoptions = array();
		foreach ($perpageoptionsdata as $key=>$value) {
			$perpageoption = array('value' => $key,'caption' => $value);
			if ($perpage == $value) $perpageoption['selected'] = TRUE;
			$perpageoptions[] = (object) $perpageoption;
		}
		$dictionary['perpageoptions'] = $perpageoptions;
		return $dictionary;
	}
}
