<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
/*
	add actions for register, login, logout, account + profile, confirm, profiles, blogs
*/
class Controller_Member Extends Controller_Site {
	protected $_datamodel;
	public function action_index()
	{
		if ($this->_user->authenticated()) {
			$this->request->redirect('member/account');
		} else {
			$this->request->redirect('member/login');
		}
	}
	
	public function before()
	{
		// first modify settings, notably _navigation_block and _navigation_item
		if ($this->_action == 'account')
			$this->_template = 'template_page_2col';
		parent::before();
	}
	
	public function action_register()
	{
		$followup = $this->request->param('id');
		if ($followup)
			return $this->register_followup($followup);
		$authorized = Access::instance()->permission('account_register','RE');
		if (!$authorized) {
			// redirect to notauthorized
			$this->request->redirect('member/notauthorized/register');
		}
		$datamodel = $this->_datamodel = new DataModel($this);
//		$datamodel->datapool->variables['base'] = $this->_base;
		$datamodel->dataframe('registerdata');
		$datamodel->datapool->constraints['registerdirect'] = Access::instance()->permission('account_register','CR');
		$datamodel->datapool->variables['moderated'] = (bool) System::instance()->ModerateSubscription;
		if ($this->_is_request()) {
			// call datamodel with metadata
			$result = $datamodel
				->controlframe('registercontrols')
				->responses(
					array(
						'register'=>array($this,'response_register'),
						'registerdirect'=>array($this,'response_registerdirect')
					))
				->respond();
		} else {
			$session = session::instance();
			$messages = $session->get_once('messages');
			if ($messages) {
				$index = $messages['index'];
				unset($messages['index']);
				$messages = (object) $messages;
				$datamodel->datapool->variables[$index] = $messages;
			}
		}

		$datamodel->retrieve();
		
		$dictionary = $datamodel->output();
		$dictionary['base'] = $this->_base;
		$dictionary['common'] = $this->_common;
		
		$pageview = $this->_pageview;
		$pageview->title = 'Register';
		$registerview = Generator::view(array('register'),$dictionary);
		$pageview->centercolumn = $registerview;
			
		$this->response->body($pageview);
	}
	// called by action_register with "followup" argument (from redirection)
	protected function register_followup($followup)
	{
		if ($followup == 'confirmation') {
			$pageview = $this->_pageview;
			$pageview->title = 'Confirmation sent';
			$dictionary = array('base' => $this->_base);
			$dictionary['common'] = $this->_common;
			$dictionary['website'] = System::instance()->Website;
			
			$registerview = Generator::view(array('register_confirm'),$dictionary);
			$pageview->centercolumn = $registerview;
			$this->response->body($pageview);
		}
	}
	public function action_confirm()
	{
		$token = $this->request->param('id');
		$pageview = $this->_pageview;
		$pageview->title = 'Confirmation';
		$dictionary = array('base' => $this->_base);
		$dictionary['common'] = $this->_common;
		
		$dictionary['email'] = System::instance()->ReturnEmailAddress;
		$dictionary['website'] = System::instance()->Website;
		if (!$token) {
			$dictionary['completionerror'] = TRUE;
			$dictionary['error'] = 'No confirmation token was provided'; // ** s/b taken from messages
		} else {
			$useraccount = new User_Account();
			$result = $useraccount->confirm($token,
				User_Account::CONFIRMATION_SEND_EMAIL, // email to user
				User_Account::CONFIRMATION_SEND_EMAIL); // email to moderator
			if (!$result) {
				$dictionary['completionerror'] = TRUE;
				$dictionary['error'] = $useraccount->error_message();
			} else {
				$dictionary['moderated'] = (bool) System::instance()->ModerateSubscription;
				$subjectuser_record = $useraccount->model()->activerecord();
				$dictionary['name'] = $subjectuser_record->Name;
			}
		}
		$confirmview = Generator::view(array('register_complete'),$dictionary);
		$pageview->centercolumn = $confirmview;
		$this->response->body($pageview);
	}
	public function action_login()
	{
		$uri = NULL;
		$session = session::instance();
		if (isset($_GET['uri'])) {
			$session->set('loginuri',$_GET['uri']);
			$this->request->redirect('member/login');
		}
		$datamodel = new DataModel($this);
		$datamodel->dataframe('logindata');
//		$datamodel->datapool->variables['base'] = $this->_base;
		
		if ($this->_user->authenticated()) {
			// user gets reminder that he/she already logged in
			$datamodel->datapool->records['useraccount'] = 
				$this->_user->model()->activerecord()->current();
		} else { 
			$datamodel->datapool->constraints['login'] = TRUE;
			if ($this->_is_request()) { 
				// create metadata
				// call datamodel with metadata
				$result = $datamodel
					->controlframe('logincontrols')
					->responses(
						array(
							'login'=>array($this,'response_login'),
							'newpassword'=>array($this,'response_newpassword'),
							'usercodelist'=>array($this,'response_usercodelist'),
						)
					)
					->respond();
				// success would not reach here, instead redirect, so login has failed.
			}
		}
		$dictionary = $datamodel->retrieve()->output();
		$dictionary['base'] = $this->_base;
		$pageview = $this->_pageview;
		$pageview->title = 'Member Login';
		$loginview = Generator::view(array('page_login'),$dictionary);
		$pageview->centercolumn = $loginview;
		$this->response->body($pageview);
	}
	public function action_confirmpassword()
	{
		$token = $this->request->param('id');
		$pageview = $this->_pageview;
		$pageview->title = 'Confirm Password Request';
		$dictionary = array('base' => $this->_base);
		
		$dictionary['email'] = System::instance()->ReturnEmailAddress;
		if (!$token) {
			$dictionary['completionerror'] = TRUE;
			$dictionary['error'] = 'No password request confirmation token was provided'; // ** s/b taken from messages
		} else {
			$useraccount = new User_Account();
			$result = $useraccount->confirm_password_request(
				$token,
				User_Account::CONFIRMATION_SEND_EMAIL); // email to user
			if (!$result) {
				$dictionary['completionerror'] = TRUE;
				$dictionary['error'] = $useraccount->error_message();
			} else {
				$subjectuser_record = $useraccount->model()->activerecord();
				$dictionary['name'] = $subjectuser_record->Name;
			}
		}
		$confirmview = Generator::view(array('register_complete_password'),$dictionary);
		$pageview->centercolumn = $confirmview;
		$this->response->body($pageview);
	}
	public function action_logout()
	{
		$pageview = $this->_pageview;
		$pageview->title = 'Member Logout';
		$dictionary = array('base' => $this->_base);
		
		if ($this->_user->authenticated())
		{
			$this->_user->logout(TRUE); // ** true s/b replaced by constant
			if (isset($_GET['uri'])) {
				$this->request->redirect($_GET['uri']);
			}
			$this->request->redirect('member/logout');
		} else {
			$logoutview = Generator::view(array('page_logout'),$dictionary);
			$pageview->centercolumn = $logoutview;
		}
		$this->response->body($pageview);
	}
	public function action_account()
	{
		$usercode = $this->request->param('id');
		# authorize access
		$user_authenticated = $this->_user->authenticated();
		if (!$user_authenticated) {
			$authorized = FALSE;
		} elseif ($usercode) { // specific account requested
			if ($usercode == $this->_user->model()->activerecord()->Code) { // own account
				$authorized = TRUE;
			} else { // administered account
				$authorized = Access::instance()->permission('member_account','RE');
			}
		} else {
			$usercode = $this->_user->model()->activerecord()->Code;
			$authorized = TRUE;
		}
		if (!$authorized) {
			// redirect to notauthorized
			$this->request->redirect('member/notauthorized/account');
		}
		# get owneraccount and existence and check ranking access
		$owneraccount = new User_Account();
		$ownermodel = $owneraccount->model();
		$table = $ownermodel->table();
		$table->query()->where('Code','=',$usercode);
		$ownerrecord = $table->activerecord();
		$ownerrecord->load();
		// if not exist, redirect...
		if (!$ownerrecord->current()) 
			$this->request->redirect('error/invalidmembercode');
		// check ranking...
		$userrank = $this->_user->rank();
		$ownerrank = $owneraccount->rank();
		$ownerid = $ownerrecord->ID;
		
		if (!($this->_user->userid() == ($ownerid))) { // administered record
			// rank must exceed administered rank
			if (!(($userrank > $ownerrank) or ($userrank == System::instance()->MaxRank)))
				$this->request->redirect('member/notauthorized/account');
		}
		# proceed with account administration
		// common dictionary requirement
		// initialize datamodel
		$datamodel = $this->_datamodel = new DataModel($this);
		$datamodel->ownerid = $ownerid; // for recordowner rights
//		$datamodel->datapool->variables['base'] = $this->_base;
		$datamodel->dataframe('accountdata');
		$datamodel->datapool->variables['gender'] = array('male' => 'Male','female'=>'Female');
		$datamodel->datapool->records['useraccount'] = $ownerrecord;
		$accountdata = $datamodel->datapool->objects['accountdata'] = $owneraccount->account_data();
		// assemble view dictionary, respond to request if there is one...
		if ($this->_is_request()) { // respond to command
			// call datamodel with metadata
			$result = $datamodel
				->controlframe('accountcontrols')
				->responses(
					array(
						'basicupdate'=>array($this,'account_basicupdate'),
						
						'updateimagesettings'=>array($this,'account_updateimagesettings'),
						'uploadimage'=>array($this,'account_uploadimage'),
						
						'changepassword'=>array($this,'account_changepassword'),
						
						'savedetailprofile'=>array($this,'account_savedetailprofile'),
						'saveandeditdetailprofile'=>array($this,'account_saveandeditdetailprofile'),
						'canceldetailupdate'=>array($this,'account_canceldetailupdate'),
						'previewdetailprofile'=>array($this,'account_previewdetailprofile'),
						'editdetailprofile'=>array($this,'account_editdetailprofile'),
						
						'savescratchpad'=>array($this,'account_savescratchpad'),
						'adminupdate'=>array($this,'account_adminupdate'),
						'saveadminnotes'=>array($this,'account_saveadminnotes'),
						'updateblogid'=>array($this,'account_updateblogid'),
						'clearblogid'=>array($this,'account_clearblogid'),
						'updateinboxid'=>array($this,'account_updateinboxid'),
						'clearinboxid'=>array($this,'account_clearinboxid'),
					))
				->respond();
			if ($result) {
				$url = 'member/account/' . $usercode;
				$this->request->redirect($url);
			}
		} else {
			$session = session::instance();
			$messages = $session->get_once('messages');
			if ($messages) {
				$index = $messages['index'];
				unset($messages['index']);
				$messages = (object) $messages;
				$datamodel->datapool->variables[$index] = $messages;
			}
		}
		// common for browse and post failure
		$cacherecord = $datamodel->cache->records['useraccount'] = clone $ownerrecord->current();
		$this->account_setconstraints($cacherecord,$datamodel);
		$datamodel
			->recordcaching(TRUE)
			->retrieve();
		$dictionary = $datamodel->output();
		$dictionary['base'] = $this->_base;
		$dictionary['common'] = $this->_common;
		// generate output
		$pageview = $this->_pageview;
		$pageview->title = 'Member Account';
		$accountview = Generator::view(array('account_record'),$dictionary);
		$pageview->centercolumn = $accountview;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function account_setconstraints($cacherecord,$datamodel) 
	{
		if (is_string($cacherecord->Permissions)) {
			$permissions = $cacherecord->Permissions = json_decode($cacherecord->Permissions);
		} else {
			$cacherecord->Permissions = NULL;
		}
		// set constraints set by permissions
		$permissions = $cacherecord->Permissions;
		if (!$permissions) $permissions = new StdClass();
		$articlesconstraint = property_exists($permissions,"article")
			? $permissions->article
			: FALSE;
		$commentsconstraint = property_exists($permissions,"comment")
			? $permissions->comment
			: FALSE;
		$bloggingconstraint = property_exists($permissions,"blog")
			? $permissions->blog
			: FALSE;
		$constraints = &$datamodel->datapool->constraints;
		$constraints['articles'] = $articlesconstraint;
		$constraints['comments'] = $commentsconstraint;
		$constraints['blogs'] = $bloggingconstraint;
	}
	
	public function action_notauthorized($action = NULL)
	{
		$action = $this->request->param('id');
		$pageview = $this->_pageview;
		$pageview->title = 'Not Authorized';
		$dictionary = array('base' => $this->_base);
		$dictionary['action'] = Kohana::message('notauthorized',$action);
		$naview = Generator::view(array('page_notauthorized'),$dictionary);
		$pageview->centercolumn = $naview;
		
		$this->response->body($pageview);
	}
	public function action_profiles()
	{
		$redirect = Route::get('profiles')->uri(array(
			'controller' => 'profiles',
			'id' => $this->request->param('id')));
		$this->request->redirect($redirect);
	}
	# ============================================================================
	# callbacks
	public function response_register($datamodel)
	{
		$datamodel->input();
		$registererrors = $datamodel->datapool->variables['registererrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('registererrors')
			->errormessageindex('register')
			->validate();
		$datamodel->finishinput();
        if ($result)
        {
            // Data has been validated, register the user
			$useraccount = new User_Account();
			$framedata = $datamodel->dataframe->as_array();
			$result = $useraccount->subscribe($framedata,User_Account::SUBSCRIBE_SEND_EMAIL);
			if (!$result) {
				$errorinfo = $useraccount->error_info();
				$message = Kohana::message('register',$errorinfo[0]);
				$registererrors->errors[] = array('error' => $message);
			}
		}
		if ($result) 
			$this->request->redirect('member/register/confirmation');
		return $result;
	}
	public function response_registerdirect($datamodel)
	{
		$datamodel->input();
		$registererrors = $datamodel->datapool->variables['registererrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('registererrors')
			->errormessageindex('register')
			->validate();
		$datamodel->finishinput();
        if ($result)
        {
            // Data has been validated, register the user
			$useraccount = new User_Account();
			$framedata = $datamodel->dataframe->as_array();
			$result = $useraccount->register($framedata);
			if (!$result) {
				$errorinfo = $useraccount->error_info();
				$message = Kohana::message('register',$errorinfo[0]);
				$registererrors->errors[] = array('error' => $message);
			}
		}
		if ($result) {
		
			$account_data = User_Account::instance()->account_data();
			$config = Kohana::$config->load('registeremails');
			$to      = $account_data->Name . ' <' . $account_data->Email . '>';
			$subject = $config['direct']['subject'];
			$headers = $config['headers'];
			
			$user = User::instance()->model()->activerecord();
			
			$dictionary = array(
				'username' => $account_data->Name,
				'website' => System::instance()->Website,
				'usercode' => $account_data->Code,
				'queryemail' => System::instance()->ReturnEmailAddress,
				'password' => $datamodel->datapool->arrays['post']['password'],
				'administratorname' => $user->Name,
				'administratorusercode' => $user->Code,
			);
			$emailview = Generator::view(array('filename' => 'email_registerdirect','ext'=>'txt'),$dictionary);
			
			$result = mail($to, $subject, $emailview, $headers);
		
			unset($datamodel->datapool->variables['registererrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','registerupdate')),),
					'index'=>'registermessages',
				)
			);
			$this->request->redirect('member/register');
		}
		return $result;
	}
	public function response_login($datamodel)
	{
		$datamodel->input();
		$data = $datamodel->dataframe->as_array();
		$usercode = $data['usercode'];
		$remember = $data['remember'];
		$result = $this->_user->login($usercode,$data['password'],$remember);
		if ($result) {
			$session = session::instance();
			$uri = $session->get_once('loginuri');
			if ($uri) $datamodel->controller->request->redirect($uri);
			$datamodel->controller->request->redirect('member/login');
		}
		// failed login
		$loginerrors = $datamodel->datapool->variables['loginerrors'] = (object) array('errors'=>array());
		$error = $this->_user->error_message();
		$loginerrors->errors[] = array('error' => $error);
		return FALSE;
	}
	public function response_newpassword($datamodel)
	{
		$datamodel->input();
		$data = $datamodel->dataframe->as_array();
		$email = $data['passwordemail'];
		$usercode = $data['usercode'];
		$validation = new Validation($data);
		$validation
			->rule('passwordemail', 'not_empty')
			->rule('passwordemail', 'email')
			->rule('usercode','not_empty');
		$loginerrors = $datamodel->datapool->variables['loginerrors'] = (object) array('errors'=>array());
		if (!($success = $validation->check())) {
			$loginerrors = $datamodel->datapool->variables['loginerrors'] = (object) array('errors'=>array());
			$errors = $validation->errors('login');
			foreach ($errors as $error) {
				$loginerrors->errors[] = array('error' => $error);
			}
		} else {
			$useraccount = new User_Account();
			$table = $useraccount->model()->table();
			$table->reset()->query()->where('Code','=',$usercode);
			$userrecord = $table->activerecord();
			$userrecord->load();
			$success = (bool) $userrecord->current();
			if (!$success) {
				$loginerrors->errors[] = array('error',Kohana::message('login','invalidusercode'));
				return FALSE;
			}
		}
		if ($success) {
			$data = array('usercode' => $usercode,'email' => $email);
			$success = $useraccount->petition_for_password($data,TRUE); // send email
			if (!$success) {
				$error = $useraccount->error_message();
				$loginerrors->errors[] = array('error'=>$error);
			} else {
				unset($datamodel->datapool->variables['loginerrors']);
				$loginmessages = $datamodel->datapool->variables['loginmessages'] = (object) array('messages'=>array());
				$loginmessages->messages[] = array('message' => Kohana::message('login','passwordrequestsent'));
			}
		}
		return FALSE;
	}
	public function response_usercodelist($datamodel)
	{
		$datamodel->input();
		$data = $datamodel->dataframe->as_array();
		$email = $data['usercodeemail'];
		$validation = new Validation($data);
		$validation
			->rule('usercodeemail', 'not_empty')
			->rule('usercodeemail', 'email');
		$loginerrors = $datamodel->datapool->variables['loginerrors'] = (object) array('errors'=>array());
		if (!($success = $validation->check())) {
			$loginerrors = $datamodel->datapool->variables['loginerrors'] = (object) array('errors'=>array());
			$errors = $validation->errors('login');
			foreach ($errors as $error) {
				$loginerrors->errors[] = array('error' => $error);
			}
		} else {
			$db = User::instance()->model()->db();
			$resultlist = DB::select('Code')
				->from('Access_Users')
				->where('Email','=',$email)
				->prepare($db)
				->execute()->as_array();
			if (empty($resultlist)) {
				$success = FALSE;
				$loginerrors->errors[] = array('error' => Kohana::message('login','noresults'));
			} else {
				$usercodes = array();
				foreach ($resultlist as $usercode) {
					$usercodes[] = array('usercode' => $usercode['Code']);
				}
			}
		}
		if ($success) {
			$config = Kohana::$config->load('registeremails');
			$to      = '<' . $email . '>';
			$subject = $config['usercodes']['subject'];
			$headers = $config['headers'];
			
			$user = User::instance()->model()->activerecord();
			
			$dictionary = array(
				'website' => System::instance()->Website,
				'usercodes' => $usercodes,
				'queryemail' => System::instance()->ReturnEmailAddress,
			);
			$emailview = Generator::view(array('filename' => 'email_loginusercodes','ext'=>'txt'),$dictionary);
			$emailview->render();
			$result = mail($to, $subject, $emailview, $headers);
			
			unset($datamodel->datapool->variables['loginerrors']);
			$loginmessages = $datamodel->datapool->variables['loginmessages'] = (object) array('messages'=>array());
			$loginmessages->messages[] = array('message' => Kohana::message('login','usercodesent'));
		}
		return FALSE;
	}
	public function account_basicupdate($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'basic';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['basicerrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('basicerrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			if (is_string($ownerrecord->Permissions)) {
				$permissions = $ownerrecord->Permissions = json_decode($ownerrecord->Permissions);
			} else {
				$permissions = new StdClass();
			}
			$cacherecord = $datamodel->cache->records['useraccount'] = new StdClass();
			$cacherecord->Permissions = $permissions;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$cacherecord->Permissions = json_encode($cacherecord->Permissions);
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['basicerrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
			unset($datamodel->datapool->variables['basicerrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','basicupdate')),),
					'index'=>'basicmessages',
				));
		} else {
			$datamodel->recoverinput();
		}
		unset($datamodel->datapool->constraints['edit']);
		return $result;
	}
	public function account_updateimagesettings($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'picture';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['pictureerrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('pictureerrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			if (is_string($ownerrecord->Permissions)) {
				$permissions = $ownerrecord->Permissions = json_decode($ownerrecord->Permissions);
			} else {
				$permissions = new StdClass();
			}
			$cacherecord = $datamodel->cache->records['useraccount'] = new StdClass();
			$cacherecord->Permissions = $permissions;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$cacherecord->Permissions = json_encode($cacherecord->Permissions);
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['pictureerrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
			unset($datamodel->datapool->variables['pictureerrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','pictureupdate')),),
					'index'=>'picturemessages',
				));
		} else {
			$datamodel->recoverinput();
		}
		unset($datamodel->datapool->constraints['edit']);
		return $result;
	}
	public function account_uploadimage($datamodel)
	{
		$activerecord = $datamodel->datapool->records['useraccount'];
		$usercode = $activerecord->Code;
		$filer = Filer::instance();
		$success = $filer->import($_FILES['file'])->as_image()->base('profiles')->to($usercode)->filename($usercode)->go();
		if ($success) {
			$activerecord->Image = $filer->filename();
			$success = $activerecord->save();
			if (!$success) {
				$errorobject = new StdClass();
				$errorobject->errors = array('error'=>kohana::message('useraccount','imagenamenotsaved'));
				$datamodel->datapool->variables['pictureerrors']=$errorobject;
			}
		} else {
			$errors = $filer->errors();
			$errorlist = array();
			foreach ($errors as $error) {
				$errorlist[] = array('error' => $error);
			}
			$errorobject = new StdClass();
			$errorobject->errors = $errorlist;
			$datamodel->datapool->variables['pictureerrors']=$errorobject;
		}
		if ($success) { // modify
			$filespec = $filer->target_filespec();
			$filer->modify($filespec)->as_image()->resize(100)->go();
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','pictureupload')),),
					'index'=>'picturemessages',
				)
			);
		}
		return $success;
	}
	public function account_changepassword($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'password';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['passworderrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('passworderrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			$cacherecord = $datamodel->cache->records['useraccount'] = $ownerrecord;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['passworderrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
			unset($datamodel->datapool->variables['passworderrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','passwordupdate')),),
					'index'=>'passwordmessages',
				));
		} else {
			$datamodel->recoverinput();
		}
		unset($datamodel->datapool->constraints['edit']);
		return $result;
	}
	public function account_editdetailprofile($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'profile';
		return FALSE;
	}
	public function account_previewdetailprofile($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'profile';
		$datamodel->datapool->constraints['preview'] = TRUE;
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['profileerrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('profileerrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) {
			unset($datamodel->datapool->variables['profileerrors']);
			$messages = $datamodel->datapool->variables['profilemessages'] = (object) array('messages' => array());
			$messages->messages[] = array('message'=>kohana::message('useraccount','profilepreview'));
		} 
		$datamodel->recoverinput();
		return FALSE;
	}
	public function account_savedetailprofile($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'profile';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['profileerrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('profileerrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			if (is_string($ownerrecord->Permissions)) {
				$permissions = $ownerrecord->Permissions = json_decode($ownerrecord->Permissions);
			} else {
				$permissions = new StdClass();
			}
			$cacherecord = $datamodel->cache->records['useraccount'] = new StdClass();
			$cacherecord->Permissions = $permissions;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$cacherecord->Permissions = json_encode($cacherecord->Permissions);
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['profileerrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
			unset($datamodel->datapool->variables['profileerrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','profileupdate')),),
					'index'=>'profilemessages',
				));
			unset($datamodel->datapool->constraints['edit']);
		} else {
			$datamodel->recoverinput();
		}
		return $result;
	}
	public function account_saveandeditdetailprofile($datamodel)
	{
		$result = $this->account_savedetailprofile($datamodel);
		if ($result) {
			$session = session::instance();
			$messages = $session->get_once('messages');
			if ($messages) {
				$index = $messages['index'];
				unset($messages['index']);
				$messages = (object) $messages;
				$datamodel->datapool->variables[$index] = $messages;
			}
			$datamodel->dataframe->reset();
			return $this->account_editdetailprofile($datamodel);
		} else {
			return $result;
		}
	}
	public function account_canceldetailupdate($datamodel)
	{
		return TRUE; // this just causes redirect. Done.
	}
	public function account_savescratchpad($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'scratchpad';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['scratchpaderrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('scratchpaderrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			$cacherecord = $datamodel->cache->records['useraccount'] = $ownerrecord;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['scratchpaderrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
			unset($datamodel->datapool->variables['scratchpaderrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','scratchpadupdate')),),
					'index'=>'scratchpadmessages',
				));
		} else {
			$datamodel->recoverinput();
		}
		unset($datamodel->datapool->constraints['edit']);
		return $result;
	}
	public function account_adminupdate($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'admin';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['adminerrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('adminerrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			if (is_string($ownerrecord->Permissions)) {
				$permissions = $ownerrecord->Permissions = json_decode($ownerrecord->Permissions);
			} else {
				$permissions = new StdClass();
			}
			$cacherecord = $datamodel->cache->records['useraccount'] = new StdClass();
			$cacherecord->Permissions = $permissions;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$cacherecord->Permissions = json_encode($cacherecord->Permissions);
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['adminerrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
		
			$config = Kohana::$config->load('registeremails');
			$to      = $owneractive->Name . ' <' . $owneractive->Email . '>';
			$subject = $config['adminchange']['subject'];
			$headers = $config['headers'];
			
			$options = array();
			
			$account_data = User_Account::instance()->account_data();
			
			$db = User_Account::instance()->model()->db();
			
			$arguments = (object) array(
				'name' => 'Access_Group_Roles',
				'index' => 'ID',
				'value' => 'Name',
			);
			$value = NULL;
			$record = DB::select($arguments->index,$arguments->value)
				->from($arguments->name)
				->where($arguments->index,'=',$account_data->RoleID)
				->as_object()
				->execute($db)->current();
			if ($record) {
				$field = $arguments->value;
				$value = $record->$field;
			}

			$options[] = array(
				'option' => 'Role',
				'value' => $value,
			);
			
			$arguments = (object) array(
				'name' => 'Access_Lookup_User_Status',
				'index' => 'ID',
				'value' => 'Name',
			);
			$value = NULL;
			$record = DB::select($arguments->index,$arguments->value)
				->from($arguments->name)
				->where($arguments->index,'=',$account_data->StatusID)
				->as_object()
				->execute($db)->current();
			if ($record) {
				$field = $arguments->value;
				$value = $record->$field;
			}
			$options[] = array(
				'option' => 'Status', 
				'value' => $value,
			);
			if ($account_data->Permissions->comment) {
				$options[] = array(
					'option' => 'Add comments', 
					'value' => 'Authorized',
				);
			}
			if ($account_data->Permissions->blog) {
				$options[] = array(
					'option' => 'Personal blog',
					'value' => 'Authorized',
				);
			}
			if ($account_data->Permissions->multiblog) {
				$options[] = array(
					'option' => 'Multiple blogs', 
					'value' => 'Authorized',
				);
			}
			if ($account_data->Permissions->article) {
				$options[] = array(
					'option' => 'Create documents',
					'value' => 'Authorized',
				);
			}
			if ($account_data->Permissions->unmoderated) {
				$options[] = array(
					'option' => 'Unmoderated',
					'value' => 'Authorized',
				);
			}
			$dictionary = array(
				'username' => $owneractive->Name,
				'website' => System::instance()->Website,
				'usercode' => $owneractive->Code,
				'options' => $options,
				'queryemail' => System::instance()->ReturnEmailAddress,
			);
			$emailview = Generator::view(array('filename' => 'email_registerchange','ext'=>'txt'),$dictionary);
			
			$result = mail($to, $subject, $emailview, $headers);

			unset($datamodel->datapool->variables['adminerrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','adminupdate')),),
					'index'=>'adminmessages',
				));
		} else {
			$datamodel->recoverinput();
		}
		unset($datamodel->datapool->constraints['edit']);
		return $result;
	}
	public function account_saveadminnotes($datamodel)
	{
		$datamodel->datapool->constraints['edit'] = 'adminnotes';
		$datamodel->input(); // collect data from browser
		$datamodel->datapool->variables['adminnoteserrors'] = (object) array('errors' => array());
		$result = $datamodel
			->errordictionaryindex('adminnoteserrors')
			->errormessageindex('useraccount')
			->validate();
		$datamodel->finishinput();
		if ($result) { // set up for store
			$owneractive = User_Account::instance()->model()->activerecord();
			$ownerrecord = clone $owneractive->current();
			$cacherecord = $datamodel->cache->records['useraccount'] = $ownerrecord;
			$datamodel
				->recordcaching(TRUE)
				->store(); // store to cacherecord
			$result = (bool) $owneractive->update((array) $cacherecord)->save();
			if (!$result) {
				// set error messages
				$errorinfo = $owneractive->error_info();
				$message = Kohana::message('useraccount',$errorinfo[0]);
				if (!$message) $message = $errorinfo[2];
				$datamodel->datapool->variables['adminnoteserrors']->errors[] = array('error'=>$message);
			}
		}
		if ($result) {
			unset($datamodel->datapool->variables['adminnoteserrors']);
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('useraccount','adminnotesupdate')),),
					'index'=>'adminnotesmessages',
				));
		} else {
			$datamodel->recoverinput();
		}
		unset($datamodel->datapool->constraints['edit']);
		return $result;
	}
	public function account_updateblogid($datamodel)
	{
		$parameters = (object) $datamodel->get_parameters();
		$myblogid = $parameters->myblogid;
		$owneractiverecord = $datamodel->datapool->records['useraccount'];
		$objecttype = Services::dblookup('Articles','ObjectType','ID',$myblogid);
		if ($objecttype == 'blog') {
			$owneractiverecord->BlogID = $myblogid;
			$owneractiverecord->save();
		}
		return TRUE;
	}
	public function account_clearblogid($datamodel)
	{
		$owneractiverecord = $datamodel->datapool->records['useraccount'];
		$owneractiverecord->BlogID = NULL;
		$owneractiverecord->save();
		return TRUE;
	}
	public function account_updateinboxid($datamodel)
	{
		$parameters = (object) $datamodel->get_parameters();
		$myinboxid = $parameters->myinboxid;
		$owneractiverecord = $datamodel->datapool->records['useraccount'];
		$objecttype = Services::dblookup('Articles','ObjectType','ID',$myinboxid);
		if ($objecttype == 'inbox') {
			$owneractiverecord->InboxID = $myinboxid;
			$owneractiverecord->save();
		}
		return TRUE;
	}
	public function account_clearinboxid($datamodel)
	{
		$owneractiverecord = $datamodel->datapool->records['useraccount'];
		$owneractiverecord->InboxID = NULL;
		$owneractiverecord->save();
		return TRUE;
	}
}
