<?php defined('SYSPATH') or die('No direct script access.');
/*
 ** Picture handling html code should accomodate use classes, attributes, and styles
 by adding custom data to $node->decoration arrays
*/
class Controller_Site Extends Controller_Base {
	public $markup_datapool;
	public function simplewiki_instance()
	{
		if (!$this->_simplewiki_instance) {
		
			$this->markup_datapool['base'] = $this->_base;
			$this->markup_datapool['loggedin'] = $this->_user->authenticated();
			$this->markup_datapool['true'] = TRUE;
			$this->markup_datapool['false'] = FALSE;
			$this->markup_datapool['website'] = System::instance()->Website;
			
			$simplewiki = $this->_simplewiki_instance = New SimpleWiki();
			$simplewiki->parser()->argchars('(?s:.)'); // any character
			// initialize
			$simplewiki->register_symlinks(
				array(
					'Uploads' => $this->_base . 'public/uploads/',
					'Local' => $this->_base,
					'Images' => $this->_base . $this->_common . 'images/',
					'Profiles' => $this->_base. 'public/profiles/',
					
					# specific controllers
					'Document' => $this->_base . 'documents/',
					'Topic' => $this->_base . 'topics/',
					'Blog' => $this->_base . 'blogs/',
					'Post' => $this->_base . 'blogs/entry/',
					'File' => $this->_base . 'library/docfiles/',
					'Extract' => $this->_base . 'library/docextracts/',
					'Link' => $this->_base . 'library/links/',
					'Folder' => $this->_base . 'library/folders/',
					'Page' => $this->_base . 'pages/',
					// Directory not listed; not appropriate
				)
			);
			$this->_handled_symlinks = array(
				'Picture' => $this->_base . 'library/pictures/', // translates to file path in {{
				'Node' => $this->_base . 'node/', // looks up appropriate controller for the node
				'Widget' => $this->_base . 'library/widgets/', // translates to embed block in {{
			);
			$simplewiki->register_symlink_handler(array($this,'symlink_handler'));
			$simplewiki->register_macro_callbacks(array(
				'var'=>array($this,'macro_var'),
				'template'=>array($this,'macro_template'),
			));
			$simplewiki->register_class_callbacks(
				array(
					'image' => array(
						'lframe'=> array($this,'callback_image_frame'),
						'rframe'=> array($this,'callback_image_frame'),
						'frame'=> array($this,'callback_image_frame'),
					),
			));
			$simplewiki->register_property_callbacks(
				array(
					'blockdef' => array(
						'condition'=> array($this,'callback_block_condition'),
						'notcondition'=> array($this,'callback_block_notcondition'),
					),
			));
		}
		return $this->_simplewiki_instance;
	}
	public function macro_var($node)
	{
		$varname = reset($node->decoration->classes); // first element
		if (!empty($varname)) {
			if (isset($this->markup_datapool[$varname])) {
				$node->output = $this->markup_datapool[$varname];
				$node->processed = TRUE;
			}
		}
		return $node;
	}
	public function macro_template($node)
	{
		$templatename = reset($node->decoration->classes); // first element
		if (!empty($templatename)) {
			$template = Template::view(array($templatename));
			$node->output = $template->render();
			$node->processed = TRUE;
		}
		return $node;
	}
	public function callback_block_condition($node)
	{
		$condition = $node->decoration->properties['condition'];
		if (!empty($condition)) {
			$result = $this->_get_condition($condition);
			if (!$result) {
				$this->_clear_node($node);
			}
		}
		return $node;
	}
	public function callback_block_notcondition($node)
	{
		$condition = $node->decoration->properties['notcondition'];
		if (!empty($condition)) {
			$result = $this->_get_condition($condition);
			if ($result) {
				$this->_clear_node($node);
			}
		}
		return $node;
	}
	protected function _get_condition($condition)
	{
		$result = FALSE;
		if (isset($this->markup_datapool[$condition])) {
			$result = (bool) $this->markup_datapool[$condition];
		}
		return $result;
	}
	protected function _clear_node($node)
	{
		$node->decoration = NULL;
		$node->opentag_head = NULL;
		$node->opentag_tail = NULL;
		$node->closetag = NULL;
		$node->elementcontent = NULL;
	}
	public function symlink_handler($node)
	{
		$symlinks = $this->_handled_symlinks;
		if (isset($symlinks[$node->linkparts->symlink])) {
			$node->linkparts->symlinkpath = $symlinks[$node->linkparts->symlink];
			if ($node->type == 'image') { // import
				$this->_process_import_node($node);
			}
			if ($node->type == 'link') {
				$this->_process_link_node($node);
			}
		} else {
			$node->unknown = TRUE;
			$node->linkparts->symlinkpath = $node->linkparts->symlink . ':'; // stub
		}
		return $node;
	}
	// *** check that symlink name is either Picture or Widget
	protected function _process_import_node($node)
	{
		$symlink = $node->linkparts->symlink;
		$selector = $node->linkparts->internalselector;
		$version = $node->linkparts->internalversion;
		$record = NULL;
		if (!empty($symlink) and is_numeric($selector)) {
			$operation = DB::select('Title','Caption','Description','Directory','FileName','ObjectType','EmbedCode')
				->from('Nodes_View')
				->where('ID','=',$selector)
				->as_object()
				->prepare()
				->execute();
			if ($operation->success()) {
				$record = $operation->fetch();
			}
			if (!empty($record)) {
				if (($record->ObjectType == 'picture') and ($symlink == 'Picture')) {
					$node->linkparts->symlinkpath = $this->_get_filepath($record);
					$node->linkparts->internalselector = $this->_get_filename($record,$version);
					// assert properties
					property_exists($node,'title') or $node->title = NULL;
					property_exists($node,'caption') or $node->caption = NULL;
					if ($node->title == '+') $node->title = $record->Caption;
					if ($node->caption == '+') $node->caption = $record->Title;
				} elseif (($record->ObjectType == 'widget') and ($symlink == 'Widget')) {
					$embed = $node->linkparts->embedcode = $record->EmbedCode; // available for class call
					$node->opentag_head = '<div';
					$node->opentag_tail = '>' . $embed . '</div>';
					$node->linkparts->symlinkpath = NULL;
					$node->linkparts->internalselector = NULL;
					// assert properties
					property_exists($node,'title') or $node->title = NULL;
					property_exists($node,'caption') or $node->caption = NULL;
					if ($node->title == '+') $node->title = $record->Caption;
					if ($node->caption == '+') $node->caption = $record->Title;
				} else {
					$record = NULL;
				}
			} 
		}
		if (empty($record)) {
			$node->unknown = TRUE;
			$node->linkparts->symlinkpath = $node->linkparts->symlink . ':'; // stub
		}
	}
	protected function _get_filepath($record)
	{
		$system = System::instance();
		return $this->_base.$system->PublicBase.$system->UploadBase.$record->Directory;
	}
	protected function _get_filename($record,$version)
	{
		if (!empty($version)) {
			$pathinfo = pathinfo($record->FileName);
			$filename = $pathinfo['filename'].'.'.$version.'.'.$pathinfo['extension'];
		} else {
			$filename = $record->FileName;
		}
		return $filename;
	}
	public function callback_image_frame($node)
	{
		$lframeindex = array_search('lframe',$node->decoration->classes);
		$rframeindex = array_search('rframe',$node->decoration->classes);
		$frameindex = array_search('frame',$node->decoration->classes);
		if ($rframeindex !== FALSE) // must be rframe
		{
			unset($node->decoration->classes[$rframeindex]);
			$orientation = 'rframe';
		}
		elseif ($lframeindex !== FALSE) // must be lframe;
		{
			unset($node->decoration->classes[$lframeindex]);
			$orientation = 'lframe';
		}
		else // must be frame
		{
			unset($node->decoration->classes[$frameindex]);
			$orientation = '';
		}
		if (!empty($node->linkparts->symlink) and ($node->linkparts->symlink == 'Picture')) {
			$symlink = $node->linkparts->symlink;
			$sympath = $this->_handled_symlinks[$symlink];
			// assert properties
			property_exists($node,'title') or $node->title = NULL;
			property_exists($node,'caption') or $node->caption = NULL;
			$dictionary['orientation'] = $orientation;
			$dictionary['caption'] = $node->title;
			$dictionary['title'] = $node->caption;
			// width
			$version = $node->linkparts->internalversion;
			if (!empty($version)) {
				$versions = (array) System::instance()->PictureSizes;
				if (!empty($versions[$version])) {
					$dictionary['width'] = $versions[$version].'px';
				} else {
					$dictionary['width'] = 'auto';
				}
			}
			// recordspec
			$replace = array('Picture:' => $sympath);
			$dictionary['recordspec'] = strtr($node->linkparts->internaladdress,$replace);
			$opentemplate = 
			'<div class="frame {{orientation}}" style="width:{{width}}">
			<h6>{{title}}</h6>
			<a target="_blank" href="{{recordspec}}">';
			$closetemplate = 
			'</a>{{?caption}}<br />{{caption}}{{/?caption}}</div>
			';
			$node->opentag_head = Template::view($opentemplate,$dictionary)->render() . $node->opentag_head;
			$node->opentag_tail .= Template::view($closetemplate,$dictionary)->render();
		} elseif (!empty($node->linkparts->symlink) and ($node->linkparts->symlink == 'Widget')) {
			$symlink = $node->linkparts->symlink;
			$sympath = $this->_handled_symlinks[$symlink];
			// assert properties
			property_exists($node,'title') or $node->title = NULL;
			property_exists($node,'caption') or $node->caption = NULL;
			$dictionary['caption'] = $node->title;
			$dictionary['title'] = $node->caption;
			isset($node->decoration->attributes['title']) and  ($node->decoration->attributes['title'] = '');
			$replace = array('Widget:' => $sympath);
			$dictionary['recordspec'] = strtr($node->linkparts->internaladdress,$replace);
			$node->decoration->classes[] = 'frame';
			$node->decoration->classes[] = $orientation;
			$opentemplate = '<div';
			$closetemplate = 
			'>
			<h6>{{title}}</h6>' . $node->linkparts->embedcode .
			'<p>{{?caption}}{{caption}} | {{/?caption}}see <a target="_blank" href="{{recordspec}}">widget page</a></p>
			</div>
			';
			$node->opentag_head = $opentemplate;
			$node->opentag_tail = Template::view($closetemplate,$dictionary)->render();;
		} else {
			$opentag_head = $node->opentag_head;
			$opentag_tail = $node->opentag_tail;
			$opentag_head = "<div class='frame $orientation'>" . $opentag_head;
			$opentag_tail .= "<br>{$node->caption}</div>";
			$node->opentag_head = $opentag_head;
			$node->opentag_tail = $opentag_tail;
		}
		
		return $node;
	}
	protected function _process_link_node($node)
	{
		$selector = $node->linkparts->internalselector;
		$ref = strstr($selector,'?',TRUE); // record selector only, no parameters
		if ($ref === FALSE) $ref = $selector;
		$symlink = $node->linkparts->symlink;
		if (!empty($symlink) and ($symlink == 'Node'))
			$sympath = $this->_handled_symlinks[$symlink];
		// assert properties
		property_exists($node,'title') or $node->title = NULL;
		property_exists($node,'caption') or $node->caption = NULL;
		if (($node->title == '+') or ($node->caption == '+')) {
			if (!empty($ref) and is_numeric($ref)) {
				$operation = DB::select('Title','Caption')
					->from('Nodes_View')
					->where('ID','=',$ref)
					->as_object()
					->prepare()
					->execute();
				if ($operation->success()) {
					$record = $operation->fetch();
				}
				if (!empty($record)) {
					if ($node->title == '+') $node->title = $record->Caption;
					if ($node->caption == '+') $node->elementcontent = $node->caption = $record->Title;
				}
			}
			if (empty($record)) {
				$node->unknown = TRUE;
				$node->linkparts->symlinkpath = $node->linkparts->symlink . ':'; // stub
			}
		}
		return $node;
	}
}
