<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
class Controller_SubLists Extends Controller_Site {
	public function before()
	{
		$this->_template = 'template_frame'; // minimal template for iframes
		parent::before();
	}
	public function action_index()
	{
		$type = $this->request->param('type');
		switch ($type) {
			case 'collaborators':
				$this->_collaborators();
				break;
			case 'containers':
				$this->_containers();
				break;
			case 'contents':
				$this->_contents();
				break;
			case 'comments':
				$this->_comments();
				break;
			case 'attachments':
				$this->_attachments();
				break;
			case 'itemlist':
				$this->_itemlist();
				break;
			default: die('incorrect list type');
		}
	}
	protected function _protect_admin($articleid)
	{
		if (!is_numeric($articleid))
			die('master record number required');
		$user = $this->_user;
		$authenticated = $user->authenticated();
		if (!$authenticated) {
			die('must be logged in to access administrative list service');
		}
		$record = Services::dbrecordlookup('Articles',array('ObjectType','AuthorID'),'ID',$articleid);
		if (!$record)
			die('invalid master record number');
		$objecttype = $record->ObjectType;
		$tokens = Services::get_objecttype_tokenset($objecttype);
		$ownerid = $record->AuthorID;
		$authorized = Access::instance()->permission($tokens->content,'R',$ownerid,$articleid);
		if (!$authorized)
			die('unauthorized use of embedded administrative list');
	}
	// actually a master record with collaborators attached
	protected function _collaborators()
	{
		$selector = $this->request->param('selector');
		if ($selector != 'admin') 
			die('invalid collaborator list call');
		$articleid = $this->request->param('code');
		$this->_protect_admin($articleid);
		$node = Base\Node::collaborator_list($this); // actually a record node of the master record
		$record = Services::dbrecordlookup('Articles',array('ObjectType','Frozen'),'ID',$articleid);
		if (!$record) die('unknown master record');
		$frozen = (bool) $record->Frozen;
		$objecttype = $record->ObjectType;
		$subcontext = new StdClass;
		$subcontext->articleid = $articleid;
		$subcontext->objecttype = $objecttype;
		$subcontext->frozen = $frozen;
		$node->extend_context($subcontext);
		$data = $node->generate();
		$dictionary = $data['dictionary'];
		$view = Template::view(array('collaborators_list'),$dictionary);
		
		$pageview = $this->_pageview;
		$pageview->centercolumn = $view;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _containers()
	{
		$selector = $this->request->param('selector');
		if ($selector != 'admin')
			die('incorrect mode for embedded containers form');
		$code = $this->request->param('code');
		if (!is_numeric($code))
			die('incorrect articleid for embedded containers form');
		$this->_protect_admin($code);
		# get data
		$node = Base\Node::container_list($this);
		$subcontext = new StdClass;
		$articleid = $code;
		$subcontext->articleid = $articleid;
		$subcontext->foreignkeyvalue = $articleid;
		$subcontext->redirect = 'sublists/containers/admin/' . $articleid;
		$node->extend_context($subcontext);
		$data = $node->generate();
		$dictionary = $data['dictionary'];
		# create templates
		$containerview = Template::view(array('containers_list'),$dictionary);
		# combine templates
		$pageview = $this->_pageview;
		$pageview->centercolumn = $containerview;
		# finish
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _contents()
	{
		$selector = $this->request->param('selector');
		if ($selector == 'admin')
			return $this->_contents_admin();
		
		$pageview = $this->_pageview;
		$pageview->centercolumn = 'unexpected contents list';
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _contents_admin()
	{
		# get data
		$articleid = $this->request->param('code');
		$this->_protect_admin($articleid);
		$node = Base\Node::contentsadmin_list($this);
		$frozen = (bool) Services::dblookup('Articles','Frozen','ID',$articleid);
		$subcontext = new StdClass;
		$subcontext->redirect = 'sublists/contents/admin/' . $articleid;
		$subcontext->articleid = $articleid;
		$subcontext->foreignkeyvalue = $articleid;
		$subcontext->frozen = $frozen;
		$node->extend_context($subcontext);
		$data = $node->generate();
		extract($data);
		# set template
		$listview = Template::view(array('contents_admin_list'),$dictionary);
		# finish
		$pageview = $this->_pageview;
		$pageview->centercolumn = $listview;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _comments()
	{
		$articleid = $this->request->param('selector');

		# get data
		$commentsclosed = (bool) Services::dblookup('Articles','CommentsClosed','ID',$articleid);
		$node = Base\Node::comment_list($this);
		$subcontext = new StdClass;
		$subcontext->commentsclosed = $commentsclosed;
		$subcontext->articleid = $articleid;
		$subcontext->foreignkeyvalue = $articleid;
		$node->extend_context($subcontext);
		$data = $node->generate();
		$dictionary = $data['dictionary'];
		# create templates
		$wrapperview = Template::view(array('comment_list_wrapper'),$dictionary);
		$listview = Template::view(array('comment_list'),$dictionary);
		$wrapperview->listcontents = $listview;
		# combine templates
		$pageview = $this->_pageview;
		$pageview->headerelements = (object) array('headerelement' => '<base target="_top" />');
		$pageview->centercolumn = $wrapperview;
		# finish
		$pageview->title = 'Comments - List';
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _attachments()
	{
		$selector = $this->request->param('selector');
		if ($selector == 'admin')
			return $this->_attachments_admin();
		# get data
		$articleid = $this->request->param('selector');
		$node = Base\Node::attachment_list($this);
		$subcontext = new StdClass;
		$subcontext->articleid = $articleid;
		$subcontext->foreignkeyvalue = $articleid;
		$node->extend_context($subcontext);
		$data = $node->generate();
		$dictionary = $data['dictionary'];
//		var_dump($dictionary);
		# set template
		$view = Template::view(array('attachments_list'),$dictionary);
		# finish
		$pageview = $this->_pageview;
		$pageview->headerelements = (object) array('headerelement' => '<base target="_top" />');
		$pageview->centercolumn = $view;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _attachments_admin()
	{
		# get data
		$articleid = $this->request->param('code');
		$this->_protect_admin($articleid);
		$node = Base\Node::attachmentadmin_list($this);
		$frozen = (bool) Services::dblookup('Articles','Frozen','ID',$articleid);
		$subcontext = new StdClass;
		$subcontext->redirect = 'sublists/attachments/admin/' . $articleid;
		$subcontext->articleid = $articleid;
		$subcontext->foreignkeyvalue = $articleid;
		$subcontext->frozen = $frozen;
		$node->extend_context($subcontext);
		$data = $node->generate();
		$dictionary = $data['dictionary'];
		# set template
		$view = Template::view(array('attachments_admin_list'),$dictionary);
		# finish
		$pageview = $this->_pageview;
		$pageview->centercolumn = $view;
		$this->show_resources();
		$this->response->body($pageview);
	}
	protected function _itemlist()
	{
		# get identifier
		$itemid = $this->request->param('selector');
		$itemnumber = $this->request->param('code');
		$containerid = $this->request->param('id');
		if (!is_numeric($itemid) or !is_numeric($itemnumber)) {
			echo 'incomplete information';
			return;
		}
		$_GET['perpage'] =10;
		if (empty($_GET['page'])) {
			$page = $_GET['page'] = (int) ceil($itemnumber / 10);
		}
		$subcontext = new StdClass;
		$subcontext->redirect = 'sublists/itemlist/' . $itemid . '/' . $itemnumber  . '/'  . $containerid;
		$subcontext->articleid = $containerid;
		$subcontext->foreignkeyvalue = $containerid;
		$subcontext->itempath = $_GET['itempath'];
		if (!empty($containerid)) {
			$containerrecord = Services::dbrecordlookup('Articles_Extensions',array('DefaultOrder','DefaultDirection'),'ArticleID',$containerid);
			if ($containerrecord) {
				$subcontext->secondary_orderby = $containerrecord->DefaultOrder;
				$subcontext->secondary_direction = $containerrecord->DefaultDirection;
			}
			$node = Base\Node::items_list($this,'contents',$subcontext);
		} else {
			$node = Base\Node::items_list($this,'treetops',$subcontext);
		}
		$node->extend_context($subcontext);
		$data = $node->generate();
		extract($data);
		$dictionary['itemid'] = $itemid;
		$dictionary['perpage'] = $_GET['itemperpage'];
		$dictionary['containerid'] = $containerid;
		$dictionary['itemid'] = $itemid;
		$dictionary['itemnumber'] = $itemnumber;
		$dictionary['itempath'] = $_GET['itempath'];
		$dictionary['itemperpage'] = $_GET['itemperpage'];
		# set template
		$templatedict = array(
			'base'=>$this->_base,
			'common'=>$this->_common,
		);
		$pageview = Template::view(array('template_blank_frame'),$templatedict);
		$listview = Template::view(array('items_list'),$dictionary);
		# finish
		$pageview->centercolumn = $listview;
		$this->show_resources();
		$this->response->body($pageview);
	}
}
