<?php defined('SYSPATH') or die('No direct script access.');
// Muster Software Copyright (c) Henrik Bechmann, Toronto, Canada 2012. All rights reserved.
// See "musterlicence.txt" for licencing information.
// mustersoftware.net
class Controller_System extends Controller_Site {
	public function action_index()
	{
		$this->request->redirect('system/roles');
	}
	# ===============================================================
	# ---------------------[ groups ]--------------------------------
	# ===============================================================
	public function action_groups()
	{
		$arg = $this->request->param('id');
		$access = Access::instance();
		$permission = $access->permission('system','R');
		if (!$permission)
			$this->request->redirect('error/system-disallowed');
		$datamodel = $this->_datamodel = new DataModel($this);
		$base = $datamodel->datapool->variables['base'] = $this->_base;
		
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		$dictionary['groupsclass'] = 'highlight';
		$technicalview = Generator::view(array('system_sidebar'),$dictionary);

		$record = DB::select('Name')
			->from('Access_Groups')
			->as_object()->prepare($this->_user->model()->db())
			->execute()->current();
			
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		$dictionary['groupname'] = $record->Name;
		$view = Generator::view(array('system_groups'),$dictionary);
		
		$pageview = $this->_pageview;
		$pageview->leftcolumn = $technicalview;
		$pageview->title = 'System Groups';
		$pageview->centercolumn = $view;
		$this->response->body($pageview);
	}
	
	# ===============================================================
	# ---------------------[ roles ]---------------------------------
	# ===============================================================
	public function action_roles()
	{
		$access = Access::instance();
		$permission = $access->permission('system','R');
		if (!$permission)
			$this->request->redirect('error/system-disallowed');
			
		$tokenmodel = $this->_tokenmodel = new TableModel('tokenschema',array('mastertable'=>'groups'));
		$datamodel = $this->_datamodel = new DataModel($this);
		$datapool = $datamodel->datapool;
		
		$base = $datamodel->datapool->variables['base'] = $this->_base;
		
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		$dictionary['rolesclass'] = 'highlight';
		$technicalview = Template::view(array('system_sidebar'),$dictionary);
		
		$grouptable = $tokenmodel->table();
		$grouptable->query()->order_by('Name'); // default, so that first item is properly selected
		$datapool->activerecords['grouprecord'] = $grouptable->activerecord();
		
		$roletable = $datapool->tables['roletable'] = $tokenmodel->table('roles');
		$datapool->activerecords['rolerecord'] = $roletable->activerecord();

		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		
		$datamodel->dataframe('systemroles');
		if ($this->_is_request()) { // get input before retrieval and output
			$result = $datamodel
				->controlframe('systemrolescontrols')
				->responses(
					array(
						'group'=>array($this,'role_group'),
						'save'=>array($this,'role_save'),
						'add'=>array($this,'role_add'),
						'delete'=>array($this,'role_delete'),
						'cancel'=>array($this,'role_cancel'),
					))
				->respond();
			if ($result) {
				$url = 'system/roles';
				$this->request->redirect($url);
			}
		} 
		
		$dictionary = $datamodel->retrieve()->output() + $dictionary;
		
		$view = Template::view(array('system_roles'),$dictionary);

		$pageview = $this->_pageview;
		$pageview->leftcolumn = $technicalview;
		$pageview->title = 'System Roles';
		$pageview->centercolumn = $view;
		$this->show_resources();
		$this->response->body($pageview);
	}
	
	public function role_group($datamodel)
	{
		$datamodel->datapool->constraints['changegroup'] = TRUE;
		$datamodel->input()->finishinput();
		$groupoption = $datamodel->dataframe->pointlist['groupoptions']->value;
		$grouptable = $this->_tokenmodel->table();
		$roletable->preset()->query()->where('ID','=',$groupoption);
		return FALSE;
	}

	public function role_save($datamodel)
	{
		# set up objects to receive messages
		$roleerrors = $datamodel->datapool->variables['roleerrors'] = 
			(object) array('errors' => array());
		$rolemessages = $datamodel->datapool->variables['rolemessages'] = 
			(object) array('messages' => array());
		# collect page data and validate data
		$valid = $datamodel
			->input()
			->errordictionaryindex('roleerrors')
			->errormessageindex('roles')
			->validate();
		# store valid changed data
		$datamodel
			->finishinput()
			->recordcaching(TRUE)
			->store()
			->recordcaching(FALSE);
		$posted = FALSE;
		# preserve current scope
		$groupid = $datamodel->dataframe->pointlist['groupid']->value;
		$tokenmodel = $this->_tokenmodel;
		$grouptable = $tokenmodel->table();
		$grouptable->preset()->query()->where('ID','=',$groupid);
		# collect changedrecords as stored by store() above
		$changedrecords = $datamodel->cache->recordsets['rolerecords'];
		if (empty($changedrecords)) { // nothing to do
			$rolemessages->messages[] = 
				array('message' => Kohana::message('roles','norecords'));
		} else { // post changed records
			# collect objects to manipulate
			$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
			$roletable = $tokenmodel->table('roles');
			$roleactive = $roletable->activerecord();
			# process each changed record in turn
			foreach ($changedrecords as $reference => $changedrecord) {
				if (!is_null($changedrecord->ID)) { // save changes to existing record
					$roletable->preset()->query()->where('ID','=',$changedrecord->ID);
					$result = $roleactive->load()->update((array) $changedrecord)->save();
					if (!$result) $errorinfo = $roleactive->error_info();
				} else { // save new record
					$roletable->preset();
					$result = $roleactive->insert((array) $changedrecord)->save();
					if (!$result) { // collect error info
						$errorinfo = $roleactive->error_info();
					} else { // retrieve new saved record
						$roletable->preset()->query()->where('ID','=',$roleactive->insert_id());
						$result = (bool) $roleactive->current();
						if (!$result) $errorinfo = array('norecord','','Record not found');
					}
				}
				$listitem = $listitems[$reference]; // get matching memory structure
				if ($result) { // prepare for refresh
					$listitem->pointlist = array();
					$listitem->primarykeyvalue = $roleactive->ID;
					$listitem->recordstatus = 'posted';
					$posted = TRUE;
				} else { // report errors
					$message = Kohana::message('roles',$errorinfo[0]);
					if (!$message) $message = $errorinfo[2];
					$message .= ' [' . ($listitem->index + 1) . ']';
					$roleerrors->errors[] = array('error'=> $message);
					DataModel::set_listitem_errorstate($listitem);
					$valid = FALSE;
				}
			}
			$roletable->preset();
		}
		if ($posted) {
			$rolemessages->messages[] = 
				array('message' => Kohana::message('roles','updates'));
		}
		if ($valid) {
			unset($datamodel->datapool->variables['roleerrors']);
		}
		return FALSE;
	}
	public function role_add($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		$datamodel->datapool->variables['newrecordcount'] = 1;
		return FALSE;
	}
	public function role_delete($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		
		# preserve current scope
		$groupid = $datamodel->dataframe->pointlist['groupid']->value;
		$tokenmodel = $this->_tokenmodel;
		$grouptable = $tokenmodel->table();
		$grouptable->preset()->query()->where('ID','=',$groupid);
		# collect deletions
		if (!isset($_POST['roles']['deletecheckbox'])) {
			$deletions = array();
		} else {
			$deletions = $_POST['roles']['deletecheckbox'];
		}
		// exit if no deletions
		if (empty($deletions)) {
			$message = 'no records were marked for deletion';
			$rolemessages = $datamodel->datapool->variables['rolemessages'] = 
				(object) array('messages' => array());
			$rolemessages->messages[] = array('message' => $message);
			return FALSE;
		}
		# establish error object
		$roleerrors = $datamodel->datapool->variables['roleerrors'] = 
			(object) array('errors' => array());
		# get record cross reference items
		$listitems = &$datamodel->dataframe->pointlist['records']->pointlist;
		$roletable = $tokenmodel->table('roles');
		$roleactive = $roletable->activerecord();
		$deletioncount = 0;
		# delete records
		$errors = FALSE;
		foreach ($deletions as $deletion) { // deletion is reference: <string><index>
			$listitem = $listitems[$deletion];
			$id = $listitem->pointlist['id']->value;
			$result = TRUE;
			if (!is_null($id)) { // record exists, delete it
				$roletable->preset()->query()->where('ID','=',$id);
				$result = (bool) $roleactive->current();
				if ($result) {
					$result = $roleactive->delete();
				} else {
					$result = TRUE; // record already gone
				}
			}
			// ... and remove listitem,
			if ($result){
				unset($listitems[$deletion]);
				$deletioncount++;
			} else {
				$errors = TRUE;
				$errorinfo = $roleactive->error_info();
				$error = Kohana::message('roles',$errorinfo[0]);
				if (!$error) $error = $errorinfo[2];
				$error .= ' [' . ($listitem->index + 1) . ']';
				DataModel::set_listitem_errorstate($listitem);
				$roleerrors->errors[] = array('error'=>$error);
			}
		}
		DataModel::renumber_listitems($listitems,'role');
		// create feedback message
		$message = ($deletioncount) . ' records were deleted';
		$rolemessages = $datamodel->datapool->variables['rolemessages'] = 
			(object) array('messages' => array());
		$rolemessages->messages[] = array('message' => $message);
		if (!$errors) unset($datamodel->datapool->variables['roleerrors']);
		return FALSE;
	}
	public function role_cancel($datamodel)
	{
		# collect page data without filtering
		$datamodel->input(Point::INPUT_RECYCLE);
		# get current master table primary key
		$groupid = $datamodel->dataframe->pointlist['groupid']->value;
		# remove records data so that it will be refreshed from database
		unset($datamodel->dataframe->pointlist['records']); // *** s/b done with constraint, but conflicts with 'changerole' (!)
		# set the master table according to primary key 
		$grouptable = $this->_tokenmodel->table();
		$grouptable->preset()->query()->where('ID','=',$groupid);
		# set feedback message
		$rolemessages = $datamodel->datapool->variables['rolemessages'] = 
			(object) array('messages' => array());
		$rolemessages->messages[] = array('message' => Kohana::message('roles','cancelled'));
		# driver will refresh data
		return FALSE;
	}
	
	# ===============================================================
	# ---------------------[ tokens ]--------------------------------
	# ===============================================================
	public function action_tokens()
	{
		$orderby = $this->request->param('id');
		$access = Access::instance();
		$permission = $access->permission('system','R');
		if (!$permission)
			$this->request->redirect('error/system-disallowed');
		$datamodel = $this->_datamodel = new DataModel($this);
		$base = $datamodel->datapool->variables['base'] = $this->_base;
		
		$sidebardictionary = array();
		$sidebardictionary['base'] = $base;
		$sidebardictionary['common'] = $this->_common;
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		$session = session::instance();
		$messages = $session->get_once('messages');
		if ($messages) {
			$index = $messages['index'];
			unset($messages['index']);
			$messages = (object) $messages;
			$datamodel->datapool->variables[$index] = $messages;
		}
		switch ($orderby) {
			case NULL: 
				$sidebardictionary['tokensclass'] = 'highlight';
				$view = $this->_tokens($dictionary);
				break;
			case 'byroles':
				$sidebardictionary['byrolesclass'] = 'highlight';
				$view = $this->_tokensbyrole($dictionary);
				break;
			case 'bytokens':
				$sidebardictionary['bytokensclass'] = 'highlight';
				$view = $this->_rolesbytoken($dictionary);
				break;
			default:
				$this->request->redirect('error/system-wrongtokenselection');
		}
		$technicalview = Template::view(array('system_sidebar'),$sidebardictionary);

		$pageview = $this->_pageview;
		$pageview->leftcolumn = $technicalview;
		$pageview->title = 'System Tokens';
		$pageview->centercolumn = $view;
		$this->show_resources();
		$this->response->body($pageview);
	}
	
	protected function _tokens($dictionary)
	{
		$this->_tokenmodel = $tokenmodel = new TableModel('tokenschema',array('mastertable','tokens'));
		$datamodel = $this->_datamodel;
		$datamodel->dataframe('systemtokens');
		if ($this->_is_request()) {
			$result = $datamodel
				->controlframe('systemtokenscontrols')
				->responses(
					array(
						'save'=>array($this,'tokens_save'),
						'add'=>array($this,'tokens_add'),
						'delete'=>array($this,'tokens_delete'),
						'generate'=>array($this,'tokens_generate'),
						'cancel'=>array($this,'tokens_cancel'),
					))
				->respond();
			if ($result) {
				$url = 'system/tokens';
				$this->request->redirect($url);
			}
		} else {
			$tokenmodel->table()->query()->limit(300);
			$records = $tokenmodel->table()->as_array();
			$datamodel->datapool->recordsets['tokenrecords'] = $records;
		}
		$dictionary = $datamodel->retrieve()->output() + $dictionary;
		$view = Template::view(array('system_tokens'),$dictionary);
		return $view;
	}
	
	public function tokens_save($datamodel)
	{
		# collect, validate, and store responses
		$tokenerrors = $datamodel->datapool->variables['tokenerrors'] = 
			(object) array('errors' => array());
		$valid = $datamodel
			->input()
			->errordictionaryindex('tokenerrors')
			->errormessageindex('tokens')
			->validate();
		$datamodel
			->recoverinput()
			->finishinput()
			->recordcaching(TRUE)
			->store();
		// prepare for processing of changed records
		$datamodel->recordcaching(FALSE); // anticipate standard retrieve
		$changedrecords = $datamodel->cache->recordsets['tokenrecords'];
		$datamodel->datapool->recordsets['tokenrecords'] = array();
		$retrieverecords = &$datamodel->datapool->recordsets['tokenrecords'];
		$posted = FALSE;
		$tokenmessages = $datamodel->datapool->variables['tokenmessages'] = 
			(object) array('messages' => array());
		# if there are no changed or invalid records, return and reset
		if (empty($changedrecords) and $valid) {
			$session = session::instance();
			$session->set('messages',
				array('messages' => 
					array(
						array('message'=>kohana::message('tokens','norecords')),),
					'index'=>'tokenmessages',
				));
			return TRUE;
		}
		# process changed records
		if (!empty($changedrecords)) {
			$tokenmodel = $this->_tokenmodel;
			$tokenactive = $tokenmodel->activerecord();
			$tokentable = $tokenmodel->table();
			$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
			// deal with each record individually, by session reference
			foreach ($changedrecords as $reference => $changedrecord) {
				if (is_null($changedrecord->ID)) { // new record
					$tokentable->preset();
					$result = $tokenactive->insert((array) $changedrecord)->save();
					if (!$result) {
						$errorinfo = $tokenactive->error_info();
					} else {
						$tokentable->preset()->query()->where('ID','=',$tokenactive->insert_id());
						$result = (bool) $tokenactive->current();
						if (!$result) $errorinfo = array('norecord','','Record not found');
					}
				} else { // existing record
					$tokentable->preset()->query()->where('ID','=',$changedrecord->ID);
					$result = $tokenactive->load()->update((array) $changedrecord)->save();
					if (!$result) $errorinfo = $tokenactive->error_info();
				}
				// prepare list item for refresh, by saving critical information
				// ... and clearing pointlist to force refresh
				$listitem = $listitems[$reference];
				if ($result) { // prepare for refresh
					$listitem->pointlist = array();
					$listitem->recordstatus = 'posted';
					$retrieverecords[$reference] = $tokenactive->current();
					$posted = TRUE;
				} else { // report errors
					$message = Kohana::message('tokens',$errorinfo[0]);
					if (!$message) $message = $errorinfo[2];
					$message .= ' [' . ($listitem->index + 1) . ']';
					$tokenerrors->errors[] = array('error'=> $message);
					DataModel::set_listitem_errorstate($listitem);
					$valid = FALSE;
				}
			}
		}
		if ($posted) {
			$tokenmessages->messages[] = array('message' => Kohana::message('tokens','updates'));
		} else {
			unset($datamodel->datapool->variables['tokenmessages']);
		}
		if ($valid) {
			unset($datamodel->datapool->variables['tokenerrors']);
		}
		return FALSE;
	}
	public function tokens_add($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		$datamodel->recordcaching(TRUE);
		$datamodel->cache->recordsets['tokenrecords'] = array(new StdClass());
		return FALSE;
	}
	public function tokens_delete($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		
		if (!isset($_POST['tokens']['deletecheckbox'])) {
			$deletions = array();
		} else {
			$deletions = $_POST['tokens']['deletecheckbox'];
		}
		if (empty($deletions)) {
			$message = 'no records were marked for deletion';
			$tokenmessages = $datamodel->datapool->variables['tokenmessages'] = 
				(object) array('messages' => array());
			$tokenmessages->messages[] = array('message' => $message);
			return FALSE;
		}
		// delete record,
		$listitems = &$datamodel->dataframe->pointlist['records']->pointlist;
		$tokenmodel = $this->_tokenmodel;
		$tokenactive = $tokenmodel->activerecord();
		$tokentable = $tokenmodel->table();
		$deletioncount = 0;
		foreach ($deletions as $deletion) {
			$listitem = $listitems[$deletion];
			$id = $listitem->pointlist['id']->value;
			if (!is_null($id)) {
				$tokentable->preset()->query()->where('ID','=',$id);
				$result = (bool) $tokenactive->current();
				if ($result) {
					$result = $tokenactive->delete();
				}
			}
			// ... and remove listitem,
			unset($listitems[$deletion]);
			$deletioncount++;
		}
		//  and recreate reference numbering in order
		DataModel::renumber_listitems($listitems,'token');
		// create feedback message
		$message = ($deletioncount) . ' records were deleted';
		$tokenmessages = $datamodel->datapool->variables['tokenmessages'] = 
			(object) array('messages' => array());
		$tokenmessages->messages[] = array('message' => $message);
		return FALSE;
	}
	public function tokens_cancel($datamodel)
	{
		return TRUE; // just redirect to refresh
	}
	public function tokens_generate($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		$db = User::instance()->db();
		$newrecords = $db->exec(
'INSERT IGNORE 
	INTO Access_Role_Tokens (RoleID,TokenID) 
	SELECT Access_Group_Roles.ID, Access_Tokens.ID
	FROM Access_Group_Roles, Access_Tokens'
		);
		$message = ($newrecords) . ' new records were generated';
		$tokenmessages = $datamodel->datapool->variables['tokenmessages'] = 
			(object) array('messages' => array());
		$tokenmessages->messages[] = array('message' => $message);
		return FALSE;
	}
	# ===============================================================
	# ---------------------[ tokens by role ]------------------------
	# ===============================================================
	protected function _tokensbyrole($dictionary)
	{
		$tokenmodel = $this->_tokenmodel = new TableModel('tokenschema',array('mastertable'=>'roles'));
		$datamodel = $this->_datamodel;
		$datapool = $datamodel->datapool;
		
		# set up data components in datapool
		// role table driver
		$roletable = $tokenmodel->table();
		$roletable->query()->order_by('GroupName')->order_by('Rank')->order_by('Order'); // default, so that first item is properly selected
		$datapool->activerecords['rolerecord'] = $roletable->activerecord();
		// roletoken table list
		$roletokentable = $datapool->tables['roletokentable'] = $tokenmodel->table('roletokens');
		$roletokenquery = $roletokentable->query();
		$datapool->activerecords['roletokenrecord'] = $roletokentable->activerecord();
		// token table support data
		$tokentable = $tokenmodel->table('tokens');
		$datapool->activerecords['tokenrecord'] = $tokentable->activerecord();
		# prepare for response
		$datamodel->dataframe('systemroletokens');
		if ($this->_is_request()) { // get input before retrieval and output
			$result = $datamodel
				->controlframe('systemroletokenscontrols')
				->responses(
					array(
						'role'=>array($this,'roletokens_role'),
						'save'=>array($this,'roletokens_save'),
						'cancel'=>array($this,'roletokens_cancel'),
					))
				->respond();
			if ($result) {
				$url = 'system/tokens/byroles';
				$this->request->redirect($url);
			}
		} 
		# prepare data for presentation
		$dictionary = $datamodel->retrieve()->output() + $dictionary;
		$view = Template::view(array('system_role_tokens'),$dictionary);
		return $view;
	}
	
	public function roletokens_role($datamodel)
	{
		$datamodel->datapool->constraints['changerole'] = TRUE;
		$datamodel->input()->finishinput();
		$roleoption = $datamodel->dataframe->pointlist['roleoptions']->value;
		$roletable = $this->_tokenmodel->table();
		$roletable->preset()->query()->where('ID','=',$roleoption);
		return FALSE;
	}
	public function roletokens_save($datamodel)
	{
		$roletokenerrors = $datamodel->datapool->variables['roletokenerrors'] = 
			(object) array('errors' => array());
		$roletokenmessages = $datamodel->datapool->variables['roletokenmessages'] = 
			(object) array('messages' => array());
		// no errors are possible, so validation not required, so
		//   go directly to store stage
		$datamodel
			->input()
			->finishinput()
			->recordcaching(TRUE)
			->store()
			->recordcaching(FALSE);
		$valid = TRUE; // unless contradicted below
		$posted = FALSE;
		// preserve current scope
		$roleid = $datamodel->dataframe->pointlist['roleid']->value;
		$tokenmodel = $this->_tokenmodel;
		$roletable = $tokenmodel->table();
		$roletable->preset()->query()->where('ID','=',$roleid);
		// changedrecords as stored by store() above
		$changedrecords = $datamodel->cache->recordsets['roletokenrecords'];
		if (empty($changedrecords)) {
			$roletokenmessages->messages[] = 
				array('message' => Kohana::message('roletokens','norecords'));
		} else {
			$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
			$roletokentable = $tokenmodel->table('roletokens');
			$roletokenactive = $roletokentable->activerecord();
			// no insertions, just do edits...
			foreach ($changedrecords as $reference => $changedrecord) {
				$roletokentable->preset()->query()->where('ID','=',$changedrecord->ID);
				$result = $roletokenactive->load()->update((array) $changedrecord)->save();
				$listitem = $listitems[$reference];
				if ($result) { // prepare for refresh
					$listitem->pointlist = array();
					$listitem->primarykeyvalue = $changedrecord->ID;
					$listitem->recordstatus = 'posted';
					$posted = TRUE;
				} else { // report errors
					$errorinfo = $roletokenactive->error_info();
					$message = Kohana::message('roletokens',$errorinfo[0]);
					if (!$message) $message = $errorinfo[2];
					$message .= ' [' . ($listitem->index + 1) . ']';
					$roletokenerrors->errors[] = array('error'=> $message);
					DataModel::set_listitem_errorstate($listitem);
					$valid = FALSE;
				}
			}
			$roletokentable->reset()->query()->reset();
		}
		if ($posted) {
			$roletokenmessages->messages[] = 
				array('message' => Kohana::message('roletokens','updates'));
		}
		if ($valid) {
			unset($datamodel->datapool->variables['roletokenerrors']);
		}
		return FALSE;
	}
	public function roletokens_cancel($datamodel)
	{
//		$datamodel->datapool->constraints['cancel'] = TRUE;
		$datamodel->input()->finishinput();
		$roleid = $datamodel->dataframe->pointlist['roleid']->value;
		// force refresh
		unset($datamodel->dataframe->pointlist['records']); // *** s/b done with constraint, but conflicts with 'changerole' (!)
		$roletable = $this->_tokenmodel->table();
		$roletable->preset()->query()->where('ID','=',$roleid);
		$roletokenmessages = $datamodel->datapool->variables['roletokenmessages'] = 
			(object) array('messages' => array());
		$roletokenmessages->messages[] = array('message' => Kohana::message('roletokens','cancelled'));
		
		return FALSE;
	}
	
	# ===============================================================
	# ---------------------[ roles by token ]------------------------
	# ===============================================================
	protected function _rolesbytoken($dictionary)
	{
		$tokenmodel = $this->_tokenmodel = $tokenmodel = new TableModel('tokenschema',array('mastertable'=>'tokens'));
		$datamodel = $this->_datamodel;
		$datapool = $datamodel->datapool;

		# set up data components in datapool
		// role table driver
		$tokentable = $tokenmodel->table();
		$tokentable->query()->order_by('Name'); // default, so that first item is properly selected
		$datapool->activerecords['tokenrecord'] = $tokentable->activerecord();
		// roletoken table list
		$tokenroletable = $datapool->tables['tokenroletable'] = $tokenmodel->table('roletokens');
		$datapool->activerecords['tokenrolerecord'] = $tokenroletable->activerecord();
		// token table support data
		$roletable = $tokenmodel->table('roles');
		$datapool->activerecords['rolerecord'] = $roletable->activerecord();
		# prepare for response
		$datamodel->dataframe('systemtokenroles');
		if ($this->_is_request()) { // get input before retrieval and output
			$result = $datamodel
				->controlframe('systemtokenrolescontrols')
				->responses(
					array(
						'token'=>array($this,'tokenroles_token'),
						'save'=>array($this,'tokenroles_save'),
						'cancel'=>array($this,'tokenroles_cancel'),
					))
				->respond();
			if ($result) {
				$url = 'system/tokens/bytokens';
				$this->request->redirect($url);
			}
		} 
		# prepare data for presentation
		$dictionary = $datamodel->retrieve()->output() + $dictionary;
		
		
		$view = Template::view(array('system_token_roles'),$dictionary);
		return $view;
	}
	
	public function tokenroles_token($datamodel)
	{
		$datamodel->datapool->constraints['changetoken'] = TRUE;
		$datamodel->input()->finishinput();
		$tokenoption = $datamodel->dataframe->pointlist['tokenoptions']->value;
		$tokentable = $this->_tokenmodel->table();
		$tokentable->preset()->query()->where('ID','=',$tokenoption);
		return FALSE;
	}
	public function tokenroles_save($datamodel)
	{
		$roletokenerrors = $datamodel->datapool->variables['tokenroleerrors'] = 
			(object) array('errors' => array());
		$roletokenmessages = $datamodel->datapool->variables['tokenrolemessages'] = 
			(object) array('messages' => array());
		// no errors are possible, so validation not required, so
		//   go directly to store stage
		$datamodel
			->input()
			->finishinput()
			->recordcaching(TRUE)
			->store()
			->recordcaching(FALSE);
		$valid = TRUE; // unless contradicted below
		$posted = FALSE;
		// preserve current scope
		$tokenid = $datamodel->dataframe->pointlist['tokenid']->value;
		$tokenmodel = $this->_tokenmodel;
		$tokentable = $tokenmodel->table();
		$tokentable->preset()->query()->where('ID','=',$tokenid);
		// changedrecords as stored by store() above
		$changedrecords = $datamodel->cache->recordsets['tokenrolerecords'];
		if (empty($changedrecords)) {
			$roletokenmessages->messages[] = 
				array('message' => Kohana::message('tokenroles','norecords'));
		} else {
			$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
			$tokenroletable = $tokenmodel->table('roletokens');
			$tokenroleactive = $tokenroletable->activerecord();
			// no insertions, just do edits...
			foreach ($changedrecords as $reference => $changedrecord) {
				$tokenroletable->preset()->query()->where('ID','=',$changedrecord->ID);
				$result = $tokenroleactive->load()->update((array) $changedrecord)->save();
				$listitem = $listitems[$reference];
				if ($result) { // prepare for refresh
					$listitem->pointlist = array();
					$listitem->primarykeyvalue = $changedrecord->ID;
					$listitem->recordstatus = 'posted';
					$posted = TRUE;
				} else { // report errors
					$errorinfo = $roletokenactive->error_info();
					$message = Kohana::message('tokenroles',$errorinfo[0]);
					if (!$message) $message = $errorinfo[2];
					$message .= ' [' . ($listitem->index + 1) . ']';
					$roletokenerrors->errors[] = array('error'=> $message);
					DataModel::set_listitem_errorstate($listitem);
					$valid = FALSE;
				}
			}
			$tokenroletable->reset()->query()->reset();
		}
		if ($posted) {
			$roletokenmessages->messages[] = 
				array('message' => Kohana::message('tokenroles','updates'));
		}
		if ($valid) {
			unset($datamodel->datapool->variables['tokenroleerrors']);
		}
		return FALSE;
	}
	public function tokenroles_cancel($datamodel)
	{
//		$datamodel->datapool->constraints['cancel'] = TRUE;
		$datamodel->input()->finishinput();
		$tokenid = $datamodel->dataframe->pointlist['tokenid']->value;
		unset($datamodel->dataframe->pointlist['records']); // *** s/b done with constraint, but conflicts with 'changerole' (!)
		$tokentable = $this->_tokenmodel->table();
		$tokentable->preset()->query()->where('ID','=',$tokenid);
		$tokenrolemessages = $datamodel->datapool->variables['tokenrolemessages'] = 
			(object) array('messages' => array());
		$tokenrolemessages->messages[] = array('message' => Kohana::message('tokenroles','cancelled'));
		
		return FALSE;
	}

	# ===============================================================
	# ------------------[ system properties ]------------------------
	# ===============================================================
	public function action_properties()
	{
		$access = Access::instance();
		$permission = $access->permission('system','R');
		if (!$permission)
			$this->request->redirect('error/system-disallowed');
			
		$propertiesmodel = $this->_propertiesmodel = new TableModel('systemschema',array('mastertable'=>'properties'));
		$datamodel = $this->_datamodel = new DataModel($this);
		$datapool = $datamodel->datapool;

		$base = $datamodel->datapool->variables['base'] = $this->_base;
		
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		$dictionary['propertiesclass'] = 'highlight';
		$technicalview = Template::view(array('system_sidebar'),$dictionary);

		$propertiestable = $datapool->tables['propertiestable'] = $propertiesmodel->table();
		$propertiesrecord = $datapool->activerecords['propertiesrecord'] = $propertiestable->activerecord();
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		
		$datamodel->dataframe('systemproperties');
		if ($this->_is_request()) { // get input before retrieval and output
			$result = $datamodel
				->controlframe('systempropertiescontrols')
				->responses(
					array(
						'save'=>array($this,'properties_save'),
						'add'=>array($this,'properties_add'),
						'delete'=>array($this,'properties_delete'),
						'cancel'=>array($this,'properties_cancel'),
					))
				->respond();
			if ($result) {
				$url = 'system/properties';
				$this->request->redirect($url);
			}
		} 
		
		$dictionary = $datamodel->retrieve()->output() + $dictionary;
		$view = Template::view(array('system_properties'),$dictionary);

		$pageview = $this->_pageview;
		$pageview->leftcolumn = $technicalview;
		$pageview->centercolumn = $view;
		$pageview->title = 'System Properties';
		$this->show_resources();
		$this->response->body($pageview);
	}
	public function properties_save($datamodel)
	{
		# set up objects to receive messages
		$errorsobject = $datamodel->datapool->variables['propertieserrors'] = 
			(object) array('errors' => array());
		$messagesobject = $datamodel->datapool->variables['propertiesmessages'] = 
			(object) array('messages' => array());
		# collect page data and validate data
		$valid = $datamodel
			->input()
			->errordictionaryindex('propertieserrors')
			->errormessageindex('properties')
			->validate();
		# store valid changed data
		$datamodel
			->finishinput()
			->recordcaching(TRUE)
			->store()
			->recordcaching(FALSE);
		$posted = FALSE;
		$propertiesmodel = $this->_propertiesmodel;
		# collect changedrecords as stored by store() above
		$changedrecords = $datamodel->cache->recordsets['propertiesrecords'];
		if (empty($changedrecords)) { // nothing to do
			$messagesobject->messages[] = 
				array('message' => Kohana::message('properties','norecords'));
		} else { // post changed records
			# collect objects to manipulate
			$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
			$propertiestable = $propertiesmodel->table('properties');
			$propertiesactive = $propertiestable->activerecord();
			# process each changed record in turn
			foreach ($changedrecords as $reference => $changedrecord) {
				if (!is_null($changedrecord->ID)) { // save changes to existing record
					$propertiestable->preset()->query()->where('ID','=',$changedrecord->ID);
					$result = $propertiesactive->load()->update((array) $changedrecord)->save();
					if (!$result) $errorinfo = $propertiesactive->error_info();
				} else { // save new record
					$propertiestable->preset();
					$result = $propertiesactive->insert((array) $changedrecord)->save();
					if (!$result) { // collect error info
						$errorinfo = $propertiesactive->error_info();
					} else { // retrieve new saved record
						$propertiestable->preset()->query()->where('ID','=',$propertiesactive->insert_id());
						$result = (bool) $propertiesactive->current();
						if (!$result) $errorinfo = array('norecord','','Record not found');
					}
				}
				$listitem = $listitems[$reference]; // get matching memory structure
				if ($result) { // prepare for refresh
					$listitem->pointlist = array();
					$listitem->primarykeyvalue = $propertiesactive->ID;
					$listitem->recordstatus = 'posted';
					$posted = TRUE;
				} else { // report errors
					$message = Kohana::message('properties',$errorinfo[0]);
					if (!$message) $message = $errorinfo[2];
					$message .= ' [' . ($listitem->index + 1) . ']';
					$errorsobject->errors[] = array('error'=> $message);
					DataModel::set_listitem_errorstate($listitem);
					$valid = FALSE;
				}
			}
			$propertiestable->preset();
		}
		if ($posted) {
			$messagesobject->messages[] = 
				array('message' => Kohana::message('properties','updates'));
		}
		if ($valid) {
			unset($datamodel->datapool->variables['propertieserrors']);
		}
		return FALSE;
	}
	public function properties_add($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		$datamodel->datapool->variables['newrecordcount'] = 1;
		return FALSE;
	}
	public function properties_delete($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		
		$propertiesmodel = $this->_propertiesmodel;
		# collect deletions
		if (!isset($_POST['properties']['deletecheckbox'])) {
			$deletions = array();
		} else {
			$deletions = $_POST['properties']['deletecheckbox'];
		}
		// exit if no deletions
		if (empty($deletions)) {
			$message = 'no records were marked for deletion';
			$messagesobject = $datamodel->datapool->variables['propertiesmessages'] = 
				(object) array('messages' => array());
			$messagesobject->messages[] = array('message' => $message);
			return FALSE;
		}
		# establish error object
		$errorsobject = $datamodel->datapool->variables['propertieserrors'] = 
			(object) array('errors' => array());
		# get record cross reference items
		$listitems = &$datamodel->dataframe->pointlist['records']->pointlist;
		$propertiestable = $propertiesmodel->table('properties');
		$propertiesactive = $propertiestable->activerecord();
		$deletioncount = 0;
		# delete records
		$errors = FALSE;
		foreach ($deletions as $deletion) { // deletion is reference: <string><index>
			$listitem = $listitems[$deletion];
			$id = $listitem->pointlist['id']->value;
			$result = TRUE;
			if (!is_null($id)) { // record exists, delete it
				$propertiestable->preset()->query()->where('ID','=',$id);
				$result = (bool) $propertiesactive->current();
				if ($result) {
					$result = $propertiesactive->delete();
				} else {
					$result = TRUE; // record already gone
				}
			}
			// ... and remove listitem,
			if ($result){
				unset($listitems[$deletion]);
				$deletioncount++;
			} else {
				$errors = TRUE;
				$errorinfo = $propertiesactive->error_info();
				$error = Kohana::message('properties',$errorinfo[0]);
				if (!$error) $error = $errorinfo[2];
				$error .= ' [' . ($listitem->index + 1) . ']';
				DataModel::set_listitem_errorstate($listitem);
				$roleerrors->errors[] = array('error'=>$error);
			}
		}
		DataModel::renumber_listitems($listitems,'properties');
		// create feedback message
		$message = ($deletioncount) . ' records were deleted';
		$messagesobject = $datamodel->datapool->variables['propertiesmessages'] = 
			(object) array('messages' => array());
		$messagesobject->messages[] = array('message' => $message);
		if (!$errors) unset($datamodel->datapool->variables['propertieserrors']);
		return FALSE;
	}
	public function properties_cancel($datamodel)
	{
		return FALSE;
	}

	# ===============================================================
	# ---------------------[ repositories ]--------------------------
	# ===============================================================
	// ** datamodel access to input records is way too convoluted -- 
	// 	see notes in gateway _retrieve_data_list_tableactiverecord
	public function action_repositories()
	{
		$access = Access::instance();
		$permission = $access->permission('system','R');
		if (!$permission)
			$this->request->redirect('error/system-disallowed');
			
		$repositoriesmodel = $this->_repositoriesmodel = new TableModel('repositoriesschema');
		$datamodel = $this->_datamodel = new DataModel($this);
		$datapool = $datamodel->datapool;

		$base = $datamodel->datapool->variables['base'] = $this->_base; //  ** assign to dictionary
		
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		$dictionary['repositoriesclass'] = 'highlight';
		$technicalview = Template::view(array('system_sidebar'),$dictionary);

		$repositoriestable = $datapool->tables['repositoriestable'] = $repositoriesmodel->table();
		$query = $repositoriestable->query();
		$query->limit(300); // until pagination is added
//		echo $query->sql();
		$repositoriesrecord = $datapool->activerecords['repositoriesrecord'] = $repositoriestable->activerecord();
		$dictionary = array();
		$dictionary['base'] = $base;
		$dictionary['common'] = $this->_common;
		
		$datamodel->dataframe('systemrepositories');
		if ($this->_is_request()) { // get input before retrieval and output
			$result = $datamodel
				->controlframe('systemrepositoriescontrols')
				->responses(
					array(
						'save'=>array($this,'repositories_save'),
						'add'=>array($this,'repositories_add'),
						'delete'=>array($this,'repositories_delete'),
						'cancel'=>array($this,'repositories_cancel'),
						'assertdirectories'=>array($this,'repositories_assert'),
						'importdirectories'=>array($this,'repositories_import'),
						'reconciledirectories'=>array($this,'repositories_reconcile'),
						'removeblanks'=>array($this,'repositories_remove_blanks'),
					))
				->respond();
			if ($result) {
				$redirect = 'system/repositories';
				$this->request->redirect($redirect);
			}
		} 
		
		$session = Session::instance();
		$messages = $session->get_once('repositorymessages');
		if ($messages) {
			$index = $messages['index'];
			unset($messages['index']);
			$this->_datamodel->datapool->variables[$index] = (object) $messages;
		}
		$dictionary = $datamodel->retrieve()->output() + $dictionary;
		$view = Template::view(array('system_repositories'),$dictionary);

		$pageview = $this->_pageview;
		$pageview->leftcolumn = $technicalview;
		$pageview->centercolumn = $view;
		$pageview->title = 'System Repositories';
		$this->show_resources();
		$this->response->body($pageview);
	}

	public function repositories_save($datamodel)
	{
		# set up objects to receive messages
		$messagesobject = $datamodel->datapool->variables['messagesection'] = new StdClass;
		$messagesobject->errors =  array();
		$messagesobject->messages = array();
		# collect page data and validate data
		$valid = $datamodel
			->input()
			->errordictionaryindex('messagesection')
			->errormessageindex('messagesection')
			->validate();
		# store valid changed data
		$datamodel
			->finishinput()
			->recordcaching(TRUE)
			->store()
			->recordcaching(FALSE);
		$posted = FALSE;
		$repositoriesmodel = $this->_repositoriesmodel;
		$articlemodel = $this->_articlesmodel = new TableModel('nodeschema');
		# collect changedrecords as stored by store() above
		$changedrecords = $datamodel->cache->recordsets['repositoriesrecords'];
		if (empty($changedrecords)) { // nothing to do
			$messagesobject->messages[] = 
				array('message' => Kohana::message('repositories','norecords'));
		} else { // post changed records
			# collect objects to manipulate
			$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
			$repositoriestable = $repositoriesmodel->table('repositories');
			$repositoriestable->auto_lock(FALSE);
			$repositoriesactive = $repositoriestable->activerecord();
			$articleextensiontable = $articlemodel->table('extension');
			$articlemastertable = $articlemodel->table('master');
			$articlemastertable->auto_lock(FALSE);
			$articlemasteractive = $articlemastertable->activerecord();
			$articleextensiontable->auto_lock(FALSE);
			$articleextensionactive = $articleextensiontable->activerecord();
			$db = $repositoriesmodel->db();
			$directorybinderid = Services::dblookup('Binders','ID','Code','Directories');
			$db->lock_tables(array('System_Repositories','Nodes_View'));
			# process each changed record in turn
			try {
				foreach ($changedrecords as $reference => $changedrecord) {
					if (!is_null($changedrecord->ID)) { // save changes to existing record
						$id = $changedrecord->ID;
						$repositoriestable->preset()->query()->where('ID','=',$changedrecord->ID);
						$result = $repositoriesactive->load()->update((array) $changedrecord)->save();
						if (!$result) {
							$errorinfo = $repositoriesactive->error_info();
						} else { // update Directory record
							$repositoriestable->preset()->query()->where('ID','=',$id);
							$repositoriesactive->load();
							$articlemastertable->preset()->query()
								->where('RepositoryID','=',$id)
								->where('ObjectType','=','directory');
							if ($record = $articlemasteractive->current()) {
								$articlemasteractive
									->update(array('Title'=>$repositoriesactive->Directory))
									->save();
							} else { // add directory record
								$success = $articlemasteractive
									->insert(array(
										'Title'=>$repositoriesactive->Directory,
										'ArticleDate'=>date('Y-m-d'),										
										'ObjectType'=>'directory',
										'BinderID'=>$directorybinderid,
										'Approval'=>'A',
										'CommentsClosed'=>1,
										'Hide'=>1,
										'Locked'=>1,
									))->save();
								if ($success) {
									$articleid = $articlemasteractive->insert_id();
									$articlemastertable->preset()->query()->where('ID','=',$articleid);
									$articleextensionactive->insert(array('RepositoryID'=>$id))->save();
								}
							}
						}
					} else { // save new record
						$repositoriestable->preset();
						$result = $repositoriesactive->insert((array) $changedrecord)->save();
						if (!$result) { // collect error info
							$errorinfo = $repositoriesactive->error_info();
						} else { // retrieve new saved record
							$id = $repositoriesactive->insert_id();
							$repositoriestable->preset()->query()->where('ID','=',$id);
							$result = (bool) $repositoriesactive->current();
							if (!$result) {
								$errorinfo = array('norecord','','Record not found');
							} else { // create Directory record
								$success = $articlemasteractive
									->insert(array(
										'Title'=>$repositoriesactive->Directory,
										'ArticleDate'=>date('Y-m-d'),
										'ObjectType'=>'directory',
										'BinderID'=>$directorybinderid,
										'Approval'=>'A',
										'CommentsClosed'=>1,
										'Hide'=>1,
										'Locked'=>1,
									))->save();
								if ($success) {
									$articleid = $articlemasteractive->insert_id();
									$articlemastertable->preset()->query()->where('ID','=',$articleid);
									$articleextensionactive->insert(array('RepositoryID'=>$id))->save();
								}
							}
						}
					}
					$listitem = $listitems[$reference]; // get matching memory structure
					if ($result) { // prepare for refresh
						$listitem->pointlist = array();
						$listitem->primarykeyvalue = $repositoriesactive->ID;
						$listitem->recordstatus = 'posted';
						$posted = TRUE;
					} else { // report errors
						$message = Kohana::message('repositories',$errorinfo[0]);
						if (!$message) $message = $errorinfo[2];
						$message .= ' [' . ($listitem->index + 1) . ']';
						$messagesobject->errors[] = array('error'=> $message);
						DataModel::set_listitem_errorstate($listitem);
						$valid = FALSE;
					}
				}
			} catch (Exception $e) {
				echo $e->getMessage();
			}
			$db->unlock_tables();
			$repositoriestable->preset();
		}
		if ($posted) {
			$messagesobject->messages[] = 
				array('message' => Kohana::message('repositories','updates'));
		}
		if ($valid) {
		}
		return FALSE;
	}
	public function repositories_add($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		$datamodel->datapool->variables['newrecordcount'] = 1;
		return FALSE;
	}
	// all related records must be deleted first, including the directory record
	public function repositories_delete($datamodel)
	{
		$datamodel->input(Point::INPUT_RECYCLE);
		
		$repositoriesmodel = $this->_repositoriesmodel;
		# collect deletions
		if (!isset($_POST['repositories']['selectcheckbox'])) {
			$deletions = array();
		} else {
			$deletions = $_POST['repositories']['selectcheckbox'];
		}
		// exit if no deletions
		$messagesobject = $datamodel->datapool->variables['messagesection'] = new StdClass;
		if (empty($deletions)) {
			$message = 'no records were marked for deletion';
			$messagesobject->messages = array();
			$messagesobject->messages[] = array('message' => $message);
			return FALSE;
		}
		# establish error object
		$messagesobject->errors = array();
		# get record cross reference items
		$listitems = &$datamodel->dataframe->pointlist['records']->pointlist;
		$repositoriestable = $repositoriesmodel->table('repositories');
		$repositoriesactive = $repositoriestable->activerecord();
		$deletioncount = 0;
		# delete records
		$errors = FALSE;
		$repositoriestable->auto_lock(FALSE);
		$db = $repositoriesmodel->db();
		$db->lock_tables(array('System_Repositories','Nodes_View'));
		try {
			foreach ($deletions as $deletion) { // deletion is reference: <string><index>
				$listitem = $listitems[$deletion];
				$id = $listitem->pointlist['id']->value;
				$directory = $listitem->pointlist['Directory']->value;
				$result = TRUE;
				if (!is_null($id)) { // record exists, delete it
					$stop_delete = TRUE;
					$operation = DB::select('ID')
						->from('Nodes_View')
						->where('RepositoryID','=',$id)
						->limit(1)
						->as_object()
						->prepare($db)
						->execute();
					if ($operation->success()) {
						$stop_delete = (bool) $operation->current();
						if ($stop_delete) {
							$errorinfo = array('delete','dependent','dependent record exists for ' . $directory);
							$result = FALSE;
						}
					} else {
						$errorinfo = $operation->error_info();
						$result = FALSE;
					}
					if ($result) {
						$repositoriestable->preset()->query()->where('ID','=',$id);
						$result = (bool) $repositoriesactive->current();
						if ($result) {
							$result = $repositoriesactive->delete();
						} else {
							$result = TRUE; // record already gone
						}
					}
				}
				// ... and remove listitem,
				if ($result){
					unset($listitems[$deletion]);
					$deletioncount++;
				} else {
					if (empty($errorinfo))
						$errorinfo = array('error','unknown','unknown error occurred deleting repository record');
					$errors = TRUE;
//					$error = Kohana::message('repositories',$errorinfo[0]);
					$error = $errorinfo[2];
					$error .= ' [' . ($listitem->index + 1) . ']';
					DataModel::set_listitem_errorstate($listitem);
					$messagesobject->errors[] = array('error'=>$error);
				}
			}
		} catch (Exception $e) {
			echo $e->getMessage();
		}
		$db->unlock_tables();
		DataModel::renumber_listitems($listitems,'repositories');
		// create feedback message
		$message = ($deletioncount) . ' records were deleted';
		$messagesobject->messages[] = array('message' => $message);
		if (!$errors) {
			$messagesection = (array) $messagesobject;
			$messagesection['index'] = 'messagesection';
			$session = Session::instance();
			$session->set('repositorymessages',$messagesection);
			return TRUE; // recycle
		} else {
			return FALSE;
		}
	}
	public function repositories_cancel($datamodel)
	{
		return TRUE; // recycle
	}
	public function repositories_assert($datamodel)
	{
		# set up objects to receive messages
		$messagesobject = $datamodel->datapool->variables['messagesection'] = new StdClass;
		$messagesobject->errors = array();
		$messagesobject->messages = array();
		# collect page data and validate data
		$valid = $datamodel
			->input()
			->errordictionaryindex('messagesection')
			->errormessageindex('messagesection')
			->validate();
		# store valid changed data
		$datamodel
			->finishinput()
			->recordcaching(TRUE)
			->store()
			->recordcaching(FALSE);
		$listitems = $datamodel->dataframe->pointlist['records']->pointlist;
		$base  = $this->_base;
		$publicbase = System::instance()->PublicBase;
		$uploadbase = System::instance()->UploadBase;
		$messagesobject->messages[] = array('message'=> 'all directories were asserted');
		foreach($listitems as $listitem) {
			$directory = $listitem->pointlist['Directory']->value;
			$path = $publicbase.$uploadbase.$directory;
			$isdir = is_dir($path);
			if (!$isdir) {
				$success = mkdir($path,0755,TRUE);
				if (!$success) {
					$messagesobject->errors[] = array('error'=> 'error attempting to create ' . $directory);
				} else {
					$messagesobject->messages[] = array('message'=> $directory . ' was created');
				}
			}
		}
		return FALSE;
	}
	public function repositories_remove_blanks($datamodel)
	{
		$parameters = $datamodel->control_input()->get_parameters();
		$publicbase = System::instance()->PublicBase;
		$uploadbase = System::instance()->UploadBase;
		$datamodel->input(Point::INPUT_RECYCLE);
		if (!isset($_POST['repositories']['selectcheckbox'])) {
			$selections = array();
		} else {
			$selections = $_POST['repositories']['selectcheckbox'];
		}
		$messages = array();
		$count = 0;
		try {
			$listitems = &$datamodel->dataframe->pointlist['records']->pointlist;
			$filerconfig = (object) Kohana::$config->load('filer');
			$filetypes = $filerconfig->types;
			foreach ($selections as $selection) {
				$listitem = $listitems[$selection];
				$id = $listitem->pointlist['id']->value;
				$directory = $listitem->pointlist['Directory']->value;
				$messages[] = array('message'=> 'directory: ' . $directory);
				if (!is_null($id)) {// record exists
					$path = $publicbase.$uploadbase.$directory; // relative addressing
					$messages[] = array('message'=> 'path: ' . $path);
					if (strpos($path,'..')!== FALSE) $continue; // invalid directory path
					if (is_dir($path)) {
						$filesraw = scandir($path); // get list of disk files/directories
						$files = array();
						foreach ($filesraw as $file) {// filter for files
							if (is_file($path.$file)) {
								$files[] = $file;
							}
						}
						if (!empty($files)) { // process files
							foreach ($files as $file) { // process each file
								// get tablemodel for file records
								$filename = pathinfo($file,PATHINFO_FILENAME);
								if (strpos($filename,' ')=== FALSE) { // no blank to fix
									continue;
								}
								$filetype = $this->_get_file_type($file,$filetypes);
								if ($filetype === FALSE) { // not a muster file type
									continue;
								}
								if ($filetype === Filer::FILETYPE_IMAGE) { // screen out versions - couldn't have gotten this far
									// filter out versions
									if (preg_match('/[.](largest|larger|large|medium|small|smaller|smallest|thumbnail)$/',$filename))
										continue;
								}
								$oldspec = $path.$file;
								$fileinfo = pathinfo($file);
								$newspec = $path.strtr($fileinfo['filename'],' ','_').'.'.$fileinfo['extension'];
								rename($oldspec,$newspec);
								$messages[] = array('message'=>$oldspec . ' => ' . $newspec);
								$count++;
							}
						}
					}
				}
			}
		} catch (Exception $e) {
			$messages[] = array('message' => 'error encountered in system.php repositories_remove_blanks');
			$messages[] = array('message' => htmlspecialchars(var_export($e->getMessage(), TRUE)));
		}
		$messages[] = array('message'=> $count . ' files renamed');
		$messagesection = compact('messages');
		$messagesection['index'] = 'messagesection';
		$session = Session::instance();
		$session->set('repositorymessages',$messagesection);
		return TRUE;
	}
	protected function _get_file_type($filename,$filetypes)
	{
		$extension = pathinfo($filename,PATHINFO_EXTENSION);
		if (empty($extension)) return FALSE;
		$extension = strtolower($extension);
		foreach ($filetypes as $type) {
			if (in_array($extension,$type['extensions']))
				return $type['type_code'];
		}
		return FALSE;
	}
	public function repositories_import($datamodel)
	{
		$parameters = $datamodel->control_input()->get_parameters();
		$publicbase = System::instance()->PublicBase;
		$uploadbase = System::instance()->UploadBase;
		$datamodel->input(Point::INPUT_RECYCLE);
		if (!isset($_POST['repositories']['selectcheckbox'])) {
			$selections = array();
		} else {
			$selections = $_POST['repositories']['selectcheckbox'];
		}
		$articlemodel = $this->_articlesmodel = new TableModel('nodeschema');
		$repositoriesmodel = $this->_repositoriesmodel;
		$table = $repositoriesmodel->table();
		$table->auto_lock(FALSE);
		$db = $repositoriesmodel->db();
		$activerecord = $table->activerecord();
		$db->lock_tables(array('System_Repositories','Nodes_View'));
		$directorybinderid = Services::dblookup('Binders','ID','Code','Directories');
		try {
			$listitems = &$datamodel->dataframe->pointlist['records']->pointlist;
			foreach ($selections as $selection) {
				$listitem = $listitems[$selection];
				$id = $listitem->pointlist['id']->value;
				$path = $listitem->pointlist['Directory']->value;
				if (!is_null($id)) {// record exists
					$this->_import_repository($publicbase,$uploadbase,$path,$activerecord,$directorybinderid);
				}
			}
		} catch (Exception $e) {
			var_dump($e->getMessage());
		}
		$db->unlock_tables();
		return TRUE;
	}
	protected function _import_repository($publicbase,$uploadbase,$path,$activerecord,$directorybinderid)
	{
		$startingpath = $publicbase.$uploadbase.$path;
		$baselength = strlen($publicbase.$uploadbase);
		$articlesmodel = $this->_articlesmodel;
		$articlemastertable = $articlesmodel->table();
		$articlemasteractive = $articlemastertable->activerecord();
		$articleextensionactive = $articlesmodel->activerecord('extension');
		$iterator = 
			new RecursiveIteratorIterator(
				new RecursiveDirectoryIterator($startingpath), RecursiveIteratorIterator::SELF_FIRST);
		foreach($iterator as $file) {
			if($file->isDir()) {
				$pathname = $file->getPathname();
				$basename = pathinfo($pathname,PATHINFO_BASENAME);
				if ((!empty($basename) and ($basename != '.') and ($basename !='..'))) {
					$directory = substr($pathname,$baselength).'/';
					$success = $activerecord->insert(array('Directory'=>$directory))->save();
					if ($success) { // create directory record
						$repositoryid = $activerecord->insert_id();
						$success = $articlemasteractive
							->insert(array(
								'Title'=>$directory,
								'ArticleDate'=>date('Y-m-d'),
								'ObjectType'=>'directory',
								'BinderID'=>$directorybinderid,
								'Approval'=>'A',
								'CommentsClosed'=>1,
								'Hide'=>1,
								'Locked'=>1,
							))->save();
						if ($success) {
							$articleid = $articlemasteractive->insert_id();
							$articlemastertable->preset()->query()->where('ID','=',$articleid);
							$articleextensionactive->insert(array('RepositoryID'=>$repositoryid))->save();
						}
					}
				}
			}
		}	
	}
	public function repositories_reconcile($datamodel)
	{
		$base  = $this->_base;
		$publicbase = System::instance()->PublicBase;
		$uploadbase = System::instance()->UploadBase;
		$db = Database::instance();
		$articlesmodel = new TableModel('nodeschema');
		$articlemastertable = $articlesmodel->table();
		$articlemasteractive = $articlemastertable->activerecord();
		$articleextensionactive = $articlesmodel->activerecord('extension');
		$messsages = array();
		$warnings = array();
		$errors = array();
		$db->lock_tables(array('Nodes_View','Nodes_Instances_View','System_Repositories'));
		$directorybinderid = Services::dblookup('Binders','ID','Code','Directories');
		try {
		# 1. check for orphan directory records
			// System_Directories_Orphans_View
			$recordset = DB::select('*')
				->from('System_Directories_Orphans_View')
				->as_object()
				->prepare($db)
				->execute();
			if ($recordset->success()) {
				while ($record = $recordset->fetch()) {
					$id = $record->ArticleID;
					$warnings[] = array('warning'=>'deleting orphaned directory record ' . $id);
					$operation = DB::delete('Articles_Extensions')
						->where('ArticleID','=',$id)
						->prepare($db)
						->execute();
					$operation = DB::delete('Articles')
						->where('ID','=',$id)
						->prepare($db)
						->execute();
				}
				$warnings[] = array('warning'=>'number of orphaned directory records deleted: ' . $recordset->row_count());
			}
			# 2. check for orphan repository records; create records
			$recordset = DB::select('*')
				->from('System_Repositories_Orphans_View')
				->as_object()
				->prepare($db)
				->execute();
			$number_of_directory_records_created = $recordset->row_count();
			if ($recordset->success()) {
				while ($record = $recordset->fetch()) {
					$repositoryid = $record->RepositoryID;
					$directory = $record->Directory;
					$messages[] = array('message'=>'creating directory record for ' . $repositoryid . ': ' . $directory);
					$success = $articlemasteractive
						->insert(array(
							'Title'=>$directory,
							'ArticleDate'=>date('Y-m-d'),
							'ObjectType'=>'directory',
							'BinderID'=>$directorybinderid,
							'Approval'=>'A',
							'CommentsClosed'=>1,
							'Hide'=>1,
							'Locked'=>1,
						))->save();
					if ($success) {
						$articleid = $articlemasteractive->insert_id();
						$articlemastertable->preset()->query()->where('ID','=',$articleid);
						$articleextensionactive->insert(array('RepositoryID'=>$repositoryid))->save();
					} else {
						$errorinfo = $articlemasteractive->error_info();
						if (!empty($errorinfo[2]))
							$errmessage = $errorinfo[2];
						else
							$errmessage = 'unable to add directory record for ' . $directory;
						$errors[] = array('error'=>$errmessage);
					}
				}
			}
			$messages[] = array('message'=>'number of directory records created: ' . $number_of_directory_records_created);
			# 3. add file record instances to directories
			$sql = 'INSERT IGNORE INTO `Nodes_Instances` (`FolderID`,`NodeID`) SELECT `FolderID`,`NodeID` from `System_Select_FileDirectoryInstances_View`';
			$number_of_rows_affected = $db->exec($sql);
			$messages[] = array('message'=>'number of instances created: ' . $number_of_rows_affected);
			# 4. check for orphan file records (not assigned to a directory)
			$recordset = DB::select('*')
				->from('System_Files_Orphans_View')
				->as_object()
				->prepare($db)
				->execute();
			$number_of_orphan_files = $recordset->row_count();
			$warnings[] = array('warning'=>'number of remaining orphan file records: ' . $number_of_orphan_files);
			
		} catch (Exception $e) {
			$errors[] = array('error'=>$e->getMessage());
		}
		$db->unlock_tables();
		$messagesection = compact('errors','messages','warnings');
		if (!empty($errors)) {
			$datamodel->datapool->variables['messagesection'] = (object) $messagesection;
			return FALSE;
		} else {
			$messagesection['index'] = 'messagesection';
			$session = Session::instance();
			$session->set('repositorymessages',$messagesection);
			return TRUE;
		}
	}
}
